/**
 * Copyright (C) 2002
 */

package cmp2_0;

import cmp2_0.customer.CustomerHomeRemote;
import cmp2_0.customer.CustomerRemote;
import cmp2_0.customer.Name;
import cmp2_0.creditcard.CreditCardHomeRemote;
import cmp2_0.creditcard.CreditCardRemote;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class Client {

    private static UserTransaction utx = null;

    public static void main(String[] args) {

        // get JNDI initial context
        Context initialContext = null;
        try {
            initialContext = new InitialContext();
        }
        catch (Exception e) {
            System.err.println("Cannot get initial context for JNDI: " + e);
            System.exit(2);
        }

        // We want to start transactions from client: get UserTransaction
        System.out.println("Getting a UserTransaction object from JNDI");
        try {

            // Comment the following lines if you want to use a David Client:
            utx = (UserTransaction) initialContext.lookup("javax.transaction.UserTransaction");

            // Uncomment the following lines if you want to use a David Client:
            /*
            Properties prop = new Properties();
            prop.put("java.naming.factory.initial", "com.sun.jndi.rmi.registry.RegistryContextFactory");
            prop.put("java.naming.provider.url", "rmi://localhost:1099"); // update the port if necessary
            InitialContext ictx = new InitialContext(prop);
            utx = (UserTransaction) ictx.lookup("javax.transaction.UserTransaction");
            */

        } catch (Exception e) {
            System.err.println("Cannot lookup UserTransaction: "+e);
            System.exit(2);
        }

        // Connecting to Home thru JNDI
        System.out.println("Connecting to the CustomerHomeRemote");
        CustomerHomeRemote home1 = null;
        CreditCardHomeRemote home2 = null;
        String beanName = null;
        try {
            beanName = "cmp2_0.CustomerEjb";
            home1 = (CustomerHomeRemote) PortableRemoteObject.narrow(
                initialContext.lookup(beanName),
                CustomerHomeRemote.class);

            beanName = "cmp2_0.CreditCardEjb";
            home2 = (CreditCardHomeRemote) PortableRemoteObject.narrow(
                initialContext.lookup(beanName),
                CreditCardHomeRemote.class);
        }
        catch (Exception e) {
            System.err.println("Cannot lookup " + beanName + ": " + e);
            System.exit(2);
        }
        String command = "";
        while (command != null) {
            System.out.println("-------------------------------------------------|");
            System.out.println("| Commands:                                      |");
            System.out.println("| f1 - find all Customer                         |");
            System.out.println("| f2 - find all Credit Card                      |");
            System.out.println("| c1 - create new Customer                       |");
            System.out.println("| c2 - create new Credit card                    |");
            System.out.println("| l1 - load an existing Customer                 |");
            System.out.println("| l2 - load an existing Credit card              |");
            System.out.println("| a1 - assign to a Customer a new Credit Card    |");
            System.out.println("| a2 - assign to a Customer a Credit Card by id  |");
            System.out.println("| r1  - remove an existing Customer              |");
            System.out.println("| r2  - remove an existing Credit Card           |");
            System.out.println("| q  - exit from the client                      |");
            System.out.println("-------------------------------------------------|");
            try {
                command = readString("=>");
                if ("f1".equalsIgnoreCase(command.trim())) {
                    findAllCustomer(home1);
                }
                else if ("f2".equalsIgnoreCase(command.trim())) {
                    findAllCreditCard(home2);
                }
                else if ("c1".equalsIgnoreCase(command.trim())) {
                    Integer id = Integer.valueOf(readString("Enter the id of the customer:"));
                    String ln = readString("Enter the last name  of the customer:");
                    String fn = readString("Enter the first name  of the customer:");
                    boolean hgc =
                        Boolean.getBoolean(readString("Has a good credit (true|false):"));
                    boolean setLink =
                        !"n".equals(readString("Do you want to set a CreditCard (y|n):"));
                    Integer ccId = null;
                    if (setLink) {
                        ccId = Integer.valueOf(readString("Enter the id of the creditcard:"));
                    }
                    createCustomer(home1, id, ln, fn, hgc, ccId);
                }
                else if ("c2".equalsIgnoreCase(command.trim())) {
                    Integer id = Integer.valueOf(readString("Enter the id of the credit card:"));
                    String numb = readString("Enter the number of the credit card:");
                    createCreditCard(home2, id, numb);
                }
                else if ("l1".equalsIgnoreCase(command.trim())) {
                    Integer id = Integer.valueOf(readString("Enter the id of the customer:"));
                    loadCustomer(home1, id);
                }
                else if ("l2".equalsIgnoreCase(command.trim())) {
                    Integer id = Integer.valueOf(readString("Enter the id of the credit card:"));
                    loadCreditCard(home2, id);
                }
                else if ("a1".equalsIgnoreCase(command.trim())) {
                    Integer custId = Integer.valueOf(readString("Enter the id of the customer:"));
                    Integer ccId = Integer.valueOf(readString("Enter the id of the new credit card:"));
                    String numb = readString("Enter the number of the new credit card:");
                    assignCC2Customer(home1, custId, ccId, numb);
                }
                else if ("a2".equalsIgnoreCase(command.trim())) {
                    Integer custId = Integer.valueOf(readString("Enter the id of the customer:"));
                    Integer ccId = Integer.valueOf(readString("Enter the id of the credit card:"));
                    assignCC2Customer(home1, custId, home2, ccId);
                }
                else if ("r1".equalsIgnoreCase(command.trim())) {
                    Integer id = Integer.valueOf(readString("Enter the id of the customer:"));
                    removeCustomer(home1, id);
                }
                else if ("r2".equalsIgnoreCase(command.trim())) {
                    Integer id = Integer.valueOf(readString("Enter the id of the credit card:"));
                    removeCreditCard(home2, id);
                }
                else if ("q".equalsIgnoreCase(command.trim())) {
                    command = null;
                    // Exit program
                    System.out.println("Client terminated");
                }
            }
            catch (Exception e) {
                e.printStackTrace();
                System.exit(-2);
            }
        }
    }

    public static CustomerRemote createCustomer(CustomerHomeRemote home,
                                      Integer id,
                                      String ln,
                                      String fn,
                                      boolean hgc,
                                      Integer ccId) throws Exception {

        // Create a first Account
        System.out.println("Creating a new Customer in database");
        CustomerRemote c1 = null;
        try {
            utx.begin();	// faster if made inside a Tx
            c1 = home.create(id, ccId);
        }
        catch (Exception e) {
            System.err.println("Cannot create Customer: " + e);
            System.exit(2);
        }

        // Fill the customer with data
        try {
            Name name = new Name(ln, fn);
            c1.setName(name);
            c1.assignHasGoodCredit(hgc);
            utx.commit();
        }
        catch (Exception e) {
            System.err.println("Cannot fill data into the Customer bean: " + e);
            System.exit(2);
        }
        checkCustomer(c1, ln, fn, hgc, ccId);
        return c1;
    }

    public static CreditCardRemote createCreditCard(CreditCardHomeRemote home,
                                      Integer id,
                                      String numb) throws Exception {

        // Create a first Account
        System.out.println("Creating a new CreditCard in database");
        CreditCardRemote c1 = null;
        try {
            utx.begin();	// faster if made inside a Tx
            c1 = home.create(id, numb);
            utx.commit();
        }
        catch (Exception e) {
            System.err.println("Cannot create Customer: " + e);
            System.exit(2);
        }

        checkCreditCard(c1, numb);
        return c1;
    }

    public static void checkCustomer(CustomerRemote c1,String ln,
                                     String fn, boolean hgc,
                                     Integer ccId) throws Exception {
        // Check if the data exist
        try {
            Name name = c1.getName();
            if (name==null) {
                System.out.println("returned name is null");
                System.exit(-2);
            } else if (!fn.equals(name.getFirstName())
                    || !ln.equals(name.getLastName())) {
                System.out.println("Bad name: ln=" + name.getLastName()
                    + " / fn=" + name.getFirstName());
                System.exit(-2);
            }
            if (hgc == c1.HasGoodCredit()) {
                System.out.println("bad hasGoodCredit value");
                System.exit(-2);
            }
            if (ccId != c1.getCCId()) {
                System.out.println("bad CreditCard value");
                System.exit(-2);
            }
            System.out.println("Values ok");
        }
        catch (Exception e) {
            System.err.println("Cannot check data into the Customer bean: " + e);
            System.exit(2);
        }
    }
    public static void checkCreditCard(CreditCardRemote c1,String numb)
        throws Exception {
        // Check if the data exist
        try {
            String n = c1.getNumber();
            if (n == null) {
                System.out.println("returned number is null");
                System.exit(-2);
            }
            else if (!n.equals(n)) {
                System.out.println("bad number value");
                System.exit(-2);
            }
            System.out.println("Values ok");
        }
        catch (Exception e) {
            System.err.println("Cannot check data into the CreditCard bean: " + e);
            System.exit(2);
        }
    }

    public static void loadCustomer(CustomerHomeRemote home, Integer id)
            throws Exception {
        // Find a second Account
        System.out.println("Finding an Customer by its pk in database: id=" + id);
        CustomerRemote c1 = null;
        try {
            c1 = home.findByPrimaryKey(id);
            printCustomerFields(c1, id);
        }
        catch (Exception e) {
            System.err.println("Cannot find Customer: " + e);
            System.exit(2);
        }
    }
    public static void loadCreditCard(CreditCardHomeRemote home, Integer id)
            throws Exception {
        // Find a second Account
        System.out.println("Finding an Customer by its pk in database: id=" + id);
        CreditCardRemote c1 = null;
        try {
            c1 = home.findByPrimaryKey(id);
            String n = c1.getNumber();
            System.out.println("id=" + id);
            System.out.println("number=" + n);
            System.out.println("pk=" + c1.getPrimaryKey());
        }
        catch (Exception e) {
            System.err.println("Cannot find Customer: " + e);
            System.exit(2);
        }
    }

    public static void assignCC2Customer(CustomerHomeRemote home, Integer custId,
                                         Integer ccId, String ccNumber)
        throws Exception {
        System.out.println("Finding an Customer by its pk in database: id=" + custId);
        CustomerRemote c1 = null;
        try {
            utx.begin();
            c1 = home.findByPrimaryKey(custId);
            c1.createCreditCard(ccId, ccNumber);
            utx.commit();
            printCustomerFields(c1, custId);
        }
        catch (Exception e) {
            System.err.println("Cannot find Customer: " + e);
            System.exit(2);
        }
    }

    public static void assignCC2Customer(CustomerHomeRemote home1,
                                         Integer custId,
                                         CreditCardHomeRemote home2,
                                         Integer ccId) throws Exception {
        System.out.println("Finding an Customer by its pk in database: id=" + custId);
        CustomerRemote c1 = null;
        try {
            utx.begin();
            c1 = home1.findByPrimaryKey(custId);
            c1.assignCreditCard(ccId);
            utx.commit();
            printCustomerFields(c1, custId);
        }
        catch (Exception e) {
            System.err.println("Cannot find Customer: " + e);
            System.exit(2);
        }
    }

    public static void removeCreditCard(CreditCardHomeRemote home, Integer id)
        throws Exception {
        // Delete account
        System.out.println("Removing Customer previously created in database");
        CreditCardRemote c1 = null;
        try {
            c1 = home.findByPrimaryKey(id);
            c1.remove();
        }
        catch (Exception e) {
            System.err.println("exception during remove: " + e);
            System.exit(2);
        }
    }

    public static void removeCustomer(CustomerHomeRemote home, Integer id)
        throws Exception {
        // Delete account
        System.out.println("Removing Customer previously created in database");
        CustomerRemote c1 = null;
        try {
            c1 = home.findByPrimaryKey(id);
            c1.remove();
        }
        catch (Exception e) {
            System.err.println("exception during remove: " + e);
            System.exit(2);
        }
    }

    public static void printCustomerFields(CustomerRemote c1, Integer id) {
        try {
            Name n = c1.getName();
            System.out.println("customer.id=" + id);
            System.out.println("customer.name=" + n);
            if (n!=null) {
                System.out.println("customer.lastName=" + n.getLastName());
                System.out.println("customer.firstName=" + n.getFirstName());
            }
            System.out.println("customer.hasGoodCredit=" + c1.HasGoodCredit());
            System.out.println("customer.creditCard.id=" + c1.getCCId());
            System.out.println("customer.creditCard.number=" + c1.getCCNumber());
            System.out.println("pk=" + c1.getPrimaryKey());
        }
        catch (RemoteException e) {
            e.printStackTrace();
        }

    }
    public static String readString(String invitation) throws Exception {
        System.out.print(invitation);
        System.out.flush();
        StringBuffer reply_buffer = null;
        char b = (char) System.in.read();
        while (b > 0 && b != '\n' && Character.isWhitespace(b)) {
            b = (char) System.in.read();
        }
        while (b > 0 && b != '\n' && b != 13) {
            if (reply_buffer == null) {
                reply_buffer = new StringBuffer();
            }
            reply_buffer.append(b);
            b = (char) System.in.read();
        }
        if (System.in.available() == 1) {
            System.in.read();
        }
        if (reply_buffer != null) {
            return reply_buffer.toString();
        }
        else {
            throw new Exception("Impossible to read command line");
        }
    }
    public static void findAllCustomer(CustomerHomeRemote home) {
        System.out.println("Finding all Customer");
        try {
            Integer i = new Integer(0);
            int counter=0;
            for(Iterator it = home.findAllCustomers().iterator();it.hasNext();) {
                System.out.println("Customer " + counter + " :");
                System.out.println("=============");
                printCustomerFields((CustomerRemote) it.next(), i);
                counter++;
            }
        }
        catch (Exception e) {
            System.err.println("Cannot find all Customer: " + e);
            System.exit(2);
        }
    }
    public static void findAllCreditCard(CreditCardHomeRemote home) {
        System.out.println("Finding all CreditCard");
        try {
            Integer i = new Integer(0);
            int counter=0;
            for(Iterator it = home.findAllCreditCards().iterator();it.hasNext();) {
                System.out.println("CreditCard " + counter + " :");
                System.out.println("=============");
                CreditCardRemote ccr = (CreditCardRemote) it.next();
                System.out.println("number=" + ccr.getNumber());
                System.out.println("pk=" + ccr.getPrimaryKey());
                counter++;
            }
        }
        catch (Exception e) {
            System.err.println("Cannot find all CreditCard: " + e);
            System.exit(2);
        }
    }

}
