# ---------------------------------------------------------------------------
# JOnAS: Java(TM) Open Application Server
# Copyright (C) 1999 Bull S.A.
# Contact: jonas-team@objectweb.org
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): ____________________________________.
# Initial developer(s): ____________________________________.
# Contributor(s): ______________________________________.
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# common Makefile


empty :=
space := $(empty) $(empty)

.SUFFIXES: .java .class _Stub.class .jj .sh .jpp .xml .jar .bat

IS_CYGWIN=no
ifeq ($(OSTYPE),cygwin32)
IS_CYGWIN=yes
endif
ifeq ($(OSTYPE),cygwin)
IS_CYGWIN=yes
endif

ifeq ($(JDKDIR),)
ifeq ($(IS_CYGWIN),yes)
JDKDIR = /jdk1.2.2
else
JDKDIR = /usr/java1.2
endif
endif


OBJECTWEB_JDK = JAVA2


ifeq ($(JDKBIN),)
JDKBIN = $(JDKDIR)/bin
endif

ifeq ($(JAR),)
JAR = $(JDKBIN)/jar
endif
ifeq ($(JAVA),)
JAVA = $(JDKBIN)/java
endif
ifeq ($(JAVAC),)
JAVAC = $(JDKBIN)/javac
endif
ifeq ($(JAVADOC),)
JAVADOC = $(JDKBIN)/javadoc
endif
ifeq ($(RMIC),)
ifeq ($(OBJECTWEB_ORB),JEREMIE)
RMIC = $(JAVA) org.objectweb.jeremie.tools.jrmic.JRMICompiler -opt
else
ifeq ($(OBJECTWEB_ORB),DAVID)
#RMIC = $(JDKBIN)/rmic
RMIC = $(JAVA) org.objectweb.david.tools.jrmic.JRMICompiler -opt
else
RMIC = $(JDKBIN)/rmic
endif
endif
endif

GENIC = $(JONAS_ROOT)/bin/unix/GenIC

JPP = $(JAVA) org.objectweb.jpp.JPP


ifndef OUTPUT_DIR
OUTPUT_DIR = $(ROOT)/output
endif
ifndef CLASS_DIR
CLASS_DIR = $(ROOT)/classes
endif
ifndef JAR_DIR
JAR_DIR = $(JONAS_ROOT)/ejbjars
endif
ifndef SRCTMP_DIR
SRCTMP_DIR = $(ROOT)/tmp
endif
ifndef JAVADOC_DIR
JAVADOC_DIR = $(OUTPUT_DIR)/jdoc
endif
ifndef LIB_DIR
LIB_DIR = $(OUTPUT_DIR)/lib
endif
ifndef BIN_DIR
BIN_DIR = $(OUTPUT_DIR)/bin
endif
ifndef WEBAPPS
WEBAPPS = $(OUTPUT_DIR)/webapps
endif

ifdef TOMCAT_HOME
ifndef TOMCAT_BASE
TOMCAT_BASE = $(TOMCAT_HOME)
endif
else
ifdef CATALINA_HOME
TOMCAT_HOME = $(CATALINA_HOME)
ifndef CATALINA_BASE
TOMCAT_BASE = $(TOMCAT_HOME)
else
TOMCAT_BASE = $(CATALINA_BASE)
endif
endif
endif

PACKDIR = $(subst .,/,$(JAVAPACKAGE))
CLASSPACKDIR = $(CLASS_DIR)/$(PACKDIR)
COPYSRCDIR = $(SRCTMP_DIR)/$(PACKDIR)

# jonas classes that must be included in the classpath
ifeq ($(JONAS_CLASSES),)
JONAS_CLASSES = $(JONAS_ROOT)/lib/$(OBJECTWEB_ORB)_jonas.jar
endif

ifeq ($(IDB_HOME),)
	IDB_HOME = /usr/local/lib
endif
ifeq ($(IDB_CLASSES),)
	IDB_CLASSES = $(IDB_HOME)/idb.jar$(PS)$(IDB_HOME)/idbexmpl.jar
endif

ifeq ($(ORACLE_CLASSES),)
	ORACLE_CLASSES = /usr/local/lib/classes12.zip
endif

ifeq ($(POSTGRE_CLASSES),)
#	POSTGRE_CLASSES = /usr/lib/pgsql/jdbc6.5-1.2.jar
	POSTGRE_CLASSES = /usr/lib/pgsql/jdbc7.0-1.2.jar
endif

ifeq ($(INTERBASE_CLASSES),)
	INTERBASE_CLASSES = /usr/interclient/interclient.jar
endif

ifeq ($(JDBC_CLASSES),)
	JDBC_CLASSES = $(IDB_CLASSES)$(PS)$(ORACLE_CLASSES)$(PS)$(POSTGRE_CLASSES)$(PS)$(INTERBASE_CLASSES)
endif

CLASSPATH := "$(XTRA_CLASSPATH)$(PS)$(JONAS_CLASSES)$(PS)$(CLASS_DIR)$(PS)$(CLASSPATH)"

JPPFLAGS += -D $(OBJECTWEB_JDK) -D $(OBJECTWEB_ORB)

ifneq ($(JCFLAGS),)
GENICFLAGS += -javacopts "$(JCFLAGS)"
endif
ifneq ($(RMICFLAGS),)
GENICFLAGS += -rmicopts "$(RMICFLAGS)"
endif

ifeq ($(EXPLICIT_PROP),true)
GENICFLAGS += -noimplpropag
endif

ifeq ($(OBJECTWEB_ORB),RMI)
RMICFLAGS = -keepgenerated -v1.1
endif
ifeq ($(OBJECTWEB_ORB),DAVID)
RMICFLAGS = -keep -iiop -c '$(JAVAC)'
endif


FILES_STUB = $(RMI_CLASSES:%=$(CLASSPACKDIR)/%_Stub.java)
FILES_RMI = $(RMI_CLASSES:%=$(CLASSPACKDIR)/%_Stub.class)
FILES_CLASS = $(CLASSES:%=$(CLASSPACKDIR)/%.class) $(RMI_CLASSES:%=$(CLASSPACKDIR)/%.class)
FILES_SRC = $(CLASSES:%=$(COPYSRCDIR)/%.java) $(RMI_CLASSES:%=$(COPYSRCDIR)/%.java)
FILES_SCRIPTS = $(SCRIPTS:%=$(CLASSPACKDIR)/%)
FILES_UNIXCOMMAND = $(UNIXCOMMANDS:%=$(BIN_DIR)/unix/%)
FILES_NTCOMMAND = $(NTCOMMANDS:%=$(BIN_DIR)/nt/%)
FILES_GENIC_XML = $(DDESCRS:%=$(CLASSPACKDIR)/%.genic_xml_output)
FILES_INSTALLED_JAR = $(EJBJARS:%=$(JAR_DIR)/%.jar)
FILES_EJBJARS = $(EJBJARS:%=$(CLASSPACKDIR)/%.jar)
FILES_IMAGES = $(IMAGES:%=$(CLASS_DIR)/images/%)


$(CLASSPACKDIR)/%.class: %.java
	@echo $? >> .classes.list

$(CLASSPACKDIR)/%_Stub.class: %.java
	@echo $(JAVAPACKAGE).$(basename $?) >>.rmi.list

$(BIN_DIR)/unix/%: %.sh
	@[ -d $(BIN_DIR)/unix ] || mkdir -p $(BIN_DIR)/unix
	cp $< $@
	chmod a+x $@

# Add the end-of-line character for DOS
$(BIN_DIR)/nt/%.bat: %.bat
	@[ -d $(BIN_DIR)/nt ] || mkdir -p $(BIN_DIR)/nt
	cat $< | sed 's%$$%
%' >$@

$(CLASSPACKDIR)/%: %.sh
	@[ -d $(CLASSPACKDIR) ] || mkdir -p $(CLASSPACKDIR)
	cp $< $@
	chmod a+x $@

$(COPYSRCDIR)/%.java: %.java
	@[ -d $(COPYSRCDIR) ] || mkdir -p $(COPYSRCDIR)
	ln $< $@

%.java:	%.jpp
	@CLASSPATH="$(CLASSPATH)" ; PATH="$(PATH)" ; export CLASSPATH PATH  ; \
	set -x ; \
	$(JPP) $(JPPFLAGS) $?
	cp $@ ~$@

$(CLASSPACKDIR)/%.genic_xml_output:	%.xml
	@CLASSPATH="$(CLASSPATH)" ; PATH="$(PATH)" ; export CLASSPATH PATH  ; \
	set -x ; \
	$(GENIC) -d $(CLASS_DIR) $(GENICFLAGS) $<
	>$@

# build ejb-jar files
$(CLASSPACKDIR)/%.jar: %.xml classes
	@[ -d $(CLASS_DIR)/META-INF ] || mkdir -p $(CLASS_DIR)/META-INF
	@cp $< $(CLASS_DIR)/META-INF/ejb-jar.xml
	@cp jonas-$< $(CLASS_DIR)/META-INF/jonas-ejb-jar.xml
	@cd $(CLASS_DIR) ; \
	set -x ; \
	$(JAR) cvf $(PACKDIR)/$*.jar META-INF/ejb-jar.xml META-INF/jonas-ejb-jar.xml $($*_CLASSES:%=$(PACKDIR)/%.class)
	@rm -f $(CLASS_DIR)/META-INF/ejb-jar.xml $(CLASS_DIR)/META-INF/jonas-ejb-jar.xml
	@CLASSPATH="$(CLASSPATH)" ; PATH="$(PATH)" ; export CLASSPATH PATH  ; \
	set -x ; \
	$(GENIC) -d $(CLASS_DIR) $(GENICFLAGS) $@

# install ejb-jar files
$(JAR_DIR)/%.jar:	$(CLASSPACKDIR)/%.jar
	@cp $< $@

all::

install::

ifneq ($(FILES_CLASS), $(space))
all::	compil

compil:	classes

classes: delete_classlist $(FILES_CLASS) compile_classlist

delete_classlist:
	@rm -f .classes.list

compile_classlist:
	@[ -d $(CLASS_DIR) ] || mkdir -p $(CLASS_DIR)
	@CLASSPATH="$(CLASSPATH)" ; PATH="$(PATH)" ; export CLASSPATH PATH  ; \
	if [ -s .classes.list ] ; \
	then	set -x ; \
		$(JAVAC) -d $(CLASS_DIR) $(JCFLAGS) $(shell if [ -s .classes.list ] ; then cat .classes.list; fi ); \
	fi

clean::
	rm -f $(CLASSPACKDIR)/*.class
endif

ifneq ($(RMI_CLASSES),)
all::	rmi

rmi:	delete_rmilist $(FILES_RMI) compile_rmilist

delete_rmilist:
	@rm -f .rmi.list

compile_rmilist:
	@CLASSPATH="$(CLASSPATH)" ; PATH="$(PATH)" ; export CLASSPATH PATH  ; \
	if [ -s .rmi.list ] ; \
	then	cd $(CLASS_DIR) ; set -x ; \
		$(RMIC) -d . $(RMICFLAGS) $(shell if [ -s .rmi.list ] ; then cat .rmi.list; fi ) ; \
	fi
endif

ifneq ($(SCRIPTS),)
all::	$(FILES_SCRIPTS)

clean::
	rm -f $(FILES_SCRIPTS)
endif

ifneq ($(UNIXCOMMANDS),)
all::	$(FILES_UNIXCOMMAND)

clean::
	rm -f $(FILES_UNIXCOMMAND)
endif

ifneq ($(NTCOMMANDS),)
all::	$(FILES_NTCOMMAND)

clean::
	rm -f $(FILES_NTCOMMAND)
endif

ifneq ($(IMAGES),)
all::
	[ -d $(CLASS_DIR)/images ] || mkdir -p $(CLASS_DIR)/images
	cp $(IMAGES) $(CLASS_DIR)/images

clean::
	rm -f $(FILES_IMAGES)
endif


ifneq ($(JARNAME),)
jar:
	[ -d $(LIB_DIR) ] || mkdir -p $(LIB_DIR)
	@(cd $(CLASS_DIR); \
	CLASSPATH="$(CLASSPATH)"; PATH="$(PATH)" ; export CLASSPATH PATH ; \
	set -x ; \
	$(JAR) cv .) >$(LIB_DIR)/$(JARNAME).jar
endif

copysrc:: $(FILES_SRC)
ifneq ($(FILES_SRC), $(space))
	echo $(JAVAPACKAGE) >> $(JAVADOC_DIR)/package.list
endif

tags::
ifneq ($(FILES_SRC), $(space))
	@etags -a -o $(PWD)/TAGS *.java *.jpp 2>/dev/null
endif

ifneq ($(EJBJARS),)
all::	$(FILES_EJBJARS) 

install:: $(FILES_INSTALLED_JAR)

clean::
	rm -f $(CLASSPACKDIR)/*.jar
	rm -f $(CLASSPACKDIR)/*.java
	rm -f $(CLASSPACKDIR)/*.java.save
endif

ifneq ($(DDESCRS),)
all::	$(FILES_GENIC_XML)

clean::
	rm -f $(CLASSPACKDIR)/*.genic_xml_output
	rm -f $(CLASSPACKDIR)/*.java
endif

ifneq ($(SUBDIRS),)
tags::
	@rm -f TAGS

all clean run tags copysrc::
	@for p in $(SUBDIRS); do (cd ./$$p; $(MAKE) $@); done
endif

javadoc: jdinit copysrc
	@CLASSPATH="$(CLASSPATH)"; export CLASSPATH; set -x ; \
	$(JAVADOC) -d $(JAVADOC_DIR) -sourcepath $(SRCTMP_DIR) -J-Xmx64m -J-Xms64m `cat $(JAVADOC_DIR)/package.list`

jdinit:
	@[ -d $(JAVADOC_DIR) ] || mkdir -p $(JAVADOC_DIR)
	rm -f $(JAVADOC_DIR)/package.list

clean::
	rm -rf $(JAVADOC_DIR) $(SRCTMP_DIR)

ifneq ($(LOCALFILES),)
ifndef LOCAL_DIR
LOCAL_DIR = $(OUTPUT_DIR)
endif

all::
	[ -d $(LOCAL_DIR) ] || mkdir -p $(LOCAL_DIR)
	cp -Rf $(LOCALFILES) $(LOCAL_DIR)
endif

# ----------------------------------------------------------
# Rules to make a web application (.war file)
# only 1 WAR_NAME per directory is allowed
# directory must be structured with: etc,web subdirectories.
# possible arguments are:
# WAR_NAME = <name>
# WAR_SUBDIRS = additional sub directories that must be processed first.
# WAR_BEANS = bean jar file used by appli
# WAR_SERVDIRS = directories containing servlets
# ----------------------------------------------------------
ifneq ($(WAR_NAME),)
WAR_DIR  = $(WEBAPPS)/$(WAR_NAME)
WAR_FILE = $(WAR_DIR).war

all::	$(WAR_FILE)

$(WEBAPPS)/%.war: wardirs $(WEBAPPS)/%/WEB-INF/copy
	cd $(@D)/$*; jar cvf ../$(@F) .

wardirs:
ifneq ($(WAR_SUBDIRS),)
	for p in $(WAR_SUBDIRS); do (cd ./$$p; $(MAKE) install); done
endif
ifneq ($(WAR_SERVDIRS),)
	for p in $(WAR_SERVDIRS); do (cd src/$$p; $(MAKE) all); done
endif

$(WEBAPPS)/%/WEB-INF/copy:
	[ -d $(@D)/lib ] || mkdir -p $(@D)/lib
ifneq ($(WAR_BEANS),)
	for p in $(WAR_BEANS); do cp $(JAR_DIR)/$$p $(@D)/lib; done
endif
	[ -d $(WEBAPPS)/$(WAR_NAME)/WEB-INF/classes ] || mkdir -p $(WEBAPPS)/$(WAR_NAME)/WEB-INF/classes
ifneq ($(WAR_SERVDIRS),)
	cd $(CLASS_DIR); find $(WAR_SERVDIRS) -name "*.class" -print | cpio -pmvdu $(WEBAPPS)/$(WAR_NAME)/WEB-INF/classes
endif
	cp etc/web.xml $(@D)
	[ -d web ] && cd web; find . ! -name "*~" -print | cpio -pmdvu ../$(@D)/..

install::	all
	rm -rf $(TOMCAT_BASE)/webapps/$(WAR_NAME)
	cp $(WAR_FILE) $(TOMCAT_BASE)/webapps
ifneq ($(APPS_XMLFILE),)
	cp $(APPS_XMLFILE) $(TOMCAT_BASE)/conf
endif

clean::
	rm -rf $(WEBAPPS)/$(WAR_NAME) $(WEBAPPS)/$(WAR_NAME).war
ifneq ($(WAR_SUBDIRS),)
	for p in $(WAR_SUBDIRS); do (cd ./$$p; $(MAKE) clean); done
endif
ifneq ($(WAR_SERVDIRS),)
	for p in $(WAR_SERVDIRS); do (cd src/$$p; $(MAKE) clean); done
	cd $(CLASS_DIR); rm -f $(WAR_SERVDIRS)/*.class
endif
endif
# -------------------------------------------

ifneq ($(WAR),)
WAR_DIR  = $(WEBAPPS)/$(WAR)
WAR_FILE = $(WAR_DIR).war
CLASS_DIR = $(WAR_DIR)/WEB-INF/classes
SERVLET_CLASSES = $(CLASSES:%=$(CLASS_DIR)/$(PACKDIR)/%.class)

all::	$(WAR_FILE)

servlets: delete_servlist $(SERVLET_CLASSES) compile_servlist

wardirs:
ifneq ($(WAR_SUBDIRS),)
	for p in $(WAR_SUBDIRS); do (cd ./$$p; $(MAKE) all; $(MAKE) install); done
endif


$(CLASS_DIR)/$(PACKDIR)/%.class: %.java
	echo $? >> .servlist

delete_servlist:
	rm -f .servlist
	[ -d $(CLASS_DIR)/$(PACKDIR) ] || mkdir -p $(CLASS_DIR)/$(PACKDIR)

compile_servlist:
	CLASSPATH="$(CLASSPATH)"; export CLASSPATH; \
	if [ -s .servlist ] ; \
	then	set -x ; \
		$(JAVAC) -d $(CLASS_DIR) -classpath $(CLASS_DIR):$(CLASSPATH) $(JCFLAGS) $(shell if [ -s .servlist ] ; then cat .servlist; fi ); \
	fi

clean::
	rm -rf $(WAR_FILE) $(WAR_DIR)
ifneq ($(WAR_SUBDIRS),)
	for p in $(WAR_SUBDIRS); do (cd ./$$p; $(MAKE) clean); done
endif

install::	all
	rm -rf $(TOMCAT_BASE)/webapps/$(WAR)
	cp $(WAR_FILE) $(TOMCAT_BASE)/webapps
ifneq ($(APPS_XMLFILE),)
	cp $(APPS_XMLFILE) $(TOMCAT_BASE)/conf
endif

$(WEBAPPS)/%.war: servlets wardirs $(WEBAPPS)/%/WEB-INF/copy
	cd $(@D)/$*; jar cvf ../$(@F) .
	rm -rf $(@D)/$*

$(WEBAPPS)/%/WEB-INF/copy: etc/web.xml
	[ -d $(@D)/lib ] || mkdir -p $(@D)/lib
ifneq ($(WAR_BEANS),)
	for p in $(WAR_BEANS); do cp $(JAR_DIR)/$$p.jar $(@D)/lib; done
endif
	cp etc/web.xml $(@D)
	[ -d web ] && cd web; find . ! -name "*~" -print | cpio -pmdvu ../$(@D)/..
endif

clean_world: clean
	-rm -rf $(OUTPUT_DIR) $(CLASS_DIR)

env:
	@echo "CLASSPATH=$(CLASSPATH)"
	@echo "PATH=$(PATH)"
	@echo "JAVAPACKAGE=$(JAVAPACKAGE)"
	@echo "FILES_SRC=$(FILES_SRC)"
	@echo "JDKDIR=$(JDKDIR)"
	@echo "JDKBIN=$(JDKBIN)"
	@echo "OSTYPE=$(OSTYPE)"
