/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.common;

import java.util.Properties;

/**
 * This class manages global variables and properties used by EJB Server. 
 * A static design pattern is used. 
 */
public class Env {

    // Variables related to Java Version
    static public  final int  JAVA_1_1_6  = 116;
    static public  final int  JAVA_1_1_7  = 117;
    static public  final int  JAVA_1_1_8  = 118;
    static public  final int  JAVA_1_2    = 120;
    static public  final int  JAVA_1_3    = 130;
    static public  final int  JAVA_1_4    = 140;
    static private       int  javaVersion = -1;

    /**
     * possible OBJECTWEB_ORB values.
     */
    static public  final int  ORB_RMI = 1;
    static public  final int  ORB_JEREMIE = 2;
    static public  final int  ORB_DAVID = 3;
    static public  int orb = 0;

    /**
     * set the orb value
     * this is called today by GenIC since it does not get property "objectweb.orb"
     */
    static public void setOrb(String ow_orb) {
	//System.out.println("ORB set to "+ow_orb);	// DEBUG
	if (ow_orb.equals("RMI")) {
	    orb = ORB_RMI;
	} else if (ow_orb.equals("JEREMIE")) {
	    orb = ORB_JEREMIE;
	} else if (ow_orb.equals("DAVID")) {
	    orb = ORB_DAVID;
	}
    }

    /**
     * get orb value 
     * If not set, get value from system property "objectweb.orb"
     */
    static private int getOrb() {
	if (orb == 0) {
	    setOrb(System.getProperty("objectweb.orb"));
	}
	if (orb == 0) {
	    System.err.println("Configuration error: OBJECTWEB_ORB is not set");
	}
	return orb;
    }

    /**
     * Return true if the os.name starts with "Windows"
     */
    static public boolean isOsWindows() {
	String osName = System.getProperty("os.name", "");
	return(osName.startsWith("Windows"));
    }

    /**
     * Gets Java Version.
     * @return javaVersion or -1 if error
     */
    static public int getJavaVersion() {

	if (javaVersion == -1) {
	    // Sets Java Version
	    String strjv = System.getProperty("java.version", "");
	    if (strjv.indexOf("1.1.6") == 0)
		javaVersion = JAVA_1_1_6;
	    if (strjv.indexOf("1.1.7") == 0)
		javaVersion = JAVA_1_1_7;
	    if (strjv.indexOf("1.1.8") == 0)
		javaVersion = JAVA_1_1_8;
	    if (strjv.indexOf("1.2") == 0)
		javaVersion = JAVA_1_2;
	    if (strjv.indexOf("1.3") == 0)
		javaVersion = JAVA_1_3;
	    if (strjv.indexOf("1.4") == 0)
		javaVersion = JAVA_1_4;
	} 
	return javaVersion;
    }

    /**
     * return true if JDK 1.2 or later
     */
    static public boolean isJAVA2() {
	return (getJavaVersion() >= JAVA_1_2);
    }

    /**
     * return true if JDK 1.4 or later
     */
    static public boolean isJAVA4() {
	return (getJavaVersion() >= JAVA_1_4);
    }

    /**
     * Return true if orb is Sun RMI over JRMP
     */
    public static boolean isSunRMI() {
	return (getOrb() == ORB_RMI);
    }

    /**
     * Return true if orb is RMI over Jeremie 
     */
    public static boolean isJeremie() {
	return (getOrb() == ORB_JEREMIE);
    }

    /**
     * Return true if orb is David RMI
     */
    public static boolean isDavid() {
	return (getOrb() == ORB_DAVID);
    }
}

