/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 * JOnAS 2.4 Murad Meghani (Murad.Meghani@compuware.com) killServer and stopServer
 * JOnAS 2.5 2002.06 Florent Benoit & Ludovic Bert : 
 *                   Methods for wars and ear files
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.adm;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import org.objectweb.jonas.ear.EarServiceException;
import org.objectweb.jonas.web.JWebContainerServiceException;
import org.objectweb.jonas.mail.MailServiceException;

/*
 * JOnAS Administration Remote Interface.
 * This is used basically by jonas admin.
 * @author Philippe Coq
 * Contributor(s):
 * Murad Meghani (Murad.Meghani@compuware.com) killServer and stopServer
 * Florent Benoit & Ludovic Bert : Methods for wars and ear files
 */
public interface AdmInterface extends Remote {

    /**
     * Deploy a given ear file with the help of the ear service.
     * @param fileName the name of the ear file.
     * @throw RemoteException if rmi call failed.
     * @throw EarServiceException if the deployment failed.
    */
    public void addEar(String fileName) throws RemoteException, EarServiceException;

    /**
     * Deploy a given war file with the help of the web container service.
     * @param fileName the name of the war file.
     * @throw RemoteException if rmi call failed.
     * @throw JWebContainerServiceException if the deployment failed.
    */
    public void addWar(String fileName) throws RemoteException, JWebContainerServiceException;

    /**
     * UnDeploy a given ear file with the help of the ear service.
     * @param fileName the name of the ear file.
     * @throw RemoteException if rmi call failed.
     * @throw EarServiceException if the undeployment failed.
    */
    public void removeEar(String fileName) throws RemoteException, EarServiceException;

    /**
     * UnDeploy a given war file with the help of the web container service.
     * @param fileName the name of the war file.
     * @throw RemoteException if rmi call failed.
     * @throw JWebContainerServiceException if the undeployment failed.
    */
    public void removeWar(String fileName) throws RemoteException, JWebContainerServiceException;

    void addBeans(String fileName) throws RemoteException;
    void removeBeans(String fileName) throws RemoteException;
    boolean isLoaded(String fileName) throws RemoteException;
    String [] listBeans() throws RemoteException;
    Vector listContext() throws RemoteException;
    Properties listEnv() throws RemoteException;
    void stopServer() throws RemoteException;
    void killServer() throws RemoteException;
    boolean isEJBContainer() throws RemoteException;
    int getServerState() throws RemoteException;
    void setTransactionTimeout(int timeout) throws RemoteException;
    void runGC() throws RemoteException;
    void syncAllEntities(boolean passivate) throws RemoteException;
    String [] getTopics() throws RemoteException;
    String getTopicLevel(String topic) throws RemoteException;
    void setTopicLevel(String topic, String l) throws RemoteException;
}
