/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import javax.naming.Context;
import java.net.URL;
import org.objectweb.jonas.service.Service;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas_ejb.container.Container;

/**
 * EJB Service interface.
 */
public interface EJBService extends Service {

    /**
     * Create a JOnAS Container for all beans that are described in a .xml 
     * file or belong to .jar file.
     * @param ctx JNDI contect in which are found the container configuration.
     */
    public Container createContainer(Context ctx) throws Exception;

    /**
     * Get the Container by its file name (.xml or .jar)
     */
    public Container getContainer(String fileName);

    /**
     * Remove a  JOnAS Container.
     * @param cont JOnAS container to remove.
     */
    public void removeContainer(Container cont);

    /**
     * List of all JOnAS Containers
     * @return an array of Container objects
     */
    public Container[] listContainers();

    /**
     * Synchronized all entity bean containers
     * @param passivate passivate instances after synchronization.
     */
    public void syncAllEntities(boolean passivate);
    
    /**
     * Deploy the given ejb-jars of an ear file with the specified parent 
     * classloader (ear classloader). (This method is only used for 
     * the ear applications, not for the ejb-jar applications).
     * @param ctx the context containing the configuration 
     * to deploy the ejbjars.<BR>
     * This context contains the following parameters :<BR>
     *    - earRootUrl the root of the ear application.<BR>
     *    - earClassLoader the ear classLoader of the ear application.<BR>
     *    - ejbClassLoader the ejb classLoader for the ejbjars.<BR>
     *    - jarURLs the list of the urls of the ejb-jars to deploy.<BR>
     *    - roleNames the role names of the security-role.<BR>
     * @return ClassLoader the ejbClassLoader created.
     * @throws ServiceException if an error occurs during the deployment.
     */
    public void deployJars(Context ctx)throws ServiceException;

    /**
     * Undeploy the given ejb-jars of an ear file with the specified parent 
     * classloader (ear classloader). (This method is only used for 
     * the ear applications, not for the ejb-jar applications).
     * @param urls the list of the urls of the ejb-jars to deploy.
     */
    public void unDeployJars(URL[] urls);

    /**
     * Make a cleanup of the cache of deployment descriptor. This method must 
     * be invoked after the ear deployment by the EAR service.
     * the deployment of an ear by .
     * @param earClassLoader the ClassLoader of the ear application to
     * remove from the cache.
     */
    public void removeCache(ClassLoader earClassLoader);

}

