/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Michel Bruno and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.util.Set;

/**
 * MBean Interface for general EJB Management 
 * MBean type: Standard
 * MBean model: Delegate (ManagableEjb: JBeanHome and JMdbFactory)
 */
public interface JmxBeanFactoryMBean {

    /**
     * @return String The Name of this Ejb
     */ 
    public String getEJBName();

    /**
     * @return Ejb File Name
     */
    public String getEJBFileName();

    /**
     * @return String Set The Name Ejb DataSource
     */
    public Set getAllDataSourceName();

 
    /**
     * @return String Set The Name Ejb Connection Factory
     */   
    public Set getAllJMSConnectionFactoryName();


    /**
     * @return String Set The Name Ejb Destination
     */  
    public Set getAllJMSDestinationName();

    /**
     * @return Set The name set of the Session Mail Factories
     */
    public Set getAllMailFactorySName();   
 
    /**
     * @return Set The name set of the MimePartDataSource Mail Factories
     */    
    public Set getAllMailFactoryMName();

    /**
     * @return Integer The current instance pool size
     */  
    public  Integer getCurrentInstancePoolSize();

    /**
     * @return String the Ejb Class
     */
    public String getEJBClass();

    /**
     * @return String the displayName of the bean.
     */
    public String getDisplayName();

    /**
     * @return String the JNDI Name of the bean.
     */
    public String getJndiName();

    /**
     * @return String the HomeClass of the bean.
     */
    public String getHomeClass();

    /**
     * @return String the RemoteClass of the bean.
     */
    public String getRemoteClass();

    /**
     * @return String the LocalHomeClass of the bean.
     */
    public String getLocalHomeClass();

    /**
     * @return String the  of the bean.
     */
    public String getLocalClass();

}
