/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.ear;

import java.rmi.RemoteException;
import java.util.Set;

/**
 * JOnAS EAR Service MBean interface.
 * This interface provides a description for the EAR service management.
 * @author Florent Benoit
 * @author Ludovic Bert
 */

public interface EarServiceImplMBean {

    /**
     * Deploy an EAR, used by the JMX Management.
     * @param fileName the fileName of the ear which must be be deployed.
     * @throws RemoteException if rmi call failed.
     * @throws EarServiceException if the deployment of the EAR failed.
     */
    public void deployEarMBean(String fileName) throws RemoteException, EarServiceException;


    /**
     * Undeploy an EAR, used by the JMX Management.
     * @param fileName the fileName of the ear which must be be undeployed.
     * @throws RemoteException if rmi call failed.
     * @throws EarServiceException if the undeployment of the EAR failed.
     */
    public void unDeployEarMBean(String fileName) throws RemoteException, EarServiceException;

    /**
     * @return current number of ears deployed in the JOnAS server
     */
    public Integer getCurrentNumberOfEars();

    /**
     * This method is added temporarily. It will disapear when Ears will have their associated MBeans
     * (when Ears will become manageable)
     * @return the names of the ears currently deployed in the JOnAS server
     */
    public Set getEarNames();
}
