/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Miroslav halas
 *
 * --------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------- 
 */

package org.objectweb.jonas.jetty;

import javax.naming.Context;
import javax.naming.NamingException;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.StringTokenizer;

import javax.management.ObjectName;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.AbsDynamicServiceImpl;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import org.mortbay.jetty.*;
import org.mortbay.jetty.jmx.*;
import org.mortbay.util.*;
import org.mortbay.util.jmx.*;

/**
 * Jetty service implementation for Jetty 4
 *
 * @author Miroslav Halas
 */
public class EmbeddedJettyImpl4 extends    AbsDynamicServiceImpl 
                                implements JettyService 
{
   /**
    * Private logger for this class.
    */
   private static Logger m_lgLogger = null;

   /**
    * Configuration used to configure Jetty.
    */
   private static String m_strConfig = null;
      
   /**
    * Name of the variable specifyin configuration file for Jetty
    */
   private static final String JETTY_CONFIG = "config";

   /**
    * Initialize Jetty server.
    *
    * @param ctx
    */
   public void doInit(
      Context ctx
   ) 
   {
      if (m_lgLogger == null)
      {
         m_lgLogger = Log.getLogger(Log.JONAS_WEB_PREFIX);
      }
      
      m_lgLogger.log(BasicLevel.DEBUG, "entering constructor()");
      
      try
      {
         m_strConfig = (String)ctx.lookup(JETTY_CONFIG);          
      } 
      catch (NamingException neExc)
      {
         // by default we take jetty.xml under JONAS_ROOT/config
         String jonasroot = System.getProperties().getProperty("install.root");
         m_strConfig = jonasroot + "/config/jetty.xml";      
      }
      m_lgLogger.log(BasicLevel.LEVEL_DEBUG, "Using configuration file "+ m_strConfig);
   }

   /**
    * Start Jetty service.
    *
    * @throws ServiceException
    */
   public void run(
   ) throws ServiceException 
   {
      m_lgLogger.log(BasicLevel.DEBUG, "entering run()");

   	try 
      {
	 // Register JettyService MBean : JettyService 
         Object obj;
         ObjectName objname;
         /*
         obj = new LogMBean();
         objname = new ObjectName("jonas:type=jetty,name=jettylog");
	      ((JmxService)ServiceManager.getJmxService()).getJmxServer().registerMBean(obj,objname);
          */
         // Once we register this bean, it will start Jetty using given configuration 
         // file. That's why we have to do it in run() instead in doStart()
         obj = new ServerMBean(m_strConfig);
         objname = JonasObjectName.wwwService();
	      ((JmxService)ServiceManager.getJmxService()).getJmxServer().registerMBean(obj,objname);
         obj = new CodeMBean();
         objname = JonasObjectName.www("Code");
	      ((JmxService)ServiceManager.getJmxService()).getJmxServer().registerMBean(obj,objname);         
   	} 
      catch (javax.management.MalformedObjectNameException moneExc) 
      {
         // this should never occur
         m_lgLogger.log(BasicLevel.LEVEL_ERROR, "Cannot start Jetty server using configuration file " 
                                                + m_strConfig, moneExc);
      }
      catch (ServiceException seExc) 
      {
	 // Jmx Service not available, do nothing
         m_lgLogger.log(BasicLevel.LEVEL_ERROR, "Cannot start Jetty server using configuration file " 
                                                + m_strConfig, seExc);
      } 
      catch (Exception eExc) 
      {
         m_lgLogger.log(BasicLevel.LEVEL_ERROR, "Cannot start Jetty server using configuration file " 
                                            + m_strConfig, eExc);
         throw new ServiceException("Cannot start Jetty server using configuration file " 
                                    + m_strConfig, eExc);
      }
   }

   /**
    * Stop Jetty service
    *
    * @throws ServiceException
    */
   public void doStop(
   ) throws ServiceException 
   {
      m_lgLogger.log(BasicLevel.DEBUG, "entering");
      if (isStarted())
      {
      	try 
         {
            ObjectName objname;
            
            objname = new ObjectName("jonas:type=service,name=jetty");
            ((JmxService)ServiceManager.getJmxService()).getJmxServer().unregisterMBean(objname);
            objname = new ObjectName("jonas:type=jetty,name=jettylog");
            ((JmxService)ServiceManager.getJmxService()).getJmxServer().unregisterMBean(objname);
            objname = new ObjectName("jonas:type=jetty,name=jettycode");
            ((JmxService)ServiceManager.getJmxService()).getJmxServer().unregisterMBean(objname);            
         } 
         catch (ServiceException seExc) 
         {
            // Jetty Service not available, do nothing
         } 
         catch (Exception eExc) 
         {
            m_lgLogger.log(BasicLevel.LEVEL_ERROR, "Cannot stop Jetty server", eExc);
            throw new ServiceException("Cannot stop Jetty server", eExc);
      	}
      }
   }
}

