/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * This class modeled based on LoggerLogSink.java from Jetty
 * Initial developer(s): Miroslav halas
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jetty;

import java.util.Map;
import java.util.HashMap;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.Level;
import org.objectweb.util.monolog.api.BasicLevel;

import org.mortbay.util.LogSink;
import org.mortbay.util.Frame;

public class MonologLogSink implements LogSink,
                                       java.io.Serializable // must implement to be managed by JMX MBeans
{
   /**
    * Private m_lgLogger for this class.
    */
   protected static Logger m_lgLogger = null;

   /**
    * Flag symbolizing if this m_lgLogger is started.
    */
   protected boolean m_bStarted = false;
   
   /**
    * This map is used to map Jetty logging level to Monolog Logging level
    */
   protected static Map s_mpLevelMap = new HashMap();

   /**
    * Static initialized
    */
   static
   {
      s_mpLevelMap.put(org.mortbay.util.Log.DEBUG, BasicLevel.LEVEL_DEBUG);
      s_mpLevelMap.put(org.mortbay.util.Log.EVENT, BasicLevel.LEVEL_INFO);
      s_mpLevelMap.put(org.mortbay.util.Log.WARN, BasicLevel.LEVEL_WARN);
      s_mpLevelMap.put(org.mortbay.util.Log.ASSERT, BasicLevel.LEVEL_WARN);
      s_mpLevelMap.put(org.mortbay.util.Log.FAIL, BasicLevel.LEVEL_FATAL);
   }
   
   /**
    * Create new instance of m_lgLogger
    */
   public MonologLogSink(
   )
   {
      if (m_lgLogger == null)
      {
         // This should bew really changed to Log.JONAS_WEBCONTAINER_PREFIX
         m_lgLogger = Log.getLogger(Log.JONAS_TOMCAT_PREFIX);
      }
      
      m_lgLogger.log(BasicLevel.LEVEL_DEBUG, "MonologLogSink created.");
   }
   
   /**
    * Set the log options.
    *
    * @param options
    */
   public void setOptions(
      String options
   ) 
   {
      m_lgLogger.log(BasicLevel.LEVEL_DEBUG, "MonologLogSink setOptions:" + options);
   }
    
   /**
    * Get the log options
    */
   public String getOptions(
   ) 
   {
      m_lgLogger.log(BasicLevel.LEVEL_DEBUG, "MonologLogSink getOptions");
      return null;
   }
    
   public void log(
      String tag,
      Object msg,
      Frame frame,
      long time
   )
   {
      // Convert Jetty level to Monolog level
      Level lvlLevel = (Level)s_mpLevelMap.get(tag);
      
      // Print exact location only in case of error or debug, but not just for 
      // information because it is too verbose
      if ((frame != null) && (!lvlLevel.equals(BasicLevel.LEVEL_INFO)))
      {
         StringBuffer sbBuffer = new StringBuffer(Long.toString(time));

         sbBuffer.append(" ");
         sbBuffer.append(msg);

         m_lgLogger.log(lvlLevel, sbBuffer.toString(), "", frame.getMethod());
      }
      else
      {
         m_lgLogger.log(lvlLevel, msg);
      }
   }
    
   /**
    * Log formatted message.
    *
    * @param formattedLog - formatted message
    */
   public void log(
      String formattedLog
   )
   {
      m_lgLogger.log(BasicLevel.LEVEL_INFO, formattedLog);
   }
    
   /**
    * Check if the m_lgLogger is started
    */
   public boolean isStarted(
   )
   {
      return m_bStarted;
   }

   /**
    * Start the m_lgLogger .
    *
    * @throws Exception
    */
   public void start(
   ) throws Exception
   {
      m_lgLogger.log(BasicLevel.LEVEL_DEBUG, "MonologLogSink start");
      m_bStarted = true;
   }

   /**
    * Stop the m_lgLogger.
    *
    * @throws InterruptedException
    */
   public void stop(
   ) throws InterruptedException
   {
      m_lgLogger.log(BasicLevel.LEVEL_DEBUG, "MonologLogSink stop");
      m_bStarted = false;
   }
}
