/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Michel Bruno and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jms;
import java.util.Set;

/**
 * MBean Interface for Jms Service Management
 * MBean type: Standard
 * MBean model: Delegate (JmsService)
 */
public interface JmsServiceImplMBean {


    /**
     * @return the current number of Topic Jms Connection Factory
     */ 
    public Integer getCurrentNumberOfJmsTopicConnectionFactory();

    /**
     * @return the current number of Queue Jms Connection Factory
     */ 
    public Integer getCurrentNumberOfJmsQueueConnectionFactory();

    /**
     * @return the current number of Topic Jms Destination
     */ 
    public Integer getCurrentNumberOfJmsTopicDestination();

    /**
     * @return the current number of Queue Jms Destination
     */ 
    public Integer getCurrentNumberOfJmsQueueDestination();

    /**
     * Create a new Jms queue destination
     * @param String jndi Name
     */
    public void createJmsQueueDestination(String jndiName); 

    /**
     * Create a new Jms topic destination
     * @param String jndi Name
     */
    public void createJmsTopicDestination(String jndiName); 

    /**
     * Remove a Jms destination
     * @param String jndi name
     */
    public void removeJmsDestination(String jndiName);

    /**
     * return Set of Queue Destinations Names
     */
    public Set getAllJmsQueueDestinationNames();

    /**
     * return Set of Topic Destination Names
     */
    public Set getAllJmsTopicDestinationNames();

    /**
     * return Set of Queue Connection Factory Names
     */
    public Set getAllJmsQueueConnectionFactoryNames();

    /**
     * return Set of Topic Destination Names
     */
    public Set getAllJmsTopicConnectionFactoryNames();
    /**
     * return String name of default Queue Connection factory
     */
    public String getDefaultQueueConnectionFactoryName();

    /**
     * return String name of default Topic Connection factory
     */
    public String getDefaultTopicConnectionFactoryName();

}
