/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin;

import javax.management.Attribute;
import javax.management.ObjectName;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.ReflectionException;
import javax.management.RuntimeOperationsException;
import javax.management.RuntimeErrorException;
import javax.management.RuntimeMBeanException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;
/**
 * A set of static methods used in all servlets
 */
public class JonasManagementRepr {

    /**
     * @return True if the MBean is already registered in the MBean server, false otherwise or if an exception is catched.
     */
    public static boolean isRegistered(ObjectName on) {
	try {
	    return ConnectorFactory.getRMIConnector().isRegistered(on);
	} catch (Exception e) {
	    return false;
	}
    }

    /**
     * @param on The ObjectName of the MBean from which the attribute is to be retrieved.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @return The value of the attribute.
     */
    public static Object getAttribute(ObjectName on, String attribute) 
	throws ManagementException 
    {
	try {
	    return ConnectorFactory.getRMIConnector().getAttribute(on, attribute);
	} catch (Exception e) {
	    throw new ManagementException("Error while getting attribute "
					  +attribute+": "
					  +e.getClass().getName(), e);
	}
    }

    /**
     * @param on The ObjectName of the MBean within which the attribute is to be set.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @param value The value to set to the attribute.
     */
    public static void setAttribute(ObjectName on, String attribute, Object value) 
	throws ManagementException 
    {
	Logger logger = Log.getLogger("org.objectweb.jonas.jmx.vjadmin");
	logger.log(BasicLevel.DEBUG, "Set Attribute called, on " + on.toString() + " to change attribute " + attribute + " value to " + (String)value);
	try {
	    ConnectorFactory.getRMIConnector().setAttribute(on, new Attribute(attribute, value));
	} catch (Exception e) {
	    throw new ManagementException("Error while setting attribute "
					  +attribute+": "
					  +e.getClass().getName(), e);
	}
    }

    /**
     * @param on
     */
    public static Object invoke(ObjectName on, String operation, Object[] param, String[] signature) 
	throws ManagementException 
    {
	Logger logger = Log.getLogger("org.objectweb.jonas.jmx.vjadmin");
	try {
	    return ConnectorFactory.getRMIConnector().invoke(on, operation, param, signature);
	} catch (Exception e) {
	    logger.log(BasicLevel.ERROR, "EXCEPTION when calling invoke. Exception class: " + e.getClass().getName());
	    String message = null;
	    Throwable exc = null;
	    if (e instanceof InstanceNotFoundException) {
		logger.log(BasicLevel.ERROR, "--------- InstanceNotFoundException -------------");
		message = "InstanceNotFoundException when trying to use MBean " + on.toString();
	    } else if (e instanceof MBeanException) {
		Exception targetExc = ((MBeanException) e).getTargetException();
		logger.log(BasicLevel.ERROR, "-------[ MBeanException ]-------");
		logger.log(BasicLevel.ERROR, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.ERROR, "-------[ " + targetExc.toString() + " ]-------");
		message = "Exception while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;		
	    } else if (e instanceof ReflectionException) {
		Exception targetExc = ((ReflectionException) e).getTargetException();		
		logger.log(BasicLevel.ERROR, "-------[ ReflectionException ]-------");
		logger.log(BasicLevel.ERROR, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.ERROR, "-------[ " + targetExc.toString() + " ]-------");
		message = "ReflectionException while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;
	    } else if (e instanceof RuntimeMBeanException) {
		Exception targetExc = ((RuntimeMBeanException) e).getTargetException();		
		logger.log(BasicLevel.ERROR, "-------[ RuntimeMBeanException ]-------");
		logger.log(BasicLevel.ERROR, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.ERROR, "-------[ " + targetExc.toString() + " ]-------");
		message = "RuntimeMBeanException while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;
	    } else if (e instanceof RuntimeOperationsException) {
		Exception targetExc = ((RuntimeOperationsException) e).getTargetException();		
		logger.log(BasicLevel.ERROR, "-------[  RuntimeOperationException]-------");
		logger.log(BasicLevel.ERROR, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.ERROR, "-------[ " + targetExc.toString() + " ]-------");
		message = "RuntimeOperationException while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;
	    } else if (e instanceof RuntimeErrorException) {
		Error targetError = ((RuntimeErrorException) e).getTargetError();
		logger.log(BasicLevel.ERROR, "-------[  RuntimeErrorException ]-------");
		logger.log(BasicLevel.ERROR, "-------[ Embedded error : ]-------");
		logger.log(BasicLevel.ERROR, "-------[ " + targetError.toString() + " ]-------");
		message = "RuntimeErrorException while invoking operation " + operation + " on MBean " + on.toString();
	    } else {
		// This situation is not possible cf. to the JMX specification
		message = "Unexpexted exception while invoking operation " + operation + " on MBean " + on.toString();
	    }
	    throw new ManagementException(message, exc);
	}
    }

    /**
     * @return A set containing the ObjectNames for the MBeans selected.
     */
    public static java.util.Set queryNames(ObjectName on) throws ManagementException 
    {
	try {
	    return (java.util.Set)ConnectorFactory.getRMIConnector().queryNames(on, null);
	} catch (Exception e) {
	    throw new ManagementException("Error while getting MBeans names: "
					  +e.getClass().getName(), e);
	}
    }
}
