/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Set;
import java.util.Hashtable;
import java.util.Iterator;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;
// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class DatabaseServiceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	// The request types treated by this servlet are : GET and POST
	boolean getRequest;
	if (request.getMethod().equals("GET"))
	    getRequest = true; // we handle a GET request
	else
	    getRequest = false; // we handle a POST request

	Properties p = new Properties();
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (getRequest) {
		    if (paramName.equals("unload")) {
			logger.log(BasicLevel.ERROR, "DatabaseServiceServlet : treate GET with param unload");
			Object[] param = { request.getParameter(paramName) };
			String[] signature = { "java.lang.String" };
			JonasManagementRepr.invoke(JonasObjectName.databaseService(), 
					       "unloadDataSource", param, signature);
		    }
		} else {
		    //logger.log(BasicLevel.ERROR, "DatabaseServiceServlet : treate a PUT request by placing the parameter's value in p (if this value is not empty)");
		    logger.log(BasicLevel.DEBUG, "DatabaseServiceServlet - PUT request parameter and value:");
		    logger.log(BasicLevel.DEBUG, "\t- " + paramName);
		    logger.log(BasicLevel.DEBUG, "\t- " + request.getParameter(paramName));
		    if (! request.getParameter(paramName).equals("")) 
			p.put(paramName, request.getParameter(paramName));
		    /*
		    Object[] param = { p };
		    String[] signature = { "java.util.properties" };
		    JonasManagementRepr.invoke(JonasObjectName.databaseService(), 
					       "loadDataSource", param, signature);
		    */
		}			
		
	    }
	    if (!getRequest) {
		logger.log(BasicLevel.DEBUG, "DatabaseServiceServlet : invoke loadDataSource");
		Object[] param = { p };
		String[] signature = { "java.util.Properties" };
		JonasManagementRepr.invoke(JonasObjectName.databaseService(), 
					   "loadDataSource", param, signature);
	    }
	} catch (ManagementException e) {
	    if (getRequest) {
		alertException = alert(e.getMessage());
		context.put("alertException", alertException);
	    } else {
		context.put("dataSourceLoadingException", e.getMessage());
		Template template = null;
		try {
		    template = getTemplate("DATABASE_SERVICE_LOAD_ERROR.vm"); 
		} catch (ParseErrorException pee ) {
		    logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
		} catch( ResourceNotFoundException rnfe) {
		    logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
		} catch( Exception ec) {
		    logger.log(BasicLevel.ERROR, "Error " + ec);
		}

		return template;
	    }
	}


	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	}

	//Get informations about the Database Service
	boolean existsDatabaseService = JonasManagementRepr.isRegistered(JonasObjectName.databaseService());
	context.put("existsDatabaseService", new Boolean(existsDatabaseService));
	if (existsDatabaseService) {
	    Integer nbOfDataSources = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.databaseService(), "CurrentNumberOfDataSource");
	    Set allDataSources = JonasManagementRepr.queryNames(JonasObjectName.allDatasources());
	    Hashtable dataSourceNames = new Hashtable();
	    for (Iterator i = allDataSources.iterator(); i.hasNext() ;) {
		ObjectName current = (ObjectName)i.next();
		String datasourceName = (String)JonasManagementRepr.getAttribute(current, "DSName");
		boolean isUnloadable;
		Object[] param = { datasourceName };
		String[] signature = { "java.lang.String" };
		if((!JonasManagementRepr.isRegistered(JonasObjectName.ejbService())) ||
		   (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(),
							       "getDataSourceDependence", 
							       param, signature)).isEmpty())
		   )
		    isUnloadable = true;
		else
		    isUnloadable = false;
		dataSourceNames.put(datasourceName, new Boolean(isUnloadable));
		logger.log(BasicLevel.ERROR, "Put one datasource name with unloadable");
	    }
	    Integer totalCurrentNumberOfJDBCConnectionOpen = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.databaseService(), 
							   "TotalCurrentNumberOfJDBCConnectionOpen");
	    context.put("nbOfDataSources", nbOfDataSources);
	    context.put("dataSourceNames", dataSourceNames);
	    context.put("totalCurrentNumberOfJDBCConnectionOpen", totalCurrentNumberOfJDBCConnectionOpen);
	}

	Template template = null;
	try {
	    template = getTemplate("DATABASE_SERVICE.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }

    
}
