/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Servlet used to display information about a DataSource
 * servlet-name : DatabaseServlet
 * url-pattern  : /servlet/Database
 * .vm template : DATABASE
 * called with the following parameters :
 * dsName = name of the DataSource
 * @author Philippe Durieux, Adriana Danes (JOnAS team)
 */
public class DatabaseServlet extends JadminServlet {
   
    /**
     * Add info in the provided context and return the template to be used.
     * @param req servlet request from client
     * @param res servlet reponse
     * @param ctx a Velocity Context object to be filled with
     *            data.  Will be used for rendering this template
     *  @return Template to be used for request
     */
    public Template handleRequest(HttpServletRequest req, HttpServletResponse res, Context ctx) { 
	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	ctx.put("date", date);

	// Get request parameters
	String dsName = null;

	try {
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("dsName")) {
		    dsName = req.getParameter("dsName");
		} else if (paramName.equals("JDBCConnMaxAge")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCConnMaxAge", 
						     new Integer(req.getParameter("JDBCConnMaxAge")));
		} else if (paramName.equals("JDBCConnCheckLevel")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCConnCheckLevel", 
						     new Integer(req.getParameter("JDBCConnCheckLevel")));
		} else if (paramName.equals("JDBCTestStatement")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCTestStatement", 
						     new String(req.getParameter("JDBCTestStatement")));
		} else if (paramName.equals("JDBCMinConnPool")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCMinConnPool", 
						     new Integer(req.getParameter("JDBCMinConnPool")));
		} else if (paramName.equals("JDBCMaxConnPool")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCMaxConnPool", 
						     new Integer(req.getParameter("JDBCMaxConnPool")));
		} else if (paramName.equals("JDBCSleepTimeOut")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCSleepTimeOut", 
						     new Integer(req.getParameter("JDBCSleepTimeOut")));
		} else if (paramName.equals("JDBCDeadLockPool")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCDeadLockPool", 
						     new Integer(req.getParameter("JDBCDeadLockPool")));
		} else if (paramName.equals("JDBCLoopWaitPool")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCLoopWaitPool", 
						     new Integer(req.getParameter("JDBCLoopWaitPool")));
		}

	    }
	} catch (Exception e) {
	    alertException = alert(e.getMessage());
	    ctx.put("alertException", alertException);
	} 

	ctx.put("dsName", dsName);
	// Get general information about the datasource
	String datasourceType = null;
	Integer currentNumberOfJDBCConnectionsOpen = null;
	String info_error = "";
	try {
	    datasourceType = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceType");
	    currentNumberOfJDBCConnectionsOpen = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "CurrentNumberOfJDBCConnectionOpen");		       
	    ctx.put("datasourceType", datasourceType);
	    ctx.put("currentNumberOfJDBCConnectionsOpen", currentNumberOfJDBCConnectionsOpen);
	} catch (ManagementException e1) {
	    info_error = "Error while getting attribute: " + e1.getMessage();
	} catch (javax.management.MalformedObjectNameException e2) {
	    info_error = "Incompatible name:" + e2.getMessage();
		
	}
	ctx.put("info_error", info_error);
	// Get dependences
	String dep_error = "";
	try {
	    Object[] param = { dsName };
	    String[] signature = { "java.lang.String" };
	    java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
									     "getDataSourceDependence", 
									     param, 
									     signature);
	    Hashtable dependences = new Hashtable();
	    if (allDep.isEmpty()) {
		logger.log(BasicLevel.ERROR, "Try to get dependences. Empty dependences set"); 
	    } else {
		for (Iterator i = allDep.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    Dependence dep = new Dependence(current);
		    dependences.put(dep.getEjbName(), dep);
		}	
	    }
	    ctx.put("dependences", dependences);
	} catch (ManagementException e) {
	    dep_error = "Error while getting attribute:" + e.toString();
	} catch (MalformedObjectNameException me) {
	    dep_error = "Incompatible name:" + me.getMessage();
	} 
	ctx.put("dep_error", dep_error);
	// Get configuration information about the datasource
	String datasourceDescription = null;
	String url = null;
	String className = null;
	String userName = null;
	String password = null;
	String datasourceFactory = null;
	String datasourceXADataName = null;
	Integer JDBCConnMaxAge = null;
	Integer JDBCConnCheckLevel = null;
	String JDBCTestStatement = null;
	String JDBCMinConnPool = null;
	String JDBCMaxConnPool = null;
	String JDBCSleepTimeOut = null;
	String JDBCDeadLockPool = null;
	String JDBCLoopWaitPool = null;
	String config_error = "";
	try {
	    datasourceDescription = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceDescription");
	    ctx.put("datasourceDescription", datasourceDescription);
	    url = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "Url");
	    ctx.put("url", url);
	    className = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "ClassName");
	    ctx.put("className", className);
	    userName = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "UserName");
	    ctx.put("userName", userName);
	    password = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "Password");
	    ctx.put("password", password);
	    datasourceFactory = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceFactory");
	    ctx.put("datasourceFactory", datasourceFactory);
	    datasourceXADataName = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceXADataName");
	    ctx.put("datasourceXADataName", datasourceXADataName);  
	    JDBCConnMaxAge = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCConnMaxAge");
	    ctx.put("JDBCConnMaxAge", JDBCConnMaxAge);
	    JDBCConnCheckLevel = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCConnCheckLevel");
	    ctx.put("JDBCConnCheckLevel", JDBCConnCheckLevel);
	    JDBCTestStatement = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCTestStatement");
	    ctx.put("JDBCTestStatement", JDBCTestStatement);
	    JDBCMinConnPool = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCMinConnPool");
	    if ( JDBCLoopWaitPool == null)
		ctx.put("JDBCMinConnPool", "");
	    else
		ctx.put("JDBCMinConnPool", JDBCMinConnPool);
	    JDBCMaxConnPool = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCMaxConnPool");
	    if (JDBCMaxConnPool == null)
		ctx.put("JDBCMaxConnPool", "");
	    else
		ctx.put("JDBCMaxConnPool", JDBCMaxConnPool);
	    JDBCSleepTimeOut = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCSleepTimeOut");
	    if (JDBCSleepTimeOut == null)
		ctx.put("JDBCSleepTimeOut", "");
	    else
		ctx.put("JDBCSleepTimeOut", JDBCSleepTimeOut);
	    JDBCDeadLockPool = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCDeadLockPool");
	    if (JDBCDeadLockPool == null)
		ctx.put("JDBCDeadLockPool", "");
	    else
		ctx.put("JDBCDeadLockPool", JDBCDeadLockPool);
	    JDBCLoopWaitPool = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCLoopWaitPool");
	    if (JDBCLoopWaitPool == null)
		ctx.put("JDBCLoopWaitPool", "");
	    else
		ctx.put("JDBCLoopWaitPool", JDBCLoopWaitPool);
	} catch (ManagementException e) {
	    config_error = "Error while getting attribute:" + e.getMessage();
	} catch (javax.management.MalformedObjectNameException e2) {
	    config_error = "Incompatible name:" + e2.getMessage();
	}
	ctx.put("config_error", config_error);

	Template template = null;
	try {
	    template = getTemplate("DATABASE.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }

}
