/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Enumeration;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;
// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class EJBServerServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   
    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) {
	// Exception treatement
	String alertException = null;

	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters	
	try {
	    String paramName = null;
	    for (Enumeration paramNames = request.getParameterNames() ; paramNames.hasMoreElements() ;) {
		paramName = (String)paramNames.nextElement();
		if (paramName.equals("run")) {
		    Object[] param = new Object[0];
		    String[] signature = new String[0];
		    JonasManagementRepr.invoke(JonasObjectName.ejbServer(), 
					       request.getParameter("run"), param, signature);
		} else if (paramName.equals("Activated")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.ejbServer(), 
						     "Activated", 
						     new Boolean(request.getParameter("Activated")));
		} else if (paramName.equals("NumberOfMeasures")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.ejbServer(), 
						     "NumberOfMeasures", 
						     new Integer(request.getParameter("NumberOfMeasures")));
		} else if (paramName.equals("Range")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.ejbServer(), 
						     "Range", 
						     new Integer(request.getParameter("Range")));
		}
	    }
	} catch (Exception e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Test if the current JOnAS server can be administrated
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));

	if (existsServer) {
	    int range = ((Integer)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "Range")).intValue();
	    int nbOfMeasures = ((Integer)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(),
								      "NumberOfMeasures")).intValue();
	    context.put("range", new Integer(range));
	    context.put("nbOfMeasures", new Integer(nbOfMeasures));
  
	    // Get information about the current JOnAS server
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    String implVersion = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasVersion");
	    String protocol = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasORB");
            Boolean isActivated = (Boolean)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "Activated");
	    Long currentTime = new Long(System.currentTimeMillis());
	    context.put("serverName", serverName);
	    context.put("implVersion", implVersion);
	    context.put("protocol", protocol);
	    context.put("isActivated", isActivated);
	    context.put("currentTime", currentTime);

	    // calculate some values for the graphic
	    Integer width = new Integer(MemoryGraphServlet.getWidth(nbOfMeasures));
	    Integer height = new Integer(MemoryGraphServlet.getHeight());
	    context.put("width", width);
	    context.put("height", height);
	    
	}
       
	Template template = null;
	try {
	    template = getTemplate("EJBSERVER.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }

}
