/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class JMSConnectionFactoryServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	String name = null;
	for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
	    String paramName = (String)e.nextElement();
	    if (paramName.equals("name")) 
		name = request.getParameter("name");
	}
	context.put("name", name);

	String dep_error = "";
	try {
	    logger.log(BasicLevel.DEBUG, "Get the dependences of the resource named: " + name);
	    Object[] param = { name };
	    String[] signature = { "java.lang.String" };
	    java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
									     "getJmsConnectionFactoryDependence", 
									     param, signature);
	    boolean no_deps = false;
	    if (allDep. isEmpty()) {
		logger.log(BasicLevel.DEBUG, "No dependences !!");
		no_deps = true;
	    } else {
		Hashtable deps = new Hashtable();
		for (Iterator i = allDep.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    Dependence dep = new Dependence(current);		
		    deps.put(dep.getEjbName(), dep);
		}
		context.put("deps", deps);
	    }
	    context.put("noDeps", new Boolean(no_deps));
	} catch (ManagementException e) {
	    dep_error = "Error while getting attribute:" + e.toString();
	} catch (MalformedObjectNameException me) {
	    dep_error = "Incompatible name:" + me.getMessage();
	}	

	// Return template
	Template template = null;
	try {
	    template = getTemplate("JMSConnectionFactory.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }
}


