/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Vector;
import java.util.StringTokenizer;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class LogServiceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   
    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	String topicToSet = null;
	String levelToSet = null;
	// Get request parameters
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("topic")) {
		    topicToSet = request.getParameter("topic");
		} else if (paramName.equals("level")) {
		    levelToSet =  request.getParameter("level");
		}
	    }
	    if ((topicToSet != null) && (levelToSet != null)) {
		Object[] param = new Object[2];
		param[0] = topicToSet ; 
		param[1] = levelToSet ;
		String[] signature = new String[2];
		signature[0] = "java.lang.String" ; 
		signature[1] = "java.lang.String" ;
		JonasManagementRepr.invoke(JonasObjectName.logService(), "setTopicLevel", param, signature);
	    }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	}

	// get information about the Log service
	boolean existsLogService = JonasManagementRepr.isRegistered(JonasObjectName.logService());
	context.put("existsLogService", new Boolean(existsLogService));
	if (existsLogService) {
	    String info_error = "";
	    Vector topics = new Vector();
	    try {
		String[] topicNames = (String [])JonasManagementRepr.getAttribute(JonasObjectName.logService(), "Topics");
		String topic = null;
		String level = null;
		for (int i = 0 ; i < topicNames.length ; i++) {
		    topic = topicNames[i];
		    Object[] param = { topic };
		    String[] signature = { "java.lang.String" };
		    level = (String) JonasManagementRepr.invoke(JonasObjectName.logService(), "getTopicLevel", param, signature);
		    if (!topic.equals("root"))
			topics.add(new TopicLevel(topic, level));
		    else
			topics.add(0, new TopicLevel(topic, level));
		}
		context.put("topics", topics);
	    } catch (ManagementException e) {
		info_error = e.getMessage();
	    }
	    context.put("info_error", info_error);
	}

	// Return template
	Template template = null;
	try {
	    template = getTemplate("LOG.vm"); 
	} catch (ParseErrorException pee) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }

    public class TopicLevel {
	String topic;
	String level;

	TopicLevel(String topic, String level) {
	    this.topic = topic;
	    this.level = level;
	}
	
	public String getTopic() {
	    return topic;
	}	
	public String getLevel() {
	    return level;
	}
    }
}
