/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Set;
import java.util.Hashtable;
import java.util.Iterator;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class MailFactoryServletM extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);


	// Get request parameters
	// The name of the mail factory
	String name = null;
	String host = null;
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("name")) {
		    name = request.getParameter("name");
		} else if (paramName.equals("host")) {
		    JonasManagementRepr.setAttribute(
                       JonasObjectName.mimeMailFactory(request.getParameter("name")),
		       "HostMBean",
		       new String(request.getParameter("host")));
		}
	    }
	} catch (MalformedObjectNameException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}
	if (name == null) {
	    alertException = alert("No mail factory name provided !");
	    context.put("alertException", alertException);
	} else {
	    context.put("factoryName", name);
	}


	// Get informations about the mail factory
	String jndiName = null;
	String factoryType = null;// the java class
	String type = "not supported"; // should be one of "Session" or "MimePartDataSource"
	String hostName = null;
	String to = null;
	String cc = null;
	String bcc = null;
	String subject = null;
	// JMX name of the current MBean 
	ObjectName mbeanName = null;
	String info_error = "";
	try {
	    mbeanName = JonasObjectName.mimeMailFactory(name);
	    jndiName = (String)JonasManagementRepr.getAttribute(mbeanName, "NameMBean");
	    context.put("jndiName", jndiName);
	    logger.log(BasicLevel.DEBUG, "Mail Factory Servlet for MimePartDS " + name + ", JNDI name is : " + jndiName);
	    factoryType = (String)JonasManagementRepr.getAttribute(mbeanName, "TypeMBean");
	    context.put("factoryType", factoryType);
	    if (factoryType.equals("javax.mail.Session"))
		type = "Session";
	    else if (factoryType.equals("javax.mail.internet.MimePartDataSource"))
		type = "MimePartDataSource";
	    context.put("type", type);
	    logger.log(BasicLevel.DEBUG, "Mail Factory Servlet for MimePartDS " + name + ", type is : " + factoryType);
	    hostName = (String)JonasManagementRepr.getAttribute(mbeanName, "HostMBean");
	    if (hostName != null)
		context.put("hostName", hostName);
	    else
		context.put("hostName", "");

	    to = (String)JonasManagementRepr.getAttribute(mbeanName, "ToMBean");
	    if (to != null)
		context.put("to", to);
	    else
		context.put("to", "");

	    cc = (String)JonasManagementRepr.getAttribute(mbeanName, "CcMBean");
	    if (cc != null)
		context.put("cc", cc);
	    else
		context.put("cc", "");

	    bcc = (String)JonasManagementRepr.getAttribute(mbeanName, "BccMBean");
	    if (bcc != null)
		context.put("bcc", bcc);
	    else
		context.put("bcc", "");

	    subject = (String)JonasManagementRepr.getAttribute(mbeanName, "SubjectMBean");
	    if (subject != null)
		context.put("subject", subject);
	    else
		context.put("subject", "");


	} catch (MalformedObjectNameException e1) {
	    logger.log(BasicLevel.ERROR, e1.getMessage());
	    info_error = "Incompatible name:" + e1.getMessage();
	} catch (ManagementException e2) {
	    logger.log(BasicLevel.ERROR, e2.getMessage());
	    info_error = "Error while getting attribute - " + e2.getMessage();
	}

	// Get dependences
	String dep_error = "";
	try {
	    Object[] param = { jndiName };
	    String[] signature = { "java.lang.String" };
	    java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(),
									     "getMailFactoryDependence",
									     param, 
									     signature);
	    Hashtable dependences = new Hashtable();
	    if (allDep.isEmpty()) {
		logger.log(BasicLevel.DEBUG, "Try to get dependences of mail factory : " + jndiName + " ??. Empty dependences set !"); 
	    } else {
		for (Iterator i = allDep.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    Dependence dep = new Dependence(current);
		    dependences.put(dep.getEjbName(), dep);
		}
	    }
	    context.put("dependences", dependences);
	} catch (ManagementException e) {
	    dep_error = "Error while getting attribute:" + e.toString();
	} catch (MalformedObjectNameException me) {
	    dep_error = "Incompatible name:" + me.getMessage();
	} 
	context.put("dep_error", dep_error);

	// Return template
	Template template = null;
	try {
	    template = getTemplate("MAIL_FACTORY.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "MailFactoryServletM : parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe ) {
            logger.log(BasicLevel.ERROR, "MailFactoryServletM : template not found " + rnfe);
        } catch( Exception e ) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }
}
