/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Set;
import java.util.Hashtable;
import java.util.Iterator;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class MailServiceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	String factoryName = null;
	// The request types treated by this servlet are : GET and POST
	boolean getRequest;
	if (request.getMethod().equals("GET"))
	    getRequest = true; // we handle a GET request
	else
	    getRequest = false; // we handle a POST request

	Properties p = new Properties();
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (getRequest) {
		    if (paramName.equals("unload")) {
			Object[] param = { request.getParameter("unload") };
			String[] signature = { "java.lang.String" };
			JonasManagementRepr.invoke(JonasObjectName.mailService(),
                                               "unbindMailFactoryMBean", param, signature);
		    }	
		} else {
		    if (paramName.equals("name")) {
			factoryName = request.getParameter("name");
		    } else if (! request.getParameter(paramName).equals("")) {
			p.put(paramName, request.getParameter(paramName));
		    }
		}	
	    }
	    if (!getRequest) {
		logger.log(BasicLevel.DEBUG, "MailServiceServlet : invoke createMailFactoryMBean");
		Object[] param = { factoryName, p };
		String[] signature = { "java.lang.String", "java.util.Properties" };
		JonasManagementRepr.invoke(JonasObjectName.mailService(), 
					   "createMailFactoryMBean", 
					   param, 
					   signature);
	    }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	}

	// Get informations about the Mail Service
	boolean existsMailService = JonasManagementRepr.isRegistered(JonasObjectName.mailService());
	context.put("existsMailService", new Boolean(existsMailService));
	if (existsMailService) {
	    try {
		// get the number of mail factories
		/*Integer nbOfMailFactories = (Integer)JonasManagementRepr.getAttribute(
							JonasObjectName.mailService(),
							"CurrentNumberOfMailFactories");
		*/
	        //logger.log(BasicLevel.DEBUG, "Mail Service : nb. of mail factories = " + nbOfMailFactories.toString());
		// get the number of session mail factories and the set of MBean names associated
		// to these factories
		Integer nbOfSessionMailFactories = (Integer)JonasManagementRepr.getAttribute(
							JonasObjectName.mailService(),
							"CurrentNumberOfSessionMailFactories");
		logger.log(BasicLevel.DEBUG, "Mail Service : nb. of session mail factories = " + nbOfSessionMailFactories.toString());
		Set allSessionMailFactories = JonasManagementRepr.queryNames(JonasObjectName.allSessionMailFactories());
		Hashtable sessionMailFactories = new Hashtable();
		for (Iterator i = allSessionMailFactories.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    String name = (String)JonasManagementRepr.getAttribute(current, "FactoryNameMBean");
		    boolean isUnloadable;
		    // determine if this mail factory is unloadable
		    String jndiName = (String)JonasManagementRepr.getAttribute(current, "NameMBean");
		    Object[] param = { jndiName };
		    String[] signature = { "java.lang.String" };
		    if((!JonasManagementRepr.isRegistered(JonasObjectName.ejbService())) ||
		          (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(),
									"getMailFactoryDependence", 
									param, signature)).isEmpty())
		       )
			isUnloadable = true;
		    else
			isUnloadable = false;
		    // register this session mail factory with its loadable/unloadable value
		    sessionMailFactories.put(name, new Boolean(isUnloadable));
		}
	        
		// get the number of MimePartDSMail mail factories and the set of MBean names associated
		// to these factories
		Integer nbOfMimeMailFactories = (Integer)JonasManagementRepr.getAttribute(
							JonasObjectName.mailService(),
							"CurrentNumberOfMimeMailFactories");
		logger.log(BasicLevel.DEBUG, "Mail Service : nb. of Mime mail factories = " + nbOfMimeMailFactories.toString());
		Set allMimeMailFactories = JonasManagementRepr.queryNames(JonasObjectName.allMimeMailFactories());
		Hashtable mimeMailFactories = new Hashtable();
		for (Iterator i = allMimeMailFactories.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    String name = (String)JonasManagementRepr.getAttribute(current, "FactoryNameMBean");
		    boolean isUnloadable;		    
		    // determine if this mail factory is unloadable
		    String jndiName = (String)JonasManagementRepr.getAttribute(current, "NameMBean");
		    Object[] param = { jndiName };
		    String[] signature = { "java.lang.String" };
		    if((!JonasManagementRepr.isRegistered(JonasObjectName.ejbService())) ||
		          (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(),
									"getMailFactoryDependence", 
									param, signature)).isEmpty())
		       )
			isUnloadable = true;
		    else
			isUnloadable = false;
		    // register this session mail factory with its loadable/unloadable value
		    mimeMailFactories.put(name, new Boolean(isUnloadable));
		}
		//context.put("nbOfMailFactories", nbOfMailFactories);
		context.put("nbOfSessionMailFactories", nbOfSessionMailFactories);
		context.put("sessionMailFactories", sessionMailFactories);
		context.put("nbOfMimeMailFactories", nbOfMimeMailFactories);
		context.put("mimeMailFactories", mimeMailFactories);
	    } catch (ManagementException me) {
		logger.log(BasicLevel.ERROR, me.getMessage());
	    }
	}

	// Return template
	Template template = null;
	try {
	    template = getTemplate("MAIL.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "MailServiceServlet : parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe ) {
            logger.log(BasicLevel.ERROR, "MailServiceServlet : template not found " + rnfe);
        } catch( Exception e ) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }
}
