/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class ResourceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	String jndiName = null;
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("JndiName")) 
		    jndiName = request.getParameter("JndiName");
	    }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}
	context.put("jndiName", jndiName);

	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	}

	if (jndiName == null) {
	    logger.log(BasicLevel.DEBUG, "No resource selected. (??)");
	} else {
	    String props_error = "";
	    Hashtable props = new Hashtable();
	    try {
		Properties p = (Properties)JonasManagementRepr.getAttribute(JonasObjectName.resourceAdapter(jndiName), "Properties");
		for (Enumeration e = p.propertyNames() ; e.hasMoreElements() ;) {
		    String key = (String)e.nextElement();
		    props.put(key, p.getProperty(key));
		}
	    } catch (ManagementException e) {
		props_error = "Error while getting attribute:" + e.getMessage();
	    } catch (MalformedObjectNameException me) {
		props_error = "Incompatible name:" + me.getMessage();
	    }
	}

	// Return template
	Template template = null;
	try {
	    template = getTemplate("RESPROPS.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }


	return template;
    }
}
