/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Servlet used to display information about a stateless session bean.
 * called with the following parameters :
 * fileName = name of the .jar file
 * ejbName  = name of the Entity bean
 * @author Philippe Durieux, Adriana Danes (JOnAS team)
 */
public class SblServlet extends SessionServlet {

    /**
     * Add info in the provided context and return the template to be used.
     * @param req servlet request from client
     * @param res servlet reponse
     * @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this template
     * @return Template to be used for request
     */
    public Template handleRequest(HttpServletRequest req, HttpServletResponse res, Context ctx) { 

	// Get the template to be returned.
	Template template = null;
	try {
	    template = getTemplate("SBL.vm");
        } catch(Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
	    return null;
        }

	// Get request parameters and put them in ctx
	String fileName = null;
	String ejbName = null;
	try {
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("fileName")) {
		    fileName = req.getParameter("fileName");
		    ctx.put("fileName", fileName);
		} else if (paramName.equals("ejbName")) {
		    ejbName = req.getParameter("ejbName");
		    ctx.put("ejbName", ejbName);
		}
	    }
	} catch (ManagementException e) {
	    ctx.put("alertException", alert(e.getMessage()));
	    return template;
	}

	// Get the current Date
	getDate(ctx);

	// Get the container's name
	try {
	    String contName = (String)JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), "ContainerName");
	    ctx.put("contName", contName);
	} catch (Exception e) {
	    ctx.put("alertException", alert("Cannot get ContainerName: "+e.getMessage()));
	    return template;
	}
	
	// Get the name of the associated MBean
	ObjectName mbean = null;
	try {
	    mbean = JonasObjectName.ejbsbl(fileName, ejbName);
	} catch (Exception e) {
	    ctx.put("alertException", alert("Cannot get mbean: "+e.getMessage()));
	    return template;
	}

	// Get all info about this EJB from EJBServer
	try {
	    getBeanInfo(ctx, mbean);
	    getSessionBeanInfo(ctx, mbean);
	} catch (Exception e) {
	    ctx.put("alertException", alert("Cannot get bean info: "+e.getMessage()));
	    return template;
	}

	getDatasourceInfo(ctx, mbean);
	getConnFactoryInfo(ctx, mbean);
	getDestinationInfo(ctx, mbean);
	getMailFactoryInfo(ctx, mbean);

	return template;
    }
}
