/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class TransactionServiceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
	    }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	}

	// Get information about the transaction service
	boolean existsTransactionService = JonasManagementRepr.isRegistered(JonasObjectName.transactionService());
	context.put("existsTransactionService", new Boolean(existsTransactionService));
	if (existsTransactionService) {
	    String info_error = "";
	    try {
		Integer currentTransactionTimeOut = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.transactionService(),
											  "CurrentTimeOut");

		String jtm = "other JVM";
		if (((Boolean)JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
							       "JTMLocal")).booleanValue()) 
		    jtm = "collocated";

		Integer portNumber = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
									       "JTMPortNumber");
		String hostName = (String)JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
								       "JTMHostName");
		String portValue;
		if (portNumber.equals(new Integer(0))) 
		    portValue = "undefined";
		else 
		    portValue = portNumber.toString();
		context.put("currentTransactionTimeOut", currentTransactionTimeOut);
		context.put("jtm", jtm);
		context.put("hostName", hostName);
		context.put("portValue", portValue);
	    } catch (ManagementException e) {
		info_error = e.getMessage();
	    }
	    context.put("info_error", info_error);

	    String status_error = "";
	    try {
		Integer [] allTxCounters = (Integer [])JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), "TxCounters");
		context.put("current_trans_number", allTxCounters[0]);
		context.put("begun_trans_number", allTxCounters[1]);
		context.put("commited_trans_number", allTxCounters[2]);
		context.put("rollbacked_trans_number", allTxCounters[3]);
		context.put("expired_time_out_number", allTxCounters[4]);
	    } catch (ManagementException e) {
		status_error = e.getMessage();
	    }
	    context.put("status_error", status_error);
	} 

	// Return template
	Template template = null;
	try {
	    template = getTemplate("TRANSACTION.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }
}
