/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Set;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class WebServiceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("remove")) {
		    Object[] param = { request.getParameter("remove") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.webContainerService(), 
					       "unRegisterWarMBean", param, signature);
		} else if (paramName.equals("create")) {
		    Object[] param = { request.getParameter("create") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.webContainerService(), 
					       "reisterWarMBean", param, signature);
		} 
	    }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	    
	    // Get information about the wars currently loaded in the server
	    // For doing this, the corresponding MBean has to be registered
	    ObjectName webContainerServiceName = JonasObjectName.webContainerService();
	    boolean isRegisteredWebService = JonasManagementRepr.isRegistered(webContainerServiceName);
	    context.put("isRegisteredWebService", new Boolean(isRegisteredWebService));
	    if (isRegisteredWebService) {
		String webContainerService_error = "";
		Vector wars = new Vector();
		try {
		    Integer nbWars = (Integer)JonasManagementRepr.getAttribute(webContainerServiceName, "CurrentNumberOfWars");
		    context.put("nbWars", nbWars);

		    // This is the way to do ! 
		    // We will do this when wars will become manageable
		    //java.util.Set allWars = JonasManagementRepr.queryNames(JonasObjectName.allWars());

		    // This the temporary solution:
		    // cut here --------
		    Object[] param = { };
		    String[] signature = { };
		    Set allWars = (Set)JonasManagementRepr.invoke(webContainerServiceName, "getWarNames", param, signature);
		    context.put("wars", allWars);
		    // cut here -------

		    /* Continue this way
		    for (Iterator i = allWars.iterator(); i.hasNext() ;) {			
			ObjectName current = (ObjectName)i.next();
			String fileName = (String)JonasManagementRepr.getAttribute(current, "FileName");
			String warName = (String)JonasManagementRepr.getAttribute(current, "WarName");
			wars.add(new WarInfo(warName, fileName.replace(' ', '+')));
		    }
		    context.put("wars", wars);	
		    */	    		    
		} catch (ManagementException e) {
		    webContainerService_error = e.getMessage();
		}
		context.put("webService_error", webContainerService_error);
	    }
	} 

	// Return template
	Template template = null;
	try {
	    template = getTemplate("WEB_SERVICE.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "WebServiceServlet: Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "WebServiceServlet: Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "WebServiceServlet: Error " + e);
        }

	return template;
    }

    /**
     * @param msg The error title.
     * @param msg The error message.
     * @return a string displaying the error msg in tr tags.
     */
    protected String alert(String msg) {
	StringBuffer sb = new StringBuffer("<script>alert(\"");
	StringTokenizer st = new StringTokenizer(msg, "\n");
	while (st.hasMoreTokens()) 
	    sb.append(st.nextToken()+"\\n");
	sb.append("\")</script>");
	return new String(sb);
    }
}
