/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.mail;


//import java
import java.rmi.RemoteException;
import java.util.Properties;

/**
 * This interface provides a description for the mail service management.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public interface MailServiceImplMBean { 

    /**
     * Create a mail factory with the specified properties and register it 
     * into the registry.
     * @param name the name of the mail factory
     * @param props the properties used to configure the mail factory.
     * @throws MailServiceException if the creation or the registration of 
     * the factory failed.
     * @throws RemoteException if rmi call failed.
     */
    public void createMailFactoryMBean(String name, Properties props) 
        throws MailServiceException;

    /**
     * Unregister the factory binded with the given name.
     * @param name the name of the factory.
     * @throws MailServiceException if the unregistration of the 
     * factory failed.
     * @throws RemoteException if rmi call failed.
     */
    public void unbindMailFactoryMBean(String name) 
        throws MailServiceException;

    /**
     * @return Integer Number of mail Session mail factories available in JOnAS
     */ 
    public Integer getCurrentNumberOfSessionMailFactories();

    /**
     * @return Integer Number of internet.MimePartDataSource mail factories available in JOnAS
     */ 
    public Integer getCurrentNumberOfMimeMailFactories();

    /**
     * @return Integer Total number of mail factories available in JOnAS
     */ 
    public Integer getCurrentNumberOfMailFactories();

    /**
     * @return mail factory configuration properties from a local file
     */
    public Properties getMailFactoryPropertiesFile(String configFile) throws Exception;

}
