/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.mail.factory;

//import java
import java.rmi.RemoteException;
import java.util.Properties;

/**
 * This interface defines all the MBean methods provided by the JMail objects.
 * @author Florent Benoit
 * @author Ludovic Bert
 */
public interface JMailMBean {

    /**
     * Get the name of the factory.
     * @return name of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getFactoryNameMBean() throws RemoteException;

    /**
     * Get the jndi name of the factory.
     * @return jndi name of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getNameMBean() throws RemoteException;

    /**
     * Get the type of the factory.
     * @return type of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getTypeMBean() throws RemoteException;

    /**
     * Get the mail server's host
     * @return the name of the host 
     * @throws RemoteException if rmi call failed.
     */
    public String getHostMBean() throws RemoteException;

    /**
     * Set the mail server's host
     * @parm host the name of the host 
     * @throws RemoteException if rmi call failed.
     */
    public void setHostMBean(String host) throws RemoteException;

    /**
     * Get the authentication properties.
     * @return properties of the authentication.
     * @throws RemoteException if rmi call failed.
     */
    public Properties getAuthenticationPropertiesMBean() throws RemoteException;

    /**
     * Get the session properties.
     * @return the properties of Session.
     * @throws RemoteException if rmi call failed.
     */
    public Properties getSessionPropertiesMBean() throws RemoteException;

    /**
     * Set the authentication properties.
     * @param props the authentication properties.
     * @throws RemoteException if rmi call failed.
     */
    public void setAuthenticationPropertiesMBean(Properties props) throws RemoteException;

    /**
     * Set the session properties.
     * @param props the session properties.
     * @throws RemoteException if rmi call failed.
     */
    public void setSessionPropertiesMBean(Properties props) throws RemoteException;
}

