/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.mail.factory;

//import java
import java.rmi.RemoteException;
import java.util.Properties;

/**
 * This interface defines all the MBean methods provided by the JMailSession objects.
 * @author Florent Benoit
 * @author Ludovic Bert
 */
public interface JMailMimePartDSMBean extends JMailMBean {

    /**
     * Get the mimePartDatasource  properties.
     * @return the properties of Session.
     * @throws RemoteException if rmi call failed.
     */
    public Properties getMimeMessagePropertiesMBean() throws RemoteException;

    /**
     * Set the mimePartDatasource properties.
     * @param props the authentication properties.
     * @throws RemoteException if rmi call failed.
     */
    public void setMimeMessagePropertiesMBean(Properties props) throws RemoteException;

    /**
     * Get the "to" property of the factory (the list of mail recipients)
     * @return the "to" property of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getToMBean() throws RemoteException;

    /**
     * Get the "cc" property of the factory (the Carbon Copy list of mail recipients)
     * @return the "cc" property of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getCcMBean() throws RemoteException;


    /**
     * Get the "bcc" property of the factory (the Blind Carbon Copy list of mail recipients)
     * @return the "bcc" property of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getBccMBean() throws RemoteException;

    /**
     * Get the "subject" property of the factory
     * @return the "subject" property of the mail factory.
     * @throws RemoteException if rmi call failed.
     */
    public String getSubjectMBean() throws RemoteException;
}
