/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 * 
 */

package org.objectweb.jonas.rmifilters ;

import org.objectweb.security.propagation.SecurityReceiver ;
import org.objectweb.security.context.SecurityCurrent ;
import org.objectweb.security.context.SecurityContext ;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class represents the implementation of a SecurityReceiver in JOnAS.
 *
 * @author Jeff Mesnil
 * Contributor(s):
 */
public class JonasSecurityReceiver implements SecurityReceiver {

    // logger of trace (used for security context)
    static private Logger logger = null;

    /**
     * Empty constructor
     */
    public JonasSecurityReceiver () {
        if(logger == null)
            logger = Log.getLogger(Log.JONAS_SECURITY_PREFIX);
    }

    
    public void received_request (int request_id, SecurityContext ctx) {
	logger.log(BasicLevel.DEBUG, "received request context " + ctx);
	 // Gets SecurityCurrent object (always existing in JOnAS Server)
	SecurityCurrent current = SecurityCurrent.getCurrent();
	if (current == null) {
	    logger.log(BasicLevel.DEBUG, "no SecurityCurrent in this server!");
	    return;
	}
	// Associates the current thread with this security context
	current.setSecurityContext (ctx) ;
    }
           
    public SecurityContext sending_reply (int request_id) {
	// Gets Current object (always existing in JOnAS Server)
	SecurityCurrent current = SecurityCurrent.getCurrent();
        if (current == null) {
	    logger.log(BasicLevel.DEBUG, "no SecurityCurrent in this server!");
	    return null;
	}
	// Get the Security Context
	SecurityContext ctx = current.getSecurityContext();
	// associates the current thread with a new default SecurityContext
	logger.log(BasicLevel.DEBUG, "security context sets to default.");
	current.setSecurityContext(new SecurityContext());
	logger.log(BasicLevel.DEBUG, "creates reply context "+ ctx+"\n");
	//System.out.println ("JOnAS send a reply : "+ ctx.getCallerPrincipal().getName()) ;
	return ctx ;
    }
}
