/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * 01/06/15 Regis Le Brettevillois - Libelis / JOnAS team - Evidian
 *          Creation.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.service;


/**
 * This interface defines a Service.
 * Objects which implement this interface must have a public constructor
 * with a string parameter which is the name of the service.
 */


import javax.naming.Context;


public interface Service {

    /**
     * Initialize the service
     * @param ctx configuration of the service
     */
    public void init(Context ctx) throws ServiceException;

    /**
     * Start the service
     */
    public void start() throws ServiceException;

    /**
     * Stop the service
     */
    public void stop() throws ServiceException;

    /**
     * Returns true if the service is started, false otherwise
     */
    public boolean isStarted();

    /**
     * Set the service's name
     */
    public void setName(String name);

    /**
     * Returns the service's name
     */
    public String getName();
}
