/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.web;

import java.rmi.RemoteException;
import java.util.Set;

/**
 * This interface provides a description for the web container management.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public interface AbsJWebContainerServiceImplMBean { 

    /**
     * Register a WAR, this method is used for the JMX Management.
     * @param fileName the name of the war to deploy.
     * @throws RemoteException if rmi call failed.
     * @throws JWebContainerServiceException if the registration of the WAR 
     * failed.
     */
    public void registerWarMBean(String fileName) 
        throws RemoteException, JWebContainerServiceException;

    /**
     * Unregister a WAR, this method is used for the JMX Management.
     * @param fileName the name of the war to undeploy.
     * @throws RemoteException if rmi call failed.
     * @throws JWebContainerServiceException if the unregistration of the WAR 
     * failed.
     */
    public void unRegisterWarMBean(String fileName) 
        throws RemoteException, JWebContainerServiceException;

    /**
     * @return current number of wars deployed in the JOnAS server
     */
    public Integer getCurrentNumberOfWars();

    /**
     * This method is added temporarily. It will disapear when Wars will have their associated MBeans
     * (when Wars will become manageable)
     * @return the names of the wars currently deployed in the JOnAS server
     */
    public Set getWarNames();
}
