/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.web;

//import java
import java.net.URL;

//import javax
import javax.naming.Context;

//import jonas
import org.objectweb.jonas.service.Service;


/**
 * JOnAS WEB Container Service interface.
 * This interface provides a description of a web container service.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public interface JWebContainerService extends Service {

    /**
     * Deploy the given wars of an ear file with the specified parent 
     * classloader (ejb classloader or ear classloader). (This method
     * is only used for the ear applications, not for the 
     * web applications).
     * @param ctx the context containing the configuration 
     * to deploy the wars.<BR>
     * This context contains the following parameters :<BR>
     *    - urls the list of the urls of the wars to deploy.<BR>
     *    - earURL the URL of the ear application file.<BR>
     *    - parentClassLoader the parent classLoader of the wars.<BR>
     *    - earClassLoader the ear classLoader of the j2ee app.<BR>
     *    - altDDs the optional URI of deployment descriptor.<BR>
     *    - contextRoots the optional context root of the wars.<BR>
     * @throws JWebContainerServiceException if an error occurs during 
     * the deployment.
     */
    public void deployWars(Context ctx) throws JWebContainerServiceException;
    
    /**
     * Undeploy the given wars of an ear file with the specified parent 
     * classloader (ejb classloader or ear classloader). (This method
     * is only used for the ear applications, not for the 
     * war applications).
     * @param urls the list of the urls of the wars to undeploy.
     */
    public void unDeployWars(URL[] urls);

    /**
     * Make a cleanup of the cache of deployment descriptor. This method must 
     * be invoked after the ear deployment by the EAR service.
     * the deployment of an ear by .
     * @param earClassLoader the ClassLoader of the ear application to
     * remove from the cache.
     */
    public void removeCache(ClassLoader earClassLoader);

}
