/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Riviere
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.container;

import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Hashtable;
import java.util.Properties;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;

/**
 * This Factory return Home or LocalHome objects to clients.
 * No need to use it in case of RMI Remote Home because Remote References are
 * registered directly in JNDI.
 * This class should be used only for local Home or in case of RMI IIOP Remote Home.
 */
public class HomeFactory implements ObjectFactory {

     public Object getObjectInstance(Object refObj, Name name, Context nameCtx, Hashtable env) throws Exception {
	 Reference ref = (Reference) refObj;
	 String clname = ref.getClassName();

	 TraceEjb.interp.log(BasicLevel.DEBUG, "HomeFactory: getObjectInstance "+clname);

	if (clname.equals("org.objectweb.jonas_ejb.container.JLocalHome")) {
	    // Local Home
	    String beanName = (String)ref.get("bean.name").getContent();
	    JLocalHome localhome = JLocalHome.getLocalHome(beanName);
	    if (localhome == null) {
		TraceEjb.error("HomeFactory: Cannot get "+beanName);
	    }
	    return localhome;
	} else if (clname.equals("org.objectweb.jonas_ejb.container.JHome")) {
	    // David Remote Home
	    String beanName = (String)ref.get("bean.name").getContent();
	    try {
		// narrrow not required here (done later by client)
		return JFactory.getCorbaInitialContext().lookup(beanName);
	    } catch(javax.naming.NamingException e) {
		TraceEjb.error("HomeFactory: RMI IIOP Remote Home not working yet");
	    }
	} else {
	    TraceEjb.error("HomeFactory: Bad class name: "+clname);
	}
	// not found !
	return null;
    }
}
