/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Philippe Coq
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.io.Serializable;
import java.rmi.RemoteException;
import javax.ejb.EJBObject;
import javax.ejb.Handle;
import javax.ejb.HomeHandle;

/**
 * This class implements javax.ejb.Handle interface.
 * For a Entity Bean  a Handle is a serializable class
 * that contains the HomeHandle and the primary key
 */
abstract public class JEntityHandle implements Handle, Serializable {

    /**
     * @serial
     */
    protected HomeHandle homehandle = null;

    /**
     * @serial
     */
    protected Serializable pk = null;

    /**
     * constructor
     */    
    public JEntityHandle(JEntityRemote remote) {
	try {
	    homehandle = remote.getEJBHome().getHomeHandle();
	} catch (RemoteException e) {
	    TraceEjb.error("Cannot get HomeHandle: ", e);
	}
    }

    
    /** 
     * Obtains the EJB object represented by this handle. 
     * @Throws RemoteException: The EJB object could not be obtained because of a system-level
     * failure. 
     */
   abstract public EJBObject getEJBObject() throws java.rmi.RemoteException;

    /**
     * Return the Primary Key embedded in the Handle
     */
    
    public java.lang.Object getPK() {
	return pk;
    }
}

