/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Philippe Coq, Philippe Durieux
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.rmi.RemoteException;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class is the Standard Home for Entity objects
 * It exists only for beans that have declared a Remote Interface.
 * It implements javax.ejb.EJBHome interface
 */
public abstract class JEntityHome extends JHome {

    /**
     * constructor
     */
    public JEntityHome(EntityDesc dd, JEntityFactory bf) throws RemoteException {
	super(dd, bf);
	TraceEjb.interp.log(BasicLevel.DEBUG, "JEntityHome constructor");
    }

    // ---------------------------------------------------------------
    // EJBHome implementation
    // both remove() methods are in the generated part.
    // other methods are in JHome (identical for Sessions and Entities)
    // ---------------------------------------------------------------

    // ---------------------------------------------------------------
    // other public methods, for internal use.
    // ---------------------------------------------------------------

    public RequestCtx preInvoke(int txa, String secu) throws RemoteException {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JEntityHome preinvoke");
	return bf.preInvokeRemote(txa, secu);
    }

    public void postInvoke(RequestCtx rctx) throws RemoteException {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JEntityHome postinvoke");
	bf.postInvokeRemote(rctx);
    }

    abstract public JEntityRemote createRemoteObject() throws RemoteException;
}
