/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Philippe Durieux
 * Contributor(s): 
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.util.Map;
import java.util.HashMap;
import java.util.Collections;
import javax.ejb.EJBLocalHome;
import javax.ejb.HomeHandle;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class represents an EJBLocalHome
 * It is shared between Sessions and Entities.
 */
public abstract class JLocalHome implements EJBLocalHome {

    protected BeanDesc dd;
    protected JFactory bf;

    // The static table of all JLocalHome objects
    protected static Map homeList = new HashMap();

    /**
     * Constructor for the base class of the specific generated Home object.
     */
    public JLocalHome(BeanDesc dd, JFactory bf) {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JLocalHome Constructor");
	this.dd = dd;
	this.bf = bf;
    }

    // ---------------------------------------------------------------
    // EJBLocalHome Implementation
    // ---------------------------------------------------------------

    /**
     * Removes an EJB object identified by its primary key.
     */
    abstract public void remove(Object primaryKey) throws RemoveException;

    // ---------------------------------------------------------------
    // other public methods
    // ---------------------------------------------------------------

    /**
     * register this bean to JNDI (rebind)
     * We register actually a Reference object.
     */
    protected void register() throws NamingException {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JLocalHome register");
	String name = dd.getJndiLocalName();

	// Keep it in the static list for later retrieving.
	homeList.put(name, this);
	
	Reference ref = new Reference("org.objectweb.jonas_ejb.container.JLocalHome",
				      "org.objectweb.jonas_ejb.container.HomeFactory",
				      null);
	ref.add(new StringRefAddr("bean.name", name));
	bf.getInitialContext().rebind(name, ref);
    }

    /**
     * unregister this bean in JNDI (unbind)
     */
    protected void unregister() throws NamingException {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JLocalHome unregister");
	String name = dd.getJndiLocalName();
	// unregister in default InitialContext
	bf.getInitialContext().unbind(name);
    }

    /**
     * Get JLocalHome by its name
     */
    public static JLocalHome getLocalHome(String beanName) {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JLocalHome getLocalHome " + beanName);
	JLocalHome lh = (JLocalHome) homeList.get(beanName);
	return lh;
    }

}
