/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Philippe Durieux
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import javax.ejb.EJBObject;
import org.objectweb.common.RemoteObject;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class is the common part of the EJBObject implementation class.
 * Here goes the code common for Entity beans and Session beans
 */
public abstract class JRemote extends RemoteObject implements EJBObject {

    protected JFactory bf;

    /**
     * constructor
     */
    public JRemote(JFactory bf) throws RemoteException {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JRemote constructor");
	this.bf = bf;
    }

    // --------------------------------------------------------------------------
    // EJBObject implementation
    // --------------------------------------------------------------------------

    // --------------------------------------------------------------------------
    // Export / UnExport Object
    // Due to the fact that this object extends RemoteObject, it is automatically
    // exported when built. We just need to unexport it when unused, and to re export
    // it if we reuse it again.
    // --------------------------------------------------------------------------

    /**
     * Make this object accessible again thru the Orb.
     * @returns True if export is OK.
     */
    public boolean exportObject() {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JRemote exportObject");
	try {
	    exportObject(this, getPort());
	} catch (Exception e) {
	    TraceEjb.error("exportObject: cannot export:"+e);
	    return false;
	}
	return true;
    }
	    
    /**
     * Make this object unaccessible thru the Orb.
     */
    public void unexportObject() throws NoSuchObjectException {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JRemote unexportObject");
	if (unexportObject(this, true) == false) {
	    TraceEjb.error("Cannot unexport EJBObject");
	}
    }
}
