/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Philippe Durieux
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import javax.ejb.EJBException;
import javax.ejb.RemoveException;
import org.objectweb.jonas_ejb.deployment.api.SessionDesc;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class is the Standard LocalHome for Session objects
 * It exists only for beans that have declared a Local Interface.
 * It implements javax.ejb.EJBLocalHome interface
 * It implements a pool of EJBLocalObject's
 */
public abstract class JSessionLocalHome extends JLocalHome {

    /**
     * constructor
     */
    public JSessionLocalHome(SessionDesc dd, JSessionFactory bf) {
	super(dd, bf);
	TraceEjb.interp.log(BasicLevel.DEBUG, "JSessionLocalHome constructor");
    }

    // ---------------------------------------------------------------
    // EJBLocalHome implementation
    // The only method is remove(pk)
    // ---------------------------------------------------------------

    /**
     * remove(pk) is not allowed for session beans
     */
    public void remove(java.lang.Object pk) throws EJBException, RemoveException {
	throw new RemoveException("remove by PK Cannot be called in a session bean");
    }

    // ---------------------------------------------------------------
    // other public methods, for internal use.
    // ---------------------------------------------------------------

    public RequestCtx preInvoke(int txa, String secu) {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JSessionLocalHome preInvoke");
	return bf.preInvoke(txa, secu);
    }

    public void postInvoke(RequestCtx rctx) {
	TraceEjb.interp.log(BasicLevel.DEBUG, "JSessionLocalHome postInvoke");
	bf.postInvoke(rctx);
    }

    /**
     * Creates the EJBLocalObject
     * this is in the generated class because it is mainly "new objectClass()"
     */
    abstract public JSessionLocal createLocalObject();

}
