/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Sebastien Chassande
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container.jorm;

import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jonas_ejb.container.JEntityFactory;
import org.objectweb.jonas_ejb.container.JEntitySwitch;
import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jonas_ejb.deployment.api.DeploymentCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.lib.JormType;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.naming.api.PBinderDelegate;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import java.util.Iterator;

/**
 * This class is an extention of the JEntityFactory class. It initializes the
 * persitant class (the bean) in the jorm mapper. This class is abstract in
 * order to be extend by the PClassMapping generate for the Bean.
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class JormFactory
    extends JEntityFactory
    implements PClassMapping {

    public JormFactory() {
        super();
    }

    public abstract Object getConnection(Object hints) throws PException;

    public abstract void releaseConnection(Object conn) throws PException;

    public abstract String getMIMappingFactory(EntityDesc bd);

    public abstract PMapper getMapper();

    public abstract PMapper getMapper(EntityDesc bd) throws PException;

    public String getMappingName(EntityDesc bd) throws PException {
        return DeploymentCmp2Desc.MAPPING_NAME;
    }

    // OVERRIDE JEntityFactory CLASS //
    //-------------------------------//

    public void init(EntityDesc ed, JContainer c) {
        TraceEjb.logger.log(BasicLevel.DEBUG, "JormFactory initialisation for the bean " + ed.getEjbName());
        // First, call the super in order to have the access to the datasource
        // and the beanNaming
        super.init(ed, c);

        EntityCmp2Desc ecd = (EntityCmp2Desc) ed;

        //----------------------- JORM INITIALIZATION -----------------------//
        TraceEjb.logger.log(BasicLevel.DEBUG, "Jorm initialisation");
        PBinderDelegate binder = null;
        try {
            try {
                binder = (PBinderDelegate)
                    Class.forName(ecd.getJormBinderClassName()).newInstance();
            }
            catch (ClassNotFoundException e) {
                binder = (PBinderDelegate)
                    c.getClassLoader().loadClass(
                        ecd.getJormBinderClassName()).newInstance();
            }
            // Instanciate the binder
            if (ecd.hasPrimaryKeyField()) {
                ((BasidBinder) binder).setCodingType(
                    JormType.getCodingType(
                        ecd.getCmpFieldDesc(
                            ecd.getPrimaryKeyFieldName()).getFieldType()));
                ((BasidBinder) binder).setNulOValue(null);
            }
            else {
                //TODO:
                // - take the initial values of the primary key
                // - put them into the generated binder (use a PNG)
            }

            TraceEjb.logger.log(BasicLevel.DEBUG, "binder "
                + ecd.getJormBinderClassName() + "instanciated");

            // Link the binder and the PClassMapping
            binder.setPClassMapping(this);
            setPBinder(binder);
            TraceEjb.logger.log(BasicLevel.DEBUG, "binder linked to the mapping");

        }
        catch (Exception e) {
            TraceEjb.logger.log(BasicLevel.ERROR,
                "Impossible to create the binder", e);
        }

        //for each reference assignes the PNamingcontext if it is possible.
        try {
            for (Iterator it = ecd.getEjbRelationshipRoleDescIterator(); it.hasNext();) {
                EjbRelationshipRoleDesc rsr = (EjbRelationshipRoleDesc) it.next();
                PClassMapping pcm2 = (PClassMapping)
                    c.getBeanFactory(rsr.getLinkedEntityCmp2Desc().getEjbName());
                if (rsr.hasCmrField() && pcm2 != null) {
                    // As the bean with which the current bean is in relation
                    // may be not load, the test of pcm2!=null is needed
                    setPNamingContext(
                        rsr.getCmrFieldName(), pcm2, true);
                }
                if (pcm2 != null
                    && rsr.isTarget()
                    && pcm2.getPNamingContext(ecd.getEjbName()) == null) {
                    // This case appears when the scheduling of the bean adding
                    // in the container is bad: A bean has been load and have
                    // a relation to current bean. We must therfore set the
                    // naming context on the PClassMapping of the first bean
                    pcm2.setPNamingContext(ecd.getEjbName(), this, true);
                }
            }
        }
        catch (Exception e) {
            TraceEjb.logger.log(BasicLevel.ERROR,
                "Impossible to assign the naming context to the PClassMapping", e);
            throw new EJBException(
                "Impossible to assign the naming context to the PClassMapping", e);
        }

        // Fetch the mapper
        PMapper m = null;
        try {
            m = getMapper(ed);
        }
        catch (Exception e) {
            TraceEjb.logger.log(BasicLevel.ERROR,
                "Impossible to fetch the mapper", e);
            throw new EJBException("Impossible to fetch the mapper", e);
        }

        // here it is possible to add the parametrization of the action to do
        // on the hosting structures (sql tables).
        // dataAction = dd.getxxxx()
        byte dataAction = PClassMapping.CLEANUP_DONOTHING;

        // Map the user class into the mapper
        Object conn = null;
        try {
            conn = getConnection(null);
            m.map(conn, this, dataAction, false);
            TraceEjb.logger.log(BasicLevel.DEBUG,
                getClassName() + " is mapped");
        }
        catch (PException pe) {
            TraceEjb.error(
                "Impossible to map the class on the rdb mapper", pe);
            throw new EJBException(
                "Impossible to map the class on the rdb mapper", pe);
        }
        finally {
            try {
                releaseConnection(conn);
            }
            catch (PException e) {
                TraceEjb.error("Impossible to close a connection", e);
            }
        }

    }

	public void stop() {
		super.stop();
		try {
			getMapper().unmap(getClassName());
		} catch (PException e) {
			TraceEjb.logger.log(BasicLevel.ERROR,
				"Impossible to unmap the class " + getClassName(), e);
		}
	}

	/**
     * This method is overrided in order to specify the JEntityContext class
     * which must be instanciated.
     * Create a new instance of the bean and its EntityContext
     * In case of CMP, the bean class is derived to manage entity persistence.
     */
    protected org.objectweb.jonas_ejb.container.JEntityContext
        createNewInstance() throws Exception {

        TraceEjb.interp.log(BasicLevel.DEBUG, "JormFactory.createNewInstance");

        // create the bean instance
        EntityBean bean = null;
        try {
            bean = (EntityBean) beanclass.newInstance();
        }
        catch (InstantiationException e) {
            TraceEjb.error(ejbname + " Cannot instantiate bean");
            throw e;
        }
        catch (IllegalAccessException e) {
            TraceEjb.error(ejbname + " Cannot instantiate bean");
            throw e;
        }
        TraceEjb.interp.log(BasicLevel.DEBUG, "JormFactory.createNewInstance: bean class instancied :" + beanclass.getName());

        // create a new EntityContext and bind it to the instance
        org.objectweb.jonas_ejb.container.JEntityContext ec = new
            org.objectweb.jonas_ejb.container.jorm.JEntityContext(this, bean);
        bean.setEntityContext(ec);

        // If too much instances, ask swapper to clean memory.
        instanceCount++;
        if (maxCacheSize != 0 && instanceCount > maxCacheSize) {
            cont.registerBF(this);
        }

        return ec;
    }

    public JEntitySwitch getJEntitySwitch() {
        Object result = null;
        try {
            result = Class.forName(
                ((EntityCmp2Desc) dd).getJormBindingClassName()).newInstance();
        }
        catch (ClassNotFoundException e) {
            try {
                result = getContainer().getClassLoader().loadClass(
                    ((EntityCmp2Desc) dd).getJormBindingClassName()).newInstance();
            }
            catch (Exception e1) {
                TraceEjb.error(
                    "Impossible to create a new JEntitySwitch as specifief in BeanNaming: "
                    + ((EntityCmp2Desc) dd).getJormBindingClassName(), e);
                return null;
            }
        }
        catch (Exception e) {
            TraceEjb.error(
                "Impossible to create a new JEntitySwitch as specifief in BeanNaming: "
                + ((EntityCmp2Desc) dd).getJormBindingClassName(), e);
            return super.getJEntitySwitch();
        }
        try {
            ((PBinding) result).init(this);
        }
        catch (PException e) {
            TraceEjb.error(
                "Impossible to initialized the new JEntitySwitch as specifief in BeanNaming: "
                + ((EntityCmp2Desc) dd).getJormBindingClassName(), e);
        }
        return (JEntitySwitch) result;
    }
}
