/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): S.Chassande_________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container.jorm;

import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.lib.BasicManager;
import org.objectweb.jorm.runtime.Mapper;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.lib.PTypeSpacePAAH;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.HashMap;
import java.util.Map;
import javax.naming.Context;

/**
 * This class is an implemention of JOnAS service in order to manage Jorm
 * mappers. This class creates and references also the PTypeSpace and the Meta
 * information manager.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class MapperManager {

    /**
     * The singleton instance of the class
     */
    private static MapperManager singleton = null;

    /**
     * It retrieves the unique instance of MapperManager.
     */
    public static MapperManager getInstance() {
        if (singleton == null) {
            singleton = new MapperManager();
        }
        return singleton;
    }

    /**
     * The Logger used in this class
     */
    private Logger logger = null;

    /**
     *  Names of the mapper to create when starting the mapper service
     */
    String mapperNames = null;

    /**
     * This fields contains the association between a mapper ant its name.
     * key: String mapper name
     * Value: PMapper a mapper instance
     */
    private HashMap mappers = new HashMap();

    /**
     *  Initial Context for Naming
     */
    private Context ictx = null;

    /**
     * This field references the PTypeSpace shared by all mappers.
     */
    private PTypeSpace ptypeSpace;

    /**
     * This fields references the meta information manager shared by all mappers
     */
    private Manager manager;

    protected MapperManager() {
        logger = TraceEjb.logger;
        ptypeSpace = new PTypeSpacePAAH();
        logger.log(BasicLevel.DEBUG, "PTypeSpace created");
        manager = new BasicManager();
        logger.log(BasicLevel.DEBUG, "Meta information manager created");

    }

    public PMapper getMapper(String mappername, JContainer c) {
		Map m = (Map) mappers.get(c);
        return (m == null ? null : (Mapper) m.get(mappername));
    }

    public PMapper addMapper(Mapper m, JContainer c) {
		Map map = (Map) mappers.get(c);
		PMapper pm =  null;
		if (map == null) {
			map = new HashMap();
			mappers.put(c, map);
			pm = null;
		} else {
			pm = (PMapper) map.get(m.getMapperName());
		}
        if (pm == null) {
            pm = m;
            String mappername = m.getMapperName();
            if (TraceEjb.loggerFactory == null) {
                m.setLogger(logger);
            }
            else {
                m.setLogger(TraceEjb.loggerFactory.getLogger(
                    TraceEjb.prefix + "." + mappername));
                m.setLoggerFactory(TraceEjb.loggerFactory);
            }
            m.setPTypeSpace(ptypeSpace);
            m.setMetaInfoManager(manager);
            map.put(mappername, m);
            logger.log(BasicLevel.DEBUG,
                "Mapper " + mappername + " initialized");
        }
        return pm;
    }

    public Manager getManager() {
        return manager;
    }

    public PTypeSpace getPtypeSpace() {
        return ptypeSpace;
    }

    public Logger getLogger() {
        return logger;
    }
}
