/**
 * Copyright (C) 2002
 */

package org.objectweb.jonas_ejb.container.jorm;

import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentCmp2Desc;
import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.runtime.Mapper;
import org.objectweb.jorm.runtime.MapperJCA;
import org.objectweb.jorm.runtime.MapperJDBC;
import org.objectweb.jorm.runtime.ConnectionSpecJDBC;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.sql.DataSource;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class RdbFactory extends MedorFactory {

    protected Mapper mapper;

    public RdbFactory() {
        super();
    }

	public void init(BeanDesc dd, JContainer cont) {
		super.init(dd, cont);
	}

	public Object getConnection(Object hints) throws PException {
        if (hints == null)
            return mapper.getConnection();
        else
            return mapper.getConnection(hints);
    }

    public void releaseConnection(Object conn) throws PException {
        mapper.closeConnection(conn);
    }

    public PMapper getMapper() {
        return mapper;
    }

    public PMapper getMapper(EntityDesc bd) throws PException {
        String[] mapperNames = ((EntityCmp2Desc) bd).getMapperNames();
		if (mapperNames == null || mapperNames.length == 0)
			mapperNames =
				((DeploymentCmp2Desc) bd.getDeploymentDesc()).getMapperNames();
		String mapperName = mapperNames[0];
		TraceEjb.interp.log(BasicLevel.DEBUG, "Bean " + bd.getEjbName()
			+ " is mapped on the mapper " + mapperName);
		MapperManager mm = MapperManager.getInstance();
        synchronized (mm) {
            mapper = (Mapper) mm.getMapper(mapperName, cont);
            if (mapper == null) {
                if (datasource instanceof DataSource)
                    mapper = new MapperJDBC();
                else
                    mapper = new MapperJCA();
                mapper.setMapperName(mapperName);
                mapper.setMappingName(getMappingName(bd));
                mapper.setConnectionFactory(datasource);
                mm.addMapper(mapper, cont);
            }
        }
        return mapper;
    }

    public String getMIMappingFactory(EntityDesc bd) {
        return "org.objectweb.jorm.metainfo.lib.rdb.BasicRDBMappingFactory";
    }
}
