/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Technologies Inc.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.deployment.api;

import java.lang.Class;

import org.objectweb.jonas_ejb.deployment.xml.EjbRef;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRef;

/**
 * Class to hold meta-information related to an ejb-ref
 * Only ejb-name and jndi-name are used in Jonas.
 * Contributor(s):
 * Florent Benoit & Ludovic Bert
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class EjbRefDesc {

    // The ejb ref name
    private String name;

    // The type of the ejb ref
    private String ejbRefType;

    // The ejb link of the ejb ref
    private String ejblink;

    // The jndi name of the ejb ref
    private String jndiName;

    /**
     * constructor to be used by parent node
     */
    public EjbRefDesc(EjbRef bRef, JonasEjbRef jBRef)
	    throws DeploymentDescException {

        name = bRef.getEjbRefName().getValue();
	ejbRefType = bRef.getEjbRefType().getValue();
	ejblink = (bRef.getEjbLink() != null) ? bRef.getEjbLink().getValue() : null;
        jndiName = (jBRef != null) ? jBRef.getJndiName().getValue() : null;
    }

    /**
     * Get the name of the ejb-ref
     * @return String representation of the name.
     */
    public String getName() {
            return name;
    }

    /**
     * Get the ejb-ref-type.
     * @return String representation of the ejb-ref-type.
     */
    public String getEjbRefType() {
            return ejbRefType;
    }

    /**
     * Get the ejb-link.
     * The value may be null when a jonas-ejb-ref is specified.
     * @return String representation of the ejb-link
     */
    public String getEjbLink() {
            return ejblink;
    }

    /**
     * Get the jndi name of the ejb-ref.
     * The value may be null when an ejb-link is specified.
     * @return String representation of the JNDI name
     */
    public String getJndiName() {
        return jndiName;
    }

    /**
     * Set the jndi name of the ejb-ref.
     * @param jndiName an string representation of the JNDI name
     */
    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\ngetName()="+getName());
        ret.append("\ngetEjbRefType()="+getEjbRefType());
        ret.append("\ngetEjbLink()="+getEjbLink());
        ret.append("\ngetJndiName()="+getJndiName());
        return ret.toString();
    }

}

