/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Technologies Inc.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.api;

import org.objectweb.jonas_ejb.deployment.xml.EjbRelation;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRelation;
import org.objectweb.jonas_ejb.deployment.xml.EjbRelationshipRole;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRelationshipRole;
import org.objectweb.jonas_ejb.deployment.xml.EjbRelationshipRoleName;

import java.util.Iterator;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * Created on Jul 7, 2002
 * Class to hold meta-information related to an ejb-relation.
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class EjbRelationDesc {

    private String name;
    private EjbRelationshipRoleDesc relationshipRoleDesc1;
    private EjbRelationshipRoleDesc relationshipRoleDesc2;

    /**
     * constructor to be used by parent node
     */
    public EjbRelationDesc(EjbRelation ejbRelation, JonasEjbRelation jonasEjbRelation) throws DeploymentDescException {
        name = ejbRelation.getEjbRelationName().getValue();

        // first role
        EjbRelationshipRole role1 = ejbRelation.getEjbRelationshipRole();
        EjbRelationshipRoleName ersrn1 = role1.getEjbRelationshipRoleName();
        if (ersrn1==null) {
            throw new DeploymentDescException(
                "You must specify a ejb-relationship-role-name in the relation "
                + name + " (first role). It is required by JOnAS");
        }
        // second role
        EjbRelationshipRole role2 = ejbRelation.getEjbRelationshipRole2();
        EjbRelationshipRoleName ersrn2 = role1.getEjbRelationshipRoleName();
        if (ersrn2==null) {
            throw new DeploymentDescException(
                "You must specify a ejb-relationship-role-name in the relation "
                + name + " (second role). It is required by JOnAS");
        }

        // store jonas roles in a HashMap
        HashMap table = new HashMap();
        for (Iterator i = jonasEjbRelation.getJonasEjbRelationshipRoleList().iterator(); i.hasNext(); ) {
            JonasEjbRelationshipRole jerr = (JonasEjbRelationshipRole)i.next();
            String rname = jerr.getEjbRelationshipRoleName().getValue();
            if (!(rname.equals(ersrn2.getValue()) || rname.equals(ersrn2.getValue()))) {
                throw new DeploymentDescException("Invalid relationship-role-name \""+rname+"\" for relation \""+name+"\" in jonas-ejb-jar.xml");
            }
            table.put(rname, jerr);
        }

        JonasEjbRelationshipRole jRole1 = (JonasEjbRelationshipRole)
            table.get(ersrn1.getValue());

        JonasEjbRelationshipRole jRole2 = (JonasEjbRelationshipRole)
            table.get(ersrn2.getValue());

        relationshipRoleDesc1 = newEjbRelationshipRoleDesc(
            role1, jRole1, role2.getCmrField()!=null, this);
        relationshipRoleDesc2 = newEjbRelationshipRoleDesc(
            role2, jRole2, role1.getCmrField()!=null, this);

    }

    /**
     * get the name of the relationship.
     */
    public String getName() {
        return name;
    }

    /**
     * get the meta-information for the first relation-ship-role
     */
    public EjbRelationshipRoleDesc getRelationshipRoleDesc1() {
        return relationshipRoleDesc1;
    }

    /**
     * get the meta-information for the second relation-ship-role
     */
    public EjbRelationshipRoleDesc getRelationshipRoleDesc2() {
        return relationshipRoleDesc2;
    }

    /**
     * EjbRelationshipRoleDesc factory method. To overwrite for jdbc implementation.
     */
    protected EjbRelationshipRoleDesc newEjbRelationshipRoleDesc(EjbRelationshipRole role,
        JonasEjbRelationshipRole jRole,boolean target,EjbRelationDesc ejbRelationDesc) throws DeploymentDescException {
        return new EjbRelationshipRoleDesc(role,jRole,target,ejbRelationDesc);
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\ngetName()="+getName());
        ret.append("\ngetRelationshipRoleDesc1() = " + getRelationshipRoleDesc1());
        ret.append("\ngetRelationshipRoleDesc2() = " + getRelationshipRoleDesc2());
        return ret.toString();
    }
}
