/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Technologies Inc.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.api;

import org.objectweb.jonas_ejb.deployment.xml.EjbRelation;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRelation;
import org.objectweb.jonas_ejb.deployment.xml.EjbRelationshipRole;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRelationshipRole;

/**
 * Created on Jul 7, 2002
 * Class to hold meta-information related to an ejb-relation when the data store is
 * jdbc based.
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class EjbRelationJdbcDesc extends EjbRelationDesc {

    String jdbcTableName = null;
    String datasourceJndiName = null;

    /**
     * constructor to be used by parent node
     */
    public EjbRelationJdbcDesc(EjbRelation ejbRelation, JonasEjbRelation jonasEjbRelation)
            throws DeploymentDescException {
        super(ejbRelation, jonasEjbRelation);
        if (jonasEjbRelation.getRelationJdbcMapping()!=null) {
            jdbcTableName = jonasEjbRelation.getRelationJdbcMapping().getJdbcTableName().getValue();
            datasourceJndiName = jonasEjbRelation.getRelationJdbcMapping().getJndiName().getValue();
        }
    }

    /**
     * get the jdbc name of the table used to store the relation
     * @return null if the relation is not stored in a join table
     */
    public String getJdbcTableName() {
        return jdbcTableName;
    }

    /**
     * get the jndi name of the datasource used to access the join table
     */
    public String getDatasourceJndiName() {
        return datasourceJndiName;
    }

    /**
     * EjbRelationshipRoleDesc factory method. To overwrite for jdbc implementation.
     */
    protected EjbRelationshipRoleDesc newEjbRelationshipRoleDesc(EjbRelationshipRole role,
        JonasEjbRelationshipRole jRole,boolean target,EjbRelationDesc ejbRelationDesc) throws DeploymentDescException {
        return new EjbRelationshipRoleJdbcDesc(role,jRole,target,ejbRelationDesc);
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
   public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\ngetJdbcTableName() = " + getJdbcTableName());
        ret.append("\ngetDatasourceJndiName() = " + getDatasourceJndiName());
        return ret.toString();
    }

}
