/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Technologies Inc.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.api;

import org.objectweb.jonas_ejb.deployment.xml.EjbRelationshipRole;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRelationshipRole;
import org.objectweb.jonas_ejb.deployment.xml.ForeignKeyJdbcMapping;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Class to hold meta-information related to an ejb-relationship-role when the data store is
 * jdbc based.
 * Created on Aug 13, 2002
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class EjbRelationshipRoleJdbcDesc extends EjbRelationshipRoleDesc {

    HashMap foreignKeyMap = new HashMap();

    /**
     * constructor to be used by parent node
     */
    public EjbRelationshipRoleJdbcDesc(EjbRelationshipRole role, JonasEjbRelationshipRole jRole,
            boolean target, EjbRelationDesc ejbRelationDesc) throws DeploymentDescException {
        super(role, jRole, target, ejbRelationDesc);
        if (jRole!=null) {
            // store the map of columns for the foreign key, carefull: the Map is inversed
            for (Iterator i = jRole.getForeignKeyJdbcMappingList().iterator(); i.hasNext();) {
                ForeignKeyJdbcMapping map = (ForeignKeyJdbcMapping) i.next();
                if (map.getKeyJdbcName()!=null) {
                    foreignKeyMap.put(map.getKeyJdbcName().getValue(),map.getForeignKeyJdbcName().getValue());
                } else {
                    // key-jdbc-name is not required for a primary-key field (non composite)
                    if (i.hasNext()) {
                        throw new DeploymentDescException("key-jdbc-name must be provided for cmr-field "+cmrFieldName+" of bean "+ejbName);
                    }
                    foreignKeyMap.put("",map.getForeignKeyJdbcName().getValue());
                }
            }
        }

    }

    /**
     * return the jdbc name (column name) of a foreign key for a given
     * primary key jdbc field name (column name)
     * @return null if there is no foreign key column for the corresponding column name
     */
    public String getForeignKeyJdbcName(String jdbcFieldName) {
        if (foreignKeyMap.size()==1) {
            // jdbcFieldName may not be set when unique
            return (String)foreignKeyMap.values().iterator().next();
        } else {
            return (String)foreignKeyMap.get(jdbcFieldName);
        }
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
   public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        for (Iterator i=foreignKeyMap.entrySet().iterator();i.hasNext();) {
            Map.Entry entry = (Map.Entry) i.next();
            ret.append("\nforeignKeyMap[" + entry.getKey()+"]="+entry.getValue());
        }
        return ret.toString();
    }


}
