/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Tecnologies Inc. , France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.deployment.api;

import java.lang.String;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;

import org.objectweb.jonas_ejb.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas_ejb.deployment.xml.Entity;
import org.objectweb.jonas_ejb.deployment.xml.CmpField;
import org.objectweb.jonas_ejb.deployment.xml.JonasEntity;
import org.objectweb.jonas_ejb.lib.BeanNaming;

/**
 * Base class to hold meta-information related to an entity of type CMP.
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class EntityCmpDesc extends EntityDesc {

    protected HashMap fieldDesc = new HashMap();
    protected String primaryKeyField = null;

    /**
     * constructor to be used by parent node
     */
    public EntityCmpDesc(ClassLoader classLoader,Entity ent, AssemblyDescriptor asd,JonasEntity jEnt)
            throws DeploymentDescException {
        super(classLoader,ent,asd,jEnt);

        // field descriptors for persistent fields
        for (Iterator i = ent.getCmpFieldList().iterator();i.hasNext();) {
            String fn = ((CmpField)i.next()).getFieldName().getValue();
            FieldDesc fd = this.newFieldDescInstance();
            fd.setFieldName(fn);
            fd.setPrimaryKey(false);
            fieldDesc.put(fn,fd);
        }

        // primary key field
        if (ent.getPrimkeyField()!=null) {
            primaryKeyField = ent.getPrimkeyField().getValue();
            if (!fieldDesc.containsKey(primaryKeyField)) {
                throw new DeploymentDescException("primkey-field "+primaryKeyField+" is not listed as cmp-field in bean "+this.ejbName);
            }
            FieldDesc fd = this.newFieldDescInstance();
            fd.setFieldName(primaryKeyField);
            fd.setPrimaryKey(true);
            fieldDesc.put(primaryKeyField,fd);
        } else {
            // public fields of primary key class
            Field[] pcf = primaryKeyClass.getFields();
            for (int i=0; i<pcf.length; i++) {
                if (Modifier.isPublic(pcf[i].getModifiers())) {
                    String pn = pcf[i].getName();
                    // exclude serialVersionUID field for jdk1.2.1 on solaris and
                    // exclude JProbe$ for JProbe using
                    if (!pn.equals("serialVersionUID") && !pn.startsWith("JProbe$")) {
                        if (!fieldDesc.containsKey(pn))
                            throw new DeploymentDescException("public field "+pn+" of primkey-class is not listed as cmp-field in bean "+this.ejbName);
                        FieldDesc fd = this.newFieldDescInstance();
                        fd.setFieldName(pn);
                        fd.setPrimaryKey(true);
                        fieldDesc.put(pn,fd);
                    }
                }
            }
        }

        String packageName = BeanNaming.getPackageName(getEjbClass().getName());
        derivedBeanName = new String("JOnAS" + getIdentifierFromEjbName(ejbName) + "Bean");
        fullDerivedBeanName = BeanNaming.getClassName(packageName, derivedBeanName);

    }


    /**
     * Assessor for primary key being a field
     * @return true if primary key is a field
     */
    public boolean hasPrimaryKeyField(){
        return primaryKeyField!=null;
    }

    /**
     * Get the primary key field for the entity.
     * Used by GenIC
     * @return Field for the primary key
     */
    public String getPrimaryKeyFieldName(){
        if (primaryKeyField==null)
            throw new Error("No primary key field defined for bean "+this.ejbName);
        return primaryKeyField;
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        for (Iterator i=fieldDesc.keySet().iterator();i.hasNext();) {
            String f = (String)i.next();
            FieldDesc fd = (FieldDesc)fieldDesc.get(f);
            ret.append("\ngetCmpFieldDesc("+f+")="+fd.getClass().getName());
            ret.append(fd.toString());
        }
        if (hasPrimaryKeyField())
            ret.append("\ngetPrimaryKeyField()="+getPrimaryKeyFieldName());
        return ret.toString();
    }

    /**
     * factory method for field descriptors
     */
    protected FieldDesc newFieldDescInstance() {
        return new FieldDesc();
    }
}

