/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.deployment.api;

import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEJBQL;
import org.objectweb.jonas_ejb.deployment.ejbql.EJBQL;
import org.objectweb.jonas_ejb.deployment.ejbql.SimpleNode;
import org.objectweb.jonas_ejb.deployment.ejbql.ParseException;
import org.objectweb.jonas_ejb.lib.EjbqlQueryTreeHolder;
import org.objectweb.jonas_ejb.lib.EjbqlSelectVisitor;
import org.objectweb.jonas_ejb.lib.EjbqlQueryFilterVisitor;
import org.objectweb.jorm.metainfo.api.Schema;

import java.lang.reflect.Method;
import java.io.CharArrayReader;


/**
 * Class to hold meta-information related to CMP2 find/select methods
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class MethodCmp2Desc extends MethodDesc  {

    protected String query = null;
    protected ASTEJBQL queryNode = null;
    protected EjbqlQueryTreeHolder queryTreeHolder = null;
    protected boolean resultTypeMappingRemote = false;


    /**
     * construtor form XML data binding structures
     */
    MethodCmp2Desc(BeanDesc beanDesc, Method meth,int index) {
        super(beanDesc, meth, index);
    }

    /**
     * get EJB-QL query when defined
     * @return possibly null String containing the EJB-QL query for the method
     */
    public String getQuery() {
        return query;
    }

    /**
     * set EJB-QL query. Because of the deployment descriptor structure, the query is
     * set after the object is created.
     */
    public void setQuery(String query) throws ParseException {
        // we want a one line query for error reporting
        this.query = query.replace('\r',' ').replace('\n',' ').replace('\t',' ');
        EJBQL parser = new EJBQL(new CharArrayReader(query.toCharArray()));
        queryNode = (ASTEJBQL)parser.EJBQL();
        queryTreeHolder=null;
    }


    /**
     * return a query tree holder allowing evaluation of the query
     */
    public EjbqlQueryTreeHolder getQueryTreeHolder() throws DeploymentDescException {
        if (queryTreeHolder==null) {
            try {
                queryTreeHolder = new EjbqlQueryTreeHolder(this,queryNode);
            } catch (Exception e) {
                throw new DeploymentDescException("ejbql query "+query+" does not map to the persistent schema",e);
            }
        }
        return queryTreeHolder;
    }

    /**
     * get result type mapping state (remote/local)
     * @return true when remote
     */
    public boolean isResultTypeMappingRemote() {
        return resultTypeMappingRemote;
    }

    /**
     * set the state of resultTypeMappingRemote
     */
    public void setResultTypeMapping(String resultTypeMapping)
        throws DeploymentDescException {
            if (resultTypeMapping.equals("Remote")) {
                resultTypeMappingRemote=true;
            } else if (resultTypeMapping.equals("Local")) {
                resultTypeMappingRemote=false;
            } else {
                throw new DeploymentDescException(resultTypeMapping+" is not a valid result-type-mapping value");
            }
    }


    /**
     * String representation of the given element <method>
     * @param  m an element <method>
     * @return String representation of the given element method
     */
    public static String queryMethodElementToString(org.objectweb.jonas_ejb.deployment.xml.QueryMethod m) {
        return methodElementToString(null, m.getMethodName(),m.getMethodParams());
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\nquery = " + query);
        ret.append("\nqueryNode = " + queryNode);
        ret.append("\nqueryTreeHolder = " + queryTreeHolder);
        ret.append("\nresultTypeMappingRemote = " + resultTypeMappingRemote);
        return ret.toString();
    }

}
