/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Tecnologies Inc. , France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.deployment.api;

import java.lang.reflect.Method;


/**
 * Class to hold meta-information related to CMP v1 attributes of a method when a jdbc
 * data store is used.
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class MethodJdbcCmp1Desc extends MethodDesc  {
    
    protected String whereClause = null;
    protected int whereClauseStatus = APPLY_TO_BEAN;
    
    /**
     * constructor to be used by parent node
     */
    public MethodJdbcCmp1Desc(BeanDesc beanDesc,Method meth,int index) {
        super(beanDesc,meth,index);
    }

    /**
     * Overwrite JdbcWhereClause
     * @param jdbcWhereClause new value for jdbcWhereClause
     * @param status applicability of given transAttribute parameter
     */
    void overwriteJdbcWhereClause(String jdbcWhereClause,int status)
            throws DeploymentDescException {
        if (status<this.whereClauseStatus)
            return;
        whereClause = jdbcWhereClause;
        whereClauseStatus = status;
    }

    /**
     * Assessor for 'WHERE' clause set
     * @return true if where clause has been set
     */
    public boolean hasWhereClause(){
        return whereClause!=null;
    }

    /**
     * Get 'WHERE' clause for the method
     * @return String containing 'WHERE' clause 
     */
    public String getWhereClause(){
        if (whereClause==null) {
            throw new Error("No whereClause defined for this method");
        }
        return whereClause;
    }

    /**
     * Get 'WHERE' clause status for the method
     */
    public int getWhereClauseStatus(){
        return whereClauseStatus;
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        if (hasWhereClause()) {
            ret.append("\ngetWhereClause()="+getWhereClause());
            ret.append("\ngetWhereClauseStatus()="+APPLY_TO[getWhereClauseStatus()]);
        }
        return ret.toString();
    }
    
}
