/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Christophe Ney cney@batisseurs.com for France Telecom
 *
 *    JOnAS 2.2.1 Joe Gittings has proposed to code method signature in order to avoid
 *                same signature for inherited methods.
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.lib;

import java.io.File;
import java.lang.reflect.Method;
import java.util.StringTokenizer;



/**
 * This class is made for hiding Naming convention in all JOnAS framework.
 * <br>
 * (Among other things, names of the implementation classes of
 * the Enterprise Bean's Home and Enterprise Bean Remote interfaces
 * generated by the GenIC tools.)
 * <br>
 * It is used by the EJB generation tools and framework
 * @author jonas
 * @author Sebastien Chassande-Barrioz
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class BeanNaming {

    /**
     * returns the name of the package of the given full name
     */
    public static String getPackageName(String fullName) {
	String pkg = new String();
	StringTokenizer stk = new StringTokenizer(fullName, ".");
	int nb = stk.countTokens();
	for (int i = 0; i < nb - 1; i++) {
	    pkg = pkg.concat(stk.nextToken());
	    if (i < nb-2) {
		pkg = pkg.concat(".");
	    }
	}
	return pkg;
    }

    /**
     * return full name of a class for given package and class names
     * @param packageName possibly empty package name
     * @param className class name
     * @return fully qualified class name
     */
    public static String getClassName(String packageName, String name) {
        return (packageName.length()==0)?name:packageName+"."+name;
    }


    /**
     * returns the full path of the file name.
     * mainly replace '.' by '/'.
     */
    public static String getPath(String fullName) {
	String pkg = new String();
	StringTokenizer stk = new StringTokenizer(fullName, ".");
	int nb = stk.countTokens();
	for (int i = 0; i < nb; i++) {
	    pkg = pkg.concat(stk.nextToken());
	    if (i < nb - 1) {
		pkg = pkg + File.separatorChar;
	    }
	}
	return pkg;
    }

    /**
     * returns the basename of the given full name
     */
    public static String getBaseName(String fullName) {
	String baseName;
	StringTokenizer stk = new StringTokenizer(fullName, ".");
	int nb = stk.countTokens();
	String pkg = new String();
	for (int i = 0; i < nb - 1; i++) {
	    pkg = pkg.concat(stk.nextToken());
	    if (i < nb - 2) {
		pkg = pkg.concat(".");
	    }
	}
	if (nb > 0) {
	    baseName = new String(stk.nextToken());
	} else {
	    baseName = new String();
	}
	return baseName;
    }

    /**
     * returns the given string with the first character converted to upper case
     */
    public static String firstToUpperCase(String s) {
	String value;
	if (s.length() > 0) {
	    char c = Character.toUpperCase(s.charAt(0));
	    value = new String(c+s.substring(1));
	} else {
	    value = new String();
	}
	return(value);
    }


    /**
     * returns the name of the JOnAS specific deployment descriptor file's name
     * corresponding to the given standard deployment descriptor file's name.
     * (ex: returns "jonas-XXX.xml" for "XXX.xml" and "../../jonas-XXX.xml" for "../../XXX.xml")
     */
    public static String getJonasXmlName(String stdXmlName) {

	String jonasXmlName = new String();
	File f = new File(stdXmlName);
	String p = f.getParent();
	String n = f.getName();
	if (p != null) {
	    jonasXmlName = jonasXmlName.concat(p);
	    jonasXmlName = jonasXmlName.concat(File.separator);
	}
	jonasXmlName = jonasXmlName.concat("jonas-");
	jonasXmlName = jonasXmlName.concat(n);

	return(jonasXmlName);
    }

    /**
     * returns a string that represents the signature of a method
     *
     */
    public static String getSignature(String prefix, Method method) {

	String signature = new String();
	String classname = method.getDeclaringClass().getName();
	String methodname = method.getName();
	Class [] types = method.getParameterTypes();
	String parameters = new String("_"+types.length+":");
	for (int i=0; i< types.length; i++) {
	    parameters += JavaType.getName(types[i])+" ";
	}
	signature = prefix +":"+ classname+"."+methodname+parameters;
	return signature;
    }
}

