/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.lib;

import org.objectweb.jonas_ejb.deployment.ejbql.ASTAbstractSchemaName;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticLiteral;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBetweenExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBooleanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBooleanLiteral;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberDeclaration;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTDatetimeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEJBQL;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEmptyCollectionComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEntityBeanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFromClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningNumerics;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningStrings;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentificationVariable;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentifier;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTInExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTInputParameter;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTLikeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTNullComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTPath;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTRangeVariableDeclaration;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSelectClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedNavigation;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTStringExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTStringLiteral;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTWhereClause;
import org.objectweb.jonas_ejb.deployment.ejbql.EJBQLVisitor;
import org.objectweb.jonas_ejb.deployment.ejbql.ParseException;
import org.objectweb.jonas_ejb.deployment.ejbql.SimpleNode;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryTree;

import java.util.Map;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.ArrayList;

/**
 * Base class with visitor utility class and default implementation of
 * visit methods
 * Created on Sep 12, 2002
 * @author  Christophe Ney  [cney@batisseurs.com]
 */
public class EjbqlAbstractVisitor implements EJBQLVisitor {

    /**
     * Values associated with each declared identifiers
     */
    protected class IdValue {
        // Paths used in the select and where clauses starting with the
        // corresponding id
        private ArrayList paths = null;

        // abstract schema name or collection values path
        public String[] name = null;

        // The query tree which corresponds to the id and the paths
        public QueryTree qt = null;

        public IdValue(String pathName) {
            this();
            name = splitPath(pathName);
        }

        public IdValue() {
            paths = new ArrayList();
        }

        public void addPath(String path) {
            if (!paths.contains(path)) {
                paths.add(path);
            }
        }
        public String[] getSplitedPath(int idx) {
            return splitPath((String) paths.get(idx));
        }
        public String getMergedPath(int idx) {
            return (String) paths.get(idx);
        }

        public int getDeclaredPathLength() {
            return paths.size();
        }
    }

	/**
	 * Runtime Exception used to wrap exceptions thrown in visit methods
	 */
	protected class VisitorException extends RuntimeException {

		Exception nestedException;

		VisitorException(Exception nestedException) {
			this.nestedException = nestedException;
		}

		Exception getNestedException() {
			return nestedException;
		}
	}

	/**
	 * split a dot separated path into tokens
	 */
	protected String[] splitPath(String path) {
		StringTokenizer st = new StringTokenizer(path, ".");
		String[] ret = new String[st.countTokens()];
		for (int i = 0; i < ret.length; i++) {
			ret[i] = st.nextToken();
		}
		return ret;
	}

    protected String mergePath(String[] path, int begin, int length) {
        if (length==0)
            return "";
        StringBuffer ret = new StringBuffer();
        for (int i=begin; i<(begin + length); i++) {
            ret.append(path[i]);
            ret.append('.');
        }
        ret.deleteCharAt(ret.length()-1);
        return ret.toString();
    }

	/**
	 * return the JormField for a given path
	 * @param path the path of the JormField to return
	 * @param ids the map of (id,QueryTree)pairs;
	 * @throw ParseException if identification variable not found.
	 * @throw MedorException if corresponding fielc does not exist
	 */
	protected Field getMedorFieldFromPath(String path, Map ids)
		throws MedorException, ParseException {
		String[] paths = splitPath(path);
		QueryTree qt = (QueryTree) ids.get(paths[0]);
//TODO
		if (paths.length > 2)
			throw new Error("recursive path not implemented: " + path);
		if (qt == null) {
			throw new ParseException("Undeclared identification variable " + paths[0] + " in path " + path);
		}
		return (Field) qt.getTupleStructure().getField(path);
	}


	/**
	 * Visit method to call from constructor.
	 * Child node visitors get a <code>java.util.Stack</code> as data parameter.
	 * @throw Exception any nested exception thrown from other visit method
	 */
	public Object visit(SimpleNode node) throws Exception {
		try {
			return visit((SimpleNode) node, new Stack());
		}
		catch (VisitorException e) {
			throw e.getNestedException();
		}
	}

	/**
	 * Generic visit method that traverses all child nodes
	 */
	public Object visit(SimpleNode node, Object data) {
		return node.childrenAccept(this, data);
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTEJBQL node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTFromClause node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTCollectionMemberDeclaration node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTRangeVariableDeclaration node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTSingleValuedPathExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTSingleValuedNavigation node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTCollectionValuedPathExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTSelectClause node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTWhereClause node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTConditionalExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTConditionalTerm node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTConditionalFactor node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTBetweenExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTInExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTLikeExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTNullComparisonExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTEmptyCollectionComparisonExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTCollectionMemberExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTComparisonExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTArithmeticExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTArithmeticTerm node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTArithmeticFactor node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTStringExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTDatetimeExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTBooleanExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTEntityBeanExpression node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTFunctionsReturningStrings node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTFunctionsReturningNumerics node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTAbstractSchemaName node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTIdentificationVariable node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTIdentifier node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTPath node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */

	public Object visit(ASTStringLiteral node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTArithmeticLiteral node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTBooleanLiteral node, Object data) {
		return null;
	}

	/**
	 * null implementation of the visit method for the corresponding parameter type
	 */
	public Object visit(ASTInputParameter node, Object data) {
		return null;
	}
}
