/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.lib;

import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticLiteral;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBetweenExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBooleanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBooleanLiteral;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTDatetimeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEJBQL;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEmptyCollectionComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEntityBeanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningNumerics;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningStrings;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentificationVariable;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTInExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTInputParameter;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTLikeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTNullComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTPath;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedNavigation;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTStringExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTStringLiteral;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTWhereClause;
import org.objectweb.jonas_ejb.deployment.ejbql.EJBQLConstants;
import org.objectweb.jonas_ejb.deployment.ejbql.ParseException;
import org.objectweb.jonas_ejb.deployment.ejbql.SimpleNode;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.lib.And;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.filter.lib.DivideBy;
import org.objectweb.medor.filter.lib.Equal;
import org.objectweb.medor.filter.lib.Greater;
import org.objectweb.medor.filter.lib.GreaterEqual;
import org.objectweb.medor.filter.lib.Lower;
import org.objectweb.medor.filter.lib.LowerEqual;
import org.objectweb.medor.filter.lib.Minus;
import org.objectweb.medor.filter.lib.Mult;
import org.objectweb.medor.filter.lib.Not;
import org.objectweb.medor.filter.lib.NotEqual;
import org.objectweb.medor.filter.lib.Or;
import org.objectweb.medor.filter.lib.Plus;
import org.objectweb.medor.filter.lib.UMinus;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;

import javax.ejb.EJBLocalObject;
import javax.ejb.EJBObject;
import java.util.Map;
import java.util.Stack;

/**
 * Implementation of a visitor that creates the filter corresponding
 * to the WHERE clause.
 * Created on Sep 6, 2002
 * @author  Christophe Ney  [cney@batisseurs.com]
 */
public class EjbqlQueryFilterVisitor extends EjbqlAbstractVisitor {

    Expression qf = null;
    QueryTree qt;
    Class[] parameterTypes;

    /**
     * Constructor
     * @param ids Map with (identifier,JormExtent) pairs
     * @param parameterTypes Type of paramaters of the finder/select method
     * @param ejbql root of the lexical tree of the query
     */
    public EjbqlQueryFilterVisitor(QueryTree _qt, Class[] parameterTypes, ASTEJBQL ejbql) throws Exception {
        qt = _qt;
        this.parameterTypes = parameterTypes;
        visit(ejbql);
    }

    /**
     * get the query filter that was built from visiting the syntaxic tree
     */
    public Expression getQueryFilter() {
        return qf;
    }

    /**
     * If query contains WHERE clause, visit child nodes<br>
     * where_clause ::= WHERE conditional_expression
     */
    public Object visit(ASTWhereClause node, Object data) {
        visit((SimpleNode) node, data);
        qf = (Expression) ((Stack) data).pop();
        return null;
    }

    /**
     * Push corresponding MedorField to the stack.<br>
     * single_valued_path_expression ::= path
     * was in initial BNF
     * single_valued_path_expression ::= {single_valued_navigation  | identification_variable}.cmp_field | single_valued_navigation
     */
    public Object visit(ASTSingleValuedPathExpression node, Object data) {
        visit((SimpleNode) node, data);
        try {
            String path = (String) ((ASTPath) ((Stack) data).pop()).value;
            Field f = qt.getTupleStructure().getField(path);
            // FIXME check type for single valued
            // test GenClassRef or ClassRef
            ((Stack) data).push(new BasicFieldOperand(f));
        }
        catch (Exception e) {
            throw new VisitorException(e);
        }
        return null;
    }

    /**
     * Push corresponding MedorField to the stack.<br>
     * single_valued_navigation ::= path
     * was in initial BNF
     * single_valued_navigation ::= identification_variable. [ single_valued_cmr_field. ] * single_valued_cmr_field
     */
    public Object visit(ASTSingleValuedNavigation node, Object data) {
        visit((SimpleNode) node, data);
        try {
            String path = (String) ((ASTPath) ((Stack) data).pop()).value;
            Field f = qt.getTupleStructure().getField(path);
            // FIXME check type for single valued cmr
            ((Stack) data).push(new BasicFieldOperand(f));
        }
        catch (Exception e) {
            throw new VisitorException(e);
        }
        return null;
    }

    /**
     * Push corresponding MedorField to the stack.<br>
     * collection_valued_path_expression ::= path
     * was in initial BNF
     * collection_valued_path_expression ::= identification_variable. [ single_valued_cmr_field. ] *collection_valued_cmr_field
     */
    public Object visit(ASTCollectionValuedPathExpression node, Object data) {
        visit((SimpleNode) node, data);
        try {
            String path = (String) ((ASTPath) ((Stack) data).pop()).value;
            Field f = qt.getTupleStructure().getField(path);
            // FIXME check type for collection valued cmr
            ((Stack) data).push(new BasicFieldOperand(f));
        }
        catch (Exception e) {
            throw new VisitorException(e);
        }
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * conditional_expression ::= conditional_term [ OR conditional_term ]*
     */
    public Object visit(ASTConditionalExpression node, Object data) {
        visit((SimpleNode) node, data);
        Expression ret = (Expression) ((Stack) data).pop();
        for (int i = 1; i < node.jjtGetNumChildren(); i++) {
            ret = new Or((Expression) ((Stack) data).pop(), ret);
        }
        // push result on Stack
        ((Stack) data).push(ret);
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * conditional_term ::= conditional_factor [ AND conditional_factor ]*
     */
    public Object visit(ASTConditionalTerm node, Object data) {
        visit((SimpleNode) node, data);
        Expression ret = (Expression) ((Stack) data).pop();
        for (int i = 1; i < node.jjtGetNumChildren(); i++) {
            ret = new And((Expression) ((Stack) data).pop(), ret);
        }
        // push result on Stack
        ((Stack) data).push(ret);
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * conditional_factor ::= [ NOT ] conditional_test
     */
    public Object visit(ASTConditionalFactor node, Object data) {
        visit((SimpleNode) node, data);
        if (node.not) {
            Expression ret = (Expression) ((Stack) data).pop();
            // push result on Stack
            ((Stack) data).push(new Not(ret));
        }
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * between_expression ::= arithmetic_expression  [ NOT ] BETWEEN arithmetic_expression AND arithmetic_expression
     */
    public Object visit(ASTBetweenExpression node, Object data) {
        visit((SimpleNode) node, data);
        Expression[] terms = new Expression[3];
        terms[2] = (Expression) ((Stack) data).pop();
        terms[1] = (Expression) ((Stack) data).pop();
        terms[0] = (Expression) ((Stack) data).pop();
        if (node.not) {
            ((Stack) data).push(new Or(new Lower(terms[0], terms[1]), new Greater(terms[0], terms[2])));
        }
        else {
            ((Stack) data).push(new And(new GreaterEqual(terms[0], terms[1]), new LowerEqual(terms[0], terms[2])));
        }
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * in_expression ::= single_valued_path_expression  [ NOT ] IN (string_literal  [ , string_literal ] * )
     */
    public Object visit(ASTInExpression node, Object data) {
        throw new Error("IN expression not implemented in Medor");
        //
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * like_expression ::= single_valued_path_expression  [ NOT ] LIKE pattern_value  [ ESCAPE escape_character ]
     */
    public Object visit(ASTLikeExpression node, Object data) {
        throw new Error("LIKE expression not implemented in Medor");  // fait
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * null_comparison_expression ::= single_valued_path_expression IS  [ NOT ] NULL
     */
    public Object visit(ASTNullComparisonExpression node, Object data) {
        throw new Error("IS NULL expression not implemented");
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * empty_collection_comparison_expression ::= collection_valued_path_expression IS  [ NOT ] EMPTY
     */
    public Object visit(ASTEmptyCollectionComparisonExpression node, Object data) {
        throw new Error("IS EMPTY expression not implemented in Medor");
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * collection_member_expression ::= {single_valued_navigation | identification_variable | input_parameter}
     * [ NOT ] MEMBER  [ OF ] collection_valued_path_expression
     */
    public Object visit(ASTCollectionMemberExpression node, Object data) {
        throw new Error("MEMBER OF expression not implemented  in Medor");
    }


    /**
     * Push corresponding Expression to the stack.<br>
     * comparison_expression ::= string_value { =|<>} string_expression
     *    | boolean_value { =|<>} boolean_expression
     *    | datetime_value { = | <> | > | < } datetime_expression
     *    | entity_bean_value { = | <> } entity_bean_expression
     *    | arithmetic_value { = | > | >= | < | <= | <> } single_value_designator
     */
    public Object visit(ASTComparisonExpression node, Object data) {
        visit((SimpleNode) node, data);
        Expression[] terms = new Expression[2];
        terms[1] = (Expression) ((Stack) data).pop();
        terms[0] = (Expression) ((Stack) data).pop();
        int op = ((Integer) node.ops.get(0)).intValue();
        switch (op) {
        case EJBQLConstants.EQ:
            ((Stack) data).push(new Equal(terms[0], terms[1]));
            break;
        case EJBQLConstants.NE:
            ((Stack) data).push(new NotEqual(terms[0], terms[1]));
            break;
        case EJBQLConstants.GT:
            ((Stack) data).push(new Greater(terms[0], terms[1]));
            break;
        case EJBQLConstants.GE:
            ((Stack) data).push(new GreaterEqual(terms[0], terms[1]));
            break;
        case EJBQLConstants.LT:
            ((Stack) data).push(new Lower(terms[0], terms[1]));
            break;
        case EJBQLConstants.LE:
            ((Stack) data).push(new LowerEqual(terms[0], terms[1]));
            break;
        }
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * arithmetic_expression ::= arithmetic_term [ { + | - } arithmetic_term ] *
     */
    public Object visit(ASTArithmeticExpression node, Object data) {
        visit((SimpleNode) node, data);
        Expression ret = (Expression) ((Stack) data).pop();
        for (int i = 0; i < node.jjtGetNumChildren() - 1; i++) {
            switch (((Integer) node.ops.get(node.jjtGetNumChildren() - 2 - i)).intValue()) {
            case EJBQLConstants.PLUS:
                ret = new Plus((Expression) ((Stack) data).pop(), ret);
                break;
            case EJBQLConstants.MINUS:
                ret = new Minus((Expression) ((Stack) data).pop(), ret);
                break;
            }
        }
        // push result to the Stack
        ((Stack) data).push(ret);
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * arithmetic_term ::= arithmetic_factor [ { * | / } arithmetic_factor ]*
     */
    public Object visit(ASTArithmeticTerm node, Object data) {
        visit((SimpleNode) node, data);
        Expression ret = (Expression) ((Stack) data).pop();
        for (int i = 0; i < node.jjtGetNumChildren() - 1; i++) {
            switch (((Integer) node.ops.get(node.jjtGetNumChildren() - 2 - i)).intValue()) {
            case EJBQLConstants.MULT:
                ret = new Mult((Expression) ((Stack) data).pop(), ret);
                break;
            case EJBQLConstants.DIV:
                ret = new DivideBy((Expression) ((Stack) data).pop(), ret);
                break;
            }
        }
        // push result to the Stack
        ((Stack) data).push(ret);
        return null;
    }

    /**
     * Push corresponding Expression to the stack.<br>
     * arithmetic_factor ::= [ + |- ] arithmetic_primary
     */
    public Object visit(ASTArithmeticFactor node, Object data) {
        visit((SimpleNode) node, data);
        Expression ret = (Expression) ((Stack) data).pop();
        if (node.ops.size() > 0) {
            if (((Integer) node.ops.get(0)).intValue() == EJBQLConstants.MINUS) {
                ret = new UMinus((Expression) ret);
            }
        }
        // push result on Stack
        ((Stack) data).push(ret);
        return null;
    }


    /**
     * Visit child nodes
     * string_expression ::= string_primary | input_parameter
     */
    public Object visit(ASTStringExpression node, Object data) {
        visit((SimpleNode) node, data);
        // FIXME check or convert type?
        return null;
    }

    /**
     * Visit child nodes
     * datetime_expression ::= datetime_value | input_parameter
     */
    public Object visit(ASTDatetimeExpression node, Object data) {
        visit((SimpleNode) node, data);
        // FIXME check or convert type?
        return null;
    }

    /**
     * Visit child nodes
     * boolean_expression ::= single_valued_path_expression | boolean_literal | input_parameter
     */
    public Object visit(ASTBooleanExpression node, Object data) {
        visit((SimpleNode) node, data);
        // FIXME check or convert type?
        return null;
    }

    /**
     * Visit child nodes
     * entity_bean_expression ::= entity_bean_value | input_parameter
     */
    public Object visit(ASTEntityBeanExpression node, Object data) {
        visit((SimpleNode) node, data);
        // FIXME check or convert type?
        return null;
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTFunctionsReturningStrings node, Object data) {
        throw new Error("Functions not implemented  in Medor");
    }


    /**
     * visit child nodes
     */
    public Object visit(ASTFunctionsReturningNumerics node, Object data) {
        throw new Error("Functions not implemented  in Medor");
    }

    /**
     * Node with value set to identification variable string.
     * Push the Node to the stack
     */
    public Object visit(ASTIdentificationVariable node, Object data) {
        String fieldname = (String) node.value + "." + Field.PNAMENAME;
        try {
            ((Stack) data).push(qt.getTupleStructure().getField(fieldname));
        }
        catch (MedorException e) {
            throw new VisitorException(e);
        }
        return null;
    }

    /**
     * Node with value set to litteral string.
     * Push the corresponding Operand to the stack
     */
    public Object visit(ASTStringLiteral node, Object data) {
        ((Stack) data).push(new BasicOperand((String) node.value));
        return null;
    }

    /**
     * Node with value set to litteral arithmetic.
     * Push the corresponding Operand to the stack
     */
    public Object visit(ASTArithmeticLiteral node, Object data) {
        if (node.value instanceof Long) {
            ((Stack) data).push(new BasicOperand(((Long) node.value).longValue()));
        }
        else {
            ((Stack) data).push(new BasicOperand(((Double) node.value).doubleValue()));
        }
        return null;
    }

    /**
     * Node with value set to litteral boolean.
     * Push the corresponding Operand to the stack
     */
    public Object visit(ASTBooleanLiteral node, Object data) {
        ((Stack) data).push(new BasicOperand(((Boolean) node.value).booleanValue()));
        return null;
    }

    /**
     * Node with value set to parameter index (1..n) string.
     * Push the corresponding Operand to the stack
     */
    public Object visit(ASTInputParameter node, Object data) {
        try {
            int pIndex = ((Integer) node.value).intValue() - 1;
            if (pIndex >= parameterTypes.length) {
                throw new ParseException("Parameter ?" + (pIndex + 1) + " is out of range (max = " + parameterTypes.length + ")");
            }
            if (parameterTypes[pIndex].isAssignableFrom(EJBObject.class)) {
                throw new Error("EJBObject params not implemented"); // FIXME use pname
            }
            if (parameterTypes[pIndex].isAssignableFrom(EJBLocalObject.class)) {
                throw new Error("EJBLocalObject params not implemented"); // FIXME use pname
            }
            Operand op = new BasicParameterOperand(JormType.getPType(parameterTypes[pIndex]), "?" + node.value);
            ((Stack) data).push(op);
            return null;
        }
        catch (ParseException e) {
            throw new VisitorException(e);
        }
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTPath node, Object data) {
        ((Stack) data).push(node);
        return null;
    }
}
