/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.lib;

import org.objectweb.jonas_ejb.deployment.ejbql.*;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;

import java.util.ArrayList;
import java.util.Map;
import java.util.Stack;
import java.util.StringTokenizer;

/**
 * Implementation of a visitor that creates the selection projection for corresponding
 * to a given WHERE clause.
 * Created on Sep 6, 2002
 * @author  Christophe Ney  [cney@batisseurs.com]
 */
public class EjbqlSelectVisitor extends EjbqlAbstractVisitor  {

    QueryTree qt;
    SelectProject sp;

    /**
     * Constructor
     * @param ejbql root of the lexical tree of the query
     * @param identifierMap Map with (identifier,JormExtent) pairs
     */
    public EjbqlSelectVisitor(ASTEJBQL ejbql, QueryTree _qt) throws Exception {
        sp = new SelectProject("", null);
        qt = _qt;
        visit(ejbql);
    }

    /**
     * get the select project that was built from visiting the lexical tree
     */
    public SelectProject getSelectProject() {
        return sp;
    }


    /**
     *  Visit child node.
     *  SELECT  [ DISTINCT ] {single_valued_path_expression | OBJECT (identification_variable)}
     */
    public Object visit(ASTSelectClause node, Object data) {
        visit((SimpleNode)node,data);
        boolean distinct = node.distinct;
//TODO: Implement the distinct case
        try {
            QueryTreeField qtf = (QueryTreeField)
                qt.getTupleStructure().getField((String)((Stack)data).pop());
            sp.addPropagatedField(
                qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
        } catch (Exception e) {
            throw new VisitorException(e);
        }
        return null;
    }

    /**
     * Visit child node
     */
    public Object visit(ASTSingleValuedPathExpression node, Object data) {
        return visit((SimpleNode)node,data);
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTIdentificationVariable node, Object data) {
        ((Stack)data).push((String) node.value + "." + Field.PNAMENAME);
        return null;
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTPath node, Object data) {
        ((Stack)data).push(node.value);
        return null;
    }

}
