/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.lib;

import org.objectweb.jonas_ejb.deployment.ejbql.ASTAbstractSchemaName;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberDeclaration;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEJBQL;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFromClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentifier;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTPath;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTRangeVariableDeclaration;
import org.objectweb.jonas_ejb.deployment.ejbql.ParseException;
import org.objectweb.jonas_ejb.deployment.ejbql.SimpleNode;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSelectClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTIdentificationVariable;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedPathExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTSingleValuedNavigation;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTWhereClause;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTConditionalFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBetweenExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTInExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTLikeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTNullComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEmptyCollectionComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTCollectionMemberExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTComparisonExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticTerm;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTArithmeticFactor;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTStringExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTDatetimeExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTBooleanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTEntityBeanExpression;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningStrings;
import org.objectweb.jonas_ejb.deployment.ejbql.ASTFunctionsReturningNumerics;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Schema;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.lib.NavigatorNodeFactory;
import org.objectweb.medor.query.lib.Cartesian;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.filter.lib.NavigatorOperator;
import org.objectweb.medor.filter.lib.BasicFieldOperand;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;
import java.util.ArrayList;
import java.util.HashSet;

/**
 * Implementation of a visitor that creates a map of pairs [id,JormExtent] for all defined
 * identifiers of the query.
 * @author  Christophe Ney  [cney@batisseurs.com]
 */


public class EjbqlVariableVisitor extends EjbqlAbstractVisitor {

    private Schema schema;
    private HashMap ids = new HashMap();
    private QueryTree qt = null;

    /**
     * constructor
     * @param ejbql root of the lexical tree
     * @param schema JORM schema on which the query applies
     */
    public EjbqlVariableVisitor(ASTEJBQL ejbql, Schema schema) throws Exception {
        this.schema = schema;
        visit(ejbql);
        // replace map values with JormExtent or Medor Field
        for (Iterator i = ids.keySet().iterator(); i.hasNext();) {
            getQueryTree((String) i.next());
        }
        HashSet s = new HashSet();
        Iterator i = null;
        for (i = ids.values().iterator(); i.hasNext();) {
            s.add(((IdValue) i.next()).qt);
        }
        i = s.iterator();
        qt = (QueryTree) i.next();
        while (i.hasNext()) {
            qt = new Cartesian(qt, (QueryTree) i.next(), null, null);
        }
    }

    private QueryTree getQueryTree(String id)
        throws ParseException, MedorException {

        IdValue iv = (IdValue) ids.get(id);
        if (iv == null) {
            throw new ParseException("Abstract schema name \""
                + id + "\" has not been declared");
        }
        if (iv.qt != null) {
            return iv.qt;
        }
        if (iv.name==null) {
            throw new ParseException("Identifer '" + id
                + "' has not been declared");
        }

        if (iv.name.length==1) {
            // Abstract schema name case
            Class theClass = schema.getClass(iv.name[0]);
            if (theClass == null) {
                throw new ParseException("Abstract schema name \"" + iv.name[0]
                    + "\" has not been declared in the jorm meta information");
            }
            iv.qt = new ClassExtent(theClass, id, Field.PNAMENAME, false);
        }
        else {
            // collection member declaration
            iv.qt = getQueryTree(iv.name[0]);
            iv.qt = getNavNode(iv.qt, iv.name, id);
        }
        for (int i=0; i<iv.getDeclaredPathLength(); i++) {
            String[] p = iv.getSplitedPath(i);
            iv.qt = getNavNode(iv.qt, p, mergePath(p, 0, p.length-1));
        }
        if (iv.name.length > 1) {
            setQueryTree(iv.name[0], iv.qt);
        }
        return iv.qt;
    }

    private QueryTree getNavNode(QueryTree qt, String[] pathName, String alias) {
        try {
            Field f = qt.getTupleStructure().getField(
                pathName[0] + "."  + Field.PNAMENAME);
            NavigatorOperator nav = new NavigatorOperator(
                new BasicFieldOperand(f), pathName[1]);
            for (int i=2; i<pathName.length; i++) {
                nav = new NavigatorOperator(nav, pathName[i]);
            }
//TODO: use the alias in the NavigatorNodeFactory
            return NavigatorNodeFactory.getNavigatorNode(nav, alias, null);
        }
        catch (MedorException e) {
            throw new VisitorException(e);
        }
    }

    private void setQueryTree(String id, QueryTree qt) {
        IdValue iv = (IdValue) ids.get(id);
        iv.qt = qt;
        if (iv.name.length > 1) {
            setQueryTree(iv.name[0], iv.qt);
        }
    }

    /**
     * get the Map that was built from visiting the lexical query tree
     */
    public QueryTree getQueryTree() {
        return qt;
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTFromClause node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTCollectionMemberDeclaration node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTRangeVariableDeclaration node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * visit child nodes
     */
    public Object visit(ASTCollectionValuedPathExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTAbstractSchemaName node, Object data) {
        ((Stack) data).push(node.value);
        return null;
    }

    /**
     * store the pair identifier,Node from the Stack in HashMap
     */
    public Object visit(ASTIdentifier node, Object data) {
        String id = (String) node.value;
        String name = (String) ((Stack) data).pop();
        IdValue iv = (IdValue) ids.get(id);
        if (iv == null) {
            iv = new IdValue(name);
            ids.put(id, iv);
        }
        else {
            iv.name = splitPath(name);
        }
        return null;
    }

    /**
     * Push the Node to the stack
     */
    public Object visit(ASTPath node, Object data) {
        ((Stack) data).push(node.value);
        return null;
    }

    // -------------- select ---------------//

    public Object visit(ASTSelectClause node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        if (!st.empty()) {
            String path = (String) st.pop();
            String id = splitPath(path)[0];
            IdValue iv = (IdValue) ids.get(id);
            if (iv == null) {
                iv = new IdValue();
            }
            iv.addPath(path);
        }
        return null;
    }

    public Object visit(ASTIdentificationVariable node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTSingleValuedPathExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    // ----------- Where -----------------//
    public Object visit(ASTWhereClause node, Object data) {
        visit((SimpleNode) node, data);
        Stack st = (Stack) data;
        while (!st.empty()) {
            String path = (String) st.pop();
            String id = splitPath(path)[0];
            IdValue iv = (IdValue) ids.get(id);
            if (iv == null) {
                iv = new IdValue();
            }
            iv.addPath(path);
        }
        return null;
    }

    public Object visit(ASTSingleValuedNavigation node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTConditionalExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTConditionalTerm node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTConditionalFactor node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTBetweenExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTInExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTLikeExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTNullComparisonExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTEmptyCollectionComparisonExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTCollectionMemberExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTComparisonExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTArithmeticExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTArithmeticTerm node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTArithmeticFactor node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTStringExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTDatetimeExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTBooleanExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTEntityBeanExpression node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTFunctionsReturningStrings node, Object data) {
        return visit((SimpleNode) node, data);
    }

    public Object visit(ASTFunctionsReturningNumerics node, Object data) {
        return visit((SimpleNode) node, data);
    }
}
