/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Sebastien Chassande-Barrioz for INRIA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ejb.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.FieldDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleDesc;

/**
 * This interface defines the methods which abstract the creation of the jorm
 * meta object specialized for a mapping type. There are only two kind of
 * specialized jorm meta object: the ClassMapping and the ValueMapping
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface MappingBuilder {

    /**
     * It call the createClassMapping method on the class parameter in order
     * to create ClassMapping object specialized for a mapper type.
     * The EntityCmp2Desc contains all information needed to fill the
     * ClassMapping.
     * @param clazz  is the jorm meta object which the method must define a
     * ClassMapping
     * @param bd is the jonas meta object which contains the data needed to
     * fill the ClassMapping.
     * @param mappingName is the name of the mapping which must be used to
     * create the ClassMapping
     */
    void createClassMapping(Class clazz, EntityCmp2Desc bd, String mappingName);

    /**
     * It call the createClassMapping method on the GenClassRef parameter in
     * order to create ClassMapping object specialized for a mapper type.
     * This ClassMapping builder is specialized for the relationship.
     * The EntityCmp2Desc contains all information needed to fill the
     * ClassMapping.
     * @param clazz  is the jorm meta object which the method must define a
     * ClassMapping
     * @param rsrd is the jonas meta object which contains the data needed to
     * fill the ClassMapping.
     * @param mappingName is the name of the mapping which must be used to
     * create the ClassMapping
     */
    void createClassMapping(GenClassRef gcr,
                            EjbRelationshipRoleDesc rsrd, String mappingName);


    /**
     * It call the createValueMapping method on the PrimitiveElement parameter
     * in order to create ValueMapping object specialized for a mapper type.
     * The FieldDesc contains all information needed to fill the ValueMapping.
     * @param pe  is the jorm meta object which the method must define a
     * ValueMapping
     * @param desc is the jonas meta object which contains the data needed to
     * fill the ValueMapping.
     * @param mappingName is the name of the mapping which must be used to
     * create the ValueMapping
     */
    void createValueMapping(PrimitiveElement pe, FieldDesc desc, String mappingName) ;

    /**
     * It call the createValueMapping method on the PrimitiveElement parameter
     * in order to create ValueMapping object specialized for a mapper type.
     * This ValueMapping builder is specialized for the relationship.
     * The FieldDesc and the EjbRelationshipRoleDesc contains all information
     * needed to fill the ValueMapping.
     * @param pe  is the jorm meta object which the method must define a
     * ValueMapping
     * @param desc is the jonas meta object which contains the data needed to
     * fill the ValueMapping.
     * @param mappingName is the name of the mapping which must be used to
     * create the ValueMapping
     */
    void createValueMapping(PrimitiveElement pe, FieldDesc desc,
                            EjbRelationshipRoleDesc rsrd,String mappingName) ;

}
