/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Sebastien Chassande-Barrioz for INRIA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/


package org.objectweb.jonas_ejb.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.rdb.RDBClassMapping;
import org.objectweb.jorm.metainfo.api.rdb.RDBValueMapping;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.FieldDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityJdbcCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.FieldJdbcDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationJdbcDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleDesc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleJdbcDesc;

import java.util.Iterator;
import java.util.Map;

/**
 * This class is an implementation of the MappingBuilder interface for the rdb
 * mapper.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class RdbMappingBuilder implements MappingBuilder {

    /**
     * This implementation is linked to the rdb mapper.
     */
    public final static String MAPPER_NAME = "rdb";

    public void createClassMapping(Class clazz, EntityCmp2Desc bd,
                                   String mappingName) {
        RDBClassMapping cm = (RDBClassMapping)
            clazz.createClassMapping(MAPPER_NAME, mappingName);

        //Assign the table name
        cm.setTableName(((EntityJdbcCmp2Desc) bd).getJdbcTableName());

        //assign the primary key definition
        for(Iterator it = bd.getCmpFieldDescIterator(); it.hasNext();) {
            FieldJdbcDesc fd = (FieldJdbcDesc) it.next();
            if (fd.isPrimaryKey()) {
                cm.addPrimaryKey(fd.getJdbcFieldName());
            }
        }
    }

    public void createValueMapping(PrimitiveElement pe, FieldDesc desc,
                                   String mappingName) {
        ((RDBValueMapping) pe.createValueMapping(MAPPER_NAME, mappingName))
            .createRDBColumnSpec(
                ((FieldJdbcDesc) desc).getJdbcFieldName(), null, false);
    }

    public void createClassMapping(GenClassRef gcr,
                                   EjbRelationshipRoleDesc rsrd,
                                   String mappingName) {
         RDBClassMapping cm = (RDBClassMapping)
             gcr.createClassMapping(MAPPER_NAME, mappingName);
         cm.setTableName(
             ((EjbRelationJdbcDesc) rsrd.getEjbRelationDesc()).getJdbcTableName() );
//TODO: fill with foreygn key
    }

    public void createValueMapping(PrimitiveElement pe, FieldDesc desc,
                                   EjbRelationshipRoleDesc rsrd,
                                   String mappingName) {

        ((RDBValueMapping) pe.createValueMapping(MAPPER_NAME, mappingName))
            .createRDBColumnSpec(
                ((EjbRelationshipRoleJdbcDesc) rsrd).getForeignKeyJdbcName(
                    ((FieldJdbcDesc) desc).getJdbcFieldName() ),
                null, false);
    }
}
