/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Laurent Chauvirey, Frederic Maistre, Nicolas Tachker
 * Contributor(s): 
 * Philippe Durieux
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_jms; 

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;
import org.objectweb.jonas_jms.api.JmsManager;

/** 
 * Factory used by JNDI lookup to get Topic or Queue Connection Factories.
 * These factories are managed by the JMS Manager.
 * @see	   javax.naming.spi.ObjectFactory
 */
public class JObjectFactory implements ObjectFactory {

    /** 
     * Always return the unique ConnectionFactory stored in the JmsManager
     */
    public Object getObjectInstance(Object refObj, Name name, Context nameCtx, Hashtable env) throws Exception {

	Reference ref = (Reference) refObj;
	JmsManager jms = JmsManagerImpl.getJmsManager();
	String clname = ref.getClassName();

	TraceJms.debug("JObjectFactory getObjectInstance "+clname);

	if (clname.equals("org.objectweb.jonas_jms.JQueueConnectionFactory")) {
	    // QueueConnectionFactory
	    return jms.getQueueConnectionFactory();
	} else if (clname.equals("org.objectweb.jonas_jms.JTopicConnectionFactory")) {
	    // TopicConnectionFactory
	    return jms.getTopicConnectionFactory();
	} else {
	    TraceJms.error("JObjectFactory - Bad class name: "+clname);
	    return null;
	}
    }
}
