/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_tm;

import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;

/**
 * Class used by JNDI to get a UserTransaction from the client
 * @author Philippe Durieux
 */
public class UserTransactionFactory implements ObjectFactory {

    public Object getObjectInstance(Object objref, Name name, Context ctx, Hashtable env) throws Exception {

	Reference ref = (Reference) objref;
	Current ut = null;

	if (ref.getClassName().equals("org.objectweb.jonas_tm.Current")) {

	    // create the UserTransaction object
	    // No need to init a TMFactory in the client!
	    ut = new Current();

	    // Get the timeout default value that was configured in the server
	    String timeoutStr = (String)ref.get("jonas.tm.timeout").getContent();
	    Integer i = new Integer(timeoutStr);
	    int timeout = i.intValue();
	    ut.setDefaultTimeout(timeout);

	    // Set corba flag
	    String corbaStr = (String)ref.get("jonas.orb.corba").getContent();
	    Boolean b = new Boolean(corbaStr);
	    ut.setCorba(b.booleanValue());
	}
	return ut;
    }
}
