/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_web.deployment.api;


//import jonas_web
import org.objectweb.jonas_web.deployment.xml.EjbRef;
import org.objectweb.jonas_web.deployment.xml.JonasEjbRef;

/**
 * Class that represents an ejb-ref tag.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public class EjbRefDesc {

    /**
     * The ejb ref name.
     */
    private String name;
    
    /**
     * The type of the ejb ref.
     */
    private String ejbRefType;

    /**
     * The ejb link of the ejb ref.
     */
    private String ejblink;

    /**
     * The jndi name of the ejb ref.
     */
    private String jndiName;

    /**
     * Construct a descriptor for an ejb ref tag.
     * @param ejbRef the ejb-ref resulting of the web.xml parsing.
     * @param jBRef the jonas-ejb-ref resulting of the jonas-web.xml parsing.
     * @throws WebContainerDeploymentDescException when missing information for
     * creating the EjbRefDesc.
     */
    public EjbRefDesc(EjbRef ejbRef, JonasEjbRef jBRef) 
        throws WebContainerDeploymentDescException {

        name = ejbRef.getEjbRefName().getValue();
        ejbRefType = ejbRef.getEjbRefType().getValue();
        ejblink = (ejbRef.getEjbLink() != null) ? ejbRef.getEjbLink().getValue() : null;
        jndiName = (jBRef != null) ? jBRef.getJndiName().getValue() : null;
    }

    /**
     * Get the name of the ejb-ref 
     * @return String representation of the name.
     */
    public String getName() {
        return name;
    }

    /**
     * Get the ejb-ref-type.
     * @return String representation of the ejb-ref-type.
     */
    public String getEjbRefType() {
        return ejbRefType;
    }

    /**
     * Get the ejb-link.
     * @return String representation of the ejb-link
     */
    public String getEjbLink() {
        return ejblink;
    }

    /**
     * Get the jndi name of the ejb-ref.
     * @return String representation of the JNDI name
     */
    public String getJndiName() {
        return jndiName;
    }

    /**
     * Set the jndi name of the ejb-ref.
     * @param jndiName representation of the JNDI name
     */
    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }
    
    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\ngetName()=" + getName());
        ret.append("\ngetEjbRefType()=" + getEjbRefType());
        ret.append("\ngetEjbLink()=" + getEjbLink());
        ret.append("\ngetJndiName()=" + getJndiName());
        return ret.toString();
    }
   
}
