/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */



package org.objectweb.security.context;

import java.io.Serializable ;

import java.security.Principal ;

/**
 * Implementation of the JOnAS Security Context
 *
 */

public class SecurityContext implements Serializable {

    private String principalName = null ;

    /**
     * Constructor SecurityContext
     *
     *   use the default principal name
     *
     */

    public SecurityContext () {
	principalName = "JOnAS_client";
    }

    /**
     * Constructor SecurityContext
     *
     *   @param String principal name
     *
     */

    public SecurityContext (String principalName) {
	this.principalName = principalName ;
    }

    /**
     * Method getCallerPrincipal
     *
     * @return the Principal in the Security Context
     *
     */

    public Principal getCallerPrincipal () {
	return new Principal () {
		public String getName() {
		    return principalName ;
		}
		
		public boolean equals (Object o) {
		    if (o instanceof Principal)
			return principalName.equals(((Principal)o).getName()) ;
		    return false ;
		}

		public String toString () {
		    return "name = "+principalName ;
		}
	    } ;
    }

    /**
     * Method toString
     *
     */
    public String toString () {
	return "principal : name = "+principalName ;
    }
 
}		

