/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 * 
 */

package org.objectweb.security.context;

import java.security.Principal;
/**
 * For handling the association SecurityContext/ Thread
 *
 */

public class SecurityCurrent {

    private static InheritableThreadLocal threadCtx ;
    
    static {
	threadCtx= new InheritableThreadLocal();
	threadCtx.set (new SecurityContext()) ;
    }
    
    private static SecurityCurrent current = new SecurityCurrent () ;

    /**
     * Method getCurrent
     *
     * @return SecurityCurrent return the current 
     *
     */

    public static SecurityCurrent getCurrent() {
	return current ;
    }

    /**
     * Method setSecurityContext
     *
     * @param SecurityContext  Security context to associate to the current thread
     *
     */

    public void setSecurityContext (SecurityContext ctx) {
	threadCtx.set(ctx) ;
    }

   /**
     * Method getSecurityContext
     *
     * @return SecurityContext  return the Security context associated to the current thread
     *
     */

    public SecurityContext getSecurityContext() {
	SecurityContext ctx = (SecurityContext) threadCtx.get() ;    
	return ctx ;
    }  

 
}  
