/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.beans.folder;

import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.ejb.CreateException;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Implementation for the bean PaperEC.
 * @author Philippe Durieux, Philippe Coq
 */
public class PaperEC implements EntityBean {

    static protected Logger logger = null;
    EntityContext ejbContext;

    // ------------------------------------------------------------------
    // State of the bean. 
    // They must be public for Container Managed Persistance.
    // ------------------------------------------------------------------
    public String name;
    public int value;

    // ------------------------------------------------------------------
    // EntityBean implementation
    // ------------------------------------------------------------------

    public void setEntityContext(EntityContext ctx) { 
	if (logger == null)
	    logger = Log.getLogger(Log.JONAS_TESTS_PREFIX);
	logger.log(BasicLevel.DEBUG, "");
	ejbContext = ctx;
    }

    public void unsetEntityContext() {
	logger.log(BasicLevel.DEBUG, "");
	ejbContext = null;
    }

    public void ejbActivate() {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbPassivate() {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbLoad() {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbStore() {
	logger.log(BasicLevel.DEBUG, "");
    }
  
    public void ejbRemove() throws RemoveException {
	logger.log(BasicLevel.DEBUG, "");
    }

    // ------------------------------------------------------------------
    // ejbCreate methods
    // ------------------------------------------------------------------

    public String ejbCreate(String name) throws CreateException {
	logger.log(BasicLevel.DEBUG, "");
	this.name = name;
	this.value = 0;
	return null;		// In CMP, should return null.
    }

    public String ejbPostCreate(String name) throws CreateException {
	logger.log(BasicLevel.DEBUG, "");
	return null;		// In CMP, should return null.
    }

    // ------------------------------------------------------------------
    // PaperLocal implementation
    // ------------------------------------------------------------------

    public String getName() {
	logger.log(BasicLevel.DEBUG, "");
	return this.name;
    }

    public int getValue() {
	logger.log(BasicLevel.DEBUG, "");
	return this.value;
    }

    public void setValue(int v) {
	logger.log(BasicLevel.DEBUG, "");
	this.value = v;
    }
}

