/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.beans.transacted;

import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.ejb.CreateException;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Entity bean with container managed persistence version 2.
 * @author Helene Joanin 
 */
public abstract class SimpleEC2 extends SimpleCommon implements EntityBean {

    protected EntityContext entityContext;

    // ------------------------------------------------------------------
    // Get and Set accessor methods of the bean's abstract schema
    // ------------------------------------------------------------------
    public abstract String getAccno();
    public abstract void setAccno(String accno);
    public abstract String getCustomer();
    public abstract void setCustomer(String customer);
    public abstract long getBalance();
    public abstract void setBalance(long balance);

    /**
     * Create without arg
     * transaction attribute = supports (by default in xml file) 
     */
    public String ejbCreate() throws CreateException {

	logger.log(BasicLevel.DEBUG, "");

	// Must init fields in any case.
	setAccno("000");
	setCustomer("Initial");
	setBalance(0);
	// In BMP, return PK.
	return getAccno();
    }

    /**
     * transaction attribute = notsupported
     */
    public String ejbCreate(int i) throws CreateException  {

	logger.log(BasicLevel.DEBUG, "");

	if (isAssociated()) {
	    throw new EJBException("ejbCreate(int i): should not be in a transaction");
	}
	Integer v = new Integer((int)i);
	setAccno(v.toString());
	setCustomer("by int");
	setBalance(1000 + i);
	
	// In BMP, return PK.
	return getAccno();
    }

    /**
     * transaction attribute = required
     */
    public String ejbCreateForRequired(long i)  throws CreateException {
	logger.log(BasicLevel.DEBUG, "");
	if (!isAssociated()) {
	    throw new EJBException("ejbCreate(long i): should be in a transaction");
	}
	Integer v = new Integer((int)i);
	setAccno(v.toString());
	setCustomer("Required");
	setBalance(1000 + i);
	
	// In BMP, return PK.
	return getAccno();
    }

    /**
     * transaction attribute = never
     */
    public String ejbCreateForNever(short i)  throws CreateException {
	logger.log(BasicLevel.DEBUG, "");

	if (isAssociated()) {
	    throw new EJBException("ejbCreateForNever(short i): should not be in a transaction");
	}
	Integer v = new Integer((int)i);
	setAccno(v.toString());
	setCustomer("by short");
	setBalance(1000 + i);
	
	// In BMP, return PK.
	return getAccno();
    }

    /**
     * transaction attribute = requiresnew
     */
    public String ejbCreateForRequiresNew(String i)  throws CreateException {
	logger.log(BasicLevel.DEBUG, "");

	if (!isAssociated()) {
	    throw new EJBException("ejbCreate(String i): should be in a transaction");
	}
	setAccno(i);
	setCustomer("by string");
	setBalance(100);

	// In BMP, return PK.
	return getAccno();
    }

    /**
     * transaction attribute = mandatory
     */
    public String ejbCreateForMandatory(char i)  throws CreateException {
	logger.log(BasicLevel.DEBUG, "");

	if (!isAssociated()) {
	    throw new EJBException("ejbCreateForMandatory(char i): should be in a transaction");
	}
	Integer v = new Integer((int)i);
	setAccno(v.toString());
	setCustomer("by char");
	setBalance(200);
	
	// In BMP, return PK.
	return getAccno();
    }

    /**
     * transaction attribute = supports
     */
    public String ejbCreateForSupports(boolean intx)  throws CreateException {
	logger.log(BasicLevel.DEBUG, "");

	if (intx) {
	    setAccno("TRUE");
	    setBalance(101);
	    if (!isAssociated()) {
		throw new EJBException("ejbCreateForSupports(true): should be in a transaction");
	    }
	} else {
	    setAccno("FALSE");
	    setBalance(102);
	    if (isAssociated()) {
		throw new EJBException("ejbCreate(false): should not be in a transaction");
	    }
	}
	setCustomer("by boolean");
	
	// In BMP, return PK.
	return getAccno();
    }


    /**
     * transaction attribute = default (= supports)
     */
    public void ejbPostCreate() {
	logger.log(BasicLevel.DEBUG, "");
    }

    /**
     * transaction attribute = notsupported
     */
    public void ejbPostCreate(int i) {
	logger.log(BasicLevel.DEBUG, "");
	if (isAssociated()) {
	    throw new EJBException("ejbPostCreate(int i): should not be in a transaction");
	}
    }

    /**
     * transaction attribute = required
     */
    public void ejbPostCreateForRequired(long i) {
	logger.log(BasicLevel.DEBUG, "");
	if (!isAssociated()) {
	    throw new EJBException("ejbPostCreateForRequired(long i): should be in a transaction");
	}
    }

    /**
     * transaction attribute = never
     */
    public void ejbPostCreateForNever(short i) {
	logger.log(BasicLevel.DEBUG, "");

	if (isAssociated()) {
	    throw new EJBException("ejbPostCreateForNever(short i): should not be in a transaction");
	}
    }

    /**
     * transaction attribute = requiresnew
     */
    public void ejbPostCreateForRequiresNew(String i) {
	logger.log(BasicLevel.DEBUG, "");

	if (!isAssociated()) {
	    throw new EJBException("ejbPostCreateForRequiresNew(String i): should be in a transaction");
	}
    }

    /**
     * transaction attribute = mandatory
     */
    public void ejbPostCreateForMandatory(char i) {
	logger.log(BasicLevel.DEBUG, "");

	if (!isAssociated()) {
	    throw new EJBException("ejbPostCreateForMandatory(char i): should be in a transaction");
	}
    }

    /**
     * transaction attribute = supports
     */
    public void ejbPostCreateForSupports(boolean intx) {
	logger.log(BasicLevel.DEBUG, "");

	if (intx) {
	    if (!isAssociated()) {
		throw new EJBException("ejbPostCreateForSupports: should be in a transaction");
	    }
	} else {
	    if (isAssociated()) {
		throw new EJBException("ejbPostCreateForSupports: should not be in a transaction");
	    }
	}
    }


    /**
     * This method return true if there is an association of a transaction with this thread
     */
    public boolean  ejbHomeOpwith_notsupported() {
	logger.log(BasicLevel.DEBUG, "");
	return isAssociated();
    }

    /**
     * This method return true if there is an association of a transaction with this thread
     */
    public boolean ejbHomeOpwith_supports() {
	logger.log(BasicLevel.DEBUG, "");
	return isAssociated();
    }
   
    /**
     * This method return true if there is an association of a transaction with this thread
     */
    public boolean  ejbHomeOpwith_required() {
	logger.log(BasicLevel.DEBUG, "");
	return isAssociated();
    }

    /**
     * This method return true if there is an association of a transaction with this thread
     */
    public boolean ejbHomeOpwith_requires_new() {
	logger.log(BasicLevel.DEBUG, "");
	return isAssociated();
    }

    /**
     * This method return true if there is an association of a transaction with this thread
     */
    public boolean ejbHomeOpwith_mandatory() {
	logger.log(BasicLevel.DEBUG, "");
	return isAssociated();
    }
  
    /**
     * This method return true if there is an association of a transaction with this thread
     */
    public boolean ejbHomeOpwith_never() {
	logger.log(BasicLevel.DEBUG, "");
	return isAssociated();
    }

    public void ejbRemove() throws RemoveException {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbPassivate() {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbActivate() {
	logger.log(BasicLevel.DEBUG, "");
    }
    
    public void ejbLoad() {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbStore() {
	logger.log(BasicLevel.DEBUG, "");
    }

    public void setEntityContext(EntityContext ctx) {
	initLogger();
	logger.log(BasicLevel.DEBUG, "");
	this.entityContext = ctx;
    }

    public void unsetEntityContext() {
	logger.log(BasicLevel.DEBUG, "");
	this.entityContext = null;
    }
	
 	
}
