/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.distribution;

import java.rmi.RemoteException;
import java.util.Enumeration;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.folder.Folder;
import org.objectweb.jonas.jtests.beans.folder.FolderHome;
import org.objectweb.jonas.jtests.beans.folder.FileHome;
import org.objectweb.jonas.jtests.beans.folder.File;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * Test a session bean remote accessing entities local.
 * Beans used: folder
 * @author Philippe Durieux
 */
public class F_Frontal extends JTestCase {

    protected static FolderHome fhome = null;
    protected Folder folder = null;

    public F_Frontal(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
	if (fhome == null) {
	    useBeans("folder", true);
	    try {
		fhome = (FolderHome) PortableRemoteObject.narrow(ictx.lookup("FolderSY"), FolderHome.class);
		assertNotNull(fhome);
	    } catch (NamingException e) {
		fail("Cannot get bean home");
	    }
	}
	if (folder == null) {
	    try {
		folder = fhome.create();
		assertNotNull(folder);
	    } catch (Exception e) {
		fail("Cannot create folder session");
	    }
	}
    }

    /**
     * create 1 entity and remove it by its remote reference.
     */
    public void testCreate1F() throws Exception {
	File f = folder.newFile("file1");
	assertNotNull(f);
	f.remove();
    }

    public static Test suite() {
	return new TestSuite(F_Frontal.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_Frontal(testtorun));
	}
    }
}
