/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.rmi.RemoteException;
import java.util.Enumeration;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.annuaire.Personne;
import org.objectweb.jonas.jtests.beans.annuaire.PersonneHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This test isModified feature on entity bean CMP.
 * Beans used: annuaire
 * @author Philippe Coq, Philippe Durieux (jonas team)
 */
public class F_IsModified extends JTestCase {

    private static String BEAN_HOME = "annuairePersonneECHome";
    protected static PersonneHome ehome = null;
    String mynum = "63800";
    String myname = "Max";

    public F_IsModified(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
	if (ehome == null) {
	    useBeans("annuaire", true);
	    try {
		ehome = (PersonneHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), PersonneHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home");
	    }
	}
    }

    /**
     * test that readonly methods does not trigger ejbStore
     */
    public void testNotModified() throws Exception {

	Personne acc = ehome.create(myname, mynum);

	// readonly transaction
	acc.reset();
	utx.begin();
	acc.getNom();
	assertTrue(! acc.isDirty());
	utx.commit();
	assertTrue(! acc.ejbStoreCalled());
	assertTrue(acc.isModifiedCalled());

	acc.remove();
    }

    /**
     * test that write methods do trigger ejbStore
     */
    public void testModified() throws Exception {

	Personne acc = ehome.create(myname, mynum);
	String os = "100";

	// write transaction
	acc.reset();
	utx.begin();
	acc.setNumero(os);
	assertTrue(acc.isDirty());
	utx.commit();
	assertTrue(acc.ejbStoreCalled());
	assertTrue(acc.isModifiedCalled());

	String ns = acc.getNumero();
	assertTrue(ns.equals(os));
	assertTrue(! acc.isDirty());

	acc.remove();
    }

    public static Test suite() {
	return new TestSuite(F_IsModified.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_IsModified(testtorun));
	}
    }
}
