/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.rmi.RemoteException;
import java.util.Collection;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.ebasic.Account;
import org.objectweb.jonas.jtests.beans.ebasic.AccountHome;
import org.objectweb.jonas.jtests.beans.ebasic.Person;
import org.objectweb.jonas.jtests.beans.ebasic.PersonHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * These are tests about the different cases of the primary key type for CMP version 1 entity.
 * All common tests to CMP version 1 and CMP version 2 are in the inherited class.
 * @author Helene Joanin (jonas team)
 */
public class F_VariousPKEC extends A_VariousPKEC {

    private static String BEAN_HOME_ACCOUNT = "ebasicAccountECHome";
    protected static AccountHome ahome = null;
    private static String BEAN_HOME_PERSON = "ebasicPersonECHome";
    protected static PersonHome phome = null;

    public F_VariousPKEC(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
    }

    public AccountHome getAccountHome() {
	if (ahome == null) {
	    try {
		ahome = (AccountHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_ACCOUNT), AccountHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home: "+BEAN_HOME_ACCOUNT);
	    }
	}
	return ahome;
    }

    public PersonHome getPersonHome() {
	if (phome == null) {
	    try {
		phome = (PersonHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_PERSON), PersonHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home: "+BEAN_HOME_PERSON);
	    }
	}
	return phome;
    }

    public static Test suite() {
	return new TestSuite(F_VariousPKEC.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_VariousPKEC(testtorun));
	}
    }
}
