/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.exception;

import java.rmi.RemoteException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.beanexc.AccountS;
import org.objectweb.jonas.jtests.beans.beanexc.AccountSHome;
import org.objectweb.jonas.jtests.beans.beanexc.AppException;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * Exception  test cases specific for session with bean-managed transaction
 * @see Spec 2.0 table 16 page377
 *
 * @author Ph.Coq
 */
public class F_CatcherBMT extends JTestCase {

    // Note: the <ejb-name> is taken as <jndi-name> when not present 
    // in jonas specific deployment descriptor
    private static String BEAN_HOME = "BTAccountSL";
    protected static AccountSHome home = null;

    public F_CatcherBMT(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
	useBeans("beanexc", true);
    }

    public AccountSHome getHome() {
        if (home == null) {
	    try {
		home = (AccountSHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), AccountSHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home");
	    }
	}
	return home;
    }

    /**
     * return an AccountS
     */
    public AccountS getAccountS(int i) {
        AccountS acs = null;
        try {
            acs = getHome().create();
        } catch (Exception e) {
            // should never occurs if the test is well written
            fail("Exception in getAccountS  should never occur:"+e);
        }
        return acs;
    }

    /**
     * Test business method throwing an Application exception.
     * It must be caught by the client.
     * 
     * Spec 2.0 page 377 (table16)
     */
    public void testApplicationBMT() throws Exception {
	AccountS acs  = getAccountS(83);
	try {
	    acs.doAppException_3();
	    fail("No AppException");
	} catch (AppException e) {
	}
    }

    /**
     * Test business method throwing an unchecked exception.
     * The client must receive RemoteException 
     * 
     * See EJB2.0 specs page 377 (table 16)
     */
    public void testUncheckedBMT() throws Exception {
	AccountS acs = getAccountS(84);
	try {
	    acs.doUncheckedException_3(); 
	    fail("No RemoteException");
	} catch (RemoteException e) {
	}

    }

    public static Test suite() {
	return new TestSuite(F_CatcherBMT.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_CatcherBMT(testtorun));
	}
    }

}
