/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.security;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.secured.BaseS;
import org.objectweb.jonas.jtests.beans.secured.BaseEHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * Security Management Suite with a stateless session as first bean
 *
 * @author Ph.Coq, Ph.Durieux
 *
 */

public class F_AccessControlEC  extends A_AccessControl {   
 
    private static String BEAN_HOME = "securedBaseECHome";
    protected static BaseEHome home = null;

    public F_AccessControlEC(String name) {
	super(name);
    }

    /**
     * return the Home
     */

    public BaseEHome getHome() {
	if (home == null) {
	    try {
		home = (BaseEHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), BaseEHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home "+BEAN_HOME);
	    }
	}
	return home;
    }

    public BaseS getBaseS(String name) throws Exception{
        return getHome().create(name, name+"_info");
    }

   /**
     * init environment:
     * - load beans
     */
    protected void setUp() {
	super.setUp();
	useBeans("secured", true);
    }


    /**
     * This suite is all BMP test cases
     */
    public static Test suite() {
	return new TestSuite(F_AccessControlEC.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_AccessControlEC(testtorun));
	}
    }
}
