/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.transaction;

import java.rmi.RemoteException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.transacted.SynchroSimple;
import org.objectweb.jonas.jtests.beans.transacted.SimpleSHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * Test for Bean to Bean transactions
 * @author Ph.Durieux
 */
public class F_BeanToLocalTx extends JTestCase {

    protected static SimpleSHome syhome = null;

    public F_BeanToLocalTx(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
	useBeans("transacted", true);
	if (syhome == null) {
	    String BEAN_HOME = "transactedSimpleSYHome";
	    try {
		syhome = (SimpleSHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), SimpleSHome.class);
	    } catch (NamingException e) {
		fail("Cannot get "+BEAN_HOME+":"+e);
	    }
	}
    }

    /**
     * SY/Required -> Local/RequiresNew
     */
    public void testSY2LRN() throws Exception {
	SynchroSimple s1 = (SynchroSimple) syhome.create();
	assertEquals(true, s1.call_requires_new_local());
	s1.remove();
    }

    /**
     * SY/Required -> Local/NotSupported
     */
    public void testSY2LNS() throws Exception {
	SynchroSimple s1 = (SynchroSimple) syhome.create();
	assertEquals(true, s1.call_notsupported_local());
	s1.remove();
    }

    public static Test suite() {
	return new TestSuite(F_BeanToLocalTx.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_BeanToLocalTx(testtorun));
	}
    }
}
