/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.transaction;

import java.rmi.RemoteException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.transaction.RollbackException;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.annuaire.Personne;
import org.objectweb.jonas.jtests.beans.annuaire.PersonneHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * Test for Bean Managed Transaction entity beans
 * @author Philippe Durieux
 */
public class F_EntityCMT extends JTestCase {

    private static String BEAN_HOME = "annuairePersonneECHome";
    protected static PersonneHome ehome = null;
    String mynum = "63801";
    String badnum = "99999";
    String newnum = "12341";
    String myname = "Phil";

    public F_EntityCMT(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
	if (ehome == null) {
	    useBeans("annuaire", true);
	    try {
		ehome = (PersonneHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), PersonneHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean Home:"+e);
	    }
	}
    }

    /**
     * Test that rollback will not lost old bean state
     * - create bean
     * - make a transaction that change the num and roll it back
     * - check num has not been changed.
     */
    public void testStateAfterRollBack() throws Exception {
	Personne acc = ehome.create(myname, mynum);
	utx.begin();
	acc.setNumero(badnum);
	utx.rollback();
	assertEquals(mynum, acc.getNumero());
	acc.remove();		// cleaning
    }

    /**
     * Test that rollback will not lost old bean state
     * - access bean 
     * - make a transaction rolled back
     * - check state has not been changed.
     */
    public void testStateAfterRollBackNoTx() throws Exception {
	Personne acc = ehome.create(myname, mynum);
	acc.setNumero(newnum);
	utx.begin();
	acc.setNumero(badnum);
	utx.rollback();
	assertEquals(newnum, acc.getNumero());
	acc.remove();		// cleaning
    }

    /**
     * Test that rollback will not lost old bean state
     * - access bean 
     * - make a transaction rolled back
     * - check state has not been changed.
     */
    public void testStateAfterRollBackTx() throws Exception {
	Personne acc = ehome.create(myname, mynum);
	utx.begin();
	acc.setNumero(newnum);
	utx.commit();
	utx.begin();
	acc.setNumero(badnum);
	utx.rollback();
	assertEquals(newnum, acc.getNumero());
	acc.remove();		// cleaning
    }

    public static Test suite() {
	return new TestSuite(F_EntityCMT.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_EntityCMT(testtorun));
	}
    }
}
