/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.api;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jonas_ejb.lib.EjbqlQueryTreeHolder;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Schema;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.Equal;
import org.objectweb.medor.query.api.PropagatedField;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.lib.SelectProject;

/**
 * Test case based on the <code>annuaire</code> EJB module.
 * @author  Christophe Ney - cney@batisseurs.com
 */
public class TestAnnuaireDeployment extends TestCase {

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * create a suite with all test methods of this class
     */
    public static Test suite() {
        return new TestSuite(TestAnnuaireDeployment.class);
    }

    /**
     * constructor called before running each testMethod (except when serialisation is involved)
     */
    public TestAnnuaireDeployment(String n) {
        super(n);
    }

    /**
     * method called before running each testMethod
     */
    protected void setUp() throws Exception {
        super.setUp();
        AssertDeploymentHelper.setJarName("annuaire.jar");
    }

    /**
     * method called after running each testMethod
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * test CMP2 properties of JOnAS meta-info
     */
    public void testAnnuaireDeploymentDesc() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        // check CMP2 detected
        assertTrue("DeploymentDesc instanceof DeploymentCmp2Desc", dd instanceof DeploymentCmp2Desc);
        // check bean found
        BeanDesc bd = dd.getBeanDesc("PersonneEC2");
        assertTrue("PersonneEC2 bean of type EntityCmp2", bd instanceof EntityCmp2Desc);
        MethodDesc md = AssertDeploymentHelper.getMethodDesc(bd, "findByNom", null);
        assertTrue("findByNom instanceof MethodCmp2Desc", md instanceof MethodCmp2Desc);
        md = AssertDeploymentHelper.getMethodDesc(bd, "findByNumero", null);
        assertTrue("findByNumero instanceof MethodCmp2Desc", md instanceof MethodCmp2Desc);
        md = AssertDeploymentHelper.getMethodDesc(bd, "findAll", null);
        assertTrue("findAll instanceof MethodCmp2Desc", md instanceof MethodCmp2Desc);
    }


    /**
     * test the struture of the generated JORM schema
     */
    public void testAnnuaireJormSchema() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        EntityJdbcCmp2Desc bd = (EntityJdbcCmp2Desc) dd.getBeanDesc("PersonneEC2");
        assertNotNull("PersonneEC2 bean descriptor", bd);
        Manager jm = dd.getManager();
        // find jorm class
        Class personne = jm.getClass("Personne");
        assertNotNull("Class Personne ", personne);
        // check schema
        assertEquals("Schema", ((Schema) personne.getParent()).getName(), ((EntityCmp2Desc) bd).getJormSchemaName());
        // check fields
        TypedElement te = personne.getTypedElement("nom");
        assertEquals("Field nom type ", te.getType().getTypeCode(), PType.TYPECODE_STRING);
        te = personne.getTypedElement("numero");
        assertEquals("Field numero type ", te.getType().getTypeCode(), PType.TYPECODE_STRING);
        // TODO check query

    }

    /**
     * test meta-information of the <code>FindAll</code> method
     */
    public void testAnnuaireQueryFindAll() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        EntityJdbcCmp2Desc bd = (EntityJdbcCmp2Desc) dd.getBeanDesc("PersonneEC2");
        MethodCmp2Desc md = (MethodCmp2Desc) AssertDeploymentHelper.getMethodDesc(bd, "findAll", null);
        assertEquals("findAll query", "SELECT OBJECT(o) FROM Personne o", md.getQuery());
        EjbqlQueryTreeHolder qth = md.getQueryTreeHolder();
        // projected field is pname
        SelectProject sp = (SelectProject) qth.getQueryTree();
        PropagatedField f = (PropagatedField) sp.getField(Field.PNAMENAME);
        assertNotNull("Query result is pName", f);
        assertTrue("Query result type is reference", f.getType().getTypeCode() == PType.TYPECODE_REFERENCE);
        // project child is JormExtent
        QueryTree je = ((QueryNode) sp).getChildren()[0];
        assertTrue("JormExtent is Personne", ((ClassExtent) je).getMetaObject().getName().equals("Personne"));
        // no filter
        assertNull("No filter", sp.getQueryFilter());
    }


    /**
     * test meta-information of the <code>FindByNom</code> method
     */
    public void testAnnuaireQueryFindByNom() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        EntityJdbcCmp2Desc bd = (EntityJdbcCmp2Desc) dd.getBeanDesc("PersonneEC2");
        MethodCmp2Desc md = (MethodCmp2Desc) AssertDeploymentHelper.getMethodDesc(bd, "findByNom", null);
        assertEquals("findByNom query", "SELECT OBJECT(o) FROM Personne o WHERE o.nom = ?1", md.getQuery());
        EjbqlQueryTreeHolder qth = md.getQueryTreeHolder();
        // projected field is pname
        SelectProject sp = (SelectProject) qth.getQueryTree();
        PropagatedField f = (PropagatedField) sp.getField(Field.PNAMENAME);
        assertNotNull("Query result is pName", f);
        assertTrue("Query result type is reference", f.getType().getTypeCode() == PType.TYPECODE_REFERENCE);
        // project child is JormExtent
        QueryTree je = ((QueryNode) sp).getChildren()[0];
        assertTrue("JormExtent is Personne", ((ClassExtent) je).getMetaObject().getName().equals("Personne"));
        // filter
        Expression e = sp.getQueryFilter();
        assertTrue("Filter of type Equal ", e instanceof Equal);
        Expression l = ((Equal) e).getLeftExpression();
        assertTrue(l instanceof BasicFieldOperand);
        assertEquals(((BasicFieldOperand) l).getField().getName(), "nom");
        Expression r = ((Equal) e).getRightExpression();
        assertTrue(r instanceof ParameterOperand);
        assertEquals(((ParameterOperand) r).getName(), "?1");
    }

    /**
     * test meta-information of the <code>FindByNumero</code> method
     */
    public void testAnnuaireQueryFindByNumero() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        EntityJdbcCmp2Desc bd = (EntityJdbcCmp2Desc) dd.getBeanDesc("PersonneEC2");
        MethodCmp2Desc md = (MethodCmp2Desc) AssertDeploymentHelper.getMethodDesc(bd, "findByNumero", null);
        assertEquals("findByNumero query", "SELECT OBJECT(o) FROM Personne o WHERE o.numero = ?1", md.getQuery());
        EjbqlQueryTreeHolder qth = md.getQueryTreeHolder();
        // projected field is pname
        SelectProject sp = (SelectProject) qth.getQueryTree();
        PropagatedField f = (PropagatedField) sp.getField(Field.PNAMENAME);
        assertNotNull("Query result is pName", f);
        assertTrue("Query result type is reference", f.getType().getTypeCode() == PType.TYPECODE_REFERENCE);
        // project child is JormExtent
        QueryTree je = ((QueryNode) sp).getChildren()[0];
        assertTrue("JormExtent is Personne", ((ClassExtent) je).getMetaObject().getName().equals("Personne"));
        // filter
        Expression e = sp.getQueryFilter();
        assertTrue("Filter of type Equal ", e instanceof Equal);
        Expression l = ((Equal) e).getLeftExpression();
        assertTrue(l instanceof BasicFieldOperand);
        assertEquals(((BasicFieldOperand) l).getField().getName(), "numero");
        Expression r = ((Equal) e).getRightExpression();
        assertTrue(r instanceof ParameterOperand);
        assertEquals(((ParameterOperand) r).getName(), "?1");
    }

}
