/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.api;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jonas.jtests.beans.relatives.RelativeLocal;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.type.api.PType;

import java.util.Iterator;
import java.util.Set;

/**
 * Test case based on the <code>relatives</code> EJB module.
 * This modules include one-to-one, one=to-many, and many-to-many relationships.
 * @author Christophe Ney - cney@batisseurs.com
 */
public class TestRelativesDeployment extends TestCase {

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * create a suite with all test methods of this class
     */
    public static Test suite() {
        return new TestSuite(TestRelativesDeployment.class);
    }

    /**
     * constructor called before running each testMethod (except when serialisation is involved)
     */
    public TestRelativesDeployment(String n) {
        super(n);
    }

    /**
     * method called before running each testMethod
     */
    protected void setUp() throws Exception {
        super.setUp();
        AssertDeploymentHelper.setJarName("relatives.jar");
    }

    /**
     * method called after running each testMethod
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * test CMP2 properties of JOnAS meta-info
     */
    public void testRelativesDeploymentDesc() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        // check CMP2 detected
        assertTrue("DeploymentDesc instanceof DeploymentCmp2Desc", dd instanceof DeploymentCmp2Desc);
        // check bean found
        BeanDesc bd = dd.getBeanDesc("RelativeEC2");
        assertNotNull("RelativeEC2 bean descriptor", bd);
        assertTrue("RelativeEC2 bean of type EntityJdbcCmp2Desc", bd instanceof EntityJdbcCmp2Desc);
        // check relationships
        Iterator i = ((EntityCmp2Desc) bd).getEjbRelationshipRoleDescIterator();
        // is-spouse-of
        EjbRelationshipRoleJdbcDesc errd = (EjbRelationshipRoleJdbcDesc) i.next();
        assertEquals("spouse", errd.getCmrFieldName());
        assertEquals("spouse type", RelativeLocal.class, errd.getCmrFieldType());
        assertEquals(errd.getLinkedEntityCmp2Desc().getEjbName(), "RelativeEC2");
        assertTrue("one", !errd.isRelationMultiple());
        assertEquals("key column for foreign key c_spouse", "c_spouse", errd.getForeignKeyJdbcName("c_name"));
        // has-for-spouse
        errd = (EjbRelationshipRoleJdbcDesc) i.next();
        assertTrue(!errd.hasCmrField());
        assertEquals("RelativeEC2", errd.getLinkedEntityCmp2Desc().getEjbName());
        assertTrue("one", !errd.isRelationMultiple());
        // has-for-sibblings
        errd = (EjbRelationshipRoleJdbcDesc) i.next();
        assertEquals("sibblings", errd.getCmrFieldName());
        assertEquals("sibblings type", Set.class, errd.getCmrFieldType());
        assertEquals("RelativeEC2", errd.getLinkedEntityCmp2Desc().getEjbName());
        assertTrue("many", errd.isRelationMultiple());
        assertEquals("key column for foreign key c_sibblings", "c_sibblings", errd.getForeignKeyJdbcName("c_name"));
        // in-sibbling-set
        errd = (EjbRelationshipRoleJdbcDesc) i.next();
        assertTrue(!errd.hasCmrField());
        assertEquals("RelativeEC2", errd.getLinkedEntityCmp2Desc().getEjbName());
        assertTrue("one", !errd.isRelationMultiple());
        // has-visited
        errd = (EjbRelationshipRoleJdbcDesc) i.next();
        assertEquals("visitedRelatives", errd.getCmrFieldName());
        assertEquals("visitedRelatives type", Set.class, errd.getCmrFieldType());
        assertEquals("RelativeEC2", errd.getLinkedEntityCmp2Desc().getEjbName());
        assertTrue("many", errd.isRelationMultiple());
        assertEquals("key column for foreign key c_visited", "c_visited", errd.getForeignKeyJdbcName("c_name"));
        // has-been-visited-by
        errd = (EjbRelationshipRoleJdbcDesc) i.next();
        assertTrue(!errd.hasCmrField());
        assertEquals("RelativeEC2", errd.getLinkedEntityCmp2Desc().getEjbName());
        assertTrue("many", errd.isRelationMultiple());
    }


    /**
     * test the struture of the generated JORM schema
     */
    public void testRelativesJormSchema() throws Exception {
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        EntityJdbcCmp2Desc bd = (EntityJdbcCmp2Desc) dd.getBeanDesc("RelativeEC2");
        Manager jm = dd.getManager();
        // find jorm class
        Class relative = jm.getClass("Relative");
        assertNotNull("Class Relative", relative);
        // check schema
        assertEquals("Schema", ((Schema) relative.getParent()).getName(), ((EntityCmp2Desc) bd).getJormSchemaName());
        // check fields
        TypedElement te = relative.getTypedElement("fullName");
        assertEquals("Field fullName type ", te.getType().getTypeCode(), PType.TYPECODE_STRING);
        te = relative.getTypedElement("birthdate");
        assertEquals("Field birthdate type ", te.getType().getTypeCode(), PType.TYPECODE_DATE);
        te = relative.getTypedElement("luckyNumber");
        assertEquals("Field luckyNumber type ", te.getType().getTypeCode(), PType.TYPECODE_INT);
        //check reference
        te = relative.getTypedElement("spouse");
        assertEquals("Field spouse type ", te.getType().getTypeCode(), PType.TYPECODE_REFERENCE);
        assertTrue("spouse instanceof ClassRef", te instanceof ClassRef);
        assertEquals("Relative", ((ClassRef) te).getClassName());
        te = relative.getTypedElement("sibblings");
        assertEquals("Field sibblings type ", te.getType().getTypeCode(), PType.TYPECODE_REFERENCE);
        assertTrue("sibblings instanceof GenClassRef", te instanceof GenClassRef);
        assertEquals("GenClass named by cmr-field", "sibblings", ((GenClassRef) te).getGenClass().getName());
        te = relative.getTypedElement("visitedRelatives");
        assertTrue("visitedRelatives instanceof GenClassRef", te instanceof GenClassRef);
        assertEquals("Field visitedRelatives type ", te.getType().getTypeCode(), PType.TYPECODE_REFERENCE);
        assertEquals("GenClass named by cmr-field", "visitedRelatives", ((GenClassRef) te).getGenClass().getName());
    }

}
