/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.api;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

/**
 * Test case based on the <code>relatives</code> EJB module.
 * This modules include one-to-one, one=to-many, and many-to-many relationships.
 * The goal of this test is to have a good coverage of EJBQL queries on this
 * specific example.
 * @author Christophe Ney - cney@batisseurs.com
 */
public class TestRelativesQueries extends TestCase {

    MethodCmp2Desc md = null;

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * create a suite with all test methods of this class
     */
    public static Test suite() {
        return new TestSuite(TestRelativesQueries.class);
    }

    /**
     * constructor called before running each testMethod (except when serialisation is involved)
     */
    public TestRelativesQueries(String n) {
        super(n);
    }

    /**
     * method called before running each testMethod
     */
    protected void setUp() throws Exception {
        super.setUp();
        AssertDeploymentHelper.setJarName("relatives.jar");
        DeploymentCmp2Desc dd = (DeploymentCmp2Desc) AssertDeploymentHelper.getDeploymentDesc();
        EntityCmp2Desc bd = (EntityCmp2Desc) dd.getBeanDesc("RelativeEC2");
        md = (MethodCmp2Desc) AssertDeploymentHelper.getMethodDesc(bd, "findFromGivenQuery", null);
    }

    /**
     * method called after running each testMethod
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * FROM clause. EJB2_0 section 11.2.6.3
     */
    public void testEJB_2_0_11_2_6_3() throws Exception {
        md.setQuery("SELECT OBJECT(c2) FROM Relative c1, Relative AS c2");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * FROM clause. EJB2_0 section 11.2.6.4
     */
    public void testEJB_2_0_11_2_6_4() throws Exception {
        md.setQuery("SELECT OBJECT(s) FROM Relative c, IN(c.sibblings) s");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.1
     */
    public void testEJB_2_0_11_2_7_1_exact_numeric() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.luckyNumber>-10 and c.luckyNumber<13");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.1
     */
    public void testEJB_2_0_11_2_7_1_boolean() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.isMale = true");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.1
     */
    public void testEJB_2_0_11_2_7_1_approximate_numeric() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.averageAnnualVisits>+1.5E1");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.1
     */
    public void testEJB_2_0_11_2_7_1_string() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.fullName = 'Jean Dupont'");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.4
     */
    public void testEJB_2_0_11_2_7_4() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.fullName = ?1 and c.luckyNumber=?2");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.6
     */
    public void testEJB_2_0_11_2_7_6() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE (((c.luckyNumber<>13) AND (c.averageAnnualVisits<4)) OR ((c.isMale=TRUE) AND (c.averageAnnualVisits>7)))");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.7
     */
    public void testEJB_2_0_11_2_7_7() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.luckyNumber BETWEEN 2 AND 4");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.8
     */
    public void testEJB_2_0_11_2_7_8() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.fullName IN('Jean Dupont','Pierre Durand')");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.9
     */
    public void testEJB_2_0_11_2_7_9() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.fullName LIKE '% Du%'");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.10
     */
    public void testEJB_2_0_11_2_7_10() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.spouse IS NOT NULL");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.11
     */
    public void testEJB_2_0_11_2_7_11() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.visitedRelatives IS EMPTY");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.12
     */
    public void testEJB_2_0_11_2_7_12() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE ?1 MEMBER OF c.sibblings");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.13
     */
    public void testEJB_2_0_11_2_7_13_concat() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE c.fullName = CONCAT(?1,' Dupont')");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.13
     */
    public void testEJB_2_0_11_2_7_13_substring() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE SUBSTRING(c.fullName,0,2)=?1");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.13
     */
    public void testEJB_2_0_11_2_7_13_locate() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE LOCATE(c.fullName,' ')=6");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.13
     */
    public void testEJB_2_0_11_2_7_13_length() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE LENGTH(c.fullName)=11");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.13
     */
    public void testEJB_2_0_11_2_7_13_abs() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE ABS(c.luckyNumber-13)>=0");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * WHERE clause. EJB2_0 section 11.2.7.13
     */
    public void testEJB_2_0_11_2_7_13_sqrt() throws Exception {
        md.setQuery("SELECT OBJECT(c) FROM Relative c WHERE SQRT(c.averageAnnualVisits)>2.25");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * SELECT clause. EJB2_0 section 11.2.8
     */
    public void testEJB_2_0_11_2_8_single_range() throws Exception {
        // single range
        md.setQuery("SELECT OBJECT(c) FROM Relative c");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * SELECT clause. EJB2_0 section 11.2.8
     */
    public void testEJB_2_0_11_2_8_single_path_cmp() throws Exception {
        md.setQuery("SELECT c.fullName FROM Relative c");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * SELECT clause. EJB2_0 section 11.2.8
     */
    public void testEJB_2_0_11_2_8_single_path_cmr() throws Exception {
        md.setQuery("SELECT c.spouse FROM Relative c");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * SELECT clause. EJB2_0 section 11.2.8
     */
    public void testEJB_2_0_11_2_8_single_path_recursive() throws Exception {
        md.setQuery("SELECT c.spouse.fullName FROM Relative c");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * SELECT clause. EJB2_0 section 11.2.8
     */
    public void testEJB_2_0_11_2_8_single_range_distinct() throws Exception {
        md.setQuery("SELECT DISTINCT OBJECT(c) FROM Relative c");
        md.getQueryTreeHolder().getQueryTree();
    }

    /**
     * SELECT clause. EJB2_0 section 11.2.8
     */
    public void testEJB_2_0_11_2_8_single_path_distinct() throws Exception {
        md.setQuery("SELECT DISTINCT c.luckyNumber FROM Relative c");
        md.getQueryTreeHolder().getQueryTree();
    }


}
