/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.deployment.ejbql;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;


/**
 * Test case of the lexical parser on a wide range of EJBQL query.
 * @author  Christophe Ney - cney@batisseurs.com
 */
public class TestQueryParser extends TestCase {

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * create a suite with all test methods of this class
     */
    public static Test suite() {
        return new TestSuite(TestQueryParser.class);
    }

    /**
     * constructor called before running each testMethod (except when serialisation is involved)
     */
    public TestQueryParser(String n) {
        super(n);
    }

    /**
     * method called before running each testMethod
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /**
     * method called after running each testMethod
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * parse and do as much as possible of validation
     */
    private void parse(String query) throws Exception {
        AssertQueryHelper.checkParserFor(query);
    }

    public void test1Petstore_1_3_1() throws Exception {
        parse("Select OBJECT(a) From Inventory a");
    }

    public void test2Petstore_1_3_1() throws Exception {
        parse("Select a.b From Inventory a");
    }

    public void test3Petstore_1_3_1() throws Exception {
        parse("Select a.b.c From Inventory a");
    }

    public void test4Petstore_1_3_1() throws Exception {
        parse("SELECT DISTINCT OBJECT(a) FROM Manager a WHERE a.status = ?1");
    }

    public void test5Petstore_1_3_1() throws Exception {
        parse("Select OBJECT(a) From Customer a");
    }

    public void test6Petstore_1_3_1() throws Exception {
        parse("SELECT DISTINCT OBJECT(a) FROM SupplierOrder a WHERE a.poStatus = ?1");
    }

    public void test7Petstore_1_3_1() throws Exception {
        parse("Select OBJECT(a) From User a");
    }

    public void test8Petstore_1_3_1() throws Exception {
        parse("SELECT OBJECT(a) From PurchaseOrder a WHERE a.poDate BETWEEN ?1 AND ?2");
    }

    public void test1OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.lastName = ?1 AND c.firstName = ?2");
    }

    public void test2OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.hasGoodCredit = TRUE");
    }

    public void test3OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.homeAddress.city = ?1 AND c.homeAddress.state = ?2");
    }

    public void test4OReillyExamples() throws Exception {
        parse("SELECT a.zip FROM Address AS a  WHERE a.state = ?1");
    }

    public void test5OReillyExamples() throws Exception {
        parse("SELECT OBJECT(a) FROM Address AS a");
    }

    public void test6OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer AS c WHERE c.homeAddress = ?1");
    }

    public void test7OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Cruise c WHERE c.name = ?1");
    }

    public void test8OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Cabin as c WHERE c.deckLevel = ?1");
    }

    public void test9OReillyExamples() throws Exception {
        parse("SELECT OBJECT(cab) FROM Customer AS cust, IN(cust.reservations) AS res, IN(res.cabins) AS cab WHERE cust = ?1");
    }

    public void test10OReillyExamples() throws Exception {
        parse(" SELECT OBJECT(c) FROM Customer c  WHERE c.lastName LIKE ?1 AND c.firstName LIKE ?2");
    }

    public void test11OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.lastName LIKE ?1 AND c.firstName LIKE ?2 AND c.homeAddress.state = ?3");
    }

    public void test12OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c  WHERE c.homeAddress.state IN ('FL','TX','AZ','CA')");
    }

    public void test13OReillyExamples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.reservations IS EMPTY");
    }

    public void test14OReillyExamples() throws Exception {
        parse("SELECT OBJECT(cust) FROM Customer cust, Cruise cr WHERE cr = ?1 AND cust MEMBER OF cr.reservations.customers");
    }

    public void test15OReillyExamples() throws Exception {
        // this is a 2.1 feature
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.homeAddress.state = ?1 ORDERBY c.lastName,c.firstName");
    }

    public void test16OReillyExamples() throws Exception {
        parse("SELECT OBJECT(s) FROM Ship s WHERE s.tonnage = ?1");
    }

    public void test17OReillyExamples() throws Exception {
        parse("SELECT OBJECT(s) FROM Ship s WHERE s.tonnage BETWEEN ?1 AND ?2");
    }

    public void test1WebLogicExamples() throws Exception {
        parse("SELECT OBJECT(a) FROM AccountBean AS a WHERE a.balance > ?1");
    }

    public void test2WebLogicExamples() throws Exception {
        parse("SELECT OBJECT(a) FROM AccountBean AS a WHERE a.balance = ?1");
    }

    public void test3WebLogicExamples() throws Exception {
        parse("SELECT OBJECT(a) FROM AccountBean AS a WHERE a.accountType IS NULL");
    }

    public void test1IBMTutorials() throws Exception {
        parse("select Object(theUser) from UserBean as theUser");
    }

    public void test2IBMTutorials() throws Exception {
        parse("select Object(o) from UserBean as o");
    }

    public void test3IBMTutorials() throws Exception {
        parse("select Object(o) from RoleBean o");
    }

    public void test4IBMTutorials() throws Exception {
        parse("select Object(group) from GroupBean group");
    }

    public void test1EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order AS o, IN(o.lineItems) AS l WHERE l.shipped = FALSE");
    }

    public void test2EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE l.product.product_type = 'office_supplies'");
    }

    public void test3EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE l.product.product_type = 'office_supplies'");
    }

    public void test4EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order AS o, IN(o.lineItems) l WHERE l.product.name = 'widget'");
    }

    public void test5EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE o.lineItems IS EMPTY");
    }

    public void test6EJB2_0_specifications_examples() throws Exception {
        parse("SELECT l.product FROM Order AS o, IN(o.lineItems) l");
    }

    public void test7EJB2_0_specifications_examples() throws Exception {
        parse("SELECT o.lineItems FROM Order AS o");
    }

    public void test8EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o1) FROM Order o1, Order o2 WHERE o1.quantity > o2.quantity AND o2.customer.lastname = 'Smith' AND o2.customer.firstname= 'John'");
    }

    public void test9EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(l) FROM Order o, IN(o.lineItems) l");
    }

    public void test10EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(l) FROM LineItems AS l");
    }

    public void test11EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o");
    }

    public void test12EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o WHERE o.shipping_address.state = 'CA'");
    }

    public void test13EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT o.shipping_address.state FROM Order o");
    }

    public void test14EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order o, IN(o.lineItems) l");
    }

    public void test15EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o WHERE o.lineItems IS NOT EMPTY");
    }

    public void test16EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o WHERE o.lineItems IS EMPTY");
    }

    public void test17EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE l.shipped = FALSE");
    }

    public void test18EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o WHERE NOT (o.shipping_address.state = o.billing_address.state AND o.shipping_address.city = o.billing_address.city AND o.shipping_address.street = o.billing_address.street)");
    }

    public void test19EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o WHERE o.shipping_address <> o.billing_address");
    }

    public void test20EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE l.product.type = 'book' AND l.product.name = 'Applying Enterprise JavaBeans: Component-Based Development for the J2EE Platform'");
    }

    public void test21EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE l.product.name = ?1");
    }

    public void test22EJB2_0_specifications_examples() throws Exception {
        parse("SELECT l.product.name FROM Order o, IN(o.lineItems) l WHERE o.ordernumber = ?1");
    }

    public void test23EJB2_0_specifications_examples() throws Exception {
        parse("SELECT o.shipping_address.city FROM Order o");
    }

    public void test24EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order AS o, IN(o.lineItems) l, Product p");
    }

    public void test25EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o, IN(o.lineItems) l WHERE l.quantity > 5");
    }

    public void test26EJB2_0_specifications_examples() throws Exception {
        parse("SELECT DISTINCT o.OKEY FROM ORDERBEAN o, LINEITEM l WHERE o.OKEY = l.FKEY AND l.QUANTITY > 5");
    }

    public void test27EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(l) FROM LineItems l WHERE l.shipped <> TRUE");
    }

    public void test28EJB2_0_specifications_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Order o WHERE o.amount > 1000");
    }

    public void test1Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Personne o WHERE o.nom = ?1");
    }

    public void test2Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Personne o WHERE o.numero = ?1");
    }

    public void test3Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Personne o");
    }

    public void test4Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Account o WHERE o.number = ?1");
    }

    public void test5Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Account o WHERE o.customer = ?1");
    }

    public void test6Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Account o WHERE o.balance > ?1");
    }

    public void test7Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Account o");
    }

    public void test8Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Simple o WHERE o.testName = ?1");
    }

    public void test9Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Simple o WHERE o.numTest = ?1");
    }

    public void test10Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Simple o WHERE o.numTest = 8");
    }

    public void test11Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Simple o WHERE (o.info BETWEEN ?1 AND ?2) AND (o.numTest BETWEEN ?1 AND ?2)");
    }

    public void test12Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Simple o WHERE o.info = ?1 AND numTest = ?1");
    }

    public void test13Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Simple o WHERE o.info = ?1 AND numTest = ?2");
    }

    public void test14Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Account o WHERE o.number = ?1");
    }

    public void test15Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Account o");
    }

    public void test16Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Person o WHERE o.number = ?1");
    }

    public void test17Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Person o WHERE o.name = ?1");
    }

    public void test18Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM Person o");
    }

    public void test19Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM SimpleE o WHERE o.customer = 'NotSupported'");
    }

    public void test20Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM SimpleE o WHERE o.customer = 'Required'");
    }

    public void test21Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM SimpleE o WHERE o.customer = 'Supports'");
    }

    public void test22Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM SimpleE o WHERE o.customer = 'RequiresNew'");
    }

    public void test23Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM SimpleE o WHERE o.customer = 'Mandatory'");
    }

    public void test24Jonas_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM SimpleE o WHERE o.customer = 'Never'");
    }

    public void test1JBOSS_3_0_1_examples() throws Exception {
        parse("select object(U) from user u where U.userName = ?1");
    }

    public void test2JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(u) FROM user U");
    }

    public void test3JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c");
    }

    public void test4JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(c) FROM Customer c WHERE c.name = ?1");
    }

    public void test5JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(a) FROM Customer c, IN (c.addresses) AS a WHERE c.id = ?1 AND a.state='CA'");
    }

    public void test6JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(a) FROM Customer c, IN (c.addresses) AS a WHERE c.id = ?1 AND a.state='CA'");
    }

    public void test7JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM OrderX o");
    }

    public void test8JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM OrderX o WHERE o.orderStatus = ?1");
    }

    public void test9JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(o) FROM OrderX o WHERE o.shippingAddress.state = 'CA'");
    }

    public void test10JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT O.shippingAddress.state FROM OrderX o");
    }

    public void test11JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT o.shippingAddress.state FROM OrderX o WHERE o.shippingAddress.state = 'XXXXX'");
    }

    public void test12JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(a) FROM Address a WHERE a.state = 'CA'");
    }

    public void test13JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(a) FROM Address a WHERE a.state = 'CAXXXXX'");
    }

    public void test14JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(a) FROM Address a");
    }

    public void test15JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(l) FROM LineItem l");
    }

    public void test16JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(p) FROM Product p");
    }

    public void test17JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(p) FROM Product p WHERE p.name = ?1");
    }

    public void test18JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(pc) FROM ProductCategory pc");
    }

    public void test19JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(b) FROM B AS b WHERE b.a = ?1");
    }

    public void test20JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT DISTINCT OBJECT(a) FROM A AS a WHERE a.bs IS NOT EMPTY");
    }

    public void test21JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(b) FROM B AS b WHERE b.bool = TRUE");
    }

    public void test22JBOSS_3_0_1_examples() throws Exception {
        parse("SELECT OBJECT(b) FROM B AS b WHERE b.bool = FALSE");
    }


    public void test1Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c2) FROM Relative c1, Relative AS c2");
    }

    public void test2Relatives_queries() throws Exception {
        parse("SELECT OBJECT(s) FROM Relative c, IN(c.sibblings) s");
    }

    public void test3Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.luckyNumber>-10 and c.luckyNumber<13");
    }

    public void test4Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.isMale = true");
    }

    public void test5Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.averageAnnualVisits>+1.5E1");
    }

    public void test6Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.fullName = 'Jean Dupont'");
    }

    public void test7Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.fullName = ?1 and c.luckyNumber=?2");
    }

    public void test8Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE (((c.luckyNumber<>13) AND (c.averageAnnualVisits<4)) OR ((c.isMale=TRUE) AND (c.averageAnnualVisits>7)))");
    }

    public void test9Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.luckyNumber BETWEEN 2 AND 4");
    }

    public void test10Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.fullName IN('Jean Dupont','Pierre Durand')");
    }

    public void test11Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.fullName LIKE '% Du%'");
    }

    public void test12Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.spouse IS NOT NULL");
    }

    public void test13Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.visitedRelatives IS EMPTY");
    }

    public void test14Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE ?1 MEMBER OF c.sibblings");
    }

    public void test15Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE c.fullName = CONCAT(?1,' Dupont')");
    }

    public void test16Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE SUBSTRING(c.fullName,0,2)=?1");
    }

    public void test17Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE LOCATE(c.fullName,' ')=6");
    }

    public void test18Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE LENGTH(c.fullName)=11");
    }

    public void test19Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE ABS(c.luckyNumber-13+2)>=0");
    }

    public void test20Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c WHERE SQRT(c.averageAnnualVisits)>2.25");
    }

    public void test21Relatives_queries() throws Exception {
        parse("SELECT OBJECT(c) FROM Relative c");
    }

    public void test22Relatives_queries() throws Exception {
        parse("SELECT c.fullName FROM Relative c");
    }

    public void test23Relatives_queries() throws Exception {
        parse("SELECT c.spouse FROM Relative c");
    }

    public void test24Relatives_queries() throws Exception {
        parse("SELECT c.spouse.fullName FROM Relative c");
    }

    public void test25Relatives_queries() throws Exception {
        parse("SELECT DISTINCT OBJECT(c) FROM Relative c");
    }

    public void test26Relatives_queries() throws Exception {
        parse("SELECT DISTINCT c.luckyNumber FROM Relative c");
    }


}
