/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.stests.manac;

import java.rmi.RemoteException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.transaction.RollbackException;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.stests.util.JTestCase;

/**
 * set of testCases used to load the EJBServer
 */
public abstract class A_stress extends A_manac {

    public A_stress(String name) {
	super(name);
    }

    /*
     * no rollback, monothreaded
     */
    public void test_a8_d0_l1200_m1_t1() throws Exception {
	manac(8, 0, 1200, 1, 1, 1);
    }

    /**
     * 2 threads - many loops
     */
    public void test_a8_d0_l600_m1_t2() throws Exception {
	manac(8, 0, 600, 1, 2, 1);
    }

    /**
     * 3 threads - many loops
     */
    public void test_a8_d0_l400_m1_t3() throws Exception {
	manac(8, 0, 400, 1, 3, 1);
    }

    /**
     * 4 threads - many loops
     */
    public void test_a8_d0_l300_m1_t4() throws Exception {
	manac(8, 0, 300, 1, 4, 1);
    }

    /**
     * 6 threads - many loops
     */
    public void test_a8_d0_l200_m1_t6() throws Exception {
	manac(8, 0, 200, 1, 6, 1);
    }

    /*
     * no rollback, multithreaded
     */
    public void test_amax_d0_l100_m1_t10() throws Exception {
	manac(initAccounts, 0, 100, 1, 10, 1);
    }

    /*
     * rollback, monothreaded
     */
    public void test_a4_d0_l1000_m100_t1() throws Exception {
	manac(4, 0, 1000, 100, 1, 1);
    }

    /*
     * rollback, multithreaded
     */
    public void test_a4_d0_l100_m100_t5() throws Exception {
	manac(4, 0, 100, 100, 5, 1);
    }

    /*
     * no rollback, monothreaded with possible create
     */
    public void test_anew_d0_l1200_m1_t1() throws Exception {
	manac(maxAccounts, 0, 1200, 1, 1, 1);
    }

    /**
     * 2 threads - many loops with possible create
     */
    public void test_anew_d0_l600_m1_t2() throws Exception {
	manac(maxAccounts, 0, 600, 1, 2, 1);
    }

    /**
     * 3 threads - many loops with possible create
     */
    public void test_anew_d0_l400_m1_t3() throws Exception {
	manac(maxAccounts, 0, 400, 1, 3, 1);
    }

    /**
     * 4 threads - many loops with possible create
     */
    public void test_anew_d0_l300_m1_t4() throws Exception {
	manac(maxAccounts, 0, 300, 1, 4, 1);
    }

    /**
     * 6 threads - many loops with possible create
     */
    public void test_anew_d0_l200_m1_t6() throws Exception {
	manac(maxAccounts, 0, 200, 1, 6, 1);
    }

    /**
     * 6 threads - many loops with possible create and checking
     */
    public void test_anew_d0_l200_m1_t6_ck() throws Exception {
	manac(maxAccounts, 0, 200, 1, 6, 2);
    }

    /*
     * no rollback, multithreaded with possible create
     */
    public void test_anew_d0_l100_m1_t10() throws Exception {
	manac(maxAccounts, 0, 100, 1, 10, 1);
    }

    /**
     * many threads with possible create
     */
    public void test_anew_d0_l100_m5_t30() throws Exception {
	manac(maxAccounts, 0, 100, 5, 30, 1);
    }

    /**
     * many threads with possible create and checking
     */
    public void test_anew_d0_l100_m5_t30_ck() throws Exception {
	manac(maxAccounts, 0, 100, 5, 30, 2);
    }

}
