/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.api;

/**
 * The PException class defines the top level of the jorm exceptions.
 * @author	P. Dchamboux
 */
public class PException extends Exception {
	/**
	 * A DS-specific nested exception.
	 */
	private Exception nestedException = null;

	/**
	 * It builds an new Jorm Exception.
	 */
	public PException() {
		super("NO ERROR MESSAGE!!");
	} // constructor

	/**
	 * It builds an new Jorm Exception with an error message.
	 * @param error	The string representation of the error message.
	 */
	public PException(String error) {
		super(error);
	} // constructor

	public PException(Exception e, String error) {
		super(error);
		nestedException = e;
	} // constructor

	/**
	 * It return the DS-specific nested exception associated with this I/O
	 * JORM exception.
	 * @return	The involved nested exception.
	 */
	public Exception getNestedException() {
		return nestedException;
	}

	/**
	 * prints the error message to the console.
	 * TBD: to a specific channel with MonoLog...
	 */
	public void print() {
		System.err.println();
		System.err.println(getMessage());
		System.err.println();
	} // method print
}
