/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.comp.cmdline;

/**

 * This class displays some informations about the current Persistence

 * manager compiler.

 * @author	X. Spengler

 */
public class CommandLineInfo {
	/**

	 * the version number of the compiler.

	 */
	static String version = "0.1";

	/**

	 * Prints the version number of the compiler.

	 */
	static public void printVersion() {
		System.out.println("Jorm " + version);
	}

	/**

	 * Prints to the console the usage (how to use and options) of the compiler.

	 */
	static public void usage() {
		System.out.println("Usage: jorm [-options] " +
			"-mappername <mapper> -mappingname <mapping> -parser <parser> " +
			"-d <path> -mappingfactory factory <file> [file]...");

		System.out.println("\nwhere:");
		System.out.println(" -mappername mapper      basic name of the mapper");
		System.out.println(" -mappingname mapping    basic name of the mapping");
		System.out.println(" -parser parser          class name of the specific parser");
		System.out.println(" -d path                 path where the generated files will be");
		System.out.println(" -mappingfactory factory class name of the specific mapping factory");
		System.out.println(" file [file]...          files are xml files");
		System.out.println("\nwhere options include:");
		System.out.println(" -ext cmdlineparser      class name of the specific command line parser");
		System.out.println(" -classpath path[:path]  classpath used to find objects");
		System.out.println(" -v                      verbose mode");
		System.out.println(" -V                      print version number");
		System.out.println(" -h|--help|-help         print the current help");
		System.out.println(" -javac jCompiler        javac compiler used to compile generated sources");
		System.out.println(" -keepsrc                do not remove the sources after compilation process");
		System.out.println(" -nocompile              do not process the compilation chain");
		System.out.println(" -conf configfile        name of the jorm configuration file");
		System.out.println(" -noverify               meta-information is not checked");
		System.out.println(" -verifier verifier      class name of the verifier system");
        System.out.println(" -metainfo metainfo      class name of the meta information manager");
        System.out.println(" -nodtdverify            the dtd is not used to validate xml files");
        System.out.println(" -bindingAbstract        define Binding object as abstract");
	}
}











