/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.comp.cmdline.api;

/**
 * The CmdLineParser interface is used to create extension of the main command
 * line. This object is composed by two methods, the first one allows to parse
 * a list of argument starting from an index, and the second one, allows to
 * parse a property (string), and assigns it a value.
 * @author	X. Spengler
 */
public interface CmdLineParser{
	/**
	 * Parses options from a command line and returns the next index in the
	 * list of arguments when encounters an unknown option.
	 * This method is used by the CommandLineParser to parse unknows option
	 * which are in link with the current object.
	 *
	 * @param args  the list of arguments. This list contains known options for
	 *              the current object to the beginning of the list, and
	 *              unknown options to the end of the list.
	 * @param i     the current index to parse the args list
	 * @return      the new current index to continue the parsing
	 */
	int parseArgument(String args[], int i);

	/**
	 * Assigns to the elem tag, the corresponding value.
	 * This method is used by the CompilerParameter when the jorm.properties
	 * is read.
	 *
	 * @param elem  the element which is found in the jorm.properties file
	 * @param value the value of the corresponding property
	 */
	void parseProperty(String elem, String value);
}
