/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.comp.cmdline.rdb;

import org.objectweb.jorm.comp.cmdline.api.CmdLineParser;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.api.Debug;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class RdbMapperOption
        implements CmdLineParser, Loggable {

    protected Logger logger = null;
    protected LoggerFactory loggerFactory = null;
    public String dataBaseName = null;

    public final static String SINGLE_CLASS_NAME = "RdbMapperOption";
    public final static String CLASS_NAME
            = "org.objectweb.jorm.comp.cmdline.rdb." + SINGLE_CLASS_NAME;
    public final static String DATABASE_OPTION_NAME = "DBName";
    public final static String DATABASE_OPTION_NAME_FULL
            = CLASS_NAME + "." + DATABASE_OPTION_NAME;
    public final static String DATABASE_OPTION_SINGLE_NAME =
            SINGLE_CLASS_NAME + "." + DATABASE_OPTION_NAME;


    public String getDataBaseName() {
        return dataBaseName;
    }

    public void setDataBaseName(String dataBaseName) {
        this.dataBaseName = dataBaseName;
    }

    // IMPLEMENTATION OF THE CmdLineParser INTERFACE //
    //-----------------------------------------------//

    /**
     * Parses options from a command line and returns the next index in the
     * list of arguments when encounters an unknown option.
     * This method is used by the CommandLineParser to parse unknows option
     * which are in link with the current object.
     *
     * @param args  the list of arguments. This list contains known options for
     *              the current object to the beginning of the list, and
     *              unknown options to the end of the list.
     * @param i     the current index to parse the args list
     * @return      the new current index to continue the parsing
     */
    public int parseArgument(String args[], int i) {
        if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
            if (args!=null)
                for (int j=0; j<args.length; j++)
                    if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
                        logger.log(BasicLevel.DEBUG, "args[" + j + "]:" + args[j]);

        if ((args.length>(i+1)) &&
                ((("-"+DATABASE_OPTION_NAME_FULL).equals("-"+args[i])) ||
                ((DATABASE_OPTION_NAME_FULL).equals(args[i])) ||
                (("-"+DATABASE_OPTION_SINGLE_NAME).equals(args[i])))) {
            dataBaseName = args[i+1];
            if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
                logger.log(BasicLevel.DEBUG, "Database name: " + dataBaseName);
        }
        return i+2;
    }


    /**
     * Assigns to the elem tag, the corresponding value.
     * This method is used by the CompilerParameter when the jorm.properties
     * is read.
     *
     * @param elem  the element which is found in the jorm.properties file
     * @param value the value of the corresponding property
     */
    public void parseProperty(String elem, String value) {
        if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
            logger.log(BasicLevel.DEBUG, "assign <"+value+"> to "+elem);
        if (elem.compareTo("DBName") == 0)
            dataBaseName = value;
    }

    // IMPLEMENTATION OF THE Loggable INTERFACE //
    //------------------------------------------//

    public Logger getLogger() {
        return logger;
    }

    public LoggerFactory getLoggerFactory() {
        return loggerFactory;
    }

    public void setLogger(Logger l) {
        logger = l;
    }

    public void setLoggerFactory(LoggerFactory lf) {
        loggerFactory = lf;
        if (logger==null)
            loggerFactory.getLogger(getClass().getName());
    }
}
