/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.comp.generator.file;

import org.apache.velocity.context.Context;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.generator.api.MOP;
import org.objectweb.jorm.comp.generator.lib.CommonHelper;
import org.objectweb.jorm.comp.generator.lib.ObjectGenerator;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.FieldName;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.util.io.api.Block;

import java.util.Iterator;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class FileBindingMOP extends CommonHelper implements MOP {

	protected static final String PACCESSOR_VAR_NAME = "paccessor";
	protected static final String PACCESSORGEN_VAR_NAME = "paccessorGen";

	// IMPLEMENTATION OF THE MOP INTERFACE //
	//-------------------------------------//

	public void initContext(Context ctx) {
		ctx.put("bindingMOPTools", this);
	}

	public String[] getTemplateLibraries() {
		String[] libs = {ObjectGenerator.TEMPLATE_DIR + "/file/FileBinding.vm"};
		return libs;
	}

	// ----- HELPER METHODS -----//
	//---------------------------//

//TODO

	// ---- OLD METHODS -----//
	//-----------------------//

	/**
	 * writes the definitions of all variables needed by
	 * a Binding object.
	 * @param block	The class definition block within which the variable
	 *			definitions are written.
	 */
	public void writeVariables(Block block, Class moc, CompilerParameter cp)
		throws Exception {
		if (containsCompositeName(moc, cp)) {
			block.writeLine("private ObjectInputStream ois = null;");
		}

	}

	/**
	 * This methods permits to specify initializations which must be done at the
	 * instanciation time.
	 * @param block	The class definition block within which the variable
	 *			definitions are written.
	 */
	public void writeConstructorBody(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * writes the code for the "write" header method needed by
	 * a Binding object.
	 * @param block	The block within which the code is written.
	 * @param moc 	The meta object which represents the jorm class
	 */
	public void writeWriteHeader(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * writes the code for the "write" tail method needed by
	 * a Binding object.
	 * @param block	The block within which the code is written.
	 * @param moc 	The meta object which represents the jorm class
	 */
	public void writeWriteTail(Block block, Class moc, CompilerParameter cp)
		throws Exception {
		block.openBlock("try");
		block.writeLine("((FileAccess)conn).writeObject("
			+ "pClassMapping.JORM_CLASS_NAME, "
			+ "pname.encodeString(), "
			+ "this, (Connection)conn);");
		block.closeBlock();
		block.openBlock("catch (Exception e)");
		block.writeLine("throw new PExceptionIO(e, "
			+ "\"Error has occured during the write operation\");");
		block.closeBlock();
	}

	/**
	 * To do nothing, because the code is common to the remove, create or update
	 * cases. Then the code is written by the writeWriteTail method.
	 */
	public void writeRemove(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * To do nothing, because the code is common to the remove, create or update
	 * cases. Then the code is written by the writeWriteTail method.
	 */
	public void writeCreate(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * To do nothing, because the code is common to the remove, create or update
	 * cases. Then the code is written by the writeWriteTail method.
	 */
	public void writeUpdate(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * No specific header
	 */
	public void writeReadHeader(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	public void writeReadBody(Block block, Class moc, CompilerParameter cp)
		throws Exception {
		block.openBlock("try");
		block.writeLine("((FileAccess)conn).readObject("
			+ "pClassMapping.JORM_CLASS_NAME, "
			+ "pname.encodeString(), "
			+ "this, (Connection)conn);");
		block.closeBlock();
		block.openBlock("catch (Exception e)");
		block.writeLine("throw new PExceptionIO(e, "
			+ "\"Error has occured during the read operation\");");
		block.closeBlock();
	}

	/**
	 * No specific tail
	 */
	public void writeReadTail(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * No specific header
	 */
	public void writeExistsHeader(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * writes the code for the "exists" method needed by
	 * a Binding object.
	 * @param block	The block within which the code is written.
	 */
	public void writeExistsBody(Block block, Class moc, CompilerParameter cp)
		throws Exception {
		block.openBlock("try");
		block.writeLine("return ((FileAccess)conn).exists("
			+ "pClassMapping.JORM_CLASS_NAME, "
			+ "pname.encodeString());");
		block.closeBlock();
		block.openBlock("catch (Exception e)");
		block.writeLine("throw new PExceptionIO(e, "
			+ "\"Error has occured during the exist operation\");");
		block.closeBlock();
	}

	/**
	 * No specific tail
	 */
	public void writeExistsTail(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * defines interfaces to implement by the current object.
	 * @return	A String tab representing the classes to implement. For
	 * 		binding object, FileReader and FileWriter are implemented.
	 */
	public String[] getImplements(Class c, CompilerParameter cp) throws Exception {
		boolean compositepname = containsCompositeName(c, cp);
		int nb = 2 + (compositepname ? 1 : 0);
		String[] res = new String[nb];
		res[0] = "FileReader";
		res[1] = "FileWriter";
		if (compositepname)
			res[2] = "PNameGetter";
		return res;
	}

	/**
	 * defines imports needed by the PBinding generation.
	 * @return	A String tab representing the class names needed by the
	 * generated object.
	 */
	public String[] getImportedClassName(Class c, CompilerParameter cp) throws Exception {
		boolean ref = this.containsReference(c);
		int nb = 6 + (ref ? 1 : 0);
		String[] res = new String[nb];
		res[0] = "org.objectweb.transaction.ra.file.FileAccess";
		res[1] = "org.objectweb.transaction.ra.file.FileReader";
		res[2] = "org.objectweb.transaction.ra.file.FileWriter";
		res[3] = "java.io.ObjectInputStream";
		res[4] = "java.io.ObjectOutputStream";
		res[5] = "javax.resource.cci.Connection";
		if (ref)
			res[6] = "org.objectweb.jorm.naming.api.PNamingContext";
		return res;
	}

	/**
	 * writes the code for other methods needed by
	 * a Binding object.
	 * @param block	The block within which the code is written.
	 */
	public void writeOtherMethods(Block block, Class moc, CompilerParameter cp)
		throws Exception {
		boolean containscompositepname = containsCompositeName(moc, cp);
		if (containscompositepname) {
			block.openBlock("private void setObjectInputStream(ObjectInputStream _ois)");
			block.writeLine("ois = _ois;");
			block.closeBlock();

			block.newLine();
			block.newLine();
			block.writeLine("// IMPLEMENTATION OF THE PNameGetter INTERFACE");

			//byte
			block.openBlock("public byte pngetByteField(String fn) throws PExceptionTyping");
			block.writeLine("return " + getReadCall("ois", PTypeSpace.BYTE) + ";");
			block.closeBlock();

			//char
			block.openBlock("public char pngetCharField(String fn) throws PExceptionTyping");
			block.writeLine("return " + getReadCall("ois", PTypeSpace.CHAR) + ";");
			block.closeBlock();

			//short
			block.openBlock("public short pngetShortField(String fn) throws PExceptionTyping");
			block.writeLine("return " + getReadCall("ois", PTypeSpace.SHORT) + ";");
			block.closeBlock();

			// int
			block.openBlock("public int pngetIntField(String fn) throws PExceptionTyping");
			block.writeLine("return " + getReadCall("ois", PTypeSpace.INT) + ";");
			block.closeBlock();

			// long
			block.openBlock("public long pngetLongField(String fn) throws PExceptionTyping");
			block.writeLine("return " + getReadCall("ois", PTypeSpace.LONG) + ";");
			block.closeBlock();

			// String
			block.openBlock("public String pngetStringField(String fn) throws PExceptionTyping");
			block.writeLine("return " + getReadCall("ois", PTypeSpace.STRING) + ";");
			block.closeBlock();
		}

		block.newLine();
		block.newLine();
		block.writeLine("// IMPLEMENTATION OF THE FileReader INTERFACE //");
		block.writeLine("//--------------------------------------------//");

		block.openBlock("public void readFile(ObjectInputStream ois) throws Exception");
		block.writeLine("throw new UnsupportedOperationException(\"A connection is needed !\");");
		block.closeBlock();

		block.openBlock("public void readFile(ObjectInputStream ois, Connection conn) throws Exception");
		NameDef nd = moc.getNameDef(cp.getMapperName(), cp.getMappingName());
		block.writeLine("PNamingContext refpnc;");
		for (int i = 0; i < 2; i++) {
			if (i == 0)
				block.openBlock("if (" + PACCESSOR_VAR_NAME + "!=null)");
			else
				block.openBlock("if (" + PACCESSORGEN_VAR_NAME + "!=null)");

			// Read the pname of the class
			block.writeLine("pname = " +
				writeReadName(nd, "pClassMapping", "ois", block, moc) + ";");

			// Read each field
			Iterator it = moc.iterateAllField();
			while (it.hasNext()) {
				TypedElement te = (TypedElement) it.next();
				if (te instanceof PrimitiveElement) {
					writeReadPrimitiveElement((PrimitiveElement) te, "ois",
						block, i == 0);
				}
				else if (te instanceof Reference) {
					Reference ref = (Reference) te;
					String fn = te.getName();
					// Fetch the naming context associated to the field
					block.writeLine("refpnc = pClassMapping.getPNamingContext(\"" + fn
						+ "\");");

					// Calculate the pname of the field
					NameDef fieldnd =
						ref.getNameDef(cp.getMapperName(), cp.getMappingName());
					String pnvalue =
						writeReadName(fieldnd, "refpnc", "ois", block, moc);

					// call the paSet method on the accessor to fix the pname
					block.openBlock("if (pClassMapping." + fn + "CodingOnly)");
					if (i == 0)
						block.writeLine(PACCESSOR_VAR_NAME + "."
							+ "paSet" + upperFL(ref.getName()) + "("
							+ pnvalue + ", conn);");
					else
						block.writeLine(PACCESSORGEN_VAR_NAME + "."
							+ getPaSetMethod(ref.getType()) + "("
							+ "\"" + ref.getName() + "\", " + pnvalue + ", conn);"
						);
					block.closeBlock();
					block.openBlock("else");
					if (i == 0)
						block.writeLine(PACCESSOR_VAR_NAME + "."
							+ "paSet" + upperFL(ref.getName()) + "("
							+ pnvalue + ".resolve(conn), conn);");
					else
						block.writeLine(PACCESSORGEN_VAR_NAME + "."
							+ getPaSetMethod(ref.getType()) + "("
							+ "\"" + ref.getName() + "\", " + pnvalue + ".resolve(conn), conn);");
					block.closeBlock();
				}
			}
			if (containscompositepname)
				block.writeLine("setObjectInputStream(null);");
			block.closeBlock();
		}
		block.closeBlock();

		block.writeLine("// IMPLEMENTATION OF THE FileWriter INTERFACE //");
		block.writeLine("//--------------------------------------------//");

		block.openBlock("public void writeFile(ObjectOutputStream oos) throws Exception");
		block.writeLine("throw new UnsupportedOperationException(\"A connection is needed !\");");
		block.closeBlock();

		block.openBlock("public void writeFile(ObjectOutputStream oos, Connection conn) throws Exception");
		block.writeLine("PNamingContext refpnc;");
		block.writeLine("PName refname;");
		nd = moc.getNameDef(cp.getMapperName(), cp.getMappingName());
		for (int i = 0; i < 2; i++) {
			if (i == 0)
				block.openBlock("if (" + PACCESSOR_VAR_NAME + "!=null)");
			else
				block.openBlock("if (" + PACCESSORGEN_VAR_NAME + "!=null)");

			// Write the pname of the object
			writeWriteName(nd, "pname", "oos", block, moc);

			// Read each field
			Iterator it = moc.iterateAllField();
			while (it.hasNext()) {
				TypedElement te = (TypedElement) it.next();
				if (te instanceof PrimitiveElement) {
					writeWritePrimitiveElement((PrimitiveElement) te, "oos",
						block, i == 0);
				}
				else if (te instanceof Reference) {
					Reference ref = (Reference) te;
					NameDef fieldnd =
						ref.getNameDef(cp.getMapperName(), cp.getMappingName());
					String fn = te.getName();
					block.writeLine("refpnc = pClassMapping.getPNamingContext(\"" + fn
						+ "\");");
					block.openBlock("if (pClassMapping." + fn + "CodingOnly)");
					if (i == 0)
						block.writeLine("\t"
							+ "refname = (PName) paccessor.paGet" + fn
							+ "(conn);");
					else
						block.writeLine("\t"
							+ "refname = (PName) paccessorGen."
							+ getPaGetMethod(te.getType())
							+ "(\"" + fn + "\", conn);");
					block.closeBlock();
					block.openBlock("else");
					if (i == 0)
						block.writeLine("\t"
							+ "refname = (PName) refpnc.export(conn, paccessor.paGet" + fn
							+ "(conn));");
					else
						block.writeLine("\t"
							+ "refname = (PName) refpnc.export(conn, paccessorGen."
							+ getPaGetMethod(te.getType())
							+ "(\"" + fn + "\", conn));");
					block.closeBlock();

					writeWriteName(fieldnd, "refname", "oos", block, moc);
				}
			}
			block.closeBlock();
		}
		block.closeBlock();
	}

	private String writeReadName(NameDef nd,
								 String pncvarname,
								 String oisvarname,
								 Block block,
								 Class moc
								 ) throws Exception {

		if ((nd.getFieldRefNumber() + nd.getHiddenFieldNumber()) > 1) {
			// The pname is composed of several fields
			block.writeLine("setObjectInputStream(" + oisvarname + ");");
			return pncvarname + ".decodeAbstract((PNameGetter) this);";
		}
		else {
			// The pname is composed of only one field
			Object o = nd.iterateField().next();
			TypedElement te = null;
			if (o instanceof FieldName)
				te = moc.getTypedElement(((FieldName) o).getField());
			else if (o instanceof PrimitiveElement)
				te = (TypedElement) o;
			return pncvarname + "." + getPNameDecodeFunction(te.getType())
				+ "(" + this.getReadCall(oisvarname, te.getType()) + ")";
		}
	}

	private void writeWriteName(NameDef nd,
								String pnvarname,
								String oosvarname,
								Block block,
								Class moc
								) throws Exception {

		boolean multiplefields =
			(nd.getFieldRefNumber() + nd.getHiddenFieldNumber()) > 1;
		if (multiplefields) {
			// The pname is composed of several fields
			block.writeLine("PNameGetter " + pnvarname + "Png = "
				+ pnvarname + ".encodeAbstract();");
		}
		Iterator it = nd.iterateField();
		while (it.hasNext()) {
			Object o = it.next();
			TypedElement te = null;
			if (o instanceof FieldName)
				te = moc.getTypedElement(((FieldName) o).getField());
			else if (o instanceof PrimitiveElement)
				te = (TypedElement) o;

			String value = pnvarname;
			if (multiplefields)
				value += "Png." + getPNameGetterGetFunction(te.getType())
					+ "(\"" + te.getName() + "\")";
			else
				value += "." + getPNameEncodeFunction(te.getType()) + "()";

			block.writeLine(getWriteCall(oosvarname, te.getType(), value) + ";");
		}
	}

	private void writeReadPrimitiveElement(PrimitiveElement pe,
										   String oisvarname,
										   Block block,
										   boolean specificaccessor) throws Exception {
		if (specificaccessor)
			block.writeLine(PACCESSOR_VAR_NAME + "."
				+ "paSet" + upperFL(pe.getName()) + "("
				+ getReadCall(oisvarname, pe.getType()) + ");");
		else
			block.writeLine(PACCESSORGEN_VAR_NAME + "."
				+ getPaSetMethod(pe.getType()) + "("
				+ "\"" + pe.getName() + "\", "
				+ getReadCall(oisvarname, pe.getType()) + ");");
	}

	private void writeWritePrimitiveElement(PrimitiveElement pe,
											String oosvarname,
											Block block,
											boolean specificaccessor) throws Exception {
		if (specificaccessor)
			block.writeLine(getWriteCall(oosvarname, pe.getType(),
				PACCESSOR_VAR_NAME
				+ ".paGet" + upperFL(pe.getName()) + "()"
			)
				+ ";"
			);
		else
			block.writeLine(getWriteCall(oosvarname, pe.getType(),
				PACCESSORGEN_VAR_NAME + "." + getPaGetMethod(pe.getType())
				+ "(\"" + pe.getName() + "\")"
			)
				+ ";"
			);
	}

	private String getReadCall(String varname, PType pt) throws Exception {
		if (pt == null
			|| varname == null || varname.length() == 0)
			throw new Exception("Invalid parameter is null");

		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return varname + ".readBoolean()";
		case PType.TYPECODE_BYTE:
			return varname + ".readByte()";
		case PType.TYPECODE_CHAR:
			return varname + ".readChar()";
		case PType.TYPECODE_SHORT:
			return varname + ".readShort()";
		case PType.TYPECODE_INT:
			return varname + ".readInt()";
		case PType.TYPECODE_LONG:
			return varname + ".readLong()";
		case PType.TYPECODE_FLOAT:
			return varname + ".readFloat()";
		case PType.TYPECODE_DOUBLE:
			return varname + ".readDouble()";
		case PType.TYPECODE_SERIALIZED:
			return "(java.io.Serializable) " + varname + ".readObject()";
		case PType.TYPECODE_DATE:
			return "(java.util.Date) " + varname + ".readObject()";
		case PType.TYPECODE_STRING:
			return "(String) " + varname + ".readObject()";
		default:
			throw new Exception("Not authorized PType : " + pt.getTypeCode());
		}
	}

	private String getWriteCall(String varname, PType pt, String param)
		throws Exception {
		if (pt == null
			|| varname == null || varname.length() == 0
			|| param == null || param.length() == 0)
			throw new Exception("Invalid parameter is null");

		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return varname + ".writeBoolean(" + param + ")";
		case PType.TYPECODE_BYTE:
			return varname + ".writeByte(" + param + ")";
		case PType.TYPECODE_CHAR:
			return varname + ".writeChar(" + param + ")";
		case PType.TYPECODE_SHORT:
			return varname + ".writeShort(" + param + ")";
		case PType.TYPECODE_INT:
			return varname + ".writeInt(" + param + ")";
		case PType.TYPECODE_LONG:
			return varname + ".writeLong(" + param + ")";
		case PType.TYPECODE_FLOAT:
			return varname + ".writeFloat(" + param + ")";
		case PType.TYPECODE_DOUBLE:
			return varname + ".writeDouble(" + param + ")";
		case PType.TYPECODE_SERIALIZED:
		case PType.TYPECODE_DATE:
		case PType.TYPECODE_STRING:
			return varname + ".writeObject(" + param + ")";
		default:
			throw new Exception("Not authorized PType : " + pt.getTypeCode());
		}
	}
}
