/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.comp.generator.file;

import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.generator.api.MOP;
import org.objectweb.jorm.comp.generator.lib.CommonHelper;
import org.objectweb.jorm.comp.generator.lib.ObjectGenerator;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.io.api.Block;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class FileMappingMOP extends CommonHelper implements MOP {

	// IMPLEMENTATION OF THE MOP INTERFACE //
	//-------------------------------------//

	public void initContext(Context ctx) {
		ctx.put("mappingMOPTools", this);
	}

	public String[] getTemplateLibraries() {
		String[] libs = {ObjectGenerator.TEMPLATE_DIR + "/file/FileMapping.vm"};
		return libs;
	}

	// ------ HELPER METHODS -------//
	//------------------------------//

//TODO

	// ---- OLD METHODS -----//
	//-----------------------//

	/**
	 * writes the definitions of all variables needed by
	 * a Binding object.
	 * @param block	The class definition block within which the variable
	 *			definitions are written.
	 */
	public void writeVariables(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * @param block	The class definition block within which the variable
	 *			definitions are written.
	 */
	public void writeConstructorBody(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * writes the body of method  initStruct(Object conn)
	 * that will be used to build the persistence support for the object in the
	 * object store.
	 * @param block	The block within which the method's body has to be
	 * written.
	 * @param moc 	The meta object which represents a jorm class
	 */
	public void writeinitMappingStructures(Block block, Class moc, CompilerParameter cp)
		throws Exception {
		block.openBlock("try");
		block.writeLine("((FileAccess)conn).createDbDir(JORM_CLASS_NAME);");
		block.closeBlock();
		block.openBlock("catch (Exception e)");
		block.writeLine("throw new PExceptionIO(e, \"Error has occured during"
			+ "the initialisation of the mapping structure\");");
		block.closeBlock();
	}

	/**
	 * writes the body of the removeStruct(Object conn) that will be used to
	 * remove the persistent support  for the object int the data storage.
	 * @param block	The block within which the method's body has to be
	 * written.
	 * @param moc 	The meta object which represents a jorm class
	 */
	public void writeRemoveMappingStructures(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * defines interfaces to implement by the generated mapping object.
	 * @return	A String tab representing the classes to implement. For
	 * 		mapping object, FileReader and FileWriter are implemented.
	 */
	public String[] getImplements(Class c, CompilerParameter cp) throws Exception {
		return new String[0];
	}

	/**
	 * defines imports needed by the PMapping generation.
	 * @return	A String tab representing the class names needed by the
	 * generated object.
	 */
	public String[] getImportedClassName(Class c, CompilerParameter cp) throws Exception {
		boolean ref2gc = this.containsGenClassRef(c);
		int nb = 2 + (ref2gc ? 1 : 0);
		String[] res = new String[nb];
		res[0] = "org.objectweb.transaction.ra.file.FileAccess";
		res[1] = "org.objectweb.jorm.type.api.PTypeSpace";
		if (ref2gc)
			res[2] = "org.objectweb.jorm.runtime.mapper.file.genclass.api.FileGenClassProp";
		return res;
	}

	/**
	 * This method generates the setting of the @varName varible with a Mapper
	 * specific FieldDesc either a PrimitiveElement.
	 * The mapping of a primitive element depends on a mapper. The extension
	 * of the FieldDesc class contains specific fields to the mapper.
	 *
	 * @block the block where the code must be generated
	 * @pe the primitive element
	 * @varName is the string of the variable name which must be set
	 * @cp the compiler parameter should permits to select the right
	 * ValueMapping linked to the primitiveElement
	 *
	 * Exception is raised if an error occurs during the generation
	 */
	public void writeFieldDescFilling(PrimitiveElement pe,
									  String varName,
									  Block block,
									  CompilerParameter cp) throws Exception {
		block.writeLine(varName + " = new FieldDesc(\"" + pe.getName() + "\", "
			+ ptype2String(pe.getType()) + ");");
	}

	/**
	 * This method generates the setting of the @varName with a PType equals to
	 * PTypeSpace.REFTOP. This method depends on a mapper because the constructor
	 * of the FieldDesc extension is only known by the MOP objects.
	 *
	 * @block the block where the code must be generated
	 * @varName is the string of the variable name which must be set
	 *
	 * Exception is raised if an error occurs during the generation
	 */
	public void writeCreateFieldDescRefTop(String varName,
										   Block block) throws Exception {
		block.writeLine(varName
			+ " = new FieldDesc(null, PTypeSpace.REFTOP);");
	}

	public void writeInitGenClassMapping(GenClassRef gcr,
										 String gcVarName,
										 Class sourceClass,
										 String fieldName,
										 int level,
										 Block block,
										 CompilerParameter cp)
		throws Exception {
		block.writeLine("((FileGenClassProp) " + gcVarName
			+ ").setSourceClassName(JORM_CLASS_NAME);");
		block.writeLine("((FileGenClassProp) " + gcVarName
			+ ").setFieldName(\"" + fieldName + "\");");
		block.writeLine("((FileGenClassProp) " + gcVarName
			+ ").setGenClassLevel(" + level + ");");
	}

	private String ptype2String(PType pt) throws PException {
		if (pt == null) {
			throw new PExceptionTyping("The parameter must be not null");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "PTypeSpace.BOOLEAN";
		case PType.TYPECODE_CHAR:
			return "PTypeSpace.CHAR";
		case PType.TYPECODE_BYTE:
			return "PTypeSpace.BYTE";
		case PType.TYPECODE_SHORT:
			return "PTypeSpace.SHORT";
		case PType.TYPECODE_INT:
			return "PTypeSpace.INT";
		case PType.TYPECODE_LONG:
			return "PTypeSpace.LONG";
		case PType.TYPECODE_FLOAT:
			return "PTypeSpace.FLOAT";
		case PType.TYPECODE_DOUBLE:
			return "PTypeSpace.DOUBLE";
		case PType.TYPECODE_STRING:
			return "PTypeSpace.STRING";
		case PType.TYPECODE_SERIALIZED:
			return "PTypeSpace.SERIALIZED";
		default :
			throw new PExceptionTyping(
				"Jorm does not support naming with the following PType:"
				+ pt.getTypeCode());
		}
	}

}
