/**

 * Copyright (C) 2002

 */

package org.objectweb.jorm.comp.generator.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

/**

 * This class is a helper for the generation. All methods provided in this

 * class are shortcut or facilties which simplify the generation.

 *

 * @author Sebastien Chassande-Barrioz

 */
public class CommonHelper implements Loggable {

    protected LoggerFactory loggerFactory = null;
    protected Logger logger = null;

    /**

     * This methods returns the word specified in parameter with the first

     * character in upper-case. This method is used for the generation of

     * accessor methods.

     * @param word	The String which the first character must be set in

     * 				upper-case

     * @return		The String whith the first character in upper-case

     */
    public String upperFL(String word) {

        if (word != null && word.length() > 0)
            return word.substring(0, 1).toUpperCase() + word.substring(1);
        else
            return word;
    }

    /**

     * This method tests if a class contains a reference. A reference can be a

     * designs a generic class or a class.

     * @param co	The meta object Class

     * @return true if the class contains a reference.

     */
    public boolean containsReference(Class co) {
        Iterator it = co.iterateAllField();
        while (it.hasNext()) {
            if (it.next() instanceof Reference)
                return true;
        }
        return false;
    }

    /**
     * This method tests if a class contains a composite reference. A reference can be a
     * designs a generic class or a class.
     * @param moc	The meta object Class
     * @cp the compiler parameter
     * @return true if the class contains the namedef for the composite reference.
     */
    public boolean containsCompositeReference(Class moc, CompilerParameter cp) throws Exception {
        Iterator it = moc.iterateAllField();
        while (it.hasNext()) {
            TypedElement te = (TypedElement) it.next();
            if (te instanceof Reference) {
                if ((((Reference) te).getNameDef(cp.getMappingName())).isNameRef())
                    return true;
            }
        }
        return false;
    }

    /**
     * This method return a list containing the nameref implied in composite reference field
     * @param moc	The meta object Class to be analysed.
     * @param cp the compiler parameter
     * @return an ArrayList which contains all nameref related to reference fields.
     */

    public ArrayList iterateCompositeReference(Class moc, CompilerParameter cp) throws Exception {

        ArrayList res = new ArrayList();
        Iterator it = moc.iterateAllField();
        boolean found;
        NameRef nr;
        while (it.hasNext()) {
            TypedElement te = (TypedElement) it.next();
            if (te instanceof Reference) {
                NameDef nd = ((Reference)te).getNameDef(cp.getMappingName());
                if (nd.isNameRef()){
                   res.add(nd.getNameRef());
                }
            }
        }
        return res;
    }


    /**
     * This method return a SET containing the full name of compositename implied
     * in the reference field of the class.
     * @param moc	The meta object Class
     * @param cp the compiler parameter
     * @return an iterator which contains all pname related to the Reference.
     */

    public Iterator iterateCompositeName(Class moc, CompilerParameter cp) throws Exception {

        ArrayList res = new ArrayList();
        Iterator it = moc.iterateAllField();
        boolean found;
        NameRef nr;
        while (it.hasNext()) {
            TypedElement te = (TypedElement) it.next();
            if (te instanceof Reference) {
                NameDef nd = ((Reference)te).getNameDef(cp.getMappingName());
                if (nd.isNameRef()){
                    nr = nd.getNameRef();
                    // avoiding duplicate cn type
                    Iterator it2 = res.iterator();
                    found = false;
                    while(it2.hasNext()&& (!(found))){
                        //NameRef nr1 = (NameRef)it2.next();
                        //if (nr1.equalCNType(nr)) found = true;

                        if (((String)it2.next()).equals(nr.getCNSchemaName()+"."+nr.getName()+"PNG")) found = true;
                    }
                    if (!(found))
                        res.add(nr.getCNSchemaName()+"."+nr.getName()+"PNG");
                }
            }
        }
        return res.iterator();
    }

    /**
     * Analyse the current class and build a list of PNGColInfo that provides information overs
     * composite pname of the classe and composite reference used in the class.
     * @return an hashtable. The keys are the fields names defined in composite name, the values are PNGDesc object
     * containing the pname id (pname coded as int) and the related typed element (i.e. the field meta object in
     * the class).
     *
     */
    /*
    public HashMap getPNGInfo(Class mo, CompilerParameter cp) throws Exception {

        HashMap res = new HashMap();
        NameDef nd = null;
        NameRef nr = null;
        Map proj = null;
        Iterator itCNField = null;
        String cnfield = null;
        ArrayList pngDescList = null;

        Iterator it = mo.iterateAllField();
        int pNameId = 0;
        while (it.hasNext()) {
            TypedElement te = (TypedElement) it.next();
            if (te instanceof Reference) {
                pNameId++;
                nd = ((Reference) te).getNameDef(cp.getMappingName());
                if (nd.isNameRef()) {
                    nr = nd.getNameRef();
                    proj = nr.getProjection();
                    itCNField = proj.keySet().iterator();
                    while (itCNField.hasNext()){
                        cnfield = (String) itCNField.next();
                        if (res.containsKey(cnfield)){
                            pngDescList = (ArrayList) res.get(cnfield);
                            pngDescList.add(new PNGDesc(pNameId,mo.getTypedElement((String)proj.get(cnfield))));

                        } else {
                            pngDescList = new ArrayList();
                            pngDescList.add(new PNGDesc(pNameId,mo.getTypedElement((String)proj.get(cnfield))));
                            res.put(cnfield,pngDescList);
                        }
                    }
                }
            }
        }
        return res;
    }
    */

    /**
     * This method retrieve the schema name for the meta object passed as parameter.
     * The meta object must be a child of a Schema MetaObject.
     * @param mo	The meta object
     * @return the name of the schema if it exist, null otherwise.
     */
    public String getSchemaName(MetaObject mo) {
        if (mo instanceof Manager) return null;
        while (!(mo instanceof Schema)) {
            mo = mo.getParent();
        }
        if (mo instanceof Schema)
            return ((Schema) mo).getName();
        else
            return null;
    }

    /**

     * This method tests if a class references to a generic class.

     * @param co	The meta object Class

     * @return true if the class references a generic class.

     */
    public boolean containsGenClassRef(Class co) {
        Iterator it = co.iterateAllField();
        while (it.hasNext()) {
            if (it.next() instanceof GenClassRef)
                return true;
        }
        return false;
    }

    /**

     * This method tests if a class contains a serialized field.

     * @param co	The meta object Class

     * @return true if the class contains a serialized field.

     */
    public boolean containsSerializedField(Class co) {
        Iterator it = co.iterateAllField();
        while (it.hasNext()) {
            if (((TypedElement) it.next()).getType().getTypeCode()
                    == PType.TYPECODE_SERIALIZED)
                return true;
        }
        return false;
    }

    /**

     * This method tests if a String is valid. A String is valid if it is not

     * null and if the length is greater than 0.

     * @param str   The string which must be checked

     * @return true if the string is valid

     */
    public boolean isValidString(String str) {
        return str != null && str.length() > 0;
    }


    /**

     * This method checks if the NameDef is composed by several fields.

     * @param nd the meta object NameDef which must be tested

     * @return true if the NameDef is composed by several fields.

     */
    public boolean isCompositeName(NameDef nd) {
        return nd.isNameRef();
    }

    /**
     * This method provides an iterator over the field names of the class implied in the
     * definition of the namedef passed as parameter.
     * These fields can be hidden or not.
     * @param nd the meta object NameDef which must be tested
     * @return an iterator over the field name of the class that defines the pname.
     */

    public Iterator getCompositeNameDefField(NameDef mo) throws Exception {
        if (mo.isNameRef()) {
            NameRef nr = mo.getNameRef();
            // get the projection of this composite name onto the field of the class
            Map proj = nr.getProjection();
            // classfields represents the name of the field related to the name structure.
            // classfields are either primitive field or hiddenfield of the class.
            return proj.values().iterator();
        } else
            throw new PException(
                    "impossible to obtain fields of " + mo);
    }

    /**
     * This method provides an iterator over the composite name referenced by the class
     * These composite names correspond to the reference field of the class.
     * It is used by the pbinding generator to generate specific pnamegetter.
     * A specific pnamegetter interface must be implemented by pbinding for each type of compositename
     * used as composite ref field.
     * @param class the meta object class which contain the composite name reference.
     * @return an iterator over the composite name ref.
     */
/*
    public Iterator getRefCompositeNameFromClass(Class mo, CompilerParameter cp) throws Exception {
        ArrayList res = new ArrayList();
        NameDef nd = mo.getNameDef(cp.getMappingName());
        if (nd.isNameRef()) res.add(nd.getNameRef());

        Iterator it = mo.iterateAllField();
        while (it.hasNext()) {
            TypedElement te = (TypedElement) it.next();
            if (te instanceof Reference) {
                nd = ((Reference) te).getNameDef(cp.getMappingName());
                if (nd.isNameRef()) {
                    if (!(res.contains(nd.getNameRef())))
                        res.add(nd.getNameRef());
                }
            }
        }
        return res.iterator();
    }
*/

    /**
     * This method provides the projection of the field name belonging to the class
     * over a field name belonging to the composite name corresponding definition.
     * @param nd the meta object NameDef embeding a nameref with a projection.
     * @param cnf the field name belonging to the pname in the class.
     * @return the field name of the projection. this field name is a field belonging to the class.
     */

    public String getProjectionField(NameDef nd, String cfn) throws Exception {
        if (nd.isNameRef()) {
            NameRef nr = nd.getNameRef();
            // get the projection of this composite name onto the field of the class
            // classfields represents the name of the field related to the name structure.
            // classfields are either primitive field or hiddenfield of the class.
            Iterator it = nr.getProjection().entrySet().iterator();
            while (it.hasNext()) {
                Map.Entry me = (Map.Entry) (it.next());
                if (((String) (me.getValue())).equals(cfn))
                    return (String) (me.getKey());
            }
        }
        throw new PException("impossible to obtain field " + cfn + " of " + nd);
    }

    /**

     * This method allows to know if a class represented by its Class meta

     * object, contains a composite PName. This method analyzes the NameDef of

     * the @c class parameter and the NameDef of all reference fields.

     * @c the class which must analyzed

     * @cp the compiler parameter of this generation

     */
    public boolean containsCompositeName(Class c, CompilerParameter cp) {
        if (logger != null)
            logger.log(BasicLevel.DEBUG, "");
        NameDef nd = c.getNameDef(cp.getMappingName());
        if (nd.isNameRef())
            return true;
        if (logger != null)
            logger.log(BasicLevel.DEBUG, "PName class has a single field");
        Iterator it = c.iterateAllField();
        while (it.hasNext()) {
            TypedElement te = (TypedElement) it.next();
            if (logger != null)
                logger.log(BasicLevel.DEBUG, "field: " + te.getName());
            if (te instanceof Reference) {
                if (logger != null) logger.log(BasicLevel.DEBUG, "is a reference");
                nd = ((Reference) te).getNameDef(cp.getMappingName());
                if (nd.isNameRef()) {
                    if (logger != null)
                        logger.log(BasicLevel.DEBUG, "true");
                    return true;
                }
            }
        }
        return false;
    }


    /**

     * This method checks if the parameter is an instance of

     * org.objectweb.jorm.metainfo.api.Class

     * @param o the object which the type must be checked

     * @return true the parameter is an instance of Class

     */
    public boolean isClass(Object o) {
        return o instanceof Class;
    }

    /**

     * This method checks if the parameter is an instance of

     * org.objectweb.jorm.metainfo.api.Reference

     * @param o the object which the type must be checked

     * @return true the parameter is an instance of Reference

     */
    public boolean isReference(TypedElement te) {
        return te instanceof Reference;
    }

    /**

     * This method checks if the parameter is an instance of

     * org.objectweb.jorm.metainfo.api.GenClassRef

     * @param o the object which the type must be checked

     * @return true the parameter is an instance of GenClassRef

     */
    public boolean isGenClassRef(TypedElement te) {
        return te instanceof GenClassRef;
    }

    /**

     * This method checks if the parameter is an instance of

     * org.objectweb.jorm.metainfo.api.PrimitiveElement

     * @param o the object which the type must be checked

     * @return true the parameter is an instance of PrimitiveElement

     */
    public boolean isPrimitiveElement(Object o) {
        return o instanceof PrimitiveElement;
    }

    /**

     * This method checks if the parameter is an instance of

     * org.objectweb.jorm.metainfo.api.FieldName

     * @param o the object which the type must be checked

     * @return true the parameter is an instance of FieldName

     */
    public boolean isFieldName(Object o) {
        return o instanceof FieldName;
    }

    /**

     * This method checks if the parameter designs a Serialized field.

     * @param o the object which the type must be checked

     * @return true the parameter is an instance of PrimitiveElement and if its

     * PType is SERIALIZED.

     */
    public boolean isSerializedField(Object o) {
        return (o instanceof PrimitiveElement)
                && (((PrimitiveElement) o).getType().getTypeCode()
                == PType.TYPECODE_SERIALIZED);
    }

    /**

     * This method checks if the PType parameter designs a Serialized type.

     * @param o the PType object which the type must be checked

     * @return true the parameter is a PType.SERIALIZED.

     */
    public boolean isSerializedType(PType t) {
        return t.getTypeCode() == PType.TYPECODE_SERIALIZED;
    }

    /**

     * This method does the conversion between a PType @pt and the getter

     * function to use on a PAccessorGen (generic accessor).

     * @param pt the ptype which helps to find the right paccessor method

     * @return 	The string which represents the paccessor method associated

     * 			to pt parameter

     */
    public String getPaGetMethod(PType pt) throws PException {
        if (pt == null)
            throw new PException("Unauthorized null PType");

        switch (pt.getTypeCode()) {
            case PType.TYPECODE_BOOLEAN:
                return "paGetBooleanField";
            case PType.TYPECODE_BYTE:
                return "paGetByteField";
            case PType.TYPECODE_CHAR:
                return "paGetCharField";
            case PType.TYPECODE_DATE:
                return "paGetDateField";
            case PType.TYPECODE_SHORT:
                return "paGetShortField";
            case PType.TYPECODE_INT:
                return "paGetIntField";
            case PType.TYPECODE_INT_OBJ:
                return "paGetIntObjField";
            case PType.TYPECODE_LONG:
                return "paGetLongField";
            case PType.TYPECODE_FLOAT:
                return "paGetFloatField";
            case PType.TYPECODE_DOUBLE:
                return "paGetDoubleField";
            case PType.TYPECODE_STRING:
                return "paGetStringField";
            case PType.TYPECODE_SERIALIZED:
                return "paGetSerializedField";
            case PType.TYPECODE_REFERENCE:
                return "paGetRefField";
        }
        throw new PExceptionTyping("Unauthorized PType " + pt);
    }

    /**

     * This method does the conversion between a PType @pt and the setter

     * function to use on a PAccessorGen (generic accessor).

     * @param pt the ptype which helps to find the right paccessor method

     * @return 	The string which represents the paccessor method associated

     * 			to pt parameter

     */
    public String getPaSetMethod(PType pt) throws PException {
        if (pt == null)
            throw new PException("Unauthorized null PType");

        switch (pt.getTypeCode()) {
            case PType.TYPECODE_BOOLEAN:
                return "paSetBooleanField";
            case PType.TYPECODE_BYTE:
                return "paSetByteField";
            case PType.TYPECODE_CHAR:
                return "paSetCharField";
            case PType.TYPECODE_DATE:
                return "paSetDateField";
            case PType.TYPECODE_SHORT:
                return "paSetShortField";
            case PType.TYPECODE_INT:
                return "paSetIntField";
            case PType.TYPECODE_INT_OBJ:
                return "paSetIntObjField";
            case PType.TYPECODE_LONG:
                return "paSetLongField";
            case PType.TYPECODE_FLOAT:
                return "paSetFloatField";
            case PType.TYPECODE_DOUBLE:
                return "paSetDoubleField";
            case PType.TYPECODE_STRING:
                return "paSetStringField";
            case PType.TYPECODE_SERIALIZED:
                return "paSetSerializedField";
            case PType.TYPECODE_REFERENCE:
                return "paSetRefField";
        }
        throw new PExceptionTyping("Unauthorized PType " + pt);
    }

    /**

     * This method permits to find the right decode function either the

     * PType specified by the parameter @pt. These functions name are available

     * on a PName interface.

     * Be carefull the only jorm types support for the naming are the scalar

     * types (char, byte, short, int, long and string) !

     * @param pt the ptype which helps to find the right decode method

     * @return 	The string which represents the decode method associated

     * 			to pt parameter

     */
    public String getPNameDecodeFunction(PType pt) throws PException {
        if (pt == null) {
            throw new PExceptionTyping("The parameter must be not null");
        }
        switch (pt.getTypeCode()) {
            case PType.TYPECODE_CHAR:
                return "decodeChar";
            case PType.TYPECODE_BYTE:
                return "decodeByte";
            case PType.TYPECODE_SHORT:
                return "decodeShort";
            case PType.TYPECODE_INT:
                return "decodeInt";
            case PType.TYPECODE_INT_OBJ:
                return "decodeIntObj";
            case PType.TYPECODE_LONG:
                return "decodeLong";
            case PType.TYPECODE_STRING:
                return "decodeString";
            default :
                throw new PExceptionTyping(
                        "Jorm does not support naming with the following PType:"
                        + pt.getTypeCode());
        }
    }


    /**

     * This method permits to find the right pngetXField function either the

     * PType specified by the parameter @pt. These functions name are available

     * on a PNameGetter interface.

     * Be carefull the only jorm types support for the naming are the scalar

     * types (char, byte, short, int, long and string) !

     * @param pt the ptype which helps to find the right pname getter method

     * @return 	The string which represents the pname getter method associated

     * 			to pt parameter

     */
    public String getPNameGetterGetFunction(PType pt) throws PException {
        if (pt == null) {
            throw new PExceptionTyping("The parameter must be not null");
        }
        switch (pt.getTypeCode()) {
            case PType.TYPECODE_CHAR:
                return "pngetCharField";
            case PType.TYPECODE_BYTE:
                return "pngetByteField";
            case PType.TYPECODE_SHORT:
                return "pngetShortField";
            case PType.TYPECODE_INT:
                return "pngetIntField";
            case PType.TYPECODE_INT_OBJ:
                return "pngetIntObjField";
            case PType.TYPECODE_LONG:
                return "pngetLongField";
            case PType.TYPECODE_STRING:
                return "pngetStringField";
            default :
                throw new PExceptionTyping(
                        "Jorm does not support naming with the following PType:"
                        + pt.getTypeCode());
        }

    }

    /**

     * This method permits to find the right encode function either the

     * PType specified by the parameter @pt. These functions name are available

     * on a PName interface.

     * Be carefull the only jorm types support for the naming are the scalar

     * types (char, byte, short, int, long and string) !

     * @param pt the ptype which helps to find the right encode method

     * @return 	The string which represents the encode method associated

     * 			to pt parameter

     */
    public String getPNameEncodeFunction(PType pt) throws PException {
        if (pt == null) {
            throw new PExceptionTyping("The parameter must be not null");
        }
        switch (pt.getTypeCode()) {
            case PType.TYPECODE_CHAR:
                return "encodeChar";
            case PType.TYPECODE_BYTE:
                return "encodeByte";
            case PType.TYPECODE_SHORT:
                return "encodeShort";
            case PType.TYPECODE_INT:
                return "encodeInt";
            case PType.TYPECODE_INT_OBJ:
                return "encodeIntObj";
            case PType.TYPECODE_LONG:
                return "encodeLong";
            case PType.TYPECODE_STRING:
                return "encodeString";
            default :
                throw new PExceptionTyping(
                        "Jorm does not support naming with the following PType:"
                        + pt.getTypeCode());
        }
    }

    public PType getPType(String ptn) {
        if (ptn.equals("boolean")) return PTypeSpace.BOOLEAN;
        if (ptn.equals("char")) return PTypeSpace.CHAR;
        if (ptn.equals("byte")) return PTypeSpace.BYTE;
        if (ptn.equals("short")) return PTypeSpace.SHORT;
        if (ptn.equals("int")) return PTypeSpace.INT;
        if (ptn.equals("Integer")) return PTypeSpace.INT_OBJ;
        if (ptn.equals("long")) return PTypeSpace.LONG;
        if (ptn.equals("float")) return PTypeSpace.FLOAT;
        if (ptn.equals("double")) return PTypeSpace.DOUBLE;
        if (ptn.equals("string")) return PTypeSpace.STRING;
        if (ptn.equals("date")) return PTypeSpace.DATE;
        if (ptn.equals("serialized")) return PTypeSpace.SERIALIZED;
        if (ptn.equals("reference")) return PTypeSpace.REFTOP;
        return null;
    }

    /**

     * Its retrieves the PType object which represents the boolean jorm type.

     */
    public PType getPTypeBoolean() {
        return PTypeSpace.BOOLEAN;
    }

    /**

     * Its retrieves the PType object which represents the char jorm type.

     */
    public PType getPTypeChar() {
        return PTypeSpace.CHAR;
    }

    /**

     * Its retrieves the PType object which represents the byte jorm type.

     */
    public PType getPTypeByte() {
        return PTypeSpace.BYTE;
    }

    /**

     * Its retrieves the PType object which represents the short jorm type.

     */
    public PType getPTypeShort() {
        return PTypeSpace.SHORT;
    }

    /**

     * Its retrieves the PType object which represents the int jorm type.

     */
    public PType getPTypeInt() {
        return PTypeSpace.INT;
    }

    /**

     * Its retrieves the PType object which represents the int jorm type.

     */
    public PType getPTypeIntObj() {
        return PTypeSpace.INT_OBJ;
    }

    /**

     * Its retrieves the PType object which represents the long jorm type.

     */
    public PType getPTypeLong() {
        return PTypeSpace.LONG;
    }

    /**

     * Its retrieves the PType object which represents the float jorm type.

     */
    public PType getPTypeFloat() {
        return PTypeSpace.FLOAT;
    }

    /**

     * Its retrieves the PType object which represents the double jorm type.

     */
    public PType getPTypeDouble() {
        return PTypeSpace.DOUBLE;
    }

    /**

     * Its retrieves the PType object which represents the string jorm type.

     */
    public PType getPTypeString() {
        return PTypeSpace.STRING;
    }

    /**

     * Its retrieves the PType object which represents the date jorm type.

     */
    public PType getPTypeDate() {
        return PTypeSpace.DATE;
    }

    /**

     * Its retrieves the PType object which represents the serialized jorm type.

     */
    public PType getPTypeSerialized() {
        return PTypeSpace.SERIALIZED;
    }

    /**

     * Its retrieves the PType object which represents a reference.

     */
    public PType getPTypeReference() {
        return PTypeSpace.REFTOP;
    }

    /**
     * It compute the test used to know if a pname is null.
     */
    public String getPNameNullTest(CompositeName cn) {
        String result = "";
        Iterator fs = cn.iterateAllField();
        while (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            if (f.getType().getJavaName().equals("java.lang.String"))
                result = result + f.getName() + ".equals(\"" + f.getNullValue() + "\")";
            else if (f.getType().getJavaName().equals("char"))
                result = result + f.getName() + " == \'" + f.getNullValue() + "\'";
            else
                result = result + f.getName() + " == " + f.getNullValue();
            if (fs.hasNext()) result = result + " && ";
        }
        return result;
    }

    /**
     * It compute the constructor parameter used to instanciate a null pname.
     */
    public String getPNameNullValue(CompositeName cn) {
        String result = "";
        Iterator fs = cn.iterateAllField();
        while (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            if (f.getType().getJavaName().equals("java.lang.String"))
                result = result + "\"" + f.getNullValue() + "\",";
            else if (f.getType().getJavaName().equals("char"))
                result = result + "\'" + f.getNullValue() + "\',";
            else
                result = result + f.getNullValue() + ",";
        }
        return result + " pnc";
    }

    /**
     * It compute the hashcode method for a composite pname.
     */
/*
    public String getCnHashCode(CompositeName cn) {
        String result ="";
        Iterator fs = cn.iterateAllField();
        // compositename type can be : byte | char | short | int | long | string
        // tthe hashcode value for the pname is equal to the hashcode of the first field belonging to the pname.
        while (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            if (f.getType().getJavaName().equals("String"))
                result = result + f.getName()+".hashCode()";
            else // byte | char | short | int | long
                result = result + f.getName();
            if (fs.hasNext()) result = result + "+";
        }
        return result;
    }
*/
    public String getCnHashCode(CompositeName cn) {
        String result = "";
        Iterator fs = cn.iterateAllField();
        // compositename type can be : byte | char | short | int | long | string
        // tthe hashcode value for the pname is equal to the hashcode of the first field belonging to the pname.
        if (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            if (f.getType().getJavaName().equals("java.lang.String"))
                result = f.getName() + ".hashCode()";
            else // byte | char | short | int | long
                result = f.getName();
        }
        return result;
    }

    /**
     * It compute the equality test for the equals method for pname.
     */
    public String getCnEquals(CompositeName cn) {
        String result = "";
        Iterator fs = cn.iterateAllField();
        while (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            if (f.getType().getJavaName().equals("java.lang.String"))
                result = result + f.getName() + ".equals( ((" + cn.getName() + "PNG)obj).pnGet" + upperFL(f.getName()) + "(null))";
            else // byte | char | short | int | long
                result = result + f.getName() + "==" + "((" + cn.getName() + "PNG)obj).pnGet" + upperFL(f.getName()) + "(null)";
            if (fs.hasNext()) result = result + " && ";
        }
        return result;
        // TO DO  : we must also check if the obj is not a subclass of the pname class.
    }

    public String getPNameFromSpecificPNG(CompositeName cn) {
        String result = "new " + cn.getName() + "PName(";
        Iterator fs = cn.iterateAllField();
        while (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            result = result + "((" + cn.getName() + "PNG)en).pnGet" + upperFL(f.getName()) + "(ctx),";
        }
        return result + " this.pcmbinder)";
    }

    public String getPNameFromGenericPNG(CompositeName cn) {
        String result = "new " + cn.getName() + "PName(";
        Iterator fs = cn.iterateAllField();
        while (fs.hasNext()) {
            ScalarField f = (ScalarField) fs.next();
            if (f.getType().getJormName().equals("string"))
                result = result + "((PNameGetter)en).pngetStringField(\"" + f.getName() + "\", ctx),";
            else if (f.getType().getJormName().equals("byte"))
                result = result + "((PNameGetter)en).pngetByteField(\"" + f.getName() + "\", ctx),";
            else if (f.getType().getJormName().equals("char"))
                result = result + "((PNameGetter)en).pngetCharField(\"" + f.getName() + "\", ctx),";
            else if (f.getType().getJormName().equals("short"))
                result = result + "((PNameGetter)en).pngetShortField(\"" + f.getName() + "\", ctx),";
            else if (f.getType().getJormName().equals("int"))
                result = result + "((PNameGetter)en).pngetIntField(\"" + f.getName() + "\", ctx),";
            else if (f.getType().getJormName().equals("Integer"))
                result = result + "((PNameGetter)en).pngetIntObjField(\"" + f.getName() + "\", ctx),";
            else if (f.getType().getJormName().equals("long"))
                result = result + "((PNameGetter)en).pngetLongField(\"" + f.getName() + "\", ctx),";
        }
        return result + " this.pcmbinder)";
    }


    // IMPLEMENTATION OF THE Loggable INTERFACE //

    //------------------------------------------//



    /**

     * Its assigns a logger factory.

     */
    public void setLoggerFactory(LoggerFactory lf) {
        if (logger == null && lf != null)
            logger = lf.getLogger(getClass().getName());
        loggerFactory = lf;
    }

    /**

     * Its assigns a logger.

     */
    public void setLogger(Logger l) {
        logger = l;
    }

    /**

     * Its retrives the current logger.

     */
    public Logger getLogger() {
        return logger;
    }

    /**

     * Its retrives the current logger factory.

     */
    public LoggerFactory getLoggerFactory() {
        return loggerFactory;
    }


    // PNGDesc
    public class PNGDesc {
        public int pnameId;
        public TypedElement elm;

        public PNGDesc(int pnameId, TypedElement elm) {
            this.pnameId = pnameId;
            this.elm = elm;
        }

    }

}

