/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.comp.generator.lib;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.app.Velocity;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.util.io.api.Block;
import org.objectweb.util.io.api.TargetHolder;
import org.objectweb.util.io.lib.BlockImpl;
import org.objectweb.util.monolog.api.BasicLevel;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.Iterator;

/**
 * This class is a generator of XXXPNG objects. It is generic either the
 * mapper type. This generator use the velocity tools. The used template is
 * PNG.vm . Associated to this template this generator builds a velocity
 * context which contains the following information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"compositename"</td><td>The reference to the compositename meta object</td></tr>
 * 	<tr>
 * 		<td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr>
 * 		<td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * </table>
 */
public class CompositeNameGenerator extends ObjectGenerator {

	// IMPLEMENTATION OF THE Generator INTERFACE //

	/**
	 * This method generates a XAccessor file corresponding to the pod
	 * parameter in the directory parameter
	 * @param pod	the meta object interface
	 * @param th	The target holder which allows to create files
	 * @param cp	This parameter permits to reach the compilation parameters
	 */
	public void generate(MetaObject pod,
						 TargetHolder holder,
						 CompilerParameter cp
						 ) throws Exception {
		// The MetaObject parameter must be a Class.
		CompositeName co = (CompositeName) pod;

		// Calculate fileName
		String packName = ((Schema) co.getParent()).getName();
		String fileName = co.getName() + "PNG";
		if (packName != null && packName.length() > 0) {
			fileName = packName + "." + fileName;
			fileName = fileName.replace('.', File.separatorChar);
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " interface");

		// Fetch a block with the fileName
		FileWriter fw = holder.getFileWriter(fileName + ".java");
		VelocityEngine ve = initVelocity();
		ve.init();
		Context ctx = new VelocityContext();
		ctx.put("compositename", co);
		CommonHelper ch = new CommonHelper();
		ch.setLogger(logger);
		ctx.put("tools", ch);
		ctx.put("header", TEMPLATE_DIR + "/lib/Header.vm");
		ve.getTemplate(TEMPLATE_DIR + "/lib/Png.vm").merge(ctx, fw);
		fw.flush();
		fw.close();
	}
}



