/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.comp.generator.lib;

import org.apache.velocity.app.Velocity;
import org.apache.velocity.app.VelocityEngine;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.generator.api.Generator;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.io.api.TargetHolder;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.wrapper.velocity.VelocityLogger;

/**
 * This class contains the common elements for generating jorm objects (mapping,
 * binding, accessors)
 */
public abstract class ObjectGenerator
	implements Generator, Loggable {

	/**
	 * The base directory where to find velocity template.
	 */
	public static final String TEMPLATE_DIR =
		"org/objectweb/jorm/comp/generator";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_DESCRIPTION_PROP
		= "class.resource.loader.description";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_DESCRIPTION_VALUE
		= "Velocity Classpath Resource Loader";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_CLASS_PROP
		= "class.resource.loader.class";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_CLASS_VALUE
		= "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader";

	/**
	 * The logger instance which logs the traces of the generator.
	 */
	protected Logger logger = null;

	/**
	 * The logger factory instance which permits to fetch Logger instance.
	 */
	protected LoggerFactory loggerFactory = null;

	/**
	 * It initializes a new VelocityEngine.
	 * @return 	A new VelocityEngine instance initialized for the generation of
	 * 			the JORM obbjects
	 */
	public VelocityEngine initVelocity() {
		VelocityEngine ve = new VelocityEngine();
		ve.setProperty(Velocity.RESOURCE_LOADER, "class");
		ve.setProperty(CLASS_RESOURCE_LOADER_DESCRIPTION_PROP,
			CLASS_RESOURCE_LOADER_DESCRIPTION_VALUE);
		ve.setProperty(CLASS_RESOURCE_LOADER_CLASS_PROP,
			CLASS_RESOURCE_LOADER_CLASS_VALUE);
		// Connect the velocity log system to monolog
		ve.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, new VelocityLogger(
			(loggerFactory == null
			? logger
			: loggerFactory.getLogger(logger.getName() + ".velocity"))
		));
		return ve;
	}

	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	public void setLoggerFactory(LoggerFactory lf) {
		if (logger == null && lf != null)
			logger = lf.getLogger(getClass().getName());
		loggerFactory = lf;
	}

	public void setLogger(Logger l) {
		logger = l;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}


	// IMPLEMENTATION OF THE Generator INTERFACE //

	/**
	 * A part of the generate
	 * @param pod	the meta object interface
	 * @param th	The target holder which allows to create files
	 */
	public abstract void generate(MetaObject pod, TargetHolder holder,
								  CompilerParameter cp) throws Exception;

}
