/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.comp.generator.memory;

import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.generator.api.MOP;
import org.objectweb.jorm.comp.generator.lib.CommonHelper;
import org.objectweb.jorm.comp.generator.lib.ObjectGenerator;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.FieldName;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.io.api.Block;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class implements the BindingMOP interface in order to specialize the
 * generation of the PBinding for the relationnal database spport.
 *
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 * TODO :
 * - le code g?n?r? doit utiliser des constantes pour les noms des champs et
 * le nom des colonnes:
 * 		=> Maintenir dans le generateur une hashMap (colName, constanteName)
 * - Commenter
 */
public class MemoryBindingMOP extends CommonHelper implements MOP {

	protected static final String PACCESSOR_VAR_NAME = "paccessor";
	protected static final String PACCESSORGEN_VAR_NAME = "paccessorGen";

	// IMPLEMENTATION OF THE MOP INTERFACE //
	//-------------------------------------//


	public void initContext(Context ctx) {
		ctx.put("bindingMOPTools", this);
	}

	public String[] getTemplateLibraries() {
		String[] libs = {ObjectGenerator.TEMPLATE_DIR + "/memory/MemoryBinding.vm"};
		return libs;
	}

	// ----- HELPER METHODS -----//
	//---------------------------//

//TODO

	// ---- OLD METHODS -----//
	//-----------------------//

    public ArrayList getField(Class moc, CompilerParameter cp) {
          Iterator iter = moc.iterateField();
        ArrayList ar = new ArrayList();
        while (iter.hasNext()) {
            ar.add("public Object "+((PrimitiveElement)iter.next()).getName() + " = null;");
        }
        return ar;
    }

    public boolean containsPrimitiveField(Class moc) {
        Iterator iter = moc.iterateField();
        while (iter.hasNext()) {
            Object o = iter.next();
            if (o instanceof PrimitiveElement)
                return true;
        }
        return false;
    }

}
