/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.comp.generator.memory;

import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.generator.api.MOP;
import org.objectweb.jorm.comp.generator.lib.CommonHelper;
import org.objectweb.jorm.comp.generator.lib.ObjectGenerator;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.rdb.RDBClassMapping;
import org.objectweb.jorm.metainfo.api.rdb.RDBColumnSpec;
import org.objectweb.jorm.metainfo.api.rdb.RDBValueMapping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.io.api.Block;

import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 */
public class MemoryMappingMOP extends CommonHelper implements MOP {

    // IMPLEMENTATION OF THE MOP INTERFACE //
	//-------------------------------------//

	public void initContext(Context ctx) {
		ctx.put("mappingMOPTools", this);
	}

	public String[] getTemplateLibraries() {
		String[] libs = {ObjectGenerator.TEMPLATE_DIR + "/memory/MemoryMapping.vm"};
		return libs;
	}

	// ------ HELPER METHODS -------//
	//------------------------------//

//TODO

	// ---- OLD METHODS -----//
	//-----------------------//

	/**
	 * writes the definitions of all variables needed by
	 * a Binding object.
	 * @param block	The class definition block within which the variable
	 *			definitions are written.
	 */
	public void writeVariables(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * @param block	The class definition block within which the variable
	 *			definitions are written.
	 */
	public void writeConstructorBody(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * writes the body of method  initStruct(Object conn)
	 * that will be used to build the persistence support for the object in the
	 * object store.
	 * @param block	The block within which the method's body has to be
	 * written.
	 * @param moc 	The meta object which represents a jorm class
	 */
	public void writeinitMappingStructures(Block block, Class moc, CompilerParameter cp)
		throws Exception {

	}

	/**
	 * writes the body of the removeStruct(Object conn) that will be used to
	 * remove the persistent support  for the object int the data storage.
	 * @param block	The block within which the method's body has to be
	 * written.
	 * @param moc 	The meta object which represents a jorm class
	 */
	public void writeRemoveMappingStructures(Block block, Class moc, CompilerParameter cp)
		throws Exception {
	}

	/**
	 * defines interfaces to implement by the generated mapping object.
	 * @return	A String tab representing the classes to implement. For
	 * 		mapping object, FileReader and FileWriter are implemented.
	 */
	public String[] getImplements(Class c, CompilerParameter cp) throws Exception {
		return new String[0];
	}

	/**
	 * defines imports needed by the PMapping generation.
	 * @return	A String tab representing the class names needed by the
	 * generated object.
	 */
	public String[] getImportedClassName(Class c, CompilerParameter cp) throws Exception {
		boolean ref2gc = this.containsGenClassRef(c);
		int nb = 2 + (ref2gc ? 1 : 0);
		String[] res = new String[nb];
		res[0] = "org.objectweb.transaction.ra.file.FileAccess";
		res[1] = "org.objectweb.jorm.type.api.PTypeSpace";
		if (ref2gc)
			res[2] = "org.objectweb.jorm.runtime.mapper.memory.genclass.api.MemoryGenClassProp";
		return res;
	}

	/**
	 * This method generates the setting of the @varName varible with a Mapper
	 * specific FieldDesc either a PrimitiveElement.
	 * The mapping of a primitive element depends on a mapper. The extension
	 * of the FieldDesc class contains specific fields to the mapper.
	 *
	 * @block the block where the code must be generated
	 * @pe the primitive element
	 * @varName is the string of the variable name which must be set
	 * @cp the compiler parameter should permits to select the right
	 * ValueMapping linked to the primitiveElement
	 *
	 * Exception is raised if an error occurs during the generation
	 */
	public void writeFieldDescFilling(PrimitiveElement pe,
									  String varName,
									  Block block,
									  CompilerParameter cp) throws Exception {
		block.writeLine(varName + " = new FieldDesc(\"" + pe.getName() + "\", "
			+ ptype2String(pe.getType()) + ");");
	}

	/**
	 * This method generates the setting of the @varName with a PType equals to
	 * PTypeSpace.REFTOP. This method depends on a mapper because the constructor
	 * of the FieldDesc extension is only known by the MOP objects.
	 *
	 * @block the block where the code must be generated
	 * @varName is the string of the variable name which must be set
	 *
	 * Exception is raised if an error occurs during the generation
	 */
	public void writeCreateFieldDescRefTop(String varName,
										   Block block) throws Exception {
		block.writeLine(varName
			+ " = new FieldDesc(null, PTypeSpace.REFTOP);");
	}

	public void writeInitGenClassMapping(GenClassRef gcr,
										 String gcVarName,
										 Class sourceClass,
										 String fieldName,
										 int level,
										 Block block,
										 CompilerParameter cp)
		throws Exception {
		block.writeLine("((FileGenClassProp) " + gcVarName
			+ ").setSourceClassName(JORM_CLASS_NAME);");
		block.writeLine("((FileGenClassProp) " + gcVarName
			+ ").setFieldName(\"" + fieldName + "\");");
		block.writeLine("((FileGenClassProp) " + gcVarName
			+ ").setGenClassLevel(" + level + ");");
	}

	private String ptype2String(PType pt) throws PException {
		if (pt == null) {
			throw new PExceptionTyping("The parameter must be not null");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "PTypeSpace.BOOLEAN";
		case PType.TYPECODE_CHAR:
			return "PTypeSpace.CHAR";
		case PType.TYPECODE_BYTE:
			return "PTypeSpace.BYTE";
		case PType.TYPECODE_SHORT:
			return "PTypeSpace.SHORT";
		case PType.TYPECODE_INT:
			return "PTypeSpace.INT";
		case PType.TYPECODE_LONG:
			return "PTypeSpace.LONG";
		case PType.TYPECODE_FLOAT:
			return "PTypeSpace.FLOAT";
		case PType.TYPECODE_DOUBLE:
			return "PTypeSpace.DOUBLE";
		case PType.TYPECODE_STRING:
			return "PTypeSpace.STRING";
		case PType.TYPECODE_SERIALIZED:
			return "PTypeSpace.SERIALIZED";
		default :
			throw new PExceptionTyping(
				"Jorm does not support naming with the following PType:"
				+ pt.getTypeCode());
		}
	}

}
