/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.facility.meminst.genclass.api;

import org.objectweb.jorm.api.PException;

import java.util.Iterator;
import java.util.Collection;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface Set {
	/**
	 * Returns the number of elements in this set (its cardinality).  If this
	 * set contains more than <tt>Integer.MAX_VALUE</tt> elements, returns
	 * <tt>Integer.MAX_VALUE</tt>.
	 *
	 * @param connection used to access the data store
	 * @return the number of elements in this set (its cardinality).
	 * @throws PException a internal jorm error occured
	 */
	public int size(Object connection) throws PException;

	/**
	 * Returns <tt>true</tt> if this set contains no elements.
	 *
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this set contains no elements.
	 * @throws PException a internal jorm error occured
	 */
	public boolean isEmpty(Object connection) throws PException;

	/**
	 * Returns <tt>true</tt> if this set contains the specified element.  More
	 * formally, returns <tt>true</tt> if and only if this set contains an
	 * element <code>e</code> such that <code>(o==null ? e==null :
	 * o.equals(e))</code>.
	 *
	 * @param o element whose presence in this set is to be tested.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this set contains the specified element.
	 * @throws PException a internal jorm error occured
	 */
	public boolean contains(Object o, Object connection) throws PException;

	/**
	 * Returns an iterator over the elements in this set.  The elements are
	 * returned in no particular order (unless this set is an instance of some
	 * class that provides a guarantee).
	 *
	 * @param connection used to access the data store
	 * @return an iterator over the elements in this set.
	 * @throws PException a internal jorm error occured
	 */
	public Iterator iterator(Object connection) throws PException;

	/**
	 * Returns an array containing all of the elements in this set.
	 * Obeys the general contract of the <tt>Collection.toArray</tt> method.
	 *
	 * @param connection used to access the data store
	 * @return an array containing all of the elements in this set.
	 * @throws PException a internal jorm error occured
	 */
	public Object[] toArray(Object connection) throws PException;

	/**
	 * Returns an array containing all of the elements in this set whose
	 * runtime type is that of the specified array.  Obeys the general
	 * contract of the <tt>Collection.toArray(Object[])</tt> method.
	 *
	 * @param a the array into which the elements of this set are to
	 *		be stored, if it is big enough; otherwise, a new array of the
	 * 		same runtime type is allocated for this purpose.
	 * @param connection used to access the data store
	 * @return an array containing the elements of this set.
	 * @throws    ArrayStoreException the runtime type of a is not a supertype
	 * of the runtime type of every element in this set.
	 * @throws PException a internal jorm error occured
	 */
	public Object[] toArray(Object a[], Object connection) throws PException;

	/**
	 * Adds the specified element to this set if it is not already present
	 * (optional operation).  More formally, adds the specified element,
	 * <code>o</code>, to this set if this set contains no element
	 * <code>e</code> such that <code>(o==null ? e==null :
	 * o.equals(e))</code>.  If this set already contains the specified
	 * element, the call leaves this set unchanged and returns <tt>false</tt>.
	 * In combination with the restriction on constructors, this ensures that
	 * sets never contain duplicate elements.<p>
	 *
	 * The stipulation above does not imply that sets must accept all
	 * elements; sets may refuse to add any particular element, including
	 * <tt>null</tt>, and throwing an exception, as described in the
	 * specification for <tt>Collection.add</tt>.  Individual set
	 * implementations should clearly document any restrictions on the the
	 * elements that they may contain.
	 *
	 * @param o element to be added to this set.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this set did not already contain the specified
	 *         element.
	 *
	 * @throws UnsupportedOperationException if the <tt>add</tt> method is not
	 * 	       supported by this set.
	 * @throws ClassCastException if the class of the specified element
	 * 	       prevents it from being added to this set.
	 * @throws IllegalArgumentException if some aspect of this element
	 *         prevents it from being added to this set.
	 * @throws PException a internal jorm error occured
	 */
	public boolean add(Object o, Object connection) throws PException;

	/**
	 * Removes the specified element from this set if it is present (optional
	 * operation).  More formally, removes an element <code>e</code> such that
	 * <code>(o==null ?  e==null : o.equals(e))</code>, if the set contains
	 * such an element.  Returns <tt>true</tt> if the set contained the
	 * specified element (or equivalently, if the set changed as a result of
	 * the call).  (The set will not contain the specified element once the
	 * call returns.)
	 *
	 * @param o object to be removed from this set, if present.
	 * @param connection used to access the data store
	 * @return true if the set contained the specified element.
	 * @throws UnsupportedOperationException if the <tt>remove</tt> method is
	 *         not supported by this set.
	 * @throws PException a internal jorm error occured
	 */
	public boolean remove(Object o, Object connection) throws PException;

	/**
	 * Returns <tt>true</tt> if this set contains all of the elements of the
	 * specified collection.  If the specified collection is also a set, this
	 * method returns <tt>true</tt> if it is a <i>subset</i> of this set.
	 *
	 * @param c collection to be checked for containment in this set.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this set contains all of the elements of the
	 * 	       specified collection.
	 * @throws PException a internal jorm error occured
	 */
	public boolean containsAll(Set c, Object connection) throws PException;

	/**
	 * Adds all of the elements in the specified collection to this set if
	 * they're not already present (optional operation).  If the specified
	 * collection is also a set, the <tt>addAll</tt> operation effectively
	 * modifies this set so that its value is the <i>union</i> of the two
	 * sets.  The behavior of this operation is unspecified if the specified
	 * collection is modified while the operation is in progress.
	 *
	 * @param c collection whose elements are to be added to this set.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this set changed as a result of the call.
	 *
	 * @throws UnsupportedOperationException if the <tt>addAll</tt> method is
	 * 		  not supported by this set.
	 * @throws ClassCastException if the class of some element of the
	 * 		  specified collection prevents it from being added to this
	 * 		  set.
	 * @throws IllegalArgumentException if some aspect of some element of the
	 *		  specified collection prevents it from being added to this
	 *		  set.
	 * @throws PException a internal jorm error occured
	 * @see #add(Object)
	 */
	public boolean addAll(Set c, Object connection) throws PException;

	/**
	 * Retains only the elements in this set that are contained in the
	 * specified collection (optional operation).  In other words, removes
	 * from this set all of its elements that are not contained in the
	 * specified collection.  If the specified collection is also a set, this
	 * operation effectively modifies this set so that its value is the
	 * <i>intersection</i> of the two sets.
	 *
	 * @param c collection that defines which elements this set will retain.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection changed as a result of the
	 *         call.
	 * @throws UnsupportedOperationException if the <tt>retainAll</tt> method
	 * 		  is not supported by this Collection.
	 * @throws PException a internal jorm error occured
	 *
	 * @see #remove(Object)
	 */
	public boolean retainAll(Set c, Object connection) throws PException;

	/**
	 * Removes from this set all of its elements that are contained in the
	 * specified collection (optional operation).  If the specified
	 * collection is also a set, this operation effectively modifies this
	 * set so that its value is the <i>asymmetric set difference</i> of
	 * the two sets.
	 *
	 * @param c collection that defines which elements will be removed from
	 *          this set.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this set changed as a result of the call.
	 *
	 * @throws UnsupportedOperationException if the <tt>removeAll</tt>
	 * 		  method is not supported by this Collection.
	 * @throws PException a internal jorm error occured
	 *
	 * @see #remove(Object) */
	public boolean removeAll(Set c, Object connection) throws PException;

	/**
	 * Removes all of the elements from this set (optional operation).
	 * This set will be empty after this call returns (unless it throws an
	 * exception).
	 *
	 * @param connection used to access the data store
	 * @throws UnsupportedOperationException if the <tt>clear</tt> method
	 * 		  is not supported by this set.
	 * @throws PException a internal jorm error occured
	 */
	public void clear(Object connection) throws PException;
}
