/**
 * Copyright (C) 2000
 */

package org.objectweb.jorm.facility.meminst.genclass.java.util;

import org.objectweb.jorm.api.PAccessorGenClass;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PIndexedElemFactory;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.facility.meminst.lifecycle.lazyload.LazyLoad;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.ArrayList;
import java.util.Iterator;
import javax.resource.ResourceException;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionFactory;

/**
 * @author	P. Dchamboux
 */
public class Collection
	implements java.util.Collection,
	PAccessorGenClass,
	org.objectweb.jorm.facility.meminst.genclass.api.Collection,
	LazyLoad,
	Loggable {

	/**
	 * The ArrayList used to store the indexed elements of the genclass.
	 * Then this list contains contains also the deleted elements.
	 */
	protected ArrayList elements = null;

	/**
	 * The type code of the elements embeded in the genclass.
	 */
	protected short etype = -1;

	/**
	 * A temporary variable which is used to search an element into the
	 * "elements" list.
	 */
	protected PIndexedElem tmpelem = null;

	/**
	 * The logger in which the traces will be send.
	 */
	protected Logger logger = null;

	protected PIndexedElemFactory pieFactory = null;

	protected PBinding pBinding = null;

	protected ConnectionFactory connectionFactory = null;

	protected boolean loaded = false;

	/**
	 *  instanciates a new set that must be initialized with the init method.
	 */
	public Collection(PIndexedElemFactory pief) {
		pieFactory = pief;
		tmpelem = pief.createPIndexedElem();
		elements = new ArrayList();
	}

	/**
	 * <b>Collection</b> instanciate and initialize a new collection with an
	 * initial set size
	 */
	public Collection(int size, PIndexedElemFactory pief) {
		pieFactory = pief;
		tmpelem = pief.createPIndexedElem();
		elements = new ArrayList(size);
	}

	public void setConnectionFactory(ConnectionFactory connectionFactory) {
		this.connectionFactory = connectionFactory;
	}

	/**
	 * <b>initGenSet</b>  initialize this set with an initial set size
	 * @param size is the initial size
	 */
	public void init(int size, PIndexedElemFactory pief, ConnectionFactory cf) {
		connectionFactory = cf;
		pieFactory = pief;
		elements = new ArrayList(size);
	}

	public PBinding getPBinding() {
		return pBinding;
	}

	public void setPBinding(PBinding pBinding) {
		this.pBinding = pBinding;
	}

	/**
	 * Clean the indexed element embeded in the set.
	 * Elements status are changed from the SingleIndexedElem.ELEM_CREATED to SingleIndexedElem.ELEM_UNMODIFIED.
	 * Elements having the status SingleIndexedElem.ELEM_DELETED are deleted in the mi.
	 * This method is call by the pbinding after a write.
	 */

	public void cleanElements() {
		Iterator it = elements.iterator();
		CollectionElem tmp;
		while (it.hasNext()) {
			tmp = (CollectionElem) it.next();
			if (tmp.getElemStatus() == PIndexedElem.ELEM_CREATED)
				tmp.setStatus(PIndexedElem.ELEM_UNMODIFIED);
			else if (tmp.getElemStatus() == PIndexedElem.ELEM_DELETED)
				// remove the element
				it.remove();
		}
	}

	// IMPLEMENTATION OF THE java.util.Collection INTERFACE //
	//------------------------------------------------------//

	/**
	 * Adds the specified element to this collection.
	 * @param o the object to be added.
	 */
	public boolean add(Object o) {
		try {
			return add(o, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Adds all of the elements in the specified collection to this set
	 * if they're not already present (implemented optional operation).
	 * @param c the collection to be added.
	 */
	public boolean addAll(java.util.Collection c) {
		try {
			Iterator it = c.iterator();
			boolean changed = false;
			while (it.hasNext()) {
				changed = add(it.next(), null) || changed;
			}
			return changed;
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * <b>clear</b>
	 * Removes all of the elements from this set (implemented
	 * optional operation).
	 */
	public void clear() {
		try {
			clear(null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Returns true if this set contains the specified element.
	 */
	public boolean contains(Object o) {
		try {
			return contains(o, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Returns true if this set contains all of the elements of the
	 * specified collection.
	 */
	public boolean containsAll(java.util.Collection c) {
		try {
			return containsAll((Collection) c, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Compares the specified object with this set for equality.
	 * two set are defined to be equal if they contain the same elements.
	 *
	 * This implementation checks if the 'o' parameter is a Set and it has the
	 * same size and finaly if all elements are in the current set.
	 */
	public boolean equals(Object o) {
		try {
			return (o instanceof Collection)
				&& ((Collection) o).size(null) == size(null)
				&& containsAll((Collection) o, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Returns true if this set contains no elements.
	 */
	public boolean isEmpty() {
		try {
			return isEmpty(null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Returns an iterator over the elements in this set.
	 */
	public Iterator iterator() {
		try {
			return iterator(null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * <b>remove</b>
	 * Removes the specified element from this set if it is present
	 * (implemented optional operation).
	 */
	public boolean remove(Object o) {
		try {
			return remove(o, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * Removes from this set all of its elements that are contained
	 * in the specified collection (non-implemented optional operation).
	 */
	public boolean removeAll(java.util.Collection c) {
		try {
			return removeAll((Collection) c, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * <b>retainAll</b>
	 * Retains only the elements in this set that are contained in
	 * the specified collection (non-implemented optional operation).
	 */
	public boolean retainAll(java.util.Collection c) {
		try {
			return retainAll((Collection) c, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * <b>size</b>
	 * Returns the number of elements in this set (its cardinality).
	 */
	public int size() {
		try {
			return size(null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * <b>toArray</b>
	 * Returns an array containing all of the elements in this set.
	 */
	public Object[] toArray() {
		try {
			return toArray((Object) null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}

	/**
	 * <b>toArray(a)</b>
	 * Returns an array containing all of the elements in this set
	 * whose runtime type is that of the specified array.
	 */
	public Object[] toArray(Object[] a) {
		try {
			return toArray(a, null);
		}
		catch (PException pe) {
			throw new InternalJormException(pe);
		}
	}


	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	/**
	 * It assigns a logger to an component that wants to log things.
	 * @param Logger	The logger object.
	 */
	public void setLogger(Logger _logger) {
		logger = _logger;
	}

	/**
	 * It assigns a logger factory that allows the creation of new loggers.
	 * @param loggerfactory	The LoggerFactory object to obtain new loggers.
	 */
	public void setLoggerFactory(LoggerFactory loggerfactory) {
		logger = loggerfactory.getLogger(this.getClass().getName());
	}

	/**
	 * Returns a logger to an component that wants to log things.
	 *
	 * @param logger	the logger object
	 */
	public Logger getLogger() {
		return logger;
	}

	/**
	 * Returns a logger factory that allows the creation of new loggers.
	 *
	 * @param loggerfactory	the LoggerFactory object to obtain new loggers
	 */
	public LoggerFactory getLoggerFactory() {
		return null;
	}

	// IMPLEMENTATION OF THE PAccessor INTERFACE //
	//-------------------------------------------//

	/**
	 * implementation of the PAccessor interface.
	 * @see org.objectweb.jorm.api.PAccessor.
	 */
	public Object getMemoryInstance() {
		//refresh();
		return this; // pa = mi
	}


	// IMPLEMENTATION OF THE PAccessorGenClass INTERFACE //
	//---------------------------------------------------//

	/**
	 * Implementation of the PAccessorGenClass.
	 * @see org.objectweb.jorm.api.PAccessorGenClass
	 */
	public void paAdd(PIndexedElem elem, Object conn) throws PExceptionIO {
		// the elem is read from the DS, set it to unmodified.
		((CollectionElem) (elem))
			.setStatus(PIndexedElem.ELEM_UNMODIFIED);
		elements.add(elem);
	}

	/**
	 * Implementation of the PAccessorGenClass.
	 * @see org.objectweb.jorm.api.PAccessorGenClass
	 */
	public boolean paDeltaSupported() {
		return true;
	}

	/**
	 * Implementation of the PAccessorGenClass.
	 * @see org.objectweb.jorm.api.PAccessorGenClass
	 */
	public int paGetNbElem() {
		return size();
	}

	/**
	 * Implementation of the PAccessorGenClass.
	 * @see org.objectweb.jorm.api.PAccessorGenClass
	 */
	public Iterator paIterator() {
		return elements.iterator();
	}

	/**
	 * Implementation of the PAccessorGenClass.
	 * @see org.objectweb.jorm.api.PAccessorGenClass
	 */
	public void paSetNbElem(int nbelem) {
		init(nbelem, pieFactory, connectionFactory);
	}

	// IMPLEMENTATION OF THE PIndexedElemFactory INTERFACE //
	//---------------------------------------------------//

	/**
	 * Implementation of the PAccessorGenClass.
	 * @see org.objectweb.jorm.api.PAccessorGenClass
	 */
	public PIndexedElem createPIndexedElem() {
		return pieFactory.createPIndexedElem();
	}

	// IMPLEMENTATION OF THE
	// org.objectweb.jorm.facility.meminst.genclass.api.Collection INTERFACE //
	//-----------------------------------------------------------------------//
	/**
	 * Returns the number of elements in this collection.  If this collection
	 * contains more than <tt>Integer.MAX_VALUE</tt> elements, returns
	 * <tt>Integer.MAX_VALUE</tt>.
	 *
	 * @param connection used to access the data store
	 * @return the number of elements in this collection
	 * @throws PException a internal jorm error occured
	 */
	public int size(Object connection) throws PException {
		Iterator it = elements.iterator();
		int size = 0;
		while (it.hasNext())
			if (((PIndexedElem) it.next())
				.getElemStatus() != PIndexedElem.ELEM_DELETED)
				size++;
		return size;
	}

	/**
	 * Returns <tt>true</tt> if this collection contains no elements.
	 * @param connection used to access the data store
	 *
	 * @return <tt>true</tt> if this collection contains no elements
	 */
	public boolean isEmpty(Object connection) throws PException {
		if (elements.isEmpty())
			return true;
		Iterator it = elements.iterator();
		boolean empty = true;
		// The set is empty if it contains only deleted indexed elements
		while (it.hasNext() && empty)
			empty &= ((PIndexedElem) it.next()).getElemStatus() == PIndexedElem.ELEM_DELETED;
		return empty;
	}

	/**
	 * Returns <tt>true</tt> if this collection contains the specified
	 * element.  More formally, returns <tt>true</tt> if and only if this
	 * collection contains at least one element <tt>e</tt> such that
	 * <tt>(o==null ? e==null : o.equals(e))</tt>.
	 *
	 * @param connection used to access the data store
	 * @param o element whose presence in this collection is to be tested.
	 * @return <tt>true</tt> if this collection contains the specified
	 *         element
	 * @throws PException a internal jorm error occured
	 */
	public boolean contains(Object o, Object connection) throws PException {
		// Search the element in the "elements" list
		try {
			((PIndexedElem) tmpelem).pieSetRefElem(o);
		}
		catch (PExceptionTyping e) {
			if (logger != null)
				logger.log(BasicLevel.ERROR, "", e);
			throw new UnsupportedOperationException(e.getMessage());
		}
		Iterator it = elements.iterator();
		while (it.hasNext()) {
			PIndexedElem e = (PIndexedElem) it.next();
			if (e.equals(tmpelem) && e.getElemStatus() != PIndexedElem.ELEM_DELETED)
				return true;
		}
		return false;
	}

	/**
	 * Returns an iterator over the elements in this collection.  There are no
	 * guarantees concerning the order in which the elements are returned
	 * (unless this collection is an instance of some class that provides a
	 * guarantee).
	 *
	 * @param connection used to access the data store
	 * @return an <tt>Iterator</tt> over the elements in this collection
	 * @throws PException a internal jorm error occured
	 */
	public Iterator iterator(Object connection) throws PException {
		return new PIndexedElemIterator(elements, this, connection, logger);
	}

	/**
	 * Returns an array containing all of the elements in this collection.  If
	 * the collection makes any guarantees as to what order its elements are
	 * returned by its iterator, this method must return the elements in the
	 * same order.<p>
	 *
	 * The returned array will be "safe" in that no references to it are
	 * maintained by this collection.  (In other words, this method must
	 * allocate a new array even if this collection is backed by an array).
	 * The caller is thus free to modify the returned array.<p>
	 *
	 * This method acts as bridge between array-based and collection-based
	 * APIs.
	 *
	 * @param connection used to access the data store
	 * @return an array containing all of the elements in this collection
	 * @throws PException a internal jorm error occured
	 */
	public Object[] toArray(Object connection) throws PException {
		throw new UnsupportedOperationException();
	}

	/**
	 * Returns an array containing all of the elements in this collection
	 * whose runtime type is that of the specified array.  If the collection
	 * fits in the specified array, it is returned therein.  Otherwise, a new
	 * array is allocated with the runtime type of the specified array and the
	 * size of this collection.<p>
	 *
	 * If this collection fits in the specified array with room to spare
	 * (i.e., the array has more elements than this collection), the element
	 * in the array immediately following the end of the collection is set to
	 * <tt>null</tt>.  This is useful in determining the length of this
	 * collection <i>only</i> if the caller knows that this collection does
	 * not contain any <tt>null</tt> elements.)<p>
	 *
	 * If this collection makes any guarantees as to what order its elements
	 * are returned by its iterator, this method must return the elements in
	 * the same order.<p>
	 *
	 * Like the <tt>toArray</tt> method, this method acts as bridge between
	 * array-based and collection-based APIs.  Further, this method allows
	 * precise control over the runtime type of the output array, and may,
	 * under certain circumstances, be used to save allocation costs<p>
	 *
	 * Suppose <tt>l</tt> is a <tt>List</tt> known to contain only strings.
	 * The following code can be used to dump the list into a newly allocated
	 * array of <tt>String</tt>:
	 *
	 * <pre>
	 *     String[] x = (String[]) v.toArray(new String[0]);
	 * </pre><p>
	 *
	 * Note that <tt>toArray(new Object[0])</tt> is identical in function to
	 * <tt>toArray()</tt>.
	 *
	 * @param connection used to access the data store
	 * @param a the array into which the elements of this collection are to be
	 *        stored, if it is big enough; otherwise, a new array of the same
	 *        runtime type is allocated for this purpose.
	 * @return an array containing the elements of this collection
	 *
	 * @throws ArrayStoreException the runtime type of the specified array is
	 *         not a supertype of the runtime type of every element in this
	 *         collection.
	 * @throws PException a internal jorm error occured
	 */

	public Object[] toArray(Object a[], Object connection) throws PException {
		throw new UnsupportedOperationException();
	}

	/**
	 * Ensures that this collection contains the specified element (optional
	 * operation).  Returns <tt>true</tt> if this collection changed as a
	 * result of the call.  (Returns <tt>false</tt> if this collection does
	 * not permit duplicates and already contains the specified element.)<p>
	 *
	 * Collections that support this operation may place limitations on what
	 * elements may be added to this collection.  In particular, some
	 * collections will refuse to add <tt>null</tt> elements, and others will
	 * impose restrictions on the type of elements that may be added.
	 * Collection classes should clearly specify in their documentation any
	 * restrictions on what elements may be added.<p>
	 *
	 * If a collection refuses to add a particular element for any reason
	 * other than that it already contains the element, it <i>must</i> throw
	 * an exception (rather than returning <tt>false</tt>).  This preserves
	 * the invariant that a collection always contains the specified element
	 * after this call returns.
	 *
	 * @param o element whose presence in this collection is to be ensured.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection changed as a result of the
	 *         call
	 *
	 * @throws UnsupportedOperationException add is not supported by this
	 *         collection.
	 * @throws ClassCastException class of the specified element prevents it
	 *         from being added to this collection.
	 * @throws IllegalArgumentException some aspect of this element prevents
	 *          it from being added to this collection.
	 * @throws PException a internal jorm error occured
	 */
	public synchronized boolean add(Object o, Object connection) throws PException {
		// Put it in the "elements" list
		try {
			PIndexedElem ie = createPIndexedElem();
			ie.pieSetRefElem(o);
			return elements.add(ie);
		}
		catch (PExceptionTyping e) {
			if (logger != null)
				logger.log(BasicLevel.ERROR, "", e);
			throw new UnsupportedOperationException(e.getMessage());
		}
	}

	/**
	 * Removes a single instance of the specified element from this
	 * collection, if it is present (optional operation).  More formally,
	 * removes an element <tt>e</tt> such that <tt>(o==null ?  e==null :
	 * o.equals(e))</tt>, if this collection contains one or more such
	 * elements.  Returns true if this collection contained the specified
	 * element (or equivalently, if this collection changed as a result of the
	 * call).
	 *
	 * @param o element to be removed from this collection, if present.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection changed as a result of the
	 *         call
	 *
	 * @throws UnsupportedOperationException remove is not supported by this
	 *         collection.
	 * @throws PException a internal jorm error occured
	 */
	public synchronized boolean remove(Object o, Object connection) throws PException {
		// Search the element in the "elements" list
		try {
			((PIndexedElem) tmpelem).pieSetRefElem(o);   // tmp type = etype.
		}
		catch (PExceptionTyping e) {
			e.printStackTrace();
		}
		Iterator it = elements.iterator();
		while (it.hasNext()) {
			PIndexedElem e = (PIndexedElem) it.next();
			if (e.equals(tmpelem) && e.getElemStatus() != PIndexedElem.ELEM_DELETED) {
				((CollectionElem) e).setStatus(PIndexedElem.ELEM_DELETED);
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns <tt>true</tt> if this collection contains all of the elements
	 * in the specified collection.
	 *
	 * @param c collection to be checked for containment in this collection.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection contains all of the elements
	 *	       in the specified collection
	 * @throws PException a internal jorm error occured
	 * @see #contains(Object)
	 */
	public boolean containsAll(
		org.objectweb.jorm.facility.meminst.genclass.api.Collection c,
		Object connection) throws PException {

		Iterator it = c.iterator(connection);
		boolean contain = true;
		while (it.hasNext() && contain)
			contain &= contains(it.next());
		return contain;
	}

	/**
	 * Adds all of the elements in the specified collection to this collection
	 * (optional operation).  The behavior of this operation is undefined if
	 * the specified collection is modified while the operation is in progress.
	 * (This implies that the behavior of this call is undefined if the
	 * specified collection is this collection, and this collection is
	 * nonempty.)
	 *
	 * @param c elements to be inserted into this collection.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection changed as a result of the
	 *         call
	 *
	 * @throws UnsupportedOperationException if this collection does not
	 *         support the <tt>addAll</tt> method.
	 * @throws ClassCastException if the class of an element of the specified
	 * 	       collection prevents it from being added to this collection.
	 * @throws IllegalArgumentException some aspect of an element of the
	 *	       specified collection prevents it from being added to this
	 *	       collection.
	 * @throws PException a internal jorm error occured
	 *
	 * @see #add(Object)
	 */
	public boolean addAll(
		org.objectweb.jorm.facility.meminst.genclass.api.Collection c,
		Object connection) throws PException {

		boolean hasChanged = false;
		Iterator it = c.iterator(null);
		while (it.hasNext())
			hasChanged |= add(it.next());
		return hasChanged;
	}

	/**
	 *
	 * Removes all this collection's elements that are also contained in the
	 * specified collection (optional operation).  After this call returns,
	 * this collection will contain no elements in common with the specified
	 * collection.
	 *
	 * @param c elements to be removed from this collection.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection changed as a result of the
	 *         call
	 *
	 * @throws UnsupportedOperationException if the <tt>removeAll</tt> method
	 * 	       is not supported by this collection.
	 * @throws PException a internal jorm error occured
	 *
	 * @see #remove(Object)
	 * @see #contains(Object)
	 */
	public boolean removeAll(
		org.objectweb.jorm.facility.meminst.genclass.api.Collection c,
		Object connection) throws PException {

		Iterator it = c.iterator(null);
		boolean removed = true;
		while (it.hasNext())
			removed &= remove(it.next());
		return removed;
	}

	/**
	 * Retains only the elements in this collection that are contained in the
	 * specified collection (optional operation).  In other words, removes from
	 * this collection all of its elements that are not contained in the
	 * specified collection.
	 *
	 * @param c elements to be retained in this collection.
	 * @param connection used to access the data store
	 * @return <tt>true</tt> if this collection changed as a result of the
	 *         call
	 *
	 * @throws UnsupportedOperationException if the <tt>retainAll</tt> method
	 * 	       is not supported by this Collection.
	 * @throws PException a internal jorm error occured
	 *
	 * @see #remove(Object)
	 * @see #contains(Object)
	 */
	public boolean retainAll(
		org.objectweb.jorm.facility.meminst.genclass.api.Collection c,
		Object connection) throws PException {
		throw new UnsupportedOperationException();
	}

	/**
	 * Removes all of the elements from this collection (optional operation).
	 * This collection will be empty after this method returns unless it
	 * throws an exception.
	 *
	 * @param connection used to access the data store
	 * @throws UnsupportedOperationException if the <tt>clear</tt> method is
	 *         not supported by this collection.
	 * @throws PException a internal jorm error occured
	 */
	public void clear(Object connection) throws PException {
		Iterator it = elements.iterator();
		while (it.hasNext())
			remove(((PIndexedElem) it.next()).pieGetRefElem());
	}

	// IMPLEMENTATION OF THE LazyLoad INTERFACE //
	//------------------------------------------//

	public void store(Object conn) throws PException {
		if (Debug.ON && logger!=null)
			logger.log(BasicLevel.DEBUG, "begin");
		loaded = true;
		Connection c = null;
		try {
			if (conn == null) {
				if (connectionFactory != null)
					c = connectionFactory.getConnection();
				else
					throw new PExceptionIO("Impossible to obtain a new connection: The connection factory is not set.");
			}
			pBinding.write((conn==null?c:conn));
			cleanElements();
		}
		catch (ResourceException re) {
			throw new PExceptionIO(re, "Impossible to obtain a new connection");
		}
		finally {
			if (conn == null && c != null) {
				try {
					c.close();
				}
				catch (ResourceException re) {
					if (logger != null)
						logger.log(BasicLevel.FATAL, "Cannot close a connection !!");
				}
			}
		}
		if (Debug.ON && logger!=null)
			logger.log(BasicLevel.DEBUG, "end");
	}

	public boolean isLoaded() {
		return loaded;
	}

	public void load(Object conn) throws PException {
		if (Debug.ON && logger!=null)
			logger.log(BasicLevel.DEBUG, "begin");

		if (!loaded) {
			Connection c = null;
			try {
				if (conn == null) {
					if (connectionFactory != null)
						c = connectionFactory.getConnection();
					else
						throw new PExceptionIO("Impossible to obtain a new connection: The connection factory is not set.");
				}
				pBinding.read((conn==null?c:conn));
				loaded = true;
			}
			catch (ResourceException re) {
				throw new PExceptionIO(re, "Impossible to obtain a new connection");
			}
			finally {
				if (conn == null && c != null) {
					try {
						c.close();
					}
					catch (ResourceException re) {
						if (logger != null)
							logger.log(BasicLevel.FATAL, "Cannot close a connection !!");
					}
				}
			}
		}
		if (Debug.ON && logger!=null)
			logger.log(BasicLevel.DEBUG, "end");
	}

	private class InternalJormException extends RuntimeException {
		PException nestedException = null;

		public InternalJormException(PException pe) {
			nestedException = pe;
		}

		public PException getNestedException() {
			return nestedException;
		}

		public String getMessage() {
			return nestedException.getMessage();
		}

		public void printStackTrace() {
			nestedException.printStackTrace();
		}

		public void printStackTrace(java.io.PrintStream s) {
			nestedException.printStackTrace(s);
		}

		public void printStackTrace(java.io.PrintWriter s) {
			nestedException.printStackTrace(s);
		}
	}
}
