/**
 * Copyright (C) 2000
 */

package org.objectweb.jorm.facility.meminst.genclass.java.util;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.facility.meminst.genclass.api.UserIEValue;
import org.objectweb.jorm.type.api.PExceptionTyping;

import java.io.Serializable;
import java.util.Date;

/**
 * @author	P. Dchamboux
 */
public class CollectionElem implements PIndexedElem, UserIEValue {

	/**
	 * The error message thrown when an accessor method associated to a
	 * primitive element type is called. Indeed this implementation supports
	 * only object elements.
	 */
	public final static String ERROR_MESSAGE_ONLY_OBJECTS =
		"This implementation supports only objects";

	/**
	 * The error message thrown when an accessor method associated to a
	 * primitive element type is called. Indeed this implementation supports
	 * only object elements.
	 */
	public final static String ERROR_MESSAGE_BAD_FIELD_NAME =
		"The index field name is not";

	/**
	 * This constant is the name of the index field. This value must be use with
	 * the methods associated to the management of the index.
	 */
	public final static String INDEX_FIELD_NAME = "id";

	/**
	 * This field is the element.
	 */
	protected Object element;

	/**
	 * this file is the type code of the single index
	 */
	byte indexTypeCode = -1;

	/**
	 * This field can be the index. The index type is known at instanciation
	 * time.
	 */
	protected char charIndex = '\0';

	/**
	 * This field can be the index. The index type is known at instanciation
	 * time.
	 */
	protected long longIndex = 0;

	/**
	 * This field can be the index. The index type is known at instanciation
	 * time.
	 */
	protected double doubleIndex = 0.0;

	/**
	 * This field can be the index. The index type is known at instanciation
	 * time.
	 */
	protected Object objectIndex = null;

	/**
	 * This field represents the satus of the PIndexedEleme. The possible value
	 * are ELEM_CREATED, ELEM_DELETED, ELEM_MODIFIED, ELEM_UNMODIFIED
	 */
	protected byte status = ELEM_CREATED;

	public void setStatus(byte s) {
		status = s;
	}

	public boolean equals(Object o) {
		try {
			return (o instanceof PIndexedElem) && element.equals(((PIndexedElem) o).pieGetRefElem());
		} catch (PException pe) {
			return false;
		}
	}

	// IMPLEMENTATION OF THE PIndexedElem INTERFACE //
	//----------------------------------------------//

	public byte getElemStatus() {
		return status;
	}

	public String pieGetStringElem() throws PExceptionTyping {
		return (String) element;
	}

	public Date pieGetDateElem() throws PExceptionTyping {
		return (Date) element;
	}

	public Serializable pieGetSerializedElem() throws PExceptionTyping {
		return (Serializable) element;
	}

	public Object pieGetRefElem() throws PExceptionTyping {
		return element;
	}

	public void pieSetStringElem(String value) throws PExceptionTyping {
		element = value;
	}

	public void pieSetDateElem(Date value) throws PExceptionTyping {
		element = value;
	}

	public void pieSetSerializedElem(Serializable value) throws PExceptionTyping {
		element = value;
	}

	public void pieSetRefElem(Object value) throws PExceptionTyping {
		element = value;
	}

	public boolean pieGetBooleanElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public byte pieGetByteElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public char pieGetCharElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public short pieGetShortElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public int pieGetIntElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public long pieGetLongElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public float pieGetFloatElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public double pieGetDoubleElem() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetBooleanElem(boolean value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetByteElem(byte value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetCharElem(char value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetShortElem(short value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetIntElem(int value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetLongElem(long value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetFloatElem(float value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public void pieSetDoubleElem(double value) throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	//----- Index accessor -----//
	//--------------------------//
	public void pieSetByteIndexField(String fn, byte value) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		longIndex = value;
	}

	public void pieSetCharIndexField(String fn, char value) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		charIndex = value;
	}

	public void pieSetShortIndexField(String fn, short value) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		longIndex = value;
	}

	public void pieSetIntIndexField(String fn, int value) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		longIndex = value;
	}

	public void pieSetLongIndexField(String fn, long value) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		longIndex = value;
	}

	public void pieSetStringIndexField(String fn, String value) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		objectIndex = value;
	}

	public short pieGetShortIndexField(String fn) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		return (short) longIndex;
	}

	public long pieGetLongIndexField(String fn) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		return longIndex;
	}

	public int pieGetIntIndexField(String fn) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		return (int) longIndex;
	}

	public String pieGetStringIndexField(String fn) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		return (String) objectIndex;
	}

	public byte pieGetByteIndexField(String fn) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		return (byte) longIndex;
	}

	public char pieGetCharIndexField(String fn) throws PExceptionTyping {
		if (!INDEX_FIELD_NAME.equals(fn))
			throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
		return charIndex;
	}

	// IMPLEMENTATION OF THE UserIEValue INTERFACE //
	//---------------------------------------------//
	public boolean getBooleanValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public byte getByteValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public short getShortValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public int getIntValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public long getLongValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public float getFloatValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public double getDoubleValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public char getCharValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public String getStringValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public java.util.Date getDateValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public Serializable getSerializedValue() throws PExceptionTyping {
		throw new PExceptionTyping(ERROR_MESSAGE_ONLY_OBJECTS);
	}

	public Object getRefValue(Object conn) throws PException {
		return element;
	}
}
