/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.facility.meminst.genclass.java.util;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.facility.meminst.genclass.api.UserIEValue;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class PIndexedElemIterator implements Iterator {

	ArrayList elements = null;
	Object synchro = null;
	int cursor = 0;
	int next = 0;
	Object connection = null;
	Logger logger = null;

	public PIndexedElemIterator(ArrayList _elements,
								Object _synchro,
								Object conn,
								Logger l) {
		logger = l;
		synchro = _synchro;
		elements = _elements;
		reset();
		connection = conn;
		if (Debug.ON && logger!=null)
			logger.log(BasicLevel.DEBUG,
				"Connection=" + (connection!=null ? "ok" : "null"));
	}

	public void reset() {
		cursor = -1;
		next = nextExist(cursor);
	}

	private int nextExist(int pos) {
		int tmp = pos + 1;
		while (tmp < elements.size()) {
			if (((PIndexedElem) elements.get(tmp))
				.getElemStatus() != PIndexedElem.ELEM_DELETED) {
				if (Debug.ON && logger!=null)
					logger.log(BasicLevel.DEBUG, "nextExist("+pos+")="+tmp);
				return tmp;
			}
			tmp++;
		}
		if (Debug.ON && logger!=null)
			logger.log(BasicLevel.DEBUG, "nextExist("+pos+")=-1");
		return -1;
	}

	// IMPLEMENTATION OF THE Iterator INTERFACE //
	//------------------------------------------//
	/**
	 * Returns <tt>true</tt> if the iteration has more elements. (In other
	 * words, returns <tt>true</tt> if <tt>next</tt> would return an element
	 * rather than throwing an exception.)
	 *
	 * @return <tt>true</tt> if the iterator has more elements.
	 */
	public boolean hasNext() {
		next = nextExist(cursor);
		return next != -1;
	}

	/**
	 * Returns the next element in the interation.
	 *
	 * @return the next element in the iteration.
	 * @exception NoSuchElementException iteration has no more elements.
	 */
	public Object next() {
		if (next == -1)
			throw new NoSuchElementException();
		if (next == cursor)
			next = nextExist(cursor);
		cursor = next;
		if (cursor == -1)
			throw new NoSuchElementException();
		try {
			return ((UserIEValue) elements.get(cursor)).getRefValue(connection);
		}
		catch (PException e) {
			if (logger!=null)
				logger.log(BasicLevel.ERROR, "Impossible to obtain value:", e);
			return null;
		}
	}

	/**
	 *
	 * Removes from the underlying collection the last element returned by the
	 * iterator (optional operation).  This method can be called only once per
	 * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
	 * the underlying collection is modified while the iteration is in
	 * progress in any way other than by calling this method.
	 *
	 * @exception UnsupportedOperationException if the <tt>remove</tt>
	 *		  operation is not supported by this Iterator.

	 * @exception IllegalStateException if the <tt>next</tt> method has not
	 *		  yet been called, or the <tt>remove</tt> method has already
	 *		  been called after the last call to the <tt>next</tt>
	 *		  method.
	 */
	public void remove() {
		synchronized (synchro) {
			((CollectionElem) elements.get(cursor))
				.setStatus(PIndexedElem.ELEM_DELETED);
		}
	}
}
