/**
 * Copyright (C) 2000 
 */

package org.objectweb.jorm.facility.meminst.manager.basic.lib;

import org.objectweb.jorm.facility.meminst.manager.basic.api.Manager;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;


/**
 * @author	Sebastien Chassande-Barrioz
 */
public abstract class ManagerImpl implements Manager, Loggable {

	protected Logger logger = null;
	protected LoggerFactory loggerFactory = null;

	// IMPLEMENTATION OF THE Manager INTERFACE //
	//-----------------------------------------------//
	/**
	 * This method permits to obtain a PBinding linked to a PAccessor and bound
	 * to the PName specified in parameter.
	 */
	public PBinding getPBinding(Object conn, PName pn)  throws PException {
		if (logger==null)
			logger = getLogger();
		if (loggerFactory==null)
			loggerFactory = getLoggerFactory();

		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "");

		Object o = recurseResolve(conn, pn);
		PBinding pb = null;
		if (o instanceof PBinding) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG,
					"The PBinding was found in the binder");
			pb = (PBinding) o;
			if (!bindingExtendsAccessor()) {
				if (isGenericAccessor()) {
					if (pb.getPAccessorGen()==null)
						pb.setPAccessorGen((PAccessorGen) createPAccessor());
				}
				else {
					if (pb.getPAccessor()==null)
						pb.setPAccessor(createPAccessor());
				}
			}
			return pb;
		}
		else if (o instanceof PName) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG,
					"The PBinding was not found in the binder, last pname :" + o);

			PName lastpn = (PName) o;

			// create a PBinding
			pb = createPBinding((PClassMapping) lastpn.getPNamingContext());

			// Link the PBindding to a PAccessor
			PAccessor pa = null;
			if (bindingExtendsAccessor()) {
				if (isGenericAccessor())
					pb.setPAccessorGen((PAccessorGen) pb);
				else
					pb.setPAccessor((PAccessor) pb);
			}
			else {
				if (isGenericAccessor())
					pb.setPAccessorGen((PAccessorGen) createPAccessor());
				else
					pb.setPAccessor(createPAccessor());
			}

			// bind the PBinding to the PName
			pb.bind(lastpn);

			if (pb.exist(conn)) {
				if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG,
						"The instance exists in the support => load it");
				load(pb, conn);
			}
			else {
				if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG,
						"The instance doesn't exist in the support => store it");

				// Store the data in the persitent support
				store(pb, conn);
			}
			return pb;
		}
		else
			throw new PExceptionNaming(
				"The PNamingContext return an unknown object");
	}

	/**
	 * This method permits to obtain a PBinding linked to a PAccessor and bound
	 * to the PName specified in parameter.
	 * The additionnal boolean parameter permits to know if the object must be
	 * created if is not found in the cache (the binder). Indeed a mapper is not
	 * always able to know if the object exist in the persistent support. This
	 * method must be used in this case only. In other case the previous method
	 * without the boolean must be used.
	 */
	public PBinding getPBinding(Object conn, PName pn, boolean create)
		throws PException {

		if (logger==null)
			logger = getLogger();
		if (loggerFactory==null)
			loggerFactory = getLoggerFactory();

		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "");

		Object o = recurseResolve(conn, pn);
		PBinding pb = null;
		if (o instanceof PBinding) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG,
					"The PBinding was found in the binder");
			pb = (PBinding) o;
			if (!bindingExtendsAccessor()) {
				if (isGenericAccessor()) {
					if (pb.getPAccessorGen()==null)
						pb.setPAccessorGen((PAccessorGen) createPAccessor());
				}
				else {
					if (pb.getPAccessor()==null)
						pb.setPAccessor(createPAccessor());
				}
			}
			return pb;
		}
		else if (o instanceof PName) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG,
					"The PBinding was not found in the binder");

			PName lastpn = (PName) o;

			// create a PBinding
			pb = createPBinding((PClassMapping) lastpn.getPNamingContext());

			// Link the PBindding to a PAccessor
			PAccessor pa = null;
			if (bindingExtendsAccessor()) {
				if (isGenericAccessor())
					pb.setPAccessorGen((PAccessorGen) pb);
				else
					pb.setPAccessor((PAccessor) pb);
			}
			else {
				if (isGenericAccessor())
					pb.setPAccessorGen((PAccessorGen) createPAccessor());
				else
					pb.setPAccessor(createPAccessor());
			}

			// bind the PBinding to the PName
			pb.bind(lastpn);

			if (!create) {
				if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG,
						"The instance exists in the support => load it");
				load(pb, conn);
			}
			else {
				if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG,
						"The instance doesn't exist in the support => store it");

				// Store the data in the persitent support
				store(pb, conn);
			}
			return pb;
		}
		else
			throw new PExceptionNaming(
				"The PNamingContext return an unknown object");
	}


	//----------------- OTHER METHOD ----------------//
	//-----------------------------------------------//

	protected Object recurseResolve(Object conn, PName pn)
		throws PException {
		Object res = pn.resolve(conn);
		if (res instanceof PBinding)
			return res;
		else if (res instanceof PName)
			return recurseResolve(conn, (PName) res);
		else if (res == null)
			return pn;
		else
			throw new PExceptionNaming(
				"The PNamingContext return an unknown object");
	}


	public void load(PBinding pb, Object conn) throws PException {
		pb.read(conn);
	}

	public void store(PBinding pb, Object conn) throws PException {
		pb.export(conn);
		pb.write(conn);
	}

	public PAccessor createPAccessor() {
		return null;
	}

	public PBinding createPBinding(PClassMapping pcm) throws PException {
		PBinding pb = pcm.createPBinding();
		if (pb instanceof Loggable)
			((Loggable) pb).setLogger(logger);
		return pb;
	}

	public abstract boolean bindingExtendsAccessor();

	public boolean isGenericAccessor() {
		return false;
	}

	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	/**
	 * It assigns a logger to an component that wants to log things.
	 * @param Logger	The logger object.
	 */
	public void setLogger(Logger _logger) {
		logger = _logger;
	}

	/**
	 * It assigns a logger factory that allows the creation of new loggers.
	 * @param loggerfactory	The LoggerFactory object to obtain new loggers.
	 */
	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
		if (logger==null)
			logger = loggerFactory.getLogger(this.getClass().getName());
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		 return loggerFactory;
	}
}
