/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
/**
 * Copyright (C) 2000
 */

package org.objectweb.jorm.facility.naming.gobid.binder;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.facility.meminst.manager.basic.api.Manager;
import org.objectweb.jorm.facility.meminst.manager.basic.lib.ManagerImpl;
import org.objectweb.jorm.facility.naming.gobid.generator.LGenerator;
import org.objectweb.jorm.facility.naming.gobid.generator.LGeneratorManager;
import org.objectweb.jorm.naming.api.PName;

import javax.resource.cci.ConnectionFactory;

/**
 * This abstract class permits to fetch LBinder instances. This class is
 * abstract because it doesn't implement the getMapper methods. This class
 * must be extended by the generated LBinderMapping.
 *
 * Since the lbinder pbinding is also the PAccessor,
 * bindingExtendsAccessor return true. This means that the CreatePAccessor
 * method is never called. The CreatePAccessor method is therefore not required.
 *
 * @author	Sebastien Chassande-Barrioz
 */
public abstract class LBinderManager extends ManagerImpl {

	ConnectionFactory connectionFactory = null;

	LGenerator bootGenerator = null;

	PClassMapping gm = null;

	Manager manager = null;

	public boolean bindingExtendsAccessor() {
		return true;
	}

	public void setConnectionFactory(ConnectionFactory cf) {
		connectionFactory = cf;
	}

	public void load(PBinding pb, Object conn) throws PException {
		initMI(pb, conn);
	}

	public void store(PBinding pb, Object conn) throws PException {
		initMI(pb, conn);

		PName genpn = gm.decodeString(pb.getPName().encodeString());
		PBinding pbgen = manager.getPBinding(conn, genpn);
		((PALBinder) pb).paSetGenerator(pbgen.getPName(), conn);

		if (bootGenerator == null) {
			bootGenerator = (LGenerator) manager.getPBinding(conn,
				gm.decodeString(LGeneratorManager.BOOT_GEN_NAME));
		}

		// set the binder identifier the first time where the binder is created
		((LBinder) pb).setId(bootGenerator.genId(conn));
		// export and write the binder
		pb.export(conn);
		((PALBinder) pb).store(conn);
	}

	/**
	 * This method will be provided by the generated BinderLongMapping class.
	 */
	public abstract PMapper getPMapper();

	private void initMI(PBinding pb, Object conn) throws PException {
		((PALBinder) pb).setConnectionFactory(connectionFactory);
		((PALBinder) pb).setLoggerFactory(loggerFactory);
		// do nothing because the object will be loaded later.
		if (gm == null) {
			gm = (PClassMapping)
				getPMapper().lookup(LGeneratorManager.GENERATOR_JORM_NAME);
			manager = (Manager) gm.getMIManager();
		}
	}
}
