/**
 * Copyright (C) 2000
 */

package org.objectweb.jorm.facility.naming.gobid.binder;

import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.type.api.PType;


/**
 * The class <b>BinderPNLong</b> defines A basic PName containing a long value.
 * @author nono de palma.
 */
public class LName implements PName {
	// persistent field
	/**
	 *  The id hold by this pname.
	 */
	private Long id;

	// transient attribute
	private PNamingContext pnc = null;

	/**
	 * Build a new BinderPNLong that must be initialized by the initBinderLong method.
	 */
	public LName() {
	}

	/**
	 * Build a new LName that does not require to be initialized by the initBinderLong method.
	 * @param id the id hold by this pname
	 * @param pnc the pnamingcontext which manage this pname.
	 */
	public LName(long id, PNamingContext pnc) {
		this.id = new Long(id);
		this.pnc = pnc;
	}

	/**
	 * This method initialize this name by setting it id and it pnamingcontext.
	 * @param id the id hold by this pname
	 * @param pnc the pnamingcontext which manage this pname.
	 */
	public void init(long id, PNamingContext pnc) {
		this.id = new Long(id);
		this.pnc = pnc;
	}

	/**
	 * It retrieve the identifier embeded in this PName. In the case of a LName this identifier is a long.
	 * It is used by the encode method of the binderlong.
	 * @return the long reprensing the value of this name.
	 */
	public long getId() {
		return this.id.longValue();
	}

	public String toString() {
		return id.toString();
	}


	// IMPLEMENTATION OF THE PName INTERFACE //
	//---------------------------------------//

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public boolean isNull() {
		return ((LName) pnc.getNull()).getId() == id.longValue();
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public boolean codingSupported(short codingtype) {
		return pnc.codingSupported(codingtype);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public Object encodeAbstract() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeAbstract(this);
	}

	public byte encodeByte() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeByte(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public char encodeChar() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeChar(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public int encodeInt() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeInt(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public Integer encodeIntObj() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeIntObj(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public long encodeLong() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeLong(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public short encodeShort() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeShort(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public String encodeString() throws PExceptionNaming {
		return pnc.encodeString(this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public PName export(Object conn, PNamingContext pnc)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		return pnc.export(conn, this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public PName export(Object conn, PNamingContext pnc, Object hints)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		return pnc.export(conn, this, hints);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public PNamingContext getPNamingContext() {
		return pnc;
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public PType getPType() {
		return ((LBinder) ((PClassMapping) this.pnc).getPBinder()).pt;
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public Object resolve(Object conn) throws PExceptionNaming, PExceptionIO {
		return pnc.resolve(conn, this);
	}

	/**
	 * Implementation of the PNamingContext interface.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
	public void unexport(Object conn)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		pnc.unexport(conn, this);
	}

	// OVERIDING OF THE Object CLASS //
	//-------------------------------//

	public int hashCode() {
		return this.id.hashCode();
	}

	public boolean equals(Object obj) {
		return ((((LName)(obj)).getId()) == id.longValue()) && ((((LName)(obj)).getPNamingContext()).equals(pnc));
		//return ((LName) obj).getId() == id.longValue();
	}
}

