/**
 * Copyright (C) 2000
 */

package org.objectweb.jorm.facility.naming.gobid.generator;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * Define a generator MI used by a Binder to produce persistant long identifier.
 * It is a persistant class which hold only a long representing the persistant id.
 * The MI, the accessor and the PBinding is merged : The  LongGen class is inherited by the accessor
 * (PALongGen) and the pbinding. This pbinding is generated by the jorm compiler from the LongGen.pd file.
 * @author	<A HREF="mailto:noel.depalma@rd.francetelecom.fr><b>
 *			Noel De Palma
 *			</b></A>
 */
public class LGeneratorImpl implements LGenerator, Loggable {

	/**
	 * The persistant id.
	 */
	private long nextid = 0;

	protected Logger logger = null;

	public LGeneratorImpl() {
	}

	public LGeneratorImpl(long id) {
		nextid = id;
	}

	/**
	 * Generate a new long id.
	 * @param conn the connexion to the DS.
	 * @return the long value.
	 */
	public long genId(Object conn) throws PException {
		if (nextid == Long.MAX_VALUE)
			throw new PExceptionNaming("No more identifier available");
		return nextid++;
	}

	/**
	 * It returns the value of the nextid field.
	 * @return	The actual value of the field.
	 */
	public long getNextId() {
		return nextid;
	}

	/**
	 * It set the value of the nextid field.
	 * @param id The new value of the field.
	 */
	public void setNextId(long id) {
		nextid = id;
	}

	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	/**
	 * It assigns a logger to an component that wants to log things.
	 * @param Logger	The logger object.
	 */
	public void setLogger(Logger _logger) {
		logger = _logger;
	}

	/**
	 * It assigns a logger factory that allows the creation of new loggers.
	 * @param loggerfactory	The LoggerFactory object to obtain new loggers.
	 */
	public void setLoggerFactory(LoggerFactory loggerfactory) {
		logger = loggerfactory.getLogger(this.getClass().getName());
	}
	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		 return null;
	}
}
