/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.facility.naming.gobid.generator;


import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.facility.meminst.lifecycle.lazyload.LazyLoad;
import org.objectweb.jorm.facility.meminst.manager.basic.lib.ManagerImpl;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.resource.cci.ConnectionFactory;




/**
 * This class permits to fetch LGenerator instances.
 *
 * Since the generator pbinding is also the PAccessor,
 * bindingExtendsAccessor return true. This means that the CreatePAccessor
 * method is never called. The CreatePAccessor method is therefore not required.
 * @author	Sebastien Chassande-Barrioz
 */
public class LGeneratorManager extends ManagerImpl {

	ConnectionFactory connectionFactory = null;

	/**
	 * name of the generator used to produce id for booting BinderLong's id
	 */
	public static final String BOOT_GEN_NAME = "BootGenerator";

	/**
	 * Jorm name of the generator class. This is a wellknown name.
	 */
	public static final String GENERATOR_JORM_NAME =
		"org.objectweb.jorm.facility.naming.gobid.generator.LGenerator";


	public boolean bindingExtendsAccessor() {
		return true;
	}

	public void setConnectionFactory(ConnectionFactory cf) {
		connectionFactory = cf;
	}

	public void load(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "");
		((PALGenerator) pb).setConnectionFactory(connectionFactory);
		((PALGenerator) pb).setLoggerFactory(loggerFactory);
	}

	public void store(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "");
		((PALGenerator) pb).setConnectionFactory(connectionFactory);
		((PALGenerator) pb).setLoggerFactory(loggerFactory);
		pb.export(conn);
		((PALGenerator) pb).store(conn);
	}
}

