/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
/**
 * Copyright (C) 2000 France Tlcom R&D
 * DTL/ASR
 */

package org.objectweb.jorm.facility.naming.gobid.pnc;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.facility.naming.gobid.binder.LBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;

public class LLName implements PName, PNameGetter {
	private Long idobj;
	private PNamingContext pnc;
	private PNamingContext previous_pnc;

	public LLName(long id, PNamingContext pnc, PNamingContext prevpnc) {
		this.idobj = new Long(id);
		this.pnc = pnc;
		this.previous_pnc = prevpnc;
	}

	public LLName() {
	}

	public void init(long id, PNamingContext pnc, PNamingContext prevpnc) {
		this.idobj = new Long(id);
		this.pnc = pnc;
		this.previous_pnc = prevpnc;
	}

	public long getIdObj() {
		return this.idobj.longValue();
	}

	public void setIdObj(long id) {
		this.idobj = new Long(id);
	}

	public PNamingContext getPnc() {
		return this.pnc;
	}

	public void setPnc(PNamingContext pnc) {
		this.pnc = pnc;
	}

	public PNamingContext getPrevPnc() {
		return this.previous_pnc;
	}

	public void setPrevPnc(PNamingContext pnc) {
		this.previous_pnc = pnc;
	}

	// IMPLEMENTATION OF THE PName INTERFACE //
	//---------------------------------------//

	public boolean isNull() {
		return ((LLName) pnc.getNull()).getIdObj() == idobj.longValue();
	}

	public boolean codingSupported(short codingtype) {
		return pnc.codingSupported(codingtype);
	}

	public Object encodeAbstract() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeAbstract(this);
	}

	public byte encodeByte() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeByte(this);
	}

	public char encodeChar() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeChar(this);
	}

	public int encodeInt() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeInt(this);
	}

	public Integer encodeIntObj() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeIntObj(this);
	}

	public long encodeLong() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeLong(this);
	}

	public short encodeShort() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeShort(this);
	}

	public String encodeString() throws PExceptionNaming {
		return pnc.encodeString(this);
	}

	public PName export(Object conn, PNamingContext pnc)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		return pnc.export(conn, (Object) this);
	}

	public PName export(Object conn, PNamingContext pnc, Object hints)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		return pnc.export(conn, (Object) this, hints);
	}

	public PNamingContext getPNamingContext() {
		return pnc;
	}

	public PType getPType() {
		return ((LLPnc) pnc).pt;
	}

	public Object resolve(Object conn)
		throws PExceptionNaming, PExceptionIO {
		return pnc.resolve(conn, this);
	}

	public void unexport(Object conn)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		pnc.unexport(conn, this);
	}

	public int hashCode() {
		return idobj.hashCode();
	}

	public boolean equals(Object obj) {
		return ((((LLName) obj).getIdObj()) == idobj.longValue()) &&
			((LBinder) ((LLName) obj).getPrevPnc()).getId() ==
			((LBinder) (this.previous_pnc)).getId();
	}

	// IMPLEMENTATION OF THE PNameGetter INTERFACE //
	//---------------------------------------------//

	public byte pngetByteField(String fn, Object ctx) throws PExceptionTyping {
		throw new PExceptionTyping("Unsupported field type");
	}

	public char pngetCharField(String fn, Object ctx) throws PExceptionTyping {
		throw new PExceptionTyping("Unsupported field type");
	}

	public short pngetShortField(String fn, Object ctx) throws PExceptionTyping {
		throw new PExceptionTyping("Unsupported field type");
	}

	public int pngetIntField(String fn, Object ctx) throws PExceptionTyping {
		throw new PExceptionTyping("Unsupported field type");
	}

	public Integer pngetIntObjField(String fn, Object ctx) throws PExceptionTyping {
		throw new PExceptionTyping("Unsupported field type");
	}

	public long pngetLongField(String fn, Object ctx) throws PExceptionTyping {
		if ("objId".equals(fn))
			return idobj.longValue();
		else if ("binderId".equals(fn)) {
			// previous_pnc must be a binderlong
			//if (this.previous_pnc instanceof BinderLong)
			if (this.previous_pnc == null)
				return LLPnc.NULLID;
			else
				return ((LBinder) this.previous_pnc).getId();
		}
		else
			throw new PExceptionTyping("Field Unknown");
	}

	public String pngetStringField(String fn, Object ctx) throws PExceptionTyping {
		if (fn.equals("objId"))
			return this.idobj.toString();
		else if (fn.equals("binderId"))
		// previous_pnc must be a binderlong
			if (previous_pnc == null)
				return Long.toString(LLPnc.NULLID);
			else
				return Long.toString(((LBinder) previous_pnc).getId());
		else
			throw new PExceptionTyping("Field Unknown");
	}
}

