/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
/**
 * Copyright (C) 2000
 */

package org.objectweb.jorm.facility.naming.gobid.pnc;


import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.facility.meminst.lifecycle.lazyload.LazyLoad;
import org.objectweb.jorm.facility.meminst.manager.basic.lib.ManagerImpl;
import org.objectweb.jorm.facility.meminst.manager.basic.api.Manager;

import javax.resource.cci.ConnectionFactory;

/**
 * This abstract class permits to fetch LLPnc instances. This class is
 * abstract because it doesn't implement the getGenClassMapping method. This class
 * must be extended by the generated LLPncMapping.
 *
 * Since the generator pbinding is also the PAccessor,
 * bindingExtendsAccessor return true. This means that the CreatePAccessor
 * method is never called. The CreatePAccessor method is therefore not required.
 *
 * @author	Sebastien Chassande-Barrioz
 */
public abstract class LLPncManager extends ManagerImpl {

	ConnectionFactory connectionFactory = null;


	public void setConnectionFactory(ConnectionFactory cf) {
		connectionFactory = cf;
	}

	public boolean bindingExtendsAccessor() {
		return true;
	}

	public void load(PBinding pb, Object conn) throws PException {
		((PALLPnc) pb).setConnectionFactory(connectionFactory);
		((PALLPnc) pb).setLoggerFactory(loggerFactory);

		//fetch the factory of the genclass associated to the field "classBinders"
		PClassMapping pcm = (PClassMapping) getGenClassMapping("classBinders");

		Manager manager = (Manager) pcm.getMIManager();

		// Build the PName
		PName gcpn = pcm.decodeString( pb.getPName().encodeString() );

		// fetch the PBinding of the gen class
		PBinding pbgc = manager.getPBinding(conn,gcpn, false);

		// set the pname on the accessor

		((LLPncAccessor) pb).paSetClassBinders(pbgc.getPName(), conn);

	}

	public void store(PBinding pb, Object conn) throws PException {
		((PALLPnc) pb).setConnectionFactory(connectionFactory);
		((PALLPnc) pb).setLoggerFactory(loggerFactory);

		//fetch the factory of the genclass associated to the field "classBinders"
		PClassMapping pcm = (PClassMapping) getGenClassMapping("classBinders");

		Manager manager = (Manager) pcm.getMIManager();

		// Build the PName
		PName gcpn = pcm.decodeString( pb.getPName().encodeString() );

		// fetch the PBinding of the gen class
		PBinding pbgc = manager.getPBinding(conn,gcpn, true);

		// set the pname on the accessor

		((LLPncAccessor) pb).paSetClassBinders(pbgc.getPName(), conn);

		pb.export(conn);
		((PALLPnc) pb).store(conn);
	}

	/**
	 * This method will be provided by the generated BinderLongMapping class.
	 */
	public abstract PClassMapping getGenClassMapping(String fieldName);
}
