/**
 * Copyright (C) 2000 
 */

package org.objectweb.jorm.facility.naming.usid;

import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.type.api.PType;


/**
 * The class <b>PNString</b> defines A basic PName containing a string value.
 * @author nono de palma.
 */

public class StrName implements PName {

	private String id = null;
	private PNamingContext pnc = null;

	public StrName() {
	}

	public StrName(String id, PNamingContext pnc) {
		this.id = id;
		this.pnc = pnc;
	}

	public void init(String id, PNamingContext pnc) {
		this.id = id;
		this.pnc = pnc;
	}

	/**
	 * It retrieve the identifier embeded in this PName. In the case of a StrName this identifier is a string.
	 * @return the string reprensing the value of this name.
	 */
	public String getId() {
		return this.id;
	}

	public String toString() {
		return id;
	}

	// IMPLEMENTATION OF THE PName INTERFACE //
	//---------------------------------------//

	public boolean isNull() {
		return (((StrName) pnc.getNull()).getId()).equals(id);
	}

	public boolean codingSupported(short codingtype) {
		return pnc.codingSupported(codingtype);
	}

	public Object encodeAbstract() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeAbstract(this);
	}

	public byte encodeByte() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeByte(this);
	}

	public char encodeChar() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeChar(this);
	}

	public int encodeInt() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeInt(this);
	}

	public Integer encodeIntObj() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeIntObj(this);
	}

	public long encodeLong() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeLong(this);
	}

	public short encodeShort() throws PExceptionNaming, UnsupportedOperationException {
		return pnc.encodeShort(this);
	}

	public String encodeString() throws PExceptionNaming {
		return pnc.encodeString(this);
	}

	public PName export(Object conn, PNamingContext pnc)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		return pnc.export(conn, this);
	}

	public PName export(Object conn, PNamingContext pnc, Object hints)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		return pnc.export(conn, this, hints);
	}

	public PNamingContext getPNamingContext() {
		return pnc;
	}

	public PType getPType() {
		if (pnc instanceof PClassMapping)
			return ((StrBinder) ((PClassMapping) pnc).getPBinder()).pt;
		else
			return null;
	}

	public Object resolve(Object conn) throws PExceptionNaming, PExceptionIO {
		return pnc.resolve(conn, this);
	}

	public void unexport(Object conn)
		throws PExceptionNaming, PExceptionIO, PExceptionProtocol {
		pnc.unexport(conn, this);
	}

	// METHODS USED BY THE CACHE //
	//---------------------------//

	public int hashCode() {
		return this.id.hashCode();
	}

	public boolean equals(Object obj) {
		return ((StrName) obj).getId().equals(id);
	}
}


