/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.interp.genclass.api;

import org.objectweb.jorm.type.api.PType;

/**
 * It dscribes the setters for defining the properties that can be set when
 * initialising a GenClassMapping.
 * @author	P. Dchamboux
 */
public interface GenClassProp {
	/**
	 * Defines the type property of a generic class instance.
	 * @param pt	The associated type.
	 */
	void setGcType(PType pt);

	/**
	 * Defines the fields property specifying the fields composing the
	 * identifier structure of a generic class instance.
	 * @param fds	The array of descriptions of the associated fields.
	 */
	void setIdentifierDesc(FieldDesc[] fds);

	/**
	 * Defines the fields property specifying the fields composing the
	 * index structure of a generic class instance elements.
	 * @param fds	The array of descriptions of the associated fields.
	 */
	void setIndexDesc(FieldDesc[] fds);

	/**
	 * Defines the fields property specifying the fields composing the
	 * element value structure of a generic class instance elements.
	 * @param fds	The array of descriptions of the associated fields.
	 */
	void setElemDesc(FieldDesc[] fds);

}
