/*
 */
package org.objectweb.jorm.interp.genclass.file.lib;

import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.interp.genclass.api.FieldDesc;
import org.objectweb.jorm.interp.genclass.lib.GenClassBinding;
import org.objectweb.jorm.interp.genclass.lib.GenClassMapping;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.transaction.ra.file.FileAccess;
import org.objectweb.transaction.ra.file.FileReader;
import org.objectweb.transaction.ra.file.FileWriter;

import javax.resource.cci.Connection;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Date;

/**
 * author Sebastien Chassande-Barrioz
 */
public class FileGenClassBinding
	extends GenClassBinding
	implements FileReader, FileWriter, PNameGetter {

	Object[][] pNameField = null;

	public FileGenClassBinding() {
		super();
	}

	/**
	 * It constructs a RdbGenClassBinding.
	 */
	public FileGenClassBinding(GenClassMapping mapping)
		throws PExceptionProtocol {
		super(mapping);
	}
	/**
	 * It verifies if the DSI attached to this binding exists within the DS.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @return	It returns true if the DSI exists.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 * 									LIFECYCLE_NOACC
	 */
	public boolean exist(Object conn)
		throws PExceptionIO, PExceptionNaming, PExceptionProtocol {

		byte nextstate = PStateGraph.nextStatePBinding(status, ACTION_EXIST);
		if (nextstate == LIFECYCLE_ERROR)
			throw new PExceptionProtocol(
				"Cannot perform exist on this GenClass PBinding");

		try {
			return ((FileAccess) conn).exists(
				((FileGenClassMapping) genClassMapping).getDir(),
				pName.encodeString());
		}
		catch (Exception e) {
			throw new PExceptionIO(e, "Error has occured during the exist operation");
		}
	}

	/**
	 * It reads the data store instance designated by the PName of this
	 * binding and transfers each attribute value to the accessor object.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 */
	public void read(Object conn)
		throws PExceptionIO, PExceptionNaming, PExceptionProtocol {

		byte nextstate = PStateGraph.nextStatePBinding(status, ACTION_READ);
		if (nextstate == LIFECYCLE_ERROR)
			throw new PExceptionProtocol(
				"Cannot perform read on this GenClass PBinding");

		try {
			((FileAccess) conn).readObject(
				((FileGenClassMapping) genClassMapping).getDir(),
				pName.encodeString(),
				(FileReader) this,
				(Connection) conn);
		}
		catch (Exception e) {
			throw new PExceptionIO(e, "Error has occured during the read operation");
		}
	}

	/**
	 * It transfers each attribute value taken from the PAccessor
	 * object to the data store instance designated by the PName
	 * associated with this binding.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @exception   e1 if the state of the binding does not allow writing
	 *                 to be completed (no valid name for example)
	 *              e2 if the pc parameter does not allow acces to the
	 *                 data store
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 * 									LIFECYCLE_NEWTOWRITE
	 * 									LIFECYCLE_DELTOWRITE
	 * 									LIFECYCLE_DELTOWRITENOACC
	 * @exception PExceptionTyping	It is raised when a reference to be stored
	 * 								has a type incompatible with the respective
	 * 								field definition.
	 */
	public void write(Object conn)
		throws PExceptionIO, PExceptionNaming, PExceptionProtocol,
		PExceptionTyping {

		byte nextstate = PStateGraph.nextStatePBinding(status, ACTION_WRITE);
		if (nextstate == LIFECYCLE_ERROR)
			throw new PExceptionProtocol(
				"Cannot perform write on this GenClass PBinding");

		try {
			((FileAccess) conn).writeObject(
				((FileGenClassMapping) genClassMapping).getDir(),
				pName.encodeString(),
				(FileWriter) this,
				(Connection) conn);
		}
		catch (Exception e) {
			throw new PExceptionIO(e, "Error has occured during the write operation");
		}
	}

	// IMPLEMENTATION OF THE FileReader INTERFACE //
	//--------------------------------------------//
	/**
	 * Reads the content of a persistent object from the file represented by
	 * the given ObjectInputStream.
	 * @param ois	The ObjectInputStream representing the file from which to
	 *				read.
	 */
	public void readFile(ObjectInputStream ois) throws Exception {
		throw new UnsupportedOperationException("A connection is needed !");
	}

	/**
	 * Reads the content of a persistent object from the file represented by
	 * the given ObjectInputStream.
	 * @param ois	The ObjectInputStream representing the file from which to
	 *				read.
	 * @param conn	The connection used to access the file repository.
	 */
	public void readFile(ObjectInputStream ois, Connection conn)
		throws Exception {

		//Read the number of elements
		int nbelement = ois.readInt();
		paccessorGenClass.paSetNbElem(nbelement);

		for (int j=0; j<nbelement; j++) {
			PIndexedElem pie = paccessorGenClass.createPIndexedElem();
			FieldDesc[] identifiers = genClassMapping.getIdentifierFields();
			pName = readPName(genClassMapping, identifiers, 0, ois);

			// Read the index fields
			FieldDesc[] indexes = genClassMapping.getIndexFields();
			for (int i = 0; i < indexes.length; i++) {
				switch (indexes[i].type.getTypeCode()) {
				case PType.TYPECODE_INT:
					pie.pieSetIntIndexField(indexes[i].name, ois.readInt());
					break;
				case PType.TYPECODE_LONG:
					pie.pieSetLongIndexField(indexes[i].name, ois.readLong());
					break;
				case PType.TYPECODE_SHORT:
					pie.pieSetShortIndexField(indexes[i].name, ois.readShort());
					break;
				case PType.TYPECODE_STRING:
					pie.pieSetStringIndexField(indexes[i].name,
						(String) ois.readObject());
					break;
				default:
					throw new PExceptionProtocol("Unauthorized index type: ("
						+ indexes[i].name + ", "
						+ indexes[i].type.getJormName());
				}
			}

			// Write the element fields
			FieldDesc[] elements = genClassMapping.getElemFields();
			switch (elements[0].type.getTypeCode()) {
			case PType.TYPECODE_BOOLEAN:
				pie.pieSetBooleanElem(ois.readBoolean());
				break;
			case PType.TYPECODE_BYTE:
				pie.pieSetByteElem(ois.readByte());
				break;
			case PType.TYPECODE_CHAR:
				pie.pieSetCharElem(ois.readChar());
				break;
			case PType.TYPECODE_DATE:
				pie.pieSetDateElem((Date) ois.readObject());
				break;
			case PType.TYPECODE_DOUBLE:
				pie.pieSetDoubleElem(ois.readDouble());
				break;
			case PType.TYPECODE_FLOAT:
				pie.pieSetFloatElem(ois.readFloat());
				break;
			case PType.TYPECODE_INT:
				pie.pieSetIntElem(ois.readInt());
				break;
			case PType.TYPECODE_LONG:
				pie.pieSetLongElem(ois.readLong());
				break;
			case PType.TYPECODE_SERIALIZED:
				pie.pieSetSerializedElem((Serializable) ois.readObject());
				break;
			case PType.TYPECODE_SHORT:
				pie.pieSetBooleanElem(ois.readBoolean());
				break;
			case PType.TYPECODE_STRING:
				pie.pieSetStringElem((String) ois.readObject());
				break;
			case PType.TYPECODE_REFERENCE:
				pie.pieSetRefElem(
					readPName(
						genClassMapping.getPNamingContext(), elements, 1, ois)
				);
				break;
			default:
				throw new PExceptionProtocol("Unauthorized element type: ("
					+ elements[0].name + ", "
					+ elements[0].type.getJormName());
			}

			paccessorGenClass.paAdd( pie, conn);
		}
	}


	// IMPLEMENTATION OF THE FileWriter INTERFACE //
	//--------------------------------------------//

	/**
	 * Writes the content of a persistent object to the file represented by
	 * the given ObjectOutputStream.
	 * @param oos	The ObjectOutputStream representing the file to which to
	 *				write.
	 */
	public void writeFile(ObjectOutputStream oos) throws Exception {
		throw new UnsupportedOperationException("A connection is needed !");
	}

	/**
	 * Writes the content of a persistent object to the file represented by
	 * the given ObjectOutputStream.
	 * @param oos	The ObjectOutputStream representing the file to which to
	 *				write.
	 * @param conn	The connection used to access the file repository.
	 */
	public void writeFile(ObjectOutputStream oos, Connection conn)
		throws Exception {
		//Write the number of element
		oos.writeInt(paccessorGenClass.paGetNbElem());

		Iterator pies = paccessorGenClass.paIterator();

		// For each element of the genclass
		while (pies.hasNext()) {
			PIndexedElem pie = (PIndexedElem) pies.next();

			// Write the identifier fields
			FieldDesc[] identifiers = genClassMapping.getIdentifierFields();
			writePName(pName, identifiers, 0, oos);

			// Write the index fields
			FieldDesc[] indexes = genClassMapping.getIndexFields();
			for (int i = 0; i < indexes.length; i++) {
				switch (indexes[i].type.getTypeCode()) {
				case PType.TYPECODE_INT:
					oos.writeInt(pie.pieGetIntIndexField(indexes[i].name));
					break;
				case PType.TYPECODE_LONG:
					oos.writeLong(pie.pieGetLongIndexField(indexes[i].name));
					break;
				case PType.TYPECODE_SHORT:
					oos.writeShort(pie.pieGetShortIndexField(indexes[i].name));
					break;
				case PType.TYPECODE_STRING:
					oos.writeObject(pie.pieGetStringIndexField(indexes[i].name));
					break;
				default:
					throw new PExceptionProtocol("Unauthorized index type: ("
						+ indexes[i].name + ", "
						+ indexes[i].type.getJormName());
				}
			}


			// Write the element fields
			FieldDesc[] elements = genClassMapping.getElemFields();
			switch (elements[0].type.getTypeCode()) {
			case PType.TYPECODE_BOOLEAN:
				oos.writeBoolean(pie.pieGetBooleanElem());
				break;
			case PType.TYPECODE_BYTE:
				oos.writeByte(pie.pieGetByteElem());
				break;
			case PType.TYPECODE_CHAR:
				oos.writeChar(pie.pieGetCharElem());
				break;
			case PType.TYPECODE_DATE:
				oos.writeObject(pie.pieGetDateElem());
				break;
			case PType.TYPECODE_DOUBLE:
				oos.writeDouble(pie.pieGetDoubleElem());
				break;
			case PType.TYPECODE_FLOAT:
				oos.writeFloat(pie.pieGetFloatElem());
				break;
			case PType.TYPECODE_INT:
				oos.writeInt(pie.pieGetIntElem());
				break;
			case PType.TYPECODE_LONG:
				oos.writeLong(pie.pieGetLongElem());
				break;
			case PType.TYPECODE_SERIALIZED:
				oos.writeObject(pie.pieGetSerializedElem());
				break;
			case PType.TYPECODE_SHORT:
				oos.writeShort(pie.pieGetShortElem());
				break;
			case PType.TYPECODE_STRING:
				oos.writeObject(pie.pieGetStringElem());
				break;
			case PType.TYPECODE_REFERENCE:
				writePName(pName, elements, 1, oos);
				break;
			default:
				throw new PExceptionProtocol("Unauthorized element type: ("
					+ elements[0].name + ", "
					+ elements[0].type.getJormName());
			}
		}
	}

	private void writePName(PName pn, FieldDesc[] identifiers, int start,
						   ObjectOutputStream oos) throws Exception {
		if (identifiers.length == 1) {
			// The pname of this gen class is composed by a single field
			// => use the rigth encode method
			switch (identifiers[start].type.getTypeCode()) {
			case PType.TYPECODE_BYTE:
				oos.writeByte(pn.encodeByte());
				break;
			case PType.TYPECODE_CHAR:
				oos.writeChar(pn.encodeChar());
				break;
			case PType.TYPECODE_INT:
				oos.writeInt(pn.encodeInt());
				break;
			case PType.TYPECODE_LONG:
				oos.writeLong(pn.encodeLong());
				break;
			case PType.TYPECODE_SHORT:
				oos.writeShort(pn.encodeShort());
				break;
			case PType.TYPECODE_STRING:
				oos.writeObject(pn.encodeString());
				break;
			default:
				throw new PExceptionProtocol(
					"Unauthorized identifier type: ("
					+ identifiers[start].name + ", "
					+ identifiers[start].type.getJormName());
			}
		}
		else {
			// The pname is composed by several fields => use a PNameGetter
			PNameGetter png = (PNameGetter) pn.encodeAbstract();
			for (int i = start; i < identifiers.length; i++) {
				switch (identifiers[i].type.getTypeCode()) {
				case PType.TYPECODE_BYTE:
					oos.writeByte(png.pngetByteField(identifiers[i].name,null));
					break;
				case PType.TYPECODE_CHAR:
					oos.writeChar(png.pngetCharField(identifiers[i].name, null));
					break;
				case PType.TYPECODE_INT:
					oos.writeInt(png.pngetIntField(identifiers[i].name, null));
					break;
				case PType.TYPECODE_LONG:
					oos.writeLong(png.pngetLongField(identifiers[i].name, null));
					break;
				case PType.TYPECODE_SHORT:
					oos.writeShort(png.pngetShortField(identifiers[i].name, null));
					break;
				case PType.TYPECODE_STRING:
					oos.writeObject(png.pngetStringField(identifiers[i].name, null));
					break;
				default:
					throw new PExceptionProtocol(
						"Unauthorized identifier type: ("
						+ identifiers[i].name + ", "
						+ identifiers[i].type.getJormName());
				}
			}
		}
	}

	private PName readPName(PNamingContext pnc, FieldDesc[] identifiers, int start,
						   ObjectInputStream ois) throws Exception {
		PName pn = null;
		if (identifiers.length == 1) {
			// The pname of this gen class is composed by a single field
			// => use the rigth encode method
			switch (identifiers[start].type.getTypeCode()) {
			case PType.TYPECODE_BYTE:
				pn = pnc.decodeByte(ois.readByte());
				break;
			case PType.TYPECODE_CHAR:
				pn = pnc.decodeChar(ois.readChar());
				break;
			case PType.TYPECODE_INT:
				pn = pnc.decodeInt(ois.readInt());
				break;
			case PType.TYPECODE_LONG:
				pn = pnc.decodeLong(ois.readLong());
				break;
			case PType.TYPECODE_SHORT:
				pn = pnc.decodeShort(ois.readShort());
				break;
			case PType.TYPECODE_STRING:
				pn = pnc.decodeString((String) ois.readObject());
				break;
			default:
				throw new PExceptionProtocol(
					"Unauthorized identifier type: ("
					+ identifiers[start].name + ", "
					+ identifiers[start].type.getJormName());
			}
		}
		else {
			// The pname is composed by several fields => use a PNameGetter
			Object[][] pnameField = new Object[identifiers.length-start][2];
			for (int i = start; i < identifiers.length; i++) {
				pnameField[i-start][0] = identifiers[i].name;
				switch (identifiers[i].type.getTypeCode()) {
				case PType.TYPECODE_BYTE:
					pnameField[i-start][1] = new Byte(ois.readByte());
					break;
				case PType.TYPECODE_CHAR:
					pnameField[i-start][1] = String.valueOf(ois.readChar());
					break;
				case PType.TYPECODE_INT:
					pnameField[i-start][1] = new Integer(ois.readInt());
					break;
				case PType.TYPECODE_LONG:
					pnameField[i-start][1] = new Long(ois.readLong());
					break;
				case PType.TYPECODE_SHORT:
					pnameField[i-start][1] = new Short(ois.readShort());
					break;
				case PType.TYPECODE_STRING:
					pnameField[i-start][1] = ois.readObject();
					break;
				default:
					throw new PExceptionProtocol(
						"Unauthorized identifier type: ("
						+ identifiers[i].name + ", "
						+ identifiers[i].type.getJormName());
				}
			}
			setPNameField(pnameField);
			pn = pnc.decodeAbstract(this,null);
			setPNameField(null);
		}
		return pn;
	}

	private void setPNameField(Object[][] pnf) {
		pNameField = pnf;
	}


	// IMPLEMENTATION OF THE PNameGetter INTERFACE //
	//---------------------------------------------//

	/**
	 * It retrieves the value of a byte field.
	 * @param fn	A String containing the name of the field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @return	The byte value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public byte pngetByteField(String fn, Object ctx)
		throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return ((Byte) pNameField[pos][1]).byteValue();
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}

	/**
	 * It retrieves the value of a char field.
	 * @param fn	A String containing the name of the field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @return	The char value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public char pngetCharField(String fn, Object ctx) throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return ((String) pNameField[pos][1]).charAt(0);
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}

	/**
	 * It retrieves the value of a short field.
	 * @param fn	A String containing the name of the field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @return	The short value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public short pngetShortField(String fn, Object ctx) throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return ((Short) pNameField[pos][1]).shortValue();
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}

	/**
	 * It retrieves the value of an int field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @param fn	A String containing the name of the field.
	 * @return	The int value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public int pngetIntField(String fn, Object ctx) throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return ((Integer) pNameField[pos][1]).intValue();
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}

	/**
	 * It retrieves the value of an int field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @param fn	A String containing the name of the field.
	 * @return	The int value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public Integer pngetIntObjField(String fn, Object ctx) throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return (Integer) pNameField[pos][1];
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}

	/**
	 * It retrieves the value of a long field.
	 * @param fn	A String containing the name of the field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @return	The long value of the field as stored within the composite
	 * 			name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public long pngetLongField(String fn, Object ctx) throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return ((Long) pNameField[pos][1]).longValue();
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}

	/**
	 * It retrieves the value of a string field.
	 * @param fn	A String containing the name of the field.
     * @param cxt A context used by PNameGetter to extract the related data.
	 * @return	The java.lang.String value of the field as stored within the
	 * 			composite name.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this composite name.
	 * @exception PExceptionIO	It is raised when a problem occurs while
	 * 							potentially accessing the DSI.
	 */
	public String pngetStringField(String fn, Object ctx) throws PExceptionTyping, PExceptionIO {
		int pos = 0;
		for(;pos<pNameField.length && !fn.equals(pNameField[pos][0]) ; pos++);
		if (pos<pNameField.length)
			return (String) pNameField[pos][1];
		else
			throw new PExceptionTyping(
				"Wrong type for a pname field of a GenClass");
	}
}
