/*
 */
package org.objectweb.jorm.interp.genclass.file.lib;

import org.objectweb.jorm.interp.genclass.lib.GenClassMapping;
import org.objectweb.jorm.interp.genclass.file.api.FileGenClassProp;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.Debug;
import org.objectweb.transaction.ra.file.FileAccess;
import org.objectweb.util.monolog.api.BasicLevel;

public class FileGenClassMapping
	extends GenClassMapping
	implements FileGenClassProp {

	public final static String MAPPER_NAME = "File";

	protected String className = null;
	protected String fieldName = null;
	protected int genClassLevel = -1;

	/**
	 * It creates a new PBinding.
	 * @return	The new PBinding with lifecycle state set to
	 * 			LIFECYCLE_NOTBOUNDNOACC.
	 */
	public PBinding createPBinding() {
		try {
			PBinding pb = new FileGenClassBinding();
			pb.init(this);
			return pb;
		}
		catch (PExceptionProtocol e) {
			return null;
		}
	}

	/**
	 * It initializes the persistent structures required to map a JORM class
	 * within a particular Data Store.
	 * This method has no effect when the structures already exist, or when the
	 * structure must not be manipulated via JORM (e.g., case of a legacy DS).
	 * The deletion of these structures may be requested before initializing
	 * them.
	 * @param conn		The connection that can be used to access the DS.
	 * @param cleanup	Tells if structures and/or data should be removed at
	 * 					initialization time.
	 * @exception PExceptionProtocol	A problem occured during initialisation
	 * 									that is not related to I/O.
	 * @exception PExceptionIO	A problem occured during the initialization of
	 * 							the DS.
	 */
	public void initMappingStructures(Object conn, byte cleanup)
		throws PExceptionProtocol, PExceptionIO {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "");
		try {
			((FileAccess)conn).createDbDir(getDir());
		}
		catch (Exception e) {
			throw new PExceptionIO(e,
				"Error has occured during the initialisation of the mapping structure");
		}
	}

	/**
	 * It removes the persistent structures required to map a JORM class
	 * within a particular Data Store.
	 * This method has no effect when the structures do not exist, or when the
	 * structure must not be manipulated via JORM (e.g., case of a legacy DS).
	 * @param conn	The connection that can be used to access the DS.
	 * @param contentonly	It specifies if only the content of the persistent
	 * 						structures should be removed (it only removes data).
	 * @exception PExceptionProtocol	A problem occured during the deletion
	 * 									that is not related to I/O.
	 * @exception PExceptionIO	A problem occured during the deletion of
	 * 							the DS.
	 */
	public void removeMappingStructures(Object conn, boolean contentonly)
		throws PExceptionProtocol, PExceptionIO {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "");
	}

	/**
	 * It specifies if the given kind of PMapper is supported by this
	 * PClassMapping.
	 * @param mappername	The name defining the PMapper kind.
	 * @return	true if this kind of mapper is supported.
	 */
	public boolean isConform(String mappername) {
		return MAPPER_NAME.equals(mappername);
	}

	// IMPLEMENTATION OF THE FileGenClassProp INTERFACE //
	//--------------------------------------------------//
	public void setSourceClassName(String cn) {
		className = cn;
	}

	public void setFieldName(String fn) {
		fieldName = fn;
	}

	public void setGenClassLevel(int level) {
		genClassLevel = level;
	}

	protected String getDir() {
		return className + "." + fieldName + "." + genClassLevel ;
	}
}
