/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.api;

import org.objectweb.jorm.type.api.PType;
import java.util.Iterator;


/**
 * The CompositeName interface defines methods to manage composite name.
 * Composite Name are used to define composite pname.
 * Composite Name defines the multiple fields describing the structure os a name.
 * @author	N. De Palma
 */
public interface CompositeName extends MetaObject {
     /**
     * Allows to know the name of the current CompositeName object.
     * @return	the string representation of the name of the CompositeName
     */
    String getName();

    /**
     * Add an inherited composite name to the current composite name.
     * @param cn the name of the composite name to be inherited.
     */
    void addInheritedCompositeName(CompositeName cn);

    /**
     * Provides an iterator over all the composite name inherited by the current composite name.
     * @return the iterator embeding inherited composite name objects.
     */
    Iterator iterateInheritedCompositeName();

    /**
     * Retrieve an inherited composite name form its name.
     * @param name the name of the composite name
     * @return the composite name
     */
    CompositeName getInheritedCompositeName(String name);

    /**
     * Retrieve a scalarfield describing the field identified by it name in the current compositename.
     * @param fieldname the name of the field.
     * @return the scalarfield describing the field.
     */
    ScalarField getScalarField(String fieldName);

    /**
     * Returns the number of inherited composite name for the current composite name object.
     * @return  the number of inherited compositename.
     */
    int getInheritedCompositeNameNumber();

    /**
     * Build a new scalarfield that describes a field in the composite name
     * @param fieldname the name of the field
     * @param type the ptype of the field
     * @return the scalarfield
     */
    ScalarField createCompositeNameField(String fieldName, PType type);

    /**
     * Give the number of fields composing the current composite name.
     * @return the fields number.
     */
    int getFieldNumber();

    /**
     * Provides an iterator over the field of the current composite name.
     * @return the iterator embeding scalarfield object
     */
    Iterator iterateField();

    /**
     * Provides an iterator over the field of the current composite name including the inherited fields.
     * @return the iterator embeding scalarfield object
     */

    Iterator iterateAllField();


}




