/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.api;

import org.objectweb.jorm.type.api.PType;

import java.util.Iterator;

/**
 * The GenClass interface defines generic class object which are
 * used to gather other objects.
 * @author	X. Spengler
 */
public interface GenClass extends MetaObject {
	/**
	 * Allows to know the name of the current generic class object.
	 *
	 * @return	the string representation of the generic class name
	 */
	String getName();

	/**
	 * Returns a new PrimitiveElement object, created with its name
	 * and its type (PType).
	 * As the current Class is generic, the field of the class is an
	 * index field. If the field already exists, it is returned, else a new
	 * one is created.
	 *
	 * @param indexFieldName	the name of the index field
	 * @param type	            the type of the index field
	 * @return	a new PrimitiveField for the current generic class
	 */
	PrimitiveElement createIndexField(String indexFieldName, PType type);

	/**
	 * Returns the PrimitiveElement corresponding to its name.
	 * If no field corresponds to this name, null is returned.
	 *
	 * @param name	the name of the index field
	 * @return	an existing index field. If the index field does not exist,
	 *	    	null is returned.
	 */
	PrimitiveElement getIndexField(String indexFieldName);

	/**
	 * Allows to know all the existing index field for the current generic
	 * class.
	 * This iterator contains PrimitiveElement objects. If no index field
	 * exists, an empty iterator is returned.
	 *
	 * @return	an Iterator on index field (PrimitiveElement). If there is no
	 *          index field to return, an empty iterator is returned.
	 */
	Iterator iterateIndexField();

	/**
	 * Returns the number of index fields for the current generic class.
	 *
	 * @return  the integer number of the index fields for the generic class
	 */
	int getIndexFieldNumber();
}
