/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.api;

/**
 * A MappingFactory is an object able to create (return) MetaObject which are
 * objects able to map other objects.
 * @author	X. Spengler
 */
public interface MappingFactory {
	/**
	 * Creates a ValueMapping object from the current factory.
	 * This object is created with a mapper name, a mapping name and its
	 * futur parent.
	 *
	 * @param mapperName    the name of the mapper
	 * @param mappingName   the name of the mapping
	 * @return  a ValueMapping object
	 */
	ValueMapping createValueMapping(String mapperName,
									String mappingName,
									MetaObject parent);

	/**
	 * Creates a ClassMapping object from the current factory.
	 * This object is created with a mapper name, a mapping name and its
	 * futur parent.
	 *
	 * @param mapperName    the name of the mapper
	 * @param mappingName   the name of the mapping
	 * @return  a ClassMapping object
	 */
	ClassMapping createClassMapping(String mapperName,
									String mappingName,
									MetaObject parent);

	/**
	 * Initializes the current factory.
	 */
	void init();
}

