/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
package org.objectweb.jorm.metainfo.api;

import java.util.Map;

/**
 * The NameRef interface defines methods to manage reference to composite name.
 * NameRef are used in namedef to reference a composite name that describe the structure of the namedef.
 * @author	N. De Palma
 */
public interface NameRef extends MetaObject{

    /**
     * Add a projection between a field in the composite name structure and a field in the class.
     * @param namefield the name of the field in the composite name
     * @param classfield the name of the field in the class
     */
    void addProjection(String namefield, String classfield);

    /**
     * Retrieve the list of projections between the fields of the composite name and the field of the class.
     * @return a map object describing the projection.
     */
    Map getProjection();

    /**
     * Retrieve the name of the nameref. This name is equal to the name of the referenced composite name.
     * @return the nameof the nameref used to describe the current pname.
     */
    String getName();

    /**
     * Retrieve the metaobject describing the composite name referenced by this nameref.
     * @return the composite name.
     */

    CompositeName getCompositeName();

    /**
     * test if the type of the compositename referenced in this nameref is equals to the type
     * referenced by the nameref passed as parameter.
     * @obj the nameref to compare to
     * @return true if the schema name ans the compositename name are equal.
     */
    boolean equalCNType(Object obj);

    /**
     * Return the name of the schema for the composite name referenced by this nameref.
     */
    String getCNSchemaName();

}




