/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.api;

import java.util.Iterator;

/**
 * A PrimitiveElement is a field with a primitive type. Type can be
 * boolean | byte | char | short | int | long | float | double |
 * string | serialized| date | time | timestamp
 * A primitive type can not be extended.
 * @author	X. Spengler
 */
public interface PrimitiveElement extends TypedElement {
	/**
	 * Allows to know if the current Field is a scalar field or not.
	 *
	 * @return	true, if the Field is scalar, else false, if the Field is not
	 *          scalar
	 */
	boolean isScalar();

	/**
	 * Returns a ValueMapping object for the current element object.
	 * If the corresponding object does not exist, null is returned.
	 *
	 * @param mapperName	the name of the mapping element name
	 * @param mappingName	the name of the  mapping name
	 * @return	a ValueMapping object. If there is no value mapping object,
	 *  		null is returned.
	 */
	ValueMapping getValueMapping(String mapperName, String mappingName);

	/**
	 * Creates a new ValueMapping object for the current element.
	 * This mapping is defined by the name of the mapper, and the name of the
	 * mapping. If the object already exists, it is returned.
	 *
	 * @param mapperName	the name of the mapper name
	 * @param mappingName	the name of the  mapping name
	 * @return	a new ValueMapping object. This object is dependent from
	 *		    the mapper.
	 */
	ValueMapping createValueMapping(String mapperName, String mappingName);

	/**
	 * Allows to know all the ValueMapping object used for the current element.
	 *
	 * @return	an iterator object on ValueMapping object. If no ValueMapping
	 *	    	is declared, an empty iterator is returned.
	 */
	Iterator iterateValueMapping();
}
