/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.api;

import java.util.Iterator;

/**
 * Reference interface is the top level of the Reference objects.
 * The sub-classes are ClassRef and GenClassRef.
 * @author	X. Spengler
 */
public interface Reference extends TypedElement {
	/**
	 * Creates a new NameDef object for the current reference class object.
	 * If this NameDef already exists for the current object, it is returned,
	 * in other case, a new NameDef object is created and returned.
	 *
	 * @param nameDefName   the name of the nameDef read in the persistence
	 *                      description file
	 * @param isSystem	    true, the namedef is defined outside jorm, and
	 *                      false, the namedef is define inside jorm
	 * @return	a new object used to describe the name projection for
	 *	      	the current reference class, or an existing one if it
	 *          already exists
	 */
	NameDef createNameDef();

	/**
	 * Returns an iterator on existing NameDef for the current object.
	 * If no NameDef object is defined, an empty iterator is returned.
	 *
	 * @return	an iterator on NameDef object
	 */
	Iterator iterateNameDef();

	/**
	 * Returns a NameDef object from a given mappingName.
	 * If the corresponding NameDef exists, it is returned, in other case, null
	 * is returned.
	 *
	 * @param mappingName   the name of the mapping
	 * @return  an existing namedef object if exists, either null
	 */
	NameDef getNameDef(String mappingName);

}
