/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.api.rdb;

import org.objectweb.jorm.metainfo.api.ValueMapping;

import java.util.Iterator;

/**
 * a RDBValueMapping defines all the columns which compose
 * a value mapping.
 * @author	X. Spengler
 */
public interface RDBValueMapping extends ValueMapping {
	/**
	 * Creates a new RDBColumnSpec for the current RDBValueMapping object.
	 * If the column already exists, it is returned.
	 *
	 * @param columnName	the name of the column
	 * @param type	the type of the column
	 * @param notNull	true, if "notnull" is defined, else false
	 * @return	a new RDBColumnSpec object or if the column already exists,
	 *          it is returned
	 */
	RDBColumnSpec createRDBColumnSpec(String columnName,
									  String type,
									  boolean notNull);

	/**
	 * Returns the RDBColumnSpec object corresponding to its name.
	 * If the column object does not exist, null is returned.
	 *
	 * @param columnName	the name of the RDBColumnSpec object (column name)
	 * @return	the corresponding RDBColumnSpec object or null if it does
	 *	    	not exist
	 */
	RDBColumnSpec getRDBColumnSpec(String columnName);

	/**
	 * Allows to know all the RDBColumnSpec defined in the current
	 * RDBValueMapping object.
	 * If no object is defined, an empty iterator is returned. For the moment,
	 * only one element is returned. This iterator contains RDBColumnSpec
	 * objects.
	 *
	 * @return	an iterator on RDBColumnSpec object
	 */
	Iterator iterateRDBColumnSpec();
}
