/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.jorm.util.api.Loggable;

/** * BasicClassMapping is THE basic implementation of the ClassMapping interface. * Its defines basic methods, and must be extended to different mappers.
 * @author	X. Spengler */
public class BasicClassMapping implements ClassMapping, Loggable {
	/**	 * the name which represents the mapper	 */
	private String mapperName;

	/**
    * the name of mapping
	*/
	private String mappingName;

	/**	 * the parent object of the current object	 */
	public MetaObject parent;

	/**
	 * a simple logger to log
	 */
	public Logger logger;

	/**
	 * a logger factory to create other loggers if needed
	 */
	public LoggerFactory loggerFactory;

	/**
	 * Builds a new BasicClassMapping.
	 * This object is defined by the name of the mapper, the name of the mapping
	 * and its parent.
	 *
	 * @param mapperName	the name of the mapper
	 * @param mappingName	the name of the class mapping class
	 * @param parent	the parent of the current object
	 */
	public BasicClassMapping(String mapperName,
							 String mappingName,
							 MetaObject parent) {
		this.mapperName = mapperName;
		this.mappingName = mappingName;
		this.parent = parent;
	}

	///////////////////////////////////////////////////////////////////
	//                      from ClassMapping interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the complete name of the mapper used by the current class
	 *
	 * @return	the string representation of the mapper name
	 */
	public String getMapperName() {
		return mapperName;
	}

	/**
	 * Returns the name of the mapping used by the current class.
	 *
	 * @return	the string representation of the mapping name
	 */
	public String getMappingName() {
		return mappingName;
	}

	///////////////////////////////////////////////////////////////////
	//                      from MetaObject interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Allows to know the parent MetaObject of the current MetaObject.
	 *
	 * @return	the MetaObject corresponding to the parent of the
	 *		    current object. If there is no parent, null is returned.
	 */
	public MetaObject getParent() {
		return parent;
	}

	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 *
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Defines the logger factory to obtain new logger.
	 *
	 * @param loggerFactory	the LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerFactory) {
		this.loggerFactory = loggerFactory;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		 return loggerFactory;
	}
}