/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.type.api.PType;

/**
 * A ScalarField describes corresponds to a hidden field of a ClassObject or
 * a Generic Class. For Classes, it is typically used to represent PNames. For
 * Generic Class, it can also be used for the index attributes of the
 * Generic Class.
 * @author	X. Spengler
 */
public class BasicScalarField extends BasicPrimitiveElement implements ScalarField {

    /**
     * This string represents the null value for the field.
     */
    private String nullvalue = null;

    /**
	 * Builds a new BasicScalarField object.
	 *
	 * @param name  the name of the scalar field
	 * @param type  the PType of the field
	 * @param paren the parent meta-object of the current field
	 */
	public BasicScalarField(String name, PType type, MetaObject parent) {
		super(name, type, parent);
	}

    /**
       * Set the null value for this field.
       * @param value a string representation of the nullvalue
       *
       */
      public void setNullValue(String value) {
          nullvalue = value;
      }

      /**
       * get the null value for this field.
       * @return the string representation of the null value or null if there is no null value for this field.
       */
      public String getNullValue() {
          return nullvalue;
      }

      /**
       * Test if the null value exists for this field.
       * @return true if the null value exist for this field.
       */
      public boolean hasNullValue() {
          return (nullvalue != null);
      }

}
