/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.metainfo.lib.memory;

import org.objectweb.jorm.metainfo.api.MappingFactory;
import org.objectweb.jorm.metainfo.api.ValueMapping;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.lib.BasicValueMapping;
import org.objectweb.jorm.metainfo.lib.BasicClassMapping;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 */
public class BasicMemoryMappingFactory implements MappingFactory, Loggable {

	protected Logger logger = null;
	protected LoggerFactory loggerFactory = null;

	/**
	 * Creates a ValueMapping object from the current factory.
	 * This object is created with a mapper name, a mapping name and its
	 * futur parent.
	 *
	 * @param mapperName    the name of the mapper
	 * @param mappingName   the name of the mapping
	 * @return  a ValueMapping object
	 */
	public ValueMapping createValueMapping(String mapperName,
										   String mappingName,
										   MetaObject parent) {
		return new BasicValueMapping(mapperName, mappingName, parent);
	}

	/**
	 * Creates a ClassMapping object from the current factory.
	 * This object is created with a mapper name, a mapping name and its
	 * futur parent.
	 *
	 * @param mapperName    the name of the mapper
	 * @param mappingName   the name of the mapping
	 * @return  a ClassMapping object
	 */
	public ClassMapping createClassMapping(String mapperName,
										   String mappingName,
										   MetaObject parent) {
		return new BasicClassMapping(mapperName, mappingName, parent);
	}

	/**
	 * Initializes the current factory.
	 */
	public void init() {
	}

	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
		if (loggerFactory!=null && logger==null)
			loggerFactory.getLogger(getClass().getName());
	}

}
