/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.lib.rdb;

import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.rdb.RDBColumnSpec;

/**
 * BasicRDBColumnSpec defines one column in the table where the class
 * is projected.
 * @author	X. Spengler
 */
public class BasicRDBColumnSpec implements RDBColumnSpec {
	/**
	 * the name of the column
	 */
	private String name;

	/**
	 * the type of the column
	 */
	private String type;

	/**
	 * defines if the column can be null or not
	 */
	private boolean notNull;

	/**
	 * the parent of the current object
	 */
	private MetaObject parent;

	/**
	 * Builds a new BasicRDBColumnSpec.
	 *
	 * @param name  the name of the column
	 * @param type  the type of the column
	 * @param notNull   true, if the column can not be null, else false, if the
	 *                  column can be null
	 * @param parent    the parent of the current object
	 */
	public BasicRDBColumnSpec(String name,
							  String type,
							  boolean notNull,
							  MetaObject parent) {
		this.name = name;
		this.type = type;
		this.notNull = notNull;
		this.parent = parent;
	}

	///////////////////////////////////////////////////////////////////
	//                      from RDBColumnSpec interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the name of the current column.
	 *
	 * @return	the string representation of column name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the SQL type of the current object.
	 *
	 * @return	the string representation of the sql type
	 */
	public String getType() {
		return type;
	}

	/**
	 * Allows to know if the column could be notnull or not.
	 *
	 * @return	true, if the column could be notnull, else false
	 */
	public boolean isNotNull() {
		return notNull;
	}
}
