/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.metainfo.lib.rdb;

import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.ValueMapping;
import org.objectweb.jorm.metainfo.api.rdb.RDBMappingFactory;
import org.objectweb.jorm.metainfo.api.rdb.RDBValueMapping;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * The BasicRDBMappingFactory object defines method to get mapping objects.
 * Mapping objects are ValueMapping objects and ClassMapping objects.
 * @author	X. Spengler
 */
public class BasicRDBMappingFactory implements RDBMappingFactory {
	/**
	 * this arrays contains differents set of mappingName/mapperName tuple
	 * for ValueMapping object.
	 * element: a ValueMapping object
	 */
	private ArrayList mappsValue;

	/**
	 * this arrays contains differents set of mappingName/mapperName tuple
	 * for ClassMapping object.
	 * element: a ClassMapping object
	 */
	private ArrayList mappsClass;

	///////////////////////////////////////////////////////////////////
	//                      from MappingFactory interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Initializes the current factory.
	 */
	public void init() {
		mappsValue = new ArrayList();
		mappsClass = new ArrayList();
	}

	/**
	 * Creates a ValueMapping object from the current factory.
	 * This object is created with a mapper name, a mapping name and its
	 * futur parent.
	 *
	 * @param mapperName    the name of the mapper
	 * @param mappingName   the name of the mapping
	 * @return  a ValueMapping object
	 */
	public ValueMapping createValueMapping(String mapperName,
										   String mappingName,
										   MetaObject parent) {
		Iterator iter = mappsValue.iterator();
		while (iter.hasNext()) {
			mapp amap = (mapp) iter.next();
			if ((mapperName.compareTo(amap.mapper) == 0) &&
				(mappingName.compareTo(amap.mapping) == 0) &&
				(parent.equals(amap.metaObject.getParent()))) {
				return (ValueMapping) amap.metaObject;
			}
		}
		// not in memory, build it
		mapp amap = new mapp();
		amap.mapper = mapperName;
		amap.mapping = mappingName;
        RDBValueMapping obj = new BasicRDBValueMapping(mapperName,
				mappingName,
				parent);
		amap.metaObject = obj;
		mappsValue.add(amap);
		return (ValueMapping) amap.metaObject;
	}

	/**
	 * Creates a ClassMapping object from the current factory.
	 * This object is created with a mapper name, a mapping name and its
	 * futur parent.
	 *
	 * @param mapperName    the name of the mapper
	 * @param mappingName   the name of the mapping
	 * @return  a ClassMapping object
	 */
	public ClassMapping createClassMapping(String mapperName,
										   String mappingName,
										   MetaObject parent) {

		Iterator iter = mappsClass.iterator();
		while (iter.hasNext()) {
			mapp amap = (mapp) iter.next();
			if ((mapperName.compareTo(amap.mapper) == 0) &&
				(mappingName.compareTo(amap.mapping) == 0) &&
				(parent.equals(amap.metaObject.getParent()))) {
				return (ClassMapping) amap.metaObject;
			}
		}

		// not in memory, build it
		mapp amap = new mapp();
		amap.mapper = mapperName;
		amap.mapping = mappingName;
		amap.metaObject =
			new BasicRDBClassMapping(
				mapperName,
				mappingName,
				parent);
		mappsClass.add(amap);
		return (ClassMapping) amap.metaObject;
	}

	/**
	 * the mapp object represents a simple tuple of mappingName/mapperName
	 * Strings.
	 */
	public class mapp {
		/**
		 * the name of the mapping
		 */
		public String mapping;

		/**
		 * the name of the mapper
		 */
		public String mapper;

		public MetaObject metaObject;
	}
}

