/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
/**

 * Copyright

 */

package org.objectweb.jorm.parser.lib;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import java.util.Vector;

/**

 * The ParserErrorHandler object extends the DefaultHandler from sax.

 * This object is given to the DOM parser to handle potential errors.
 * @author	X. Spengler
 */
public class ParserErrorHandler extends DefaultHandler {
	/**

	 * a vector of errors

	 */
	private Vector errors;

	/**

	 * Builds a new ParserErrorHandler for the DOM parser.

	 */
	public ParserErrorHandler() {
		errors = new Vector();
	}

	/**

	 * Allows to know the number of handled errors.
	 *

	 * @return	the current number of errors

	 */
	public int numErrors() {
		return errors.size();
	}

	/**

	 * Returns the vector of errors.
	 *

	 * @return	the current vector of errors

	 */
	public Vector getErrors() {
		return errors;
	}

	/**

	 * Resets the vector of errors.

	 */
	public void initialize() {
		errors.clear();
	}

	/**

	 * Adds a new exception to the current vector of exceptions.
	 *

	 * @param e	a new SAXParseException to register

	 */
	public void warning(SAXParseException e) throws SAXException {
		errors.add(e.toString());
	}

	/**

	 * Adds a new exception to the current vector of exceptions.
	 *

	 * @param e	a new SAXParseException to register

	 */
	public void error(SAXParseException e) throws SAXException {
		errors.add(e.toString());
	}
}

