/**
 * Copyright (C) 2002
 */

package org.objectweb.jorm.parser.lib.file;

import org.objectweb.jorm.parser.api.MappingParser;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.io.api.PathExplorer;
import org.w3c.dom.Element;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class FileParser implements MappingParser, Loggable {

	protected Logger logger = null;
	protected LoggerFactory loggerFactory = null;
	protected PathExplorer pathExplorer = null;
	protected String mapperName = null;
	protected String mappingName = null;
	protected Manager metaInfoManager = null;

	// IMPLEMENTATION OF THE MappingParser INTERFACE //
	//-----------------------------------------------//

	/**
	 * Parses the mapping part of the XML file and builds the corresponding
	 * meta-information for a given Class object.
	 * Manager, PathExplorer, MapperName and MappingName must be set before the
	 * call of this method.
	 *
	 * @param node	    the node to treat
	 * @param class_    the class to complete
	 */
	public void parseMapping(Element node, Class class_) {
		logger.log(BasicLevel.INFO, "node:" + node);
	}

	/**
	 * Parses the mapping part of the XML file and builds the corresponding
	 * meta-information for a given PrimitiveElement object.
	 * Manager, PathExplorer, MapperName and MappingName must be set before the
	 * call of this method.
	 *
	 * @param node              the node to treat
	 * @param primitiveElement  the primitive element to complete
	 */
	public void parseMapping(Element node, PrimitiveElement primitiveElement) {
		logger.log(BasicLevel.INFO, "node:" + node);
	}

	/**
	 * Parses the mapping part of the XML file and builds the corresponding
	 * meta-information for a given GenClassRef object.
	 * Manager, PathExplorer, MapperName and MappingName must be set before the
	 * call of this method.
	 *
	 * @param node          the node to treat
	 * @param genClassRef   the generic class reference to complete
	 */
	public void parseMapping(Element node, GenClassRef genClassRef) {
		logger.log(BasicLevel.INFO, "node:" + node);
	}

	/**
	 * Assigns a meta-information manager to a parser in order from it to
	 * create the meta-information related to the XML files it parses.
	 *
	 * @param mim	the meta-information Manager to be associated to this
	 * 				parser
	 */
	public void setMetaInfoManager(Manager mim) {
		metaInfoManager = mim;
	}

	/**
	 * Assigns a PathExplorer object for locating files that have to be parsed.
	 *
	 * @param pathexpl	The PathExplorer to be used for file location
	 */
	public void setPathExplorer(PathExplorer pe) {
		pathExplorer = pe;
	}

	/**
	 * Sets the name of the mapper to use.
	 * This name is passed on the command line or defined in a properties file.
	 *
	 * @param mapperName	the name of the mapper
	 */
	public void setMapperName(String mn) {
		mapperName = mn;
	}

	/**
	 * Sets the name of the mapping to use.
	 * This name is passed on the command line or defined in a properties file.
	 *
	 * @param mappingName	the name of the mapping
	 */
	public void setMappingName(String mn) {
		mappingName = mn;
	}

	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
		if (logger==null)
			loggerFactory.getLogger(getClass().getName());
	}

}
