/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.runtime.mapper.rdb.adapter.api;

import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.type.api.PType;

import java.sql.ResultSet;
import java.sql.SQLException;

/*
 * This interface permits to do the conversion between the jorm types PType and
 * the sql types. It is also possible to fetch the function name of getter and
 * setter methods associated to a PType.
 *
 * @author : S.Chassande-Barrioz
 */

public interface RdbAdpater {
	/**
	 * This methods return the SQL type linked to the PType
	 * specified in parameter.
	 */
	String getSqlType(PType pt) throws PExceptionProtocol;

	/**
	 * This methods return the name of the sql getter method linked to the PType
	 * specified in parameter.
	 */
	String getSqlGetterName(PType pt) throws PExceptionProtocol;

	/**
	 * This methods return the name of the sql setter method linked to the PType
	 * specified in parameter.
	 */
	String getSqlSetterName(PType pt) throws PExceptionProtocol;

	/**
	 * This methods return the SQL type code linked to the PType
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	int getSqlTypeCode(PType pt) throws PExceptionProtocol;

	/**
	 * This calculates the size of a result set. If this feature is not supported
	 * by a database or its jdbc driver, this method returns 0.
	 * @ResultSet rs the result set which the size must be returned
	 */
	int fetchResultSetSize(ResultSet rs) throws SQLException;
}
