/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.runtime.mapper.rdb.adapter.lib;

import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.runtime.mapper.rdb.adapter.api.RdbAdpater;
import org.objectweb.jorm.type.api.PType;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

/*
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */

public class BasicRdbAdapter implements RdbAdpater {
	/**
	 * This methods return the SQL type code linked to the PType
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	public int getSqlTypeCode(PType pt) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("Invalid parameter");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return Types.BIT;
		case PType.TYPECODE_CHAR:
			return Types.CHAR;
		case PType.TYPECODE_BYTE:
			return Types.TINYINT;
		case PType.TYPECODE_SHORT:
			return Types.SMALLINT;
		case PType.TYPECODE_INT:
		case PType.TYPECODE_INT_OBJ:
			return Types.INTEGER;
		case PType.TYPECODE_LONG:
			return Types.BIGINT;
		case PType.TYPECODE_FLOAT:
			return Types.REAL;
		case PType.TYPECODE_DOUBLE:
			return Types.DOUBLE;
		case PType.TYPECODE_STRING:
			return Types.VARCHAR;
		case PType.TYPECODE_DATE:
			return Types.DATE;
/*		case PType.TYPECODE_TIME:
			return Types.TIME;
		case PType.TYPECODE_TIMESTAMP:
			return Types.TIMESTAMP;
*/
		case PType.TYPECODE_SERIALIZED:
			return Types.BINARY;
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}

	public String getSqlType(PType pt) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("Invalid parameter");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "BIT";
		case PType.TYPECODE_CHAR:
			return "CHAR";
		case PType.TYPECODE_BYTE:
			return "TINYINT";
		case PType.TYPECODE_SHORT:
			return "SMALLINT";
		case PType.TYPECODE_INT:
		case PType.TYPECODE_INT_OBJ:
			return "INTEGER";
		case PType.TYPECODE_LONG:
			return "BIGINT";
		case PType.TYPECODE_FLOAT:
			return "REAL";
		case PType.TYPECODE_DOUBLE:
			return "DOUBLE";
		case PType.TYPECODE_STRING:
			return "VARCHAR";
		case PType.TYPECODE_DATE:
			return "DATE";
/*		case PType.TYPECODE_TIME:
			return "TIME";
		case PType.TYPECODE_TIMESTAMP:
			return "TIMESTAMP";
*/
		case PType.TYPECODE_SERIALIZED:
			return "BINARY";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}

	public String getSqlGetterName(PType pt) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("Invalid parameter");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "getBoolean";
		case PType.TYPECODE_CHAR:
			return "getString";
		case PType.TYPECODE_BYTE:
			return "getByte";
		case PType.TYPECODE_SHORT:
			return "getShort";
		case PType.TYPECODE_INT:
			return "getInt";
		case PType.TYPECODE_LONG:
			return "getLong";
		case PType.TYPECODE_FLOAT:
			return "getFloat";
		case PType.TYPECODE_DOUBLE:
			return "getDouble";
		case PType.TYPECODE_STRING:
			return "getString";
		case PType.TYPECODE_DATE:
			return "getDate";
		case PType.TYPECODE_SERIALIZED:
        case PType.TYPECODE_INT_OBJ:
			return "getObject";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}

	public String getSqlSetterName(PType pt) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("Invalid parameter");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "setBoolean";
		case PType.TYPECODE_CHAR:
			return "setString";
		case PType.TYPECODE_BYTE:
			return "setByte";
		case PType.TYPECODE_SHORT:
			return "setShort";
		case PType.TYPECODE_INT:
			return "setInt";
		case PType.TYPECODE_LONG:
			return "setLong";
		case PType.TYPECODE_FLOAT:
			return "setFloat";
		case PType.TYPECODE_DOUBLE:
			return "setDouble";
		case PType.TYPECODE_STRING:
			return "setString";
		case PType.TYPECODE_DATE:
			return "setDate";
/*
		case PType.TYPECODE_TIME:
			return "setTime";
		case PType.TYPECODE_TIMESTAMP:
			return "setTimeStamp";
*/
		case PType.TYPECODE_SERIALIZED:
        case PType.TYPECODE_INT_OBJ:
			return "setObject";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
				+ pt.getJormName() + " PType");
		}
	}

	/**
	 * This calculates the size of a result set. If this feature is not supported
	 * by a database or its jdbc driver, this method returns 0.
	 * @ResultSet rs the result set which the size must be returned
	 */
	public int fetchResultSetSize(ResultSet rs) throws SQLException {
		return rs.getFetchSize();
	}
}
