/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.runtime.mapper.rdb.adapter.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.comp.cmdline.rdb.RdbMapperOption;
import org.objectweb.jorm.runtime.mapper.rdb.adapter.api.RdbAdpater;

/**
 * This factory permits to create new instances of RdbAdpater.
 *
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */
public class RdbAdapterFactory {

	/**
	 * This constant represents the data base name for a standard jdbc database
	 */
	public static final String DATABASE_NAME_JDBC = "jdbc";

	/**
	 * This constant represents the data base name for the oracle database
	 */
	public static final String DATABASE_NAME_ORACLE = "oracle";

	/**
	 * This constant represents the data base name for the postgres database
	 */
	public static final String DATABASE_NAME_POSTGRES = "postgres";

	/**
	 * This first possibility permits to specify the name for which a
	 * RdbAdpater is needed.
	 * @dbn : the data base name
	 * @return : the type converter linked to the data base name
	 * @exception PException : if the parameter is null or if no RdbAdpater
	 * is associated to the name, a PException is throwing.
	 */
	public RdbAdpater getTypeConverter(String dbn) throws PException {
		if (dbn == null)
			throw new PException("No RdbAdpater associated to a null name");
		if (dbn.length() == 0)
			return new BasicRdbAdapter();
		if (dbn.equals(DATABASE_NAME_JDBC))
			return new BasicRdbAdapter();
		if (dbn.equals(DATABASE_NAME_ORACLE))
			return new BasicRdbAdapter();
		if (dbn.equals(DATABASE_NAME_POSTGRES))
			return new PostgresAdapter();
		throw new PException(
			"No RdbAdpater associated to the following name: " + dbn);
	}

	/**
	 * This first possibility permits to specify a compiler parameter which
	 * containsthe all needed information to determine which a RdbAdpater
	 * must be returned.
	 * @dbn : The compiler parameter which contains information
	 * @return : the type converter linked to the compiler parameter
	 * @exception PException : if the parameter is null or if no RdbAdpater
	 * is associated to the compiler parameter, a PException is throwing.
	 */
	public RdbAdpater getTypeConverter(CompilerParameter cp)
		throws PException {
		RdbMapperOption mo = (RdbMapperOption)
        //cp.getCPExtension(RdbMapperOption.SINGLE_CLASS_NAME);
			cp.getCPExtension(RdbMapperOption.CLASS_NAME);
		if (mo==null)
			throw new PException(
				"No RdbAdapter available: you must specify the extension "
				+ RdbMapperOption.CLASS_NAME + " option for the rdb mapper");
		return getTypeConverter(mo.getDataBaseName());
	}
}
