/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.type.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;

/**
 * This implementation of the PTypeconsiders that this is the PType which
 * conatains its ancestor. The PAAH letters mean PType As Ancestor Holder.
 *
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */

public class PTypePAAH extends PType {
	protected PType superTypes[] = null;

	/**
	 * It constructs the types that represent JORM classes.
	 * @param classname		The name of the class for which a PType must be
	 * 						constructed.
	 * @param typespace		The type space to which the constructed PType
	 * 						belongs.
	 * @param supertypes	The super types of that class.
	 */
	public PTypePAAH(String classname, PTypeSpace typespace,
					 PType[] supertypes) {
		super(classname, typespace);
		superTypes = supertypes;
	}

	/**
	 * It retrieves the PType directly inherited by a PType representing a
	 * JORM class.
	 * @return	The array of directly inherited PType.
	 */
	public PType[] getInheritedPType() {
		return superTypes;
	}
}
