/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.util.lib;

import org.objectweb.util.monolog.api.Logger;

import java.util.Iterator;
import java.util.Vector;

/**
 * This object is used to manage errors from the compiler.
 * @author	X. Spengler
 */
public class CompilerError {
	/**
	 * a vector of errors (String messages)
	 */
	private Vector errors;

	/**
	 * a logger object to log messages
	 */
	private Logger logger;

	/**
	 * Builds a new CompilerError object
	 */
	public CompilerError() {
		errors = new Vector();
		logger =
			CompilerLogger.getLoggerFactory().getLogger("org.objectweb.jorm.util");
	}

	/**
	 * Adds an error message to the list of messages.
	 *
	 * @param line  the message to add
	 */
	public void addError(String line) {
		errors.add(line);
	}

	/**
	 * Allows to know all the error message registered by the compiler.
	 *
	 * @return  an iterator of error messages (String objects)
	 */
	public Iterator iterateError() {
		return errors.iterator();
	}
}
