/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.util.lib;

import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.HandlerFactory;
import org.objectweb.util.monolog.api.LevelFactory;
import org.objectweb.util.monolog.api.TopicalLogger;
import org.objectweb.util.monolog.wrapper.common.Configurable;
import org.objectweb.util.monolog.file.monolog.PropertiesConfAccess;

import java.util.Properties;
import java.util.Iterator;
import java.util.Map;
import java.io.FileInputStream;

/**
 * The CompilerLogger object defines a static LoggerFactory object to create
 * logger. This object can be configured by a properties file.
 * @author	X. Spengler
 */
public class CompilerLogger {
	/**
	 * the logger factory object which creates logger object.
	 */
	private static LoggerFactory mf;

	/**
	 * Allows to know the logger factory used by jorm.
	 *
	 * @return  a static LoggerFactory object
	 */
	public static LoggerFactory getLoggerFactory() {
		return mf;
	}

	/**
	 * Configures the logger factory object with a properties object.
	 *
	 * @param properties    a properties object
	 */
	public static void configure(Properties properties) {
		debug("CompilerLogger.configure(..)");
		if (mf == null) {
			String classname =
				properties.getProperty("log.config.classname", null);
			if (classname != null) {
				debug("logger factory classname=" + classname);
				try {
					mf = (LoggerFactory) Class.forName(classname).newInstance();
					debug("loggerFactory=" + mf);
				}
				catch (Exception e) {
					e.printStackTrace(System.out);
				}
			}
		}

		if (mf != null) {
			try {
				debug("log configuration loading:" + properties);
				Properties p = properties;
				String fn = properties.getProperty(
					Configurable.LOG_CONFIGURATION_FILE, null);
				if (fn!=null) {
                    debug("dans fndifferente null");
					String useCP = properties.getProperty(
					Configurable.LOG_CONFIGURATION_FILE_USE_CLASSPATH, null);
					p = new Properties();
					if (useCP!=null && "true".equalsIgnoreCase(useCP))
						p.load(ClassLoader.getSystemResourceAsStream(fn));
					else
						p.load(new FileInputStream(fn));
				}
				debug("log configuration loading from:" + p);
				PropertiesConfAccess.load(p, mf, (HandlerFactory) mf,
					(LevelFactory) mf);
				debug("rootH="
					+ ((TopicalLogger)mf.getLogger("")).getHandler("consoleHandler"));
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private static void debug(String m) {
		//System.out.println(m);
	}
}
